# -*- coding: utf-8 -*-
"""
Created on Fri Nov  4 11:27:30 2022

@author: wjwong

Additional functions needed to calculate Gioncu's (1999) analytical solution
"""

import numpy as np
import functions as fn

def calc_chi2(b1,dl,d):
    """
    chi2 
    b1 half of flange
    dl distance from rotating pivot in hinge to bottom face of top flange
    d depth of web
    """
    return b1/dl/d

def calc_rho(Yf,Yw):
    """
    rho ratio of web to flange yield stress
    Yf flange yield stress
    Yw web yield stress
    """
    return Yw/Yf

def calc_A2(chi2,rho,b1,b2,d,tw,tf2):
    """
    A2 coefficient in moment-rotation formula for asymmetrical mode (relating to tf2)
    chi2 geometrical parameter relating to position of pivot
    rho ratio of web to flange yield stress
    b1 half of top flange width
    b2 half of bottom flange width
    d depth of web
    tw thickness of web
    tf2 thickness of bottom flange
    """ 
    return 1/4*rho*(1-b1/chi2/d)**2*tw/d \
        + (1-b1/chi2/d)*b2/b1*tf2/d \
            + 1/4*b2/b1*tf2**2/d**2
        
def calc_om1(b1,l):
    """
    om1 geometrical parameter relating to length of beam
    b1 half of top flange width
    l total length of beam
    """
    return np.sqrt(1 - 2*b1/l)
    
def calc_B2(chi2,rho,b1,d,tw,tf1,om1,l):
    """
    B2 coefficient in moment-rotation formula for asymmetrical mode (relating to tf1 and om1)
    chi2 geometrical parameter relating to position of pivot
    rho ratio of web to flange yield stress
    b1 half of top flange width
    b2 half of bottom flange width
    d depth of web
    tw thickness of web
    tf1 thickness of top flange
    om1 geometrical parameter relating to length of beam
    l total length of beam
    """
    return 1/2/om1/np.sqrt(chi2)*b1*tf1/d**2 \
        + 1/2*rho/chi2**(3/2)*(1+l/2/b1+l**2/2/b1**2*chi2**2)*b1*tw**2/d**2/l

def calc_B3(chi2,d,tf1,om1):
    """
    B3 coefficient in moment-rotation formula for asymmetrical mode (relating to tf1 and om1)
    chi2 geometrical parameter relating to position of pivot
    d depth of web
    tf1 thickness of top flange
    om1 geometrical parameter relating to length of beam
    """
    return 3/2/np.sqrt(om1)/chi2**(1/4)*tf1**2/d**2

def calc_M_norm_asym(chi2,th,b1,b2,d,tw,tf1,tf2,om1,l,Yf,Yw):
    tf = tf1 # section is doubly symmetric
    b = b1 # section is doubly symmetric
    B = 2*b # total flange width
    H = d + 2*tf
    rho = calc_rho(Yf,Yw)
    return b1*d**2*Yf / fn.calc_Mpl(Yf,Yw,tf,tw,B,H) \
        * (calc_A2(chi2,rho,b1,b2,d,tw,tf2) \
            + calc_B2(chi2,rho,b1,d,tw,tf1,om1,l)/np.sqrt(th) \
                + calc_B3(chi2,d,tf1,om1)/th**(3/4))

def calc_Ix(tf,tw,b,d):
    return tw*d**3/12+b*tf*(d+tf)**2+b*tf**3/3

def calc_Tp(Mp,L,E,Ix):
    return Mp*L/E/Ix

def calc_T_norm(T,Tp):
    return T/Tp

def calc_A3(chi2,rho,b1,b2,d,tw,tf2):
    """
    A3 coefficient in moment-rotation formula for symmetrical mode (relating to tf2)
    chi2 geometrical parameter relating to position of pivot
    rho ratio of web to flange yield stress
    b1 half of top flange width
    b2 half of bottom flange width
    d depth of web
    tw thickness of web
    tf2 thickness of bottom flange
    """ 
    return 1/2*rho*(1-b1/chi2/d)**2*tw/d \
        + 2*(1-b1/chi2/d)*b2/b1*tf2/d \
            + 1/2*b2/b1*tf2**2/d**2

def calc_om2(b1,l):
    """
    om2 geometrical parameter relating to length of beam
    b1 half of top flange width
    l total length of beam
    """
    return np.sqrt((1-6*b1/l)/(1-2*b1/l))

def calc_B4(chi2,rho,b1,d,tw,tf1,om2,l):
    """
    B4 coefficient in moment-rotation formula for symmetrical mode (relating to tf1 and om2)
    chi2 geometrical parameter relating to position of pivot
    rho ratio of web to flange yield stress
    b1 half of top flange width
    b2 half of bottom flange width
    d depth of web
    tw thickness of web
    tf1 thickness of top flange
    om1 geometrical parameter relating to length of beam
    l total length of beam
    """
    return om2/np.sqrt(chi2)/(1-6*b1/l)*b1*tf1/d**2 \
        + rho*om2/chi2**(3/2)*((1+l/2/b1)*chi2+(1+1.5*l*chi2**2/b1)/(0.5*l/b1-3)/chi2)*tw**2/d**2

def calc_B5(chi2,d,tf1,om2):
    """
    B5 coefficient in moment-rotation formula for symmetrical mode (relating to tf1 and om2)
    chi2 geometrical parameter relating to position of pivot
    d depth of web
    tf1 thickness of top flange
    om1 geometrical parameter relating to length of beam
    """
    return 3/2/np.sqrt(om2)/chi2**(1/4)*(1+1/om2)*tf1**2/d**2

def calc_M_norm_sym(chi2,th,b1,b2,d,tw,tf1,tf2,om2,l,Yf,Yw):
    tf = tf1 # section is doubly symmetric
    b = b1 # section is doubly symmetric
    B = 2*b # total flange width
    H = d + 2*tf
    rho = calc_rho(Yf,Yw)
    return b1*d**2*Yf / fn.calc_Mpl(Yf,Yw,tf,tw,B,H) \
        * (calc_A3(chi2,rho,b1,b2,d,tw,tf2) \
            + calc_B4(chi2,rho,b1,d,tw,tf1,om2,l)/np.sqrt(th) \
                + calc_B5(chi2,d,tf1,om2)/th**(1/4))
            
def calc_beta(tf,tw,d,b):
    return 0.6*(tf/tw)**(3/4)*(d/b)**(1/4)

def calc_chi1(beta,b,dl,d):
    return beta*b/dl/d
    
def calc_A1(chi1,rho,b1,b2,d,tw,tf1,tf2):
    """
    A1 coefficient in moment-rotation formula for in-plane mode
    chi1 geometrical parameter relating to position of pivot
    rho ratio of web to flange yield stress
    b1 half of top flange width
    b2 half of bottom flange width
    d depth of web
    tw thickness of web
    tf2 thickness of bottom flange
    """ 
    beta = calc_beta(tf1,tw,d,b1)
    if beta<=1:
        X = beta**2*chi1
    elif beta>1:
        X = (2*beta-1)*chi1
        
    return 1/2*rho*(beta**2*b1**2/chi1**2/d**2+(1-beta*b1/chi1/d)**2)*tw/b1 \
        + 2*(1-beta*b1/chi1/d)*b2/b1*tf2/d \
            + 1/2*b2/b1*tf2**2/d**2 \
                + b1*tf1/d**2*X
                
def calc_B1(chi1,rho,b1,d,tw,tf1,alpha):
    """
    B1 coefficient in moment-rotation formula for asymmetrical mode (relating to tf1 and om1)
    chi1 geometrical parameter relating to position of pivot
    rho ratio of web to flange yield stress
    b1 half of top flange width
    b2 half of bottom flange width
    d depth of web
    tw thickness of web
    tf1 thickness of top flange
    om1 geometrical parameter relating to length of beam
    l total length of beam
    """
    beta = calc_beta(tf1,tw,d,b1)
    if beta<=1:
        X1 = 1
    elif beta>1:
        X1 = np.sqrt(beta)
    if beta<=1:
        X2 = (beta+2)/np.sqrt(chi1)
    elif beta>1:
        X2 = 3*np.sqrt(beta/chi1)
        
    return 1/2*rho*beta/chi1**(3/2) * \
        (((2+chi1)*(1+chi1**2*alpha**2*beta**2)-chi1*(2+chi1**2))/np.sqrt(1-2*chi1+(1-alpha**2*beta**2)*chi1**2) \
            + chi1*(np.sqrt(1-2*chi1+(1-alpha**2/beta**2)*chi1**2)-X1))*tw**2/d**2 \
            + tf1**2/d**2*X2
            
def calc_M_norm_in(chi1,th,b1,b2,d,tw,tf1,tf2,Yf,Yw,alpha):
    tf = tf1 # section is doubly symmetric
    b = b1 # section is doubly symmetric
    B = 2*b # total flange width
    H = d + 2*tf
    rho = calc_rho(Yf,Yw)
    return b1*d**2*Yf / fn.calc_Mpl(Yf,Yw,tf,tw,B,H) \
        * (calc_A1(chi1,rho,b1,b2,d,tw,tf1,tf2) \
            + calc_B1(chi1,rho,b1,d,tw,tf1,alpha)/np.sqrt(th))