# -*- coding: utf-8 -*-
"""
Created on Fri Nov 11 14:58:00 2022

@author: wjwong
"""

# %% Import modules and tools

import analytic_functions as afn
import parametric_functions as pfn
import functions as fn
import matplotlib.pyplot as plt
import matplotlib.lines as mlines
import numpy as np
from scipy.interpolate import griddata
from scipy import optimize

# For 3D arrows and annotations

from mpl_toolkits.mplot3d.proj3d import proj_transform
from mpl_toolkits.mplot3d.axes3d import Axes3D
from matplotlib.text import Annotation

class Annotation3D(Annotation):

    def __init__(self, text, xyz, *args, **kwargs):
        super().__init__(text, xy=(0, 0), *args, **kwargs)
        self._xyz = xyz

    def draw(self, renderer):
        x2, y2, z2 = proj_transform(*self._xyz, self.axes.M)
        self.xy = (x2, y2)
        super().draw(renderer)
        
def _annotate3D(ax, text, xyz, *args, **kwargs):
    '''Add anotation `text` to an `Axes3d` instance.'''

    annotation = Annotation3D(text, xyz, *args, **kwargs)
    ax.add_artist(annotation)
    
setattr(Axes3D, 'annotate3D', _annotate3D)

from matplotlib.patches import FancyArrowPatch

class Arrow3D(FancyArrowPatch):

    def __init__(self, x, y, z, dx, dy, dz, *args, **kwargs):
        super().__init__((0, 0), (0, 0), *args, **kwargs)
        self._xyz = (x, y, z)
        self._dxdydz = (dx, dy, dz)

    def draw(self, renderer):
        x1, y1, z1 = self._xyz
        dx, dy, dz = self._dxdydz
        x2, y2, z2 = (x1 + dx, y1 + dy, z1 + dz)

        xs, ys, zs = proj_transform((x1, x2), (y1, y2), (z1, z2), self.axes.M)
        self.set_positions((xs[0], ys[0]), (xs[1], ys[1]))
        super().draw(renderer)
        
    def do_3d_projection(self, renderer=None):
        x1, y1, z1 = self._xyz
        dx, dy, dz = self._dxdydz
        x2, y2, z2 = (x1 + dx, y1 + dy, z1 + dz)

        xs, ys, zs = proj_transform((x1, x2), (y1, y2), (z1, z2), self.axes.M)
        self.set_positions((xs[0], ys[0]), (xs[1], ys[1]))

        return np.min(zs) 
    
def _arrow3D(ax, x, y, z, dx, dy, dz, *args, **kwargs):
    '''Add an 3d arrow to an `Axes3D` instance.'''

    arrow = Arrow3D(x, y, z, dx, dy, dz, *args, **kwargs)
    ax.add_artist(arrow)
    
setattr(Axes3D, 'arrow3D', _arrow3D)



# %% Load data into dictionary

batch_names =  ['all runs',
                'all 690 runs',
                'all 890 runs',
                'vary YoT vary Y t_HSA800 default H and B',
                'vary B vary YoT t_HSA800 Y690 Lw_40p0eps',
                'vary B vary YoT t_HSA800 Y890 Lw_40p0eps',
                'vary YoT vary Y t_SM490 default H and B',
                'vary YoT vary Y t_SM490_12mm default H and B',
                'vary H vary YoT t_SM490_12mm Y690 Lf_10p1eps',
                'vary H vary YoT t_SM490_12mm Y890 Lf_10p1eps',
                'vary YoT vary Y t_HSA800 Lw_40p0eps Lf_10p1eps',
                'vary YoT vary Y t_SM490_12mm Lf_10p1eps Lw_52p5eps',
                'vary Y t_HSA800 default H and B YoT_1',
                'vary Y t_SM490 default H and B YoT_1'
                ]

data_dict = {}

for i, batch_name in enumerate(batch_names):
    
    # mypath = r'C:\Users\wjwong\Documents\Research\plastic hinge\parametric-runs\\' + batch_name + '\\'
    # data_dict[batch_name] = pfn.load_data(mypath)

    mypath = r'C:\Users\wjwong\Documents\Research\plastic hinge\parametric-runs\Parametric runs with a = 0.53tw\\' + batch_name + '\\'
        
    # for parameter in pfn.load_data(mypath):
    loaded_data = pfn.load_data(mypath)
    data_dict[batch_name] = {
        'YoT': loaded_data[0],
        'Y': loaded_data[1],
        'Ldp': loaded_data[2],
        'B': loaded_data[3],
        'H': loaded_data[4],
        'tf': loaded_data[5],
        'tw': loaded_data[6],
        'imp': loaded_data[7],
        'Ldf': loaded_data[8],
        'Ldw': loaded_data[9],
        'Mp': loaded_data[10],
        'Tp': loaded_data[11],
        'L': loaded_data[12],
        'R': loaded_data[13],
        'Ldf_eps': loaded_data[14],
        'Ldw_eps': loaded_data[15],
        'Mnu': loaded_data[16],
        'Ru':loaded_data[17],
        'idx_R':loaded_data[18],
        'idx_Ru':loaded_data[19],
        'Mel':loaded_data[20],
        'idx_Mel':loaded_data[21],
        'data_list':loaded_data[22]
        }
# %% Plot specific moment-rotation graphs

YoT = 0.94
Y = 790
B = 220
H = 400
tf = 15
tw = 11
plt.close('all')

def plot_MR(Y,YoT,B,H,tf,tw,ax,this_batch):
    """
    Plot_MR given Y, YoT, B, H, tf, tw
    Outputs the index (frame number) near to which R occurs
    ax: axes to plot to
    """
    
    # initialize
    idx_R = np.NaN
    idx_Ru = np.NaN
    idx_Mel = np.NaN
                
    for i, dataYoT in enumerate(data_dict[this_batch]['YoT']):
        if (dataYoT==YoT 
            and data_dict[this_batch]['Y'][i]==Y 
            and data_dict[this_batch]['B'][i]==B
            and data_dict[this_batch]['H'][i]==H
            and data_dict[this_batch] ['tf'][i]==tf
            and data_dict[this_batch]['tw'][i]==tw):
            
            ax.plot(data_dict[this_batch]['data_list'][i]['UR1'].values/data_dict[this_batch]['Tp'][i],
            data_dict[this_batch]['data_list'][i]['RM1']/data_dict[this_batch]['Mp'][i],
            label=rf'$\sigma_y={Y}; \sigma_y/\sigma_u={YoT}$'.format(Y=Y,YoT=YoT))
            # import pdb; pdb.set_trace()
            
            idx_R = data_dict[this_batch]['idx_R'][i]
            if ~np.isnan(idx_R):
                ax.plot(data_dict[this_batch]['data_list'][i]['UR1'][idx_R]/data_dict[this_batch]['Tp'][i],
                        data_dict[this_batch]['data_list'][i]['RM1'][idx_R]/data_dict[this_batch]['Mp'][i],
                        marker='x')
            else:
                print('The moment does not reach Mpl.')

            idx_Ru = data_dict[this_batch]['idx_Ru'][i]
            ax.plot(data_dict[this_batch]['data_list'][i]['UR1'][idx_Ru]/data_dict[this_batch]['Tp'][i],
                    data_dict[this_batch]['data_list'][i]['RM1'][idx_Ru]/data_dict[this_batch]['Mp'][i],
                    marker='x')
            
            idx_Mel = data_dict[this_batch]['idx_Mel'][i]
            ax.plot(data_dict[this_batch]['data_list'][i]['UR1'][idx_Mel]/data_dict[this_batch]['Tp'][i],
                    data_dict[this_batch]['data_list'][i]['RM1'][idx_Mel]/data_dict[this_batch]['Mp'][i],
                    marker='x')
            
            # ax.plot(data_dict[this_batch]['data_list'][i]['UR1'][50]/data_dict[this_batch]['Tp'][i],
            #         data_dict[this_batch]['data_list'][i]['RM1'][50]/data_dict[this_batch]['Mp'][i],
            #         marker='x')
            
            print(f'Mel occurs at {idx_Mel} and Ru occurs at {idx_Ru} and R occurs at {idx_R}'.format(idx_Mel=idx_Mel,idx_Ru=idx_Ru,idx_R=idx_R))
    if np.isnan(idx_Mel):
        print('The given combination of Y, YoT, B, H, tf and tw does not exist in the parametric database.')

fig, ax = plt.subplots()

this_batch = 'all runs'
idx_R = plot_MR(Y,YoT,B,H,tf,tw,ax,this_batch)

ax.set_xlabel(r'$\theta/\theta_{pl}$',  fontsize=20)
ax.set_ylabel(r'$M/M_{pl}$', fontsize=20)
ax.grid()
ax.tick_params(axis='both', which='major', labelsize=12)
ax.tick_params(axis='both', which='minor', labelsize=12)
ax.set_title(r'',  fontsize=20)
ax.legend()

# %% Plot specific moment-rotation graphs with Gioncu solution

E = 2e5 # [MPa] Young's modulus for calculating Tp
YoT = 1
Y = 460 # [MPa] 
B = 220 # [ mm]
H = 400 # [mm]
tf = 15 # [mm]
tw = 11 # [mm]
plt.close('all')

def plot_MR(Y,YoT,B,H,tf,tw,ax,this_batch):
    """
    Plot_MR given Y, YoT, B, H, tf, tw
    Outputs length and the index (frame number) near to which R occurs
    (In the parametric runs, the length was chosen such that LdLT = 0.1)
    ax: axes to plot to. input as 0 if plot is not desired
    """
    
    # initialize
    idx_R = np.NaN
    idx_Ru = np.NaN
    idx_Mel = np.NaN
    L = np.NaN
    R = np.NaN
    Ru = np.NaN
    R_numeric = np.NaN
                
    for i, dataYoT in enumerate(data_dict[this_batch]['YoT']):
        if (dataYoT==YoT 
            and data_dict[this_batch]['Y'][i]==Y 
            and data_dict[this_batch]['B'][i]==B
            and data_dict[this_batch]['H'][i]==H
            and data_dict[this_batch] ['tf'][i]==tf
            and data_dict[this_batch]['tw'][i]==tw):
            
            L = data_dict[this_batch]['L'][i]
            R = data_dict[this_batch]['R'][i]
            Ru = data_dict[this_batch]['Ru'][i]
            
            if ax!=0:
                ax.plot(data_dict[this_batch]['data_list'][i]['UR1'].values/data_dict[this_batch]['Tp'][i],
                data_dict[this_batch]['data_list'][i]['RM1']/data_dict[this_batch]['Mp'][i],
                label=rf'$\sigma_y={Y}; \sigma_y/\sigma_u={YoT}$'.format(Y=Y,YoT=YoT))
            
            idx_R = data_dict[this_batch]['idx_R'][i]
            if ~np.isnan(idx_R):
                if ax!=0:
                    ax.plot(data_dict[this_batch]['data_list'][i]['UR1'][idx_R]/data_dict[this_batch]['Tp'][i],
                        data_dict[this_batch]['data_list'][i]['RM1'][idx_R]/data_dict[this_batch]['Mp'][i],
                        marker='x')
            else:
                print('The moment does not reach Mpl.')

            idx_Ru = data_dict[this_batch]['idx_Ru'][i]
            if ax!=0:
                ax.plot(data_dict[this_batch]['data_list'][i]['UR1'][idx_Ru]/data_dict[this_batch]['Tp'][i],
                        data_dict[this_batch]['data_list'][i]['RM1'][idx_Ru]/data_dict[this_batch]['Mp'][i],
                        marker='x')
            
            idx_Mel = data_dict[this_batch]['idx_Mel'][i]
            if ax!=0:
                ax.plot(data_dict[this_batch]['data_list'][i]['UR1'][idx_Mel]/data_dict[this_batch]['Tp'][i],
                        data_dict[this_batch]['data_list'][i]['RM1'][idx_Mel]/data_dict[this_batch]['Mp'][i],
                        marker='x')
            
            print(f'Mel occurs at {idx_Mel} and Ru occurs at {idx_Ru} and R occurs at {idx_R}'.format(idx_Mel=idx_Mel,idx_Ru=idx_Ru,idx_R=idx_R))
    if np.isnan(idx_Mel):
        print('The given combination of Y, YoT, B, H, tf and tw does not exist in the parametric database.')

    if R==0:
        R_numeric = Ru
    else:
        R_numeric = R       

    return R_numeric, R, Ru, L, idx_R

fig, ax = plt.subplots()

this_batch = 'all runs'
R_numeric, R, Ru, L, idx_R = plot_MR(Y,YoT,B,H,tf,tw,ax,this_batch)

ax.set_ylim([0,1.5])
# ax.set_xlim([0,15])
ax.set_xlabel(r'$\theta/\theta_{pl}$', fontsize=20)
ax.set_ylabel(r'$M/M_{pl}$', fontsize=20)
ax.grid()
ax.tick_params(axis='both', which='major', labelsize=12)
ax.tick_params(axis='both', which='minor', labelsize=12)
ax.set_title(r'', fontsize=20)

def plot_MR_analytic_asym(Y,B,H,tf,tw,L,ax):

    b = B/2 # mm
    d = H - 2*tf # mm web depth
    Yf = Y # MPa flange yield strength
    Yw = Y # MPa web yield strength
    b1 = b # half of total top flange width
    b2 = b # half of total bottom flange width
    tf1 = tf
    tf2 = tf

    om1 = afn.calc_om1(b1,L)
    
    Mp = fn.calc_Mpl(Yf,Yw,tf,tw,B,H)
    Ix = afn.calc_Ix(tf,tw,b,d)
    Tp = afn.calc_Tp(Mp,L,E,Ix)
    
    # Initalize
    T = np.linspace(0,1.5,500)
    M_norm = np.zeros(np.size(T)) # for varying with T
    T_norm = afn.calc_T_norm(T,Tp)
    chi2 = np.linspace(0,2.5,500)
    M_Norm = np.zeros(np.size(chi2)) # for varying with chi2

    min_chi2= b/d # physically minimum possible chi2 corresponding to delta=1
    
    # function where M/Mp=1 to be solved to find T
    def one_min_M_norm_asym(T,chi2,b1,b2,d,tw,tf1,tf2,om1,L,Yf,Yw):
        return 1-afn.calc_M_norm_asym(chi2,T,b1,b2,d,tw,tf1,tf2,om1,L,Yf,Yw)
    
    # Find an initial T_analytic and R_analytic assuming the chi2=min_chi2
    T_analytic = optimize.fsolve(one_min_M_norm_asym, x0=0.03, args=(min_chi2,b1,b2,d,tw,tf1,tf2,om1,L,Yf,Yw))
    R_analytic = T_analytic/Tp-1
    
    # Find the chi2 which leads to the minimimed M/Mp at the assumed T_analytic
    optim_chi2 = optimize.minimize(afn.calc_M_norm_asym,x0=0.12,args=(T_analytic,b1,b2,d,tw,tf1,tf2,om1,L,Yf,Yw)).x
    
    # Check whether the optim_chi2 is smaller than the minimum possible chi2
    if optim_chi2<min_chi2: # if it is smaller, use the min_chi2 as the optim_chi2 as before.
        final_chi2 = min_chi2
    else: # if not, use the calculated optim_chi2 and recalculate T_analytic
        final_chi2 = optim_chi2
        T_analytic = optimize.fsolve(one_min_M_norm_asym, x0=0.03, args=(final_chi2,b1,b2,d,tw,tf1,tf2,om1,L,Yf,Yw))
        R_analytic = T_analytic/Tp-1
    
    # Calculate M_norm wrt to chi2 using final T_analytic
    for j in range(len(chi2)): # 
        M_Norm[j] = afn.calc_M_norm_asym(chi2[j],T_analytic,b1,b2,d,tw,tf1,tf2,om1,L,Yf,Yw)
    
    # Calculate M_norm wrt to chi2 using final chi2
    for i in range(len(T)):
        M_norm[i] = afn.calc_M_norm_asym(final_chi2,T[i],b1,b2,d,tw,tf1,tf2,om1,L,Yf,Yw)
    
    if ax!=0:
        ax.plot(T_norm, M_norm, '-r', label=rf'asymmetrical; $\chi$={final_chi2}')
        ax.plot([T_analytic/Tp, T_analytic/Tp], [0,1], '-r', label=rf'$\theta/\theta_p$ = {T_analytic[0]/Tp}')
        fig2, ax2 = plt.subplots()
        ax2.plot(chi2, M_Norm, '-r', label=rf'$\theta$={T_analytic[0]}; $\theta/\theta_p$={T_analytic[0]/Tp}')
        ax2.plot([0,2.5],[1,1])
        ax2.plot([min_chi2, min_chi2], [0,1], label =rf'minimum $\chi_2$={min_chi2}')
        ax2.plot([final_chi2, final_chi2], [0,1], '-r', label =rf'final $\chi_2$={final_chi2}')
        ax2.set_xlim([0.1,final_chi2+0.1])
        ax2.set_ylim([0.5,1.05])
        ax2.legend()
        ax2.set_xlabel(r'$\chi_2$', fontsize=20)
        ax2.set_ylabel(r'$M/M_{pl}$', fontsize=20)
    
    return R_analytic, min_chi2, optim_chi2, final_chi2

def plot_MR_analytic_sym(Y,B,H,tf,tw,L,ax):

    b = B/2 # mm
    d = H - 2*tf # mm web depth
    Yf = Y # MPa flange yield strength
    Yw = Y # MPa web yield strength
    b1 = b # half of total top flange width
    b2 = b # half of total bottom flange width
    tf1 = tf
    tf2 = tf

    om2 = afn.calc_om2(b1,L)
    
    Mp = fn.calc_Mpl(Yf,Yw,tf,tw,B,H)
    Ix = afn.calc_Ix(tf,tw,b,d)
    Tp = afn.calc_Tp(Mp,L,E,Ix)
    
    # Initalize
    T = np.linspace(0,1.5,500)
    M_norm = np.zeros(np.size(T)) # for varying with T
    T_norm = afn.calc_T_norm(T,Tp)
    chi2 = np.linspace(0,1.5,500)
    M_Norm = np.zeros(np.size(chi2)) # for varying with chi2

    min_chi2= b/d # physically minimum possible chi2 corresponding to delta=1
    
    # function where M/Mp=1 to be solved to find T
    def one_min_M_norm_sym(T,chi2,b1,b2,d,tw,tf1,tf2,om2,L,Yf,Yw):
        return 1-afn.calc_M_norm_sym(chi2,T,b1,b2,d,tw,tf1,tf2,om2,L,Yf,Yw)
    
    # Find an initial T_analytic and R_analytic assuming the chi2=min_chi2
    T_analytic = optimize.fsolve(one_min_M_norm_sym, x0=0.03, args=(min_chi2,b1,b2,d,tw,tf1,tf2,om2,L,Yf,Yw))
    R_analytic = T_analytic/Tp-1
    
    # Find the chi2 which leads to the minimimed M/Mp at the assumed T_analytic
    optim_chi2 = optimize.minimize(afn.calc_M_norm_sym,x0=0.25,args=(T_analytic,b1,b2,d,tw,tf1,tf2,om2,L,Yf,Yw)).x
    
    # Check whether the optim_chi2 is smaller than the minimum possible chi2
    if optim_chi2<min_chi2: # if it is smaller, use the min_chi2 as the optim_chi2 as before.
        final_chi2 = min_chi2
    else: # if not, use the calculated optim_chi2 and recalculate T_analytic
        final_chi2 = optim_chi2
        T_analytic = optimize.fsolve(one_min_M_norm_sym, x0=0.03, args=(final_chi2,b1,b2,d,tw,tf1,tf2,om2,L,Yf,Yw))
        R_analytic = T_analytic/Tp-1
        
    # Calculate M_norm wrt to chi2 using final T_analytic
    for j in range(len(chi2)): # 
        M_Norm[j] = afn.calc_M_norm_sym(chi2[j],T_analytic,b1,b2,d,tw,tf1,tf2,om2,L,Yf,Yw)
    
    # Calculate M_norm wrt to chi2 using final chi2
    for i in range(len(T)):
        M_norm[i] = afn.calc_M_norm_sym(final_chi2,T[i],b1,b2,d,tw,tf1,tf2,om2,L,Yf,Yw)

    if ax!=0:
        ax.plot(T_norm, M_norm, '-b', label=rf'symmetrical; $\chi$={final_chi2}')
        ax.plot([T_analytic/Tp, T_analytic/Tp], [0,1], '-b', label=rf'$\theta/\theta_p$ = {T_analytic[0]/Tp}')
        fig2, ax2 = plt.subplots()
        ax2.plot(chi2, M_Norm, '-b', label=rf'$\theta$={T_analytic[0]}; $\theta/\theta_p$={T_analytic[0]/Tp}')
        ax2.plot([0,1.5],[1,1])
        ax2.plot([min_chi2, min_chi2], [0,1], label =rf'minimum $\chi_2$={min_chi2}')
        ax2.plot([final_chi2, final_chi2], [0,1], '-b', label =rf'final $\chi_2$={final_chi2}')
        ax2.set_xlim([0.1,final_chi2+0.1])
        ax2.set_ylim([0.5,1.05])
        ax2.legend()
        ax2.set_xlabel(r'$\chi_2$', fontsize=20)
        ax2.set_ylabel(r'$M/M_{pl}$', fontsize=20)
    
    return R_analytic, min_chi2, optim_chi2, final_chi2

def plot_MR_analytic_in(Y,B,H,tf,tw,L,ax):

    b = B/2 # mm
    d = H - 2*tf # mm web depth
    Yf = Y # MPa flange yield strength
    Yw = Y # MPa web yield strength
    b1 = b # half of total top flange width
    b2 = b # half of total bottom flange width
    tf1 = tf
    tf2 = tf
    beta = afn.calc_beta(tf, tw, d, b)
    alpha = 0
    
    Mp = fn.calc_Mpl(Yf,Yw,tf,tw,B,H)
    Ix = afn.calc_Ix(tf,tw,b,d)
    Tp = afn.calc_Tp(Mp,L,E,Ix)
    
    # Initalize
    T = np.linspace(0,1.5,500)
    M_norm = np.zeros(np.size(T)) # for varying with T
    T_norm = afn.calc_T_norm(T,Tp)
    chi1 = np.linspace(0,1.5,500)
    M_Norm = np.zeros(np.size(chi1)) # for varying with chi2

    min_chi1= beta*b/d # physically minimum possible chi2 corresponding to delta=1
    
    # function where M/Mp=1 to be solved to find T
    def one_min_M_norm_in(T,chi1,b1,b2,d,tw,tf1,tf2,Yf,Yw,alpha):
        return 1-afn.calc_M_norm_in(chi1,T,b1,b2,d,tw,tf1,tf2,Yf,Yw,alpha)
    
    # Find an initial T_analytic and R_analytic assuming the chi1=min_chi1
    T_analytic = optimize.fsolve(one_min_M_norm_in, x0=0.03, args=(min_chi1,b1,b2,d,tw,tf1,tf2,Yf,Yw,alpha))
    R_analytic = T_analytic/Tp-1
    # import pdb;pdb.set_trace()
    # Find the chi1 which leads to the minimimed M/Mp at the assumed T_analytic
    optim_chi1 = optimize.minimize(afn.calc_M_norm_in,x0=0.7,args=(T_analytic,b1,b2,d,tw,tf1,tf2,Yf,Yw,alpha)).x
    
    # Check whether the optim_chi1 is smaller than the minimum possible chi1
    if optim_chi1<min_chi1: # if it is smaller, use the min_chi2 as the optim_chi2 as before.
        final_chi1 = min_chi1
    else: # if not, use the calculated optim_chi2 and recalculate T_analytic
        final_chi1 = optim_chi1
        T_analytic = optimize.fsolve(one_min_M_norm_in, x0=0.03, args=(final_chi1,b1,b2,d,tw,tf1,tf2,Yf,Yw,alpha))
        R_analytic = T_analytic/Tp-1
        
    # Calculate M_norm wrt to chi1 using final T_analytic
    for j in range(len(chi1)): # 
        M_Norm[j] = afn.calc_M_norm_in(chi1[j],T_analytic,b1,b2,d,tw,tf1,tf2,Yf,Yw,alpha)
    
    # Calculate M_norm wrt to chi2 using final chi2
    for i in range(len(T)):
        M_norm[i] = afn.calc_M_norm_in(final_chi1,T[i],b1,b2,d,tw,tf1,tf2,Yf,Yw,alpha)

    if ax!=0:
        ax.plot(T_norm, M_norm, '-g', label=rf'in-plane; $\chi$={final_chi1}')
        ax.plot([T_analytic/Tp, T_analytic/Tp], [0,1], '-g', label=rf'$\theta/\theta_p$ = {T_analytic[0]/Tp}')
        fig2, ax2 = plt.subplots()
        ax2.plot(chi1, M_Norm, '-g', label=rf'$\theta$={T_analytic[0]}; $\theta/\theta_p$={T_analytic[0]/Tp}')
        ax2.plot([0,1.5],[1,1])
        ax2.plot([min_chi1, min_chi1], [0,1], label =rf'minimum $\chi_1$={min_chi1}')
        ax2.plot([final_chi1, final_chi1], [0,1], '-g', label =rf'final $\chi_1$={final_chi1}')
        ax2.set_xlim([0.1,final_chi1+0.1])
        ax2.set_ylim([0.5,1.05])
        ax2.legend()
        ax2.set_xlabel(r'$\chi_1$', fontsize=20)
        ax2.set_ylabel(r'$M/M_{pl}$', fontsize=20)
    
    return R_analytic, min_chi1, optim_chi1, final_chi1

plot_MR_analytic_asym(Y,B,H,tf,tw,L,ax)
plot_MR_analytic_sym(Y,B,H,tf,tw,L,ax)
plot_MR_analytic_in(Y,B,H,tf,tw,L,ax)
ax.legend()

# %% Compare numeric and Gioncu solution for multiple cases

this_batch = 'all runs'

E = 2e5 # [MPa] Young's modulus for calculating Tp
YoT = 1
Y = np.array([460, 550, 690, 790, 890, 960]) # [MPa] 
B = 220 # [ mm]
H = 400 # [mm]
tf = 15 # [mm]
tw = 11 # [mm]
plt.close('all')

numeric_results = list()
analytic_asym_results = list()
analytic_sym_results = list()

for k, Yk in enumerate(Y):
    numeric_results.append(plot_MR(Yk,YoT,B,H,tf,tw,0,this_batch))
    analytic_asym_results.append(plot_MR_analytic_asym(Yk,B,H,tf,tw,numeric_results[k][3],0))
    analytic_sym_results.append(plot_MR_analytic_sym(Yk,B,H,tf,tw,numeric_results[k][3],0))

# plot R analytic against R numerical 

list_R_analytic_sym = [results[0] for results in analytic_sym_results]
list_R_analytic_asym = [results[0] for results in analytic_asym_results]
list_R_numeric = [results[0] for results in numeric_results]

fig3, ax3 = plt.subplots()
ax3.plot(list_R_numeric, list_R_analytic_asym, '-x', label = 'Gioncu (1997) out-of-plane asymmetric')
# ax3.plot(list_R_numeric, list_R_analytic_sym, '-x', label = 'Gioncu (1997) out-of-plane symmetric')
ax3.set_xlabel(r'$R$ [-] from parametric study')
ax3.set_ylabel(r'$R$ [-] from Gioncu (1997)')
ax3.legend()
ax3.grid()

# plot R against Y
fig4, ax4 = plt.subplots()
ax4.plot(Y, list_R_analytic_asym, '-x', label = 'Gioncu (1997) out-of-plane asymmetric')
# ax4.plot(Y, list_R_analytic_sym, '-x', label = 'Gioncu (1997) out-of-plane symmetric')
ax4.plot(Y, list_R_numeric, '-x', label ='Numerical results')
ax4.set_xlabel(r'$\sigma_y$ [MPa]')
ax4.set_ylabel(r'$R$ [-]')
ax4.legend()
ax4.grid()