# -*- coding: utf-8 -*-
"""
Created on Fri Nov 11 14:58:00 2022

@author: wjwong
"""

# %% Import modules and tools

import analytic_functions as afn
import parametric_functions as pfn
import functions as fn
import matplotlib.pyplot as plt
import matplotlib.lines as mlines
import numpy as np
from scipy.interpolate import griddata
from scipy import optimize

# For 3D arrows and annotations

from mpl_toolkits.mplot3d.proj3d import proj_transform
from mpl_toolkits.mplot3d.axes3d import Axes3D
from matplotlib.text import Annotation

class Annotation3D(Annotation):

    def __init__(self, text, xyz, *args, **kwargs):
        super().__init__(text, xy=(0, 0), *args, **kwargs)
        self._xyz = xyz

    def draw(self, renderer):
        x2, y2, z2 = proj_transform(*self._xyz, self.axes.M)
        self.xy = (x2, y2)
        super().draw(renderer)
        
def _annotate3D(ax, text, xyz, *args, **kwargs):
    '''Add anotation `text` to an `Axes3d` instance.'''

    annotation = Annotation3D(text, xyz, *args, **kwargs)
    ax.add_artist(annotation)
    
setattr(Axes3D, 'annotate3D', _annotate3D)

from matplotlib.patches import FancyArrowPatch

class Arrow3D(FancyArrowPatch):

    def __init__(self, x, y, z, dx, dy, dz, *args, **kwargs):
        super().__init__((0, 0), (0, 0), *args, **kwargs)
        self._xyz = (x, y, z)
        self._dxdydz = (dx, dy, dz)

    def draw(self, renderer):
        x1, y1, z1 = self._xyz
        dx, dy, dz = self._dxdydz
        x2, y2, z2 = (x1 + dx, y1 + dy, z1 + dz)

        xs, ys, zs = proj_transform((x1, x2), (y1, y2), (z1, z2), self.axes.M)
        self.set_positions((xs[0], ys[0]), (xs[1], ys[1]))
        super().draw(renderer)
        
    def do_3d_projection(self, renderer=None):
        x1, y1, z1 = self._xyz
        dx, dy, dz = self._dxdydz
        x2, y2, z2 = (x1 + dx, y1 + dy, z1 + dz)

        xs, ys, zs = proj_transform((x1, x2), (y1, y2), (z1, z2), self.axes.M)
        self.set_positions((xs[0], ys[0]), (xs[1], ys[1]))

        return np.min(zs) 
    
def _arrow3D(ax, x, y, z, dx, dy, dz, *args, **kwargs):
    '''Add an 3d arrow to an `Axes3D` instance.'''

    arrow = Arrow3D(x, y, z, dx, dy, dz, *args, **kwargs)
    ax.add_artist(arrow)
    
setattr(Axes3D, 'arrow3D', _arrow3D)



# %% Load data into dictionary

batch_names =  ['all runs',
                'all 690 runs',
                'all 890 runs',
                'vary YoT vary Y t_HSA800 default H and B',
                'vary B vary YoT t_HSA800 Y690 Lw_40p0eps',
                'vary B vary YoT t_HSA800 Y890 Lw_40p0eps',
                'vary YoT vary Y t_SM490 default H and B',
                'vary YoT vary Y t_SM490_12mm default H and B',
                'vary H vary YoT t_SM490_12mm Y690 Lf_10p1eps',
                'vary H vary YoT t_SM490_12mm Y890 Lf_10p1eps',
                'vary YoT vary Y t_HSA800 Lw_40p0eps Lf_10p1eps',
                'vary YoT vary Y t_SM490_12mm Lf_10p1eps Lw_52p5eps'
                ]

data_dict = {}

for i, batch_name in enumerate(batch_names):
    
    # mypath = r'C:\Users\wjwong\Documents\Research\plastic hinge\parametric-runs\\' + batch_name + '\\'
    # data_dict[batch_name] = pfn.load_data(mypath)

    mypath = r'C:\Users\wjwong\Documents\Research\plastic hinge\parametric-runs\Parametric runs with a = 0.53tw\\' + batch_name + '\\'
        
    # for parameter in pfn.load_data(mypath):
    loaded_data = pfn.load_data(mypath)
    data_dict[batch_name] = {
        'YoT': loaded_data[0],
        'Y': loaded_data[1],
        'Ldp': loaded_data[2],
        'B': loaded_data[3],
        'H': loaded_data[4],
        'tf': loaded_data[5],
        'tw': loaded_data[6],
        'imp': loaded_data[7],
        'Ldf': loaded_data[8],
        'Ldw': loaded_data[9],
        'Mp': loaded_data[10],
        'Tp': loaded_data[11],
        'L': loaded_data[12],
        'R': loaded_data[13],
        'Ldf_eps': loaded_data[14],
        'Ldw_eps': loaded_data[15],
        'Mnu': loaded_data[16],
        'Ru':loaded_data[17],
        'idx_R':loaded_data[18],
        'idx_Ru':loaded_data[19],
        'Mel':loaded_data[20],
        'idx_Mel':loaded_data[21],
        'data_list':loaded_data[22]
        }
# %% Plot specific moment-rotation graphs

YoT = 0.94
Y = 790
B = 220
H = 400
tf = 15
tw = 11
plt.close('all')

def plot_MR(Y,YoT,B,H,tf,tw,ax,this_batch):
    """
    Plot_MR given Y, YoT, B, H, tf, tw
    Outputs the index (frame number) near to which R occurs
    ax: axes to plot to
    """
    
    # initialize
    idx_R = np.NaN
    idx_Ru = np.NaN
    idx_Mel = np.NaN
                
    for i, dataYoT in enumerate(data_dict[this_batch]['YoT']):
        if (dataYoT==YoT 
            and data_dict[this_batch]['Y'][i]==Y 
            and data_dict[this_batch]['B'][i]==B
            and data_dict[this_batch]['H'][i]==H
            and data_dict[this_batch] ['tf'][i]==tf
            and data_dict[this_batch]['tw'][i]==tw):
            
            ax.plot(data_dict[this_batch]['data_list'][i]['UR1'].values/data_dict[this_batch]['Tp'][i],
            data_dict[this_batch]['data_list'][i]['RM1']/data_dict[this_batch]['Mp'][i],
            label=rf'$\sigma_y={Y}; \sigma_y/\sigma_u={YoT}$'.format(Y=Y,YoT=YoT))
            # import pdb; pdb.set_trace()
            
            idx_R = data_dict[this_batch]['idx_R'][i]
            if ~np.isnan(idx_R):
                ax.plot(data_dict[this_batch]['data_list'][i]['UR1'][idx_R]/data_dict[this_batch]['Tp'][i],
                        data_dict[this_batch]['data_list'][i]['RM1'][idx_R]/data_dict[this_batch]['Mp'][i],
                        marker='x')
            else:
                print('The moment does not reach Mpl.')

            idx_Ru = data_dict[this_batch]['idx_Ru'][i]
            ax.plot(data_dict[this_batch]['data_list'][i]['UR1'][idx_Ru]/data_dict[this_batch]['Tp'][i],
                    data_dict[this_batch]['data_list'][i]['RM1'][idx_Ru]/data_dict[this_batch]['Mp'][i],
                    marker='x')
            
            idx_Mel = data_dict[this_batch]['idx_Mel'][i]
            ax.plot(data_dict[this_batch]['data_list'][i]['UR1'][idx_Mel]/data_dict[this_batch]['Tp'][i],
                    data_dict[this_batch]['data_list'][i]['RM1'][idx_Mel]/data_dict[this_batch]['Mp'][i],
                    marker='x')
            
            # ax.plot(data_dict[this_batch]['data_list'][i]['UR1'][50]/data_dict[this_batch]['Tp'][i],
            #         data_dict[this_batch]['data_list'][i]['RM1'][50]/data_dict[this_batch]['Mp'][i],
            #         marker='x')
            
            print(f'Mel occurs at {idx_Mel} and Ru occurs at {idx_Ru} and R occurs at {idx_R}'.format(idx_Mel=idx_Mel,idx_Ru=idx_Ru,idx_R=idx_R))
    if np.isnan(idx_Mel):
        print('The given combination of Y, YoT, B, H, tf and tw does not exist in the parametric database.')

fig, ax = plt.subplots()

this_batch = 'all runs'
idx_R = plot_MR(Y,YoT,B,H,tf,tw,ax,this_batch)

ax.set_xlabel(r'$\theta/\theta_{pl}$',  fontsize=20)
ax.set_ylabel(r'$M/M_{pl}$', fontsize=20)
ax.grid()
ax.tick_params(axis='both', which='major', labelsize=12)
ax.tick_params(axis='both', which='minor', labelsize=12)
ax.set_title(r'',  fontsize=20)
ax.legend()

# %% Plot specific moment-rotation graphs with Gioncu solution

E = 2e5 # [MPa] Young's modulus for calculating Tp
YoT = 0.99
Y = 690 # [MPa] 
B = 200 # [ mm]
H = 400 # [mm]
tf = 20 # [mm]
tw = 17.6 # [mm]
plt.close('all')

def plot_MR(Y,YoT,B,H,tf,tw,ax,this_batch):
    """
    Plot_MR given Y, YoT, B, H, tf, tw
    Outputs length and the index (frame number) near to which R occurs
    (In the parametric runs, the length was chosen such that LdLT = 0.1)
    ax: axes to plot to
    """
    
    # initialize
    idx_R = np.NaN
    idx_Ru = np.NaN
    idx_Mel = np.NaN
    L = np.NaN
                
    for i, dataYoT in enumerate(data_dict[this_batch]['YoT']):
        if (dataYoT==YoT 
            and data_dict[this_batch]['Y'][i]==Y 
            and data_dict[this_batch]['B'][i]==B
            and data_dict[this_batch]['H'][i]==H
            and data_dict[this_batch] ['tf'][i]==tf
            and data_dict[this_batch]['tw'][i]==tw):
            
            L = data_dict[this_batch]['L'][i]
            
            ax.plot(data_dict[this_batch]['data_list'][i]['UR1'].values/data_dict[this_batch]['Tp'][i],
            data_dict[this_batch]['data_list'][i]['RM1']/data_dict[this_batch]['Mp'][i],
            label=rf'$\sigma_y={Y}; \sigma_y/\sigma_u={YoT}$'.format(Y=Y,YoT=YoT))
            # import pdb; pdb.set_trace()
            
            idx_R = data_dict[this_batch]['idx_R'][i]
            if ~np.isnan(idx_R):
                ax.plot(data_dict[this_batch]['data_list'][i]['UR1'][idx_R]/data_dict[this_batch]['Tp'][i],
                        data_dict[this_batch]['data_list'][i]['RM1'][idx_R]/data_dict[this_batch]['Mp'][i],
                        marker='x')
            else:
                print('The moment does not reach Mpl.')

            idx_Ru = data_dict[this_batch]['idx_Ru'][i]
            ax.plot(data_dict[this_batch]['data_list'][i]['UR1'][idx_Ru]/data_dict[this_batch]['Tp'][i],
                    data_dict[this_batch]['data_list'][i]['RM1'][idx_Ru]/data_dict[this_batch]['Mp'][i],
                    marker='x')
            
            idx_Mel = data_dict[this_batch]['idx_Mel'][i]
            ax.plot(data_dict[this_batch]['data_list'][i]['UR1'][idx_Mel]/data_dict[this_batch]['Tp'][i],
                    data_dict[this_batch]['data_list'][i]['RM1'][idx_Mel]/data_dict[this_batch]['Mp'][i],
                    marker='x')
            
            # ax.plot(data_dict[this_batch]['data_list'][i]['UR1'][50]/data_dict[this_batch]['Tp'][i],
            #         data_dict[this_batch]['data_list'][i]['RM1'][50]/data_dict[this_batch]['Mp'][i],
            #         marker='x')
            
            print(f'Mel occurs at {idx_Mel} and Ru occurs at {idx_Ru} and R occurs at {idx_R}'.format(idx_Mel=idx_Mel,idx_Ru=idx_Ru,idx_R=idx_R))
    if np.isnan(idx_Mel):
        print('The given combination of Y, YoT, B, H, tf and tw does not exist in the parametric database.')

    return L, idx_R

fig, ax = plt.subplots()

this_batch = 'all runs'
L, idx_R = plot_MR(Y,YoT,B,H,tf,tw,ax,this_batch)

ax.set_ylim([0,1.5])
ax.set_xlim([0,15])
ax.set_xlabel(r'$\theta/\theta_{pl}$',  fontsize=20)
ax.set_ylabel(r'$M/M_{pl}$', fontsize=20)
ax.grid()
ax.tick_params(axis='both', which='major', labelsize=12)
ax.tick_params(axis='both', which='minor', labelsize=12)
ax.set_title(r'',  fontsize=20)

def plot_MR_analytic(Y,B,H,tf,tw,L,ax):

    b = B/2 # mm
    d = H - 2*tf # mm web depth
    Yf = Y # MPa flange yield strength
    Yw = Y # MPa web yield strength
    rho = afn.calc_rho(Yf,Yw)
    b1 = b # half of total top flange width
    b2 = b # half of total bottom flange width
    tf1 = tf
    tf2 = tf

    om1 = afn.calc_om1(b1,L)

    # fn.calc_norm_M(chi2,th,rho,b1,b2,d,tw,tf1,tf2,om1,l,Yf,Yw)
    min_chi2= b/d
    optim_chi2 = optimize.minimize(afn.calc_M_norm_asym,x0=0.12,args=(0.04680359,rho,b1,b2,d,tw,tf1,tf2,om1,L,Yf,Yw)).x
    optim_chi3 = optimize.minimize(afn.calc_M_norm_asym,x0=0.12,args=(0.1,rho,b1,b2,d,tw,tf1,tf2,om1,L,Yf,Yw)).x
    optim_chi4 = optimize.minimize(afn.calc_M_norm_asym,x0=0.12,args=(0.001,rho,b1,b2,d,tw,tf1,tf2,om1,L,Yf,Yw)).x
    optim_chi5 = optimize.minimize(afn.calc_M_norm_asym,x0=0.12,args=(0.04702782,rho,b1,b2,d,tw,tf1,tf2,om1,L,Yf,Yw),bounds=((min_chi2,np.inf),)).x
    # Putting a T here of e.g. 0.1 or 0.001 doesn't necessarily lead to a
    # chi for which the M/Mp = 1 at T = 0.1 or 0.001.
    # But it will lead to a chi for which M/Mp is mimimum wrt to chi at that chi,
    # if the theta = 0.1 or 0.001. 

    # Plot
   
    T = np.linspace(0,0.5,500)
    M_norm = np.zeros(np.size(T))

    for i in range(len(T)):
        M_norm[i] = afn.calc_M_norm_asym(optim_chi2,T[i],rho,b1,b2,d,tw,tf1,tf2,om1,L,Yf,Yw)
    
    Mp = fn.calc_Mpl(Yf,Yw,tf,tw,B,H)
    Ix = afn.calc_Ix(tf,tw,b,d)
    Tp = afn.calc_Tp(Mp,L,E,Ix)
    T_norm = afn.calc_T_norm(T,Tp)
    
    ax.plot(T_norm, M_norm, '-b', label=rf'$\chi$={optim_chi2}')
    
    def one_min_M_norm_asym(T,chi2,rho,b1,b2,d,tw,tf1,tf2,om1,L,Yf,Yw):
        return 1-afn.calc_M_norm_asym(chi2,T,rho,b1,b2,d,tw,tf1,tf2,om1,L,Yf,Yw)
    
    T_analytic = optimize.fsolve(one_min_M_norm_asym, x0=0.03, args=(optim_chi2,rho,b1,b2,d,tw,tf1,tf2,om1,L,Yf,Yw))
    R_analytic2 = T_analytic/Tp-1
    chi2 = np.linspace(0,0.4,50)
    M_Norm = np.zeros(np.size(chi2))
    for j in range(len(chi2)):
        M_Norm[j] = afn.calc_M_norm_asym(chi2[j],T_analytic,rho,b1,b2,d,tw,tf1,tf2,om1,L,Yf,Yw)
        
    fig2, ax2 = plt.subplots()
    ax2.plot(chi2, M_Norm, '-b', label=rf'$\theta$={T_analytic}; $\theta/\theta_p$={T_analytic/Tp}')
    ax2.plot([0,0.4],[1,1])
    # ax2.set_xlim([0.1,0.3])
    ax2.set_ylim([0.5,1.05])
    
    for i in range(len(T)):
        M_norm[i] = afn.calc_M_norm_asym(optim_chi3,T[i],rho,b1,b2,d,tw,tf1,tf2,om1,L,Yf,Yw)
    
    Mp = fn.calc_Mpl(Yf,Yw,tf,tw,B,H)
    Ix = afn.calc_Ix(tf,tw,b,d)
    Tp = afn.calc_Tp(Mp,L,E,Ix)
    T_norm = afn.calc_T_norm(T,Tp)

    ax.plot(T_norm, M_norm, '--g', label=rf'$\chi$={optim_chi3}')

    T_analytic = 0.1
    R_analytic3 = T_analytic/Tp-1        
    chi2 = np.linspace(0,0.4,50)
    M_Norm = np.zeros(np.size(chi2))
    for j in range(len(chi2)):
        M_Norm[j] = afn.calc_M_norm_asym(chi2[j],T_analytic,rho,b1,b2,d,tw,tf1,tf2,om1,L,Yf,Yw)
    ax2.plot(chi2, M_Norm, ':g', label=rf'$\theta$={T_analytic}; $\theta/\theta_p$={T_analytic/Tp}; minimisation done at $\theta$=0.1 to find optim_chi3={optim_chi3}')
    
    T_analytic = optimize.fsolve(one_min_M_norm_asym, x0=0.03, args=(optim_chi3,rho,b1,b2,d,tw,tf1,tf2,om1,L,Yf,Yw))
    R_analytic3 = T_analytic/Tp-1
    chi2 = np.linspace(0,0.4,50)
    M_Norm = np.zeros(np.size(chi2))
    for j in range(len(chi2)):
        M_Norm[j] = afn.calc_M_norm_asym(chi2[j],T_analytic,rho,b1,b2,d,tw,tf1,tf2,om1,L,Yf,Yw)
    ax2.plot(chi2, M_Norm, '--g', label=rf'$\theta$={T_analytic}; $\theta/\theta_p$={T_analytic/Tp}; solved to find $\theta$={T_analytic} for which M/Mp = 1, assuming optim_chi3={optim_chi3}')
        
    for i in range(len(T)):
        M_norm[i] = afn.calc_M_norm_asym(optim_chi4,T[i],rho,b1,b2,d,tw,tf1,tf2,om1,L,Yf,Yw)
    
    Mp = fn.calc_Mpl(Yf,Yw,tf,tw,B,H)
    Ix = afn.calc_Ix(tf,tw,b,d)
    Tp = afn.calc_Tp(Mp,L,E,Ix)
    T_norm = afn.calc_T_norm(T,Tp)

    ax.plot(T_norm, M_norm, ':r', label=rf'$\chi$={optim_chi4}')

    T_analytic = optimize.fsolve(one_min_M_norm_asym, x0=0.03, args=(optim_chi4,rho,b1,b2,d,tw,tf1,tf2,om1,L,Yf,Yw))
    R_analytic4 = T_analytic/Tp-1
    chi2 = np.linspace(0,0.4,50)
    M_Norm = np.zeros(np.size(chi2))
    for j in range(len(chi2)):
        M_Norm[j] = afn.calc_M_norm_asym(chi2[j],T_analytic,rho,b1,b2,d,tw,tf1,tf2,om1,L,Yf,Yw)
    ax2.plot(chi2, M_Norm, ':r', label=rf'$\theta$={T_analytic}; $\theta/\theta_p$={T_analytic/Tp}')
    
    for i in range(len(T)):
        M_norm[i] = afn.calc_M_norm_asym(optim_chi5,T[i],rho,b1,b2,d,tw,tf1,tf2,om1,L,Yf,Yw)
    
    Mp = fn.calc_Mpl(Yf,Yw,tf,tw,B,H)
    Ix = afn.calc_Ix(tf,tw,b,d)
    Tp = afn.calc_Tp(Mp,L,E,Ix)
    T_norm = afn.calc_T_norm(T,Tp)
    
    ax.plot(T_norm, M_norm, '-y', label=rf'$\chi$={optim_chi5}')

    T_analytic = optimize.fsolve(one_min_M_norm_asym, x0=0.03, args=(optim_chi5,rho,b1,b2,d,tw,tf1,tf2,om1,L,Yf,Yw))
    R_analytic5 = T_analytic/Tp-1
    chi2 = np.linspace(0,0.4,50)
    M_Norm = np.zeros(np.size(chi2))
    for j in range(len(chi2)):
        M_Norm[j] = afn.calc_M_norm_asym(chi2[j],T_analytic,rho,b1,b2,d,tw,tf1,tf2,om1,L,Yf,Yw)
    ax2.plot(chi2, M_Norm, '-y', label=rf'$\theta$={T_analytic}; $\theta/\theta_p$={T_analytic/Tp}')
    
    ax2.legend()
    
    return optim_chi2, optim_chi3, optim_chi4, optim_chi5, R_analytic2, R_analytic3, R_analytic4, R_analytic5
   
plot_MR_analytic(Y,B,H,tf,tw,L,ax)
ax.legend()

# The chi2 is to be found by taking the chi for which M/Mp = 1 and M/Mp is minimum 
# wrt to chi2. there is only one value of theta for which the minimum in the M/Mp 
# wrt to chi2 happens at M/Mp=1. and that theta will also be the theta which 
# marks the rotation capacity.
# However, there is a physical minimum chi allowable which corresponds to del <= 1. 
# if that is the limiting situation, then it might be that
# a chi2 is found for which this chi2 does not correspond to M/Mp being minimum 
# wrt to chi2, but that chi2 is the minimum possible, and but M/Mp does equal to 1
# indeed equal to 1 for that chi2

# conclusion: solve for the chi by minimizing M/Mp wrt chi, for a theta where the minimum 
# M/Mp wrt to chi is 1.

# Out of M/Mpl, chi2, and theta, given 2, the 3rd is fixed.
# in our case, we are fixing the chi2. chi2 stays constant throughout the
# development of the mechanism.

# in this case, this corresponds to both the smallest value of chi2 and the smallest value of
# theta for which M/Mp = 1

# Looking at the plot here of M/Mp against chi. ignore values of chi smaller than the 
# minimum possible chi. then, there is a family a curves to choose from. the bigger
# the theta, the lower the curve. choose the smallest theta for which M/Mp = 1 occurs
# at the smallest chi (the minimum chi). In this case, the yellow line.

# procedure for how to do this:
# first determine the physically possible min_chi2
# find the theta for which M/Mp = 1 for that min_chi2
# calculate the min_chi2 assuming that theta