# -*- coding: utf-8 -*-
"""
Created on Wed Jun  8 16:40:24 2022

@author: wjwong
"""
# %% * Import modules and tools

import functions as fn
import parametric_functions as pfn
import matplotlib.pyplot as plt
import matplotlib.lines as mlines
import numpy as np
from scipy.interpolate import griddata
from scipy import optimize
import mimic_alpha as ma # function to mimic transparency for .eps

# For 3D arrows and annotations

from mpl_toolkits.mplot3d.proj3d import proj_transform
from mpl_toolkits.mplot3d.axes3d import Axes3D
from matplotlib.text import Annotation

class Annotation3D(Annotation):

    def __init__(self, text, xyz, *args, **kwargs):
        super().__init__(text, xy=(0, 0), *args, **kwargs)
        self._xyz = xyz

    def draw(self, renderer):
        x2, y2, z2 = proj_transform(*self._xyz, self.axes.M)
        self.xy = (x2, y2)
        super().draw(renderer)
        
def _annotate3D(ax, text, xyz, *args, **kwargs):
    '''Add anotation `text` to an `Axes3d` instance.'''

    annotation = Annotation3D(text, xyz, *args, **kwargs)
    ax.add_artist(annotation)
    
setattr(Axes3D, 'annotate3D', _annotate3D)

from matplotlib.patches import FancyArrowPatch

class Arrow3D(FancyArrowPatch):

    def __init__(self, x, y, z, dx, dy, dz, *args, **kwargs):
        super().__init__((0, 0), (0, 0), *args, **kwargs)
        self._xyz = (x, y, z)
        self._dxdydz = (dx, dy, dz)

    def draw(self, renderer):
        x1, y1, z1 = self._xyz
        dx, dy, dz = self._dxdydz
        x2, y2, z2 = (x1 + dx, y1 + dy, z1 + dz)

        xs, ys, zs = proj_transform((x1, x2), (y1, y2), (z1, z2), self.axes.M)
        self.set_positions((xs[0], ys[0]), (xs[1], ys[1]))
        super().draw(renderer)
        
    def do_3d_projection(self, renderer=None):
        x1, y1, z1 = self._xyz
        dx, dy, dz = self._dxdydz
        x2, y2, z2 = (x1 + dx, y1 + dy, z1 + dz)

        xs, ys, zs = proj_transform((x1, x2), (y1, y2), (z1, z2), self.axes.M)
        self.set_positions((xs[0], ys[0]), (xs[1], ys[1]))

        return np.min(zs) 
    
def _arrow3D(ax, x, y, z, dx, dy, dz, *args, **kwargs):
    '''Add an 3d arrow to an `Axes3D` instance.'''

    arrow = Arrow3D(x, y, z, dx, dy, dz, *args, **kwargs)
    ax.add_artist(arrow)
    
setattr(Axes3D, 'arrow3D', _arrow3D)

# Function for saving images as emf for use in word
# First save as svg, then use Inkscape to convert to emf
import subprocess
import os
inkscapePath = r'C:\Program Files\Inkscape\bin\inkscape.exe'
savePath= os.getcwd()+'\Images'
def exportEmf(savePath, plotName, fig=None, keepSVG=False):
    """Save a figure as an emf file

    Parameters
    ----------
    savePath : str, the path to the directory you want the image saved in
    plotName : str, the name of the image 
    fig : matplotlib figure, (optional, default uses gca)
    keepSVG : bool, whether to keep the interim svg file
    """

    figFolder = savePath + r"\{}.{}"
    svgFile = figFolder.format(plotName,"svg")
    emfFile = figFolder.format(plotName,"emf")
    if fig:
        use=fig
    else:
        use=plt
    use.savefig(svgFile, bbox_inches='tight')
    subprocess.run([inkscapePath, svgFile, '--export-filename', emfFile])
 
    if not keepSVG:
        os.system('del "{}"'.format(svgFile))

def ceil_prec(a, precision=0): # for ceiling to a given number of decimal places
    return np.true_divide(np.ceil(a * 10**precision), 10**precision)

def floor_prec(a, precision=0): # for flooring to a given number of decimal places
    return np.true_divide(np.floor(a * 10**precision), 10**precision)

# %% * Load data into dictionary

batch_names =  [
                'all runs',
                'all 690 runs',
                'all 890 runs',
                'vary YoT vary Y t_HSA800 default H and B',
                'vary B vary YoT t_HSA800 Y690 Lw_40p0eps',
                'vary B vary YoT t_HSA800 Y890 Lw_40p0eps',
                'vary YoT vary Y t_SM490 default H and B',
                'vary YoT vary Y t_SM490_12mm default H and B',
                'vary H vary YoT t_SM490_12mm Y690 Lf_10p1eps',
                'vary H vary YoT t_SM490_12mm Y890 Lf_10p1eps',
                'vary YoT vary Y t_HSA800 Lw_40p0eps Lf_10p1eps',
                'vary YoT vary Y t_SM490_12mm Lf_10p1eps Lw_52p5eps'
                ]

data_dict = {}

for i, batch_name in enumerate(batch_names):
    
    # mypath = r'C:\Users\wjwong\Documents\Research\plastic hinge\plastic-hinge-postprocessing\\' + batch_name + '\\'
    # data_dict[batch_name] = pfn.load_data(mypath)

    mypath = r'C:\Users\wjwong\Documents\Research\plastic hinge\plastic-hinge-postprocessing\Parametric runs with a = 0.53tw\\' + batch_name + '\\'
        
    # for parameter in pfn.load_data(mypath):
    loaded_data = pfn.load_data(mypath)
    data_dict[batch_name] = {
        'YoT': loaded_data[0],
        'Y': loaded_data[1],
        'Ldp': loaded_data[2],
        'B': loaded_data[3],
        'H': loaded_data[4],
        'tf': loaded_data[5],
        'tw': loaded_data[6],
        'imp': loaded_data[7],
        'Ldf': loaded_data[8],
        'Ldw': loaded_data[9],
        'Mp': loaded_data[10],
        'Tp': loaded_data[11],
        'L': loaded_data[12],
        'R': loaded_data[13],
        'Ldf_eps': loaded_data[14],
        'Ldw_eps': loaded_data[15],
        'Mnu': loaded_data[16],
        'Ru':loaded_data[17],
        'idx_R':loaded_data[18],
        'idx_Ru':loaded_data[19],
        'Mel':loaded_data[20],
        'idx_Mel':loaded_data[21],
        'data_list':loaded_data[22]
        }

# %% Plot H against L

plt.close('all')

# this_batch = 'vary B vary YoT t_HSA800 Y890 Lw_40p0eps'
this_batch = 'all runs'

fig, ax = plt.subplots()
ax.plot(data_dict[this_batch]['H'], data_dict[this_batch]['L'], 'o')
ax.set_xlabel(r'$H$',  fontsize=20)
ax.set_ylabel(r'$L$', fontsize=20)
ax.grid()
ax.tick_params(axis='both', which='major', labelsize=12)
ax.tick_params(axis='both', which='minor', labelsize=12)
ax.set_title(r'',  fontsize=20)

# %% Plot rotation capacities YoT and Y - HS800 thickness

plt.close('all')

this_batch = 'vary YoT vary Y t_HSA800 default H and B'

# Plot R against YoT and Y

fig = plt.figure()
ax = plt.axes(projection='3d')

# ax.scatter3D(data_dict[this_batch]['YoT'], data_dict[this_batch]['Y'], data_dict[this_batch]['R'])

for i, imp in enumerate(data_dict[this_batch]['imp']):
    if imp==0 and data_dict[this_batch]['R'][i]!=0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Y'][i],data_dict[this_batch]['R'][i], marker='o', fillstyle='none', color='black')
    elif imp==1 and data_dict[this_batch]['R'][i]!=0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Y'][i],data_dict[this_batch]['R'][i], marker='o', color='black')
    if imp==0 and data_dict[this_batch]['R'][i]==0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Y'][i],data_dict[this_batch]['R'][i], marker='s', fillstyle='none', color='black')
    elif imp==1 and data_dict[this_batch]['R'][i]==0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Y'][i],data_dict[this_batch]['R'][i], marker='s', color='black')
        
ax.set_xlabel(r'$\sigma_y/\sigma_u$',  fontsize=20)
ax.set_ylabel(r'$Y$', fontsize=20)
ax.set_zlabel(r'$R$', fontsize=20)
ax.grid()
ax.tick_params(axis='both', which='major', labelsize=12)
ax.tick_params(axis='both', which='minor', labelsize=12)
ax.set_title(r'',  fontsize=20)

xi = np.linspace(data_dict[this_batch]['YoT'].min(),data_dict[this_batch]['YoT'].max())
yi = np.linspace(data_dict[this_batch]['Y'].min(),data_dict[this_batch]['Y'].max())
zi = griddata((data_dict[this_batch]['YoT'], data_dict[this_batch]['Y']), data_dict[this_batch]['R'], (xi[None,:], yi[:,None]), method='linear')

xig, yig = np.meshgrid(xi, yi)

surf = ax.plot_surface(xig, yig, zi, cmap='gist_gray',
                       vmin=np.floor(np.nanmin(zi)), vmax=np.ceil(np.nanmax(zi)))
fig.colorbar(surf, shrink=0.5, aspect=5)

# Plot Mnu against YoT and Y

fig = plt.figure()
ax = plt.axes(projection='3d')

# ax.scatter3D(data_dict[this_batch]['YoT'], data_dict[this_batch]['Y'], data_dict[this_batch]['R'])

for i, imp in enumerate(data_dict[this_batch]['imp']):
    if imp==0 and data_dict[this_batch]['R'][i]!=0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Y'][i],data_dict[this_batch]['Mnu'][i], marker='o', fillstyle='none', color='black')
    elif imp==1 and data_dict[this_batch]['R'][i]!=0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Y'][i],data_dict[this_batch]['Mnu'][i], marker='o', color='black')
    if imp==0 and data_dict[this_batch]['R'][i]==0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Y'][i],data_dict[this_batch]['Mnu'][i], marker='s', fillstyle='none', color='black')
    elif imp==1 and data_dict[this_batch]['R'][i]==0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Y'][i],data_dict[this_batch]['Mnu'][i], marker='s', color='black')
        
ax.set_xlabel(r'$\sigma_y/\sigma_u$',  fontsize=20)
ax.set_ylabel(r'$Y$', fontsize=20)
ax.set_zlabel(r'$M_u/M_{p}$', fontsize=20)
ax.grid()
ax.tick_params(axis='both', which='major', labelsize=12)
ax.tick_params(axis='both', which='minor', labelsize=12)
ax.set_title(r'',  fontsize=20)

xi = np.linspace(data_dict[this_batch]['YoT'].min(),data_dict[this_batch]['YoT'].max())
yi = np.linspace(data_dict[this_batch]['Y'].min(),data_dict[this_batch]['Y'].max())
zi = griddata((data_dict[this_batch]['YoT'], data_dict[this_batch]['Y']), data_dict[this_batch]['Mnu'], (xi[None,:], yi[:,None]), method='linear')

xig, yig = np.meshgrid(xi, yi)

surf = ax.plot_surface(xig, yig, zi, cmap='gist_gray',
                       vmin=np.floor(np.nanmin(zi)), vmax=np.ceil(np.nanmax(zi)))
fig.colorbar(surf, shrink=0.5, aspect=5)

# %% * Plot rotation capacities YoT and Y - HS800, SM490, SM490_12

plt.close('all')

fig = plt.figure()
ax = plt.axes(projection='3d',computed_zorder=False)

ax.set_xlabel(r'$\sigma_y/\sigma_u$',  fontsize=30, labelpad=20)
ax.set_ylabel(r'$\sigma_y$ [MPa]', fontsize=30, labelpad=20)
ax.set_zlabel(r'$R$', fontsize=30)
ax.grid()
ax.tick_params(axis='both', which='major', labelsize=20)
ax.tick_params(axis='both', which='minor', labelsize=20)
ax.set_title(r'',  fontsize=20)
plt.xticks(np.arange(0.8,1.01,0.05))
ax.set_xlim([0.78,1])
#

this_batch = 'vary YoT vary Y t_SM490_12mm default H and B'

for i, imp in enumerate(data_dict[this_batch]['imp']):
    if imp==0 and data_dict[this_batch]['R'][i]!=0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Y'][i],data_dict[this_batch]['R'][i], marker='o', fillstyle='none', color='orange')
    elif imp==1 and data_dict[this_batch]['R'][i]!=0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Y'][i],data_dict[this_batch]['R'][i], marker='o', color='orange')
    if imp==0 and data_dict[this_batch]['R'][i]==0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Y'][i],data_dict[this_batch]['R'][i], marker='s', fillstyle='none', color='orange')
    elif imp==1 and data_dict[this_batch]['R'][i]==0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Y'][i],data_dict[this_batch]['R'][i], marker='s', color='orange')

idx = np.nonzero(data_dict[this_batch]['R'])
xdat = data_dict[this_batch]['YoT'][idx]
ydat = data_dict[this_batch]['Y'][idx]
zdat = data_dict[this_batch]['R'][idx]

xi = np.linspace(xdat.min(),xdat.max())
yi = np.linspace(ydat.min(),ydat.max())
zi = griddata((xdat, ydat), zdat, (xi[None,:], yi[:,None]), method='linear')
xig, yig = np.meshgrid(xi, yi)

surf1 = ax.plot_surface(xig, yig, zi, cmap='Oranges_r', alpha=1, # use 0.7 using transparency 
                       vmin=np.floor(np.nanmin(zi)), vmax=np.ceil(np.nanmax(zi)))
#

this_batch = 'vary YoT vary Y t_SM490 default H and B'

for i, imp in enumerate(data_dict[this_batch]['imp']):
    if imp==0 and data_dict[this_batch]['R'][i]!=0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Y'][i],data_dict[this_batch]['R'][i], marker='o', fillstyle='none', color='purple')
    elif imp==1 and data_dict[this_batch]['R'][i]!=0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Y'][i],data_dict[this_batch]['R'][i], marker='o', color='purple')
    if imp==0 and data_dict[this_batch]['R'][i]==0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Y'][i],data_dict[this_batch]['R'][i], marker='s', fillstyle='none', color='purple')
    elif imp==1 and data_dict[this_batch]['R'][i]==0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Y'][i],data_dict[this_batch]['R'][i], marker='s', color='purple')

idx = np.nonzero(data_dict[this_batch]['R'])
xdat = data_dict[this_batch]['YoT'][idx]
ydat = data_dict[this_batch]['Y'][idx]
zdat = data_dict[this_batch]['R'][idx]

xi = np.linspace(xdat.min(),xdat.max())
yi = np.linspace(ydat.min(),ydat.max())
zi = griddata((xdat, ydat), zdat, (xi[None,:], yi[:,None]), method='linear')
xig, yig = np.meshgrid(xi, yi)

surf2 = ax.plot_surface(xig, yig, zi, cmap='Purples_r', alpha=1, # use 0.7 using transparency
                       vmin=np.floor(np.nanmin(zi)), vmax=np.ceil(np.nanmax(zi)))

#

this_batch = 'vary YoT vary Y t_HSA800 default H and B'

for i, imp in enumerate(data_dict[this_batch]['imp']):
    if imp==0 and data_dict[this_batch]['R'][i]!=0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Y'][i],data_dict[this_batch]['R'][i], marker='o', fillstyle='none', color='black')
    elif imp==1 and data_dict[this_batch]['R'][i]!=0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Y'][i],data_dict[this_batch]['R'][i], marker='o', color='black')
    if imp==0 and data_dict[this_batch]['R'][i]==0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Y'][i],data_dict[this_batch]['R'][i], marker='s', fillstyle='none', color='black')
    elif imp==1 and data_dict[this_batch]['R'][i]==0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Y'][i],data_dict[this_batch]['R'][i], marker='s', color='black')

idx = np.nonzero(data_dict[this_batch]['R'])
xdat = data_dict[this_batch]['YoT'][idx]
ydat = data_dict[this_batch]['Y'][idx]
zdat = data_dict[this_batch]['R'][idx]

xi = np.linspace(xdat.min(),xdat.max())
yi = np.linspace(ydat.min(),ydat.max())
zi = griddata((xdat, ydat), zdat, (xi[None,:], yi[:,None]), method='linear')
xig, yig = np.meshgrid(xi, yi)

surf3 = ax.plot_surface(xig, yig, zi, cmap='gist_gray', alpha=1, # use 0.7 using transparency
                       vmin=np.floor(np.nanmin(zi)), vmax=np.ceil(np.nanmax(zi)))

# colorbars

clb3 = fig.colorbar(surf3, shrink=0.5, aspect=5, pad=0.015)
clb3.ax.set_title(r'$t_f = 20$' ' mm \n'
                  r'$t_w = 17.6$' ' mm \n'
                  r'$H = 400$' ' mm \n' 
                  r'$B = 200$' ' mm \n' 
                  r'$\lambda_p = [0.30, 0.42]$', fontsize=20, pad=35)
clb3.ax.tick_params(labelsize=20)
clb2 = fig.colorbar(surf2, shrink=0.5, aspect=5)
clb2.ax.set_title(r'$t_f = 15.0$' ' mm \n'
                  r'$t_w = 11.0$' ' mm \n'
                  r'$H = 400$' ' mm \n'  
                  r'$B = 220$' ' mm \n'
                  r'$\lambda_p = [0.46, 0.65]$', fontsize=20, pad=35)
clb2.ax.tick_params(labelsize=20)
clb1 = fig.colorbar(surf1, shrink=0.5, aspect=5, pad=0.1)
clb1.ax.set_title(r'$t_f = 12$' ' mm \n'
                  r'$t_w = 11$' ' mm \n'
                  r'$H = 400$' ' mm \n' 
                  r'$B = 220$' ' mm \n' 
                  r'$\lambda_p = [0.54, 0.77]$', fontsize=20, pad=35)
clb1.ax.tick_params(labelsize=20)

# Legend for markers

import matplotlib.lines as mlines

filled_circle = mlines.Line2D([], [], color='black', marker='o', linestyle='None',
                          markersize=10, label='LBA peak in web; GMNIA achieves $M_{pl}$')
empty_circle = mlines.Line2D([], [], color='black', marker='o', fillstyle = 'none', linestyle='None',
                          markersize=10, label='LBA peak in flange; GMNIA achieves $M_{pl}$')
filled_square = mlines.Line2D([], [], color='black', marker='s', linestyle='None', 
                          markersize=10, label='LBA peak in web; GMNIA achieves $M_{pl}$')
empty_square = mlines.Line2D([], [], color='black', marker='s', fillstyle = 'none', linestyle='None',
                          markersize=10, label='LBA peak in flange; GMNIA does not achieve $M_{pl}$')

# plt.legend(handles=[filled_circle, empty_circle, filled_square, empty_square],
           # loc='center left', bbox_to_anchor=(1.1, 0.1))
plt.legend(handles=[empty_circle, empty_square],
            loc='center left', bbox_to_anchor=(1.05, 0.1), fontsize=20)
plt.show()

# using bbox_inches='tight' makes the color in the colobar displaced, so make code here makes figure appear as intended in window before saving
ax.view_init(elev=20, azim=315) # rotate 3D plot
plt.show()
plt.subplots_adjust(left=0, bottom=0, right=1, top=1, wspace=0, hspace=0) # to remove blank space
figManager = plt.get_current_fig_manager()
figManager.window.showMaximized()
# fig.savefig('R_against_YoT_Y.eps', format='eps') 

# fig.savefig('R against YoT and Y.png', format='png', dpi=1200, bbox_inches='tight')
# fig.savefig('R against YoT and Y.pdf', format='pdf', bbox_inches='tight')

# %% Plot Ldp against Y

plt.close('all')

this_batch = 'all runs'

fig, ax = plt.subplots()
ax.plot(data_dict[this_batch]['Y'], data_dict[this_batch]['Ldp'], 'o')
ax.set_xlabel(r'$\sigma_y$',  fontsize=20)
ax.set_ylabel(r'$\lambda_p$', fontsize=20)
ax.grid()
ax.tick_params(axis='both', which='major', labelsize=12)
ax.tick_params(axis='both', which='minor', labelsize=12)
ax.set_title(r'',  fontsize=20)

# %% * Plot rotation capacities YoT and Ldp (using Ldp in place of Y) - HS800, SM490, SM490_12

my_cmap = ma.cmap('Oranges_r', 0.6, bg="midnightblue", set_under=None, set_over=None,
         set_bad=None, out_cmap_name=None)

plt.close('all')

fig = plt.figure()
ax = plt.axes(projection='3d',computed_zorder=False)

ax.set_xlabel(r'$\sigma_y/\sigma_u$',  fontsize=30, labelpad=20)
ax.set_ylabel(r'$\lambda_p$', fontsize=30, labelpad=20)
ax.set_zlabel(r'$R$', fontsize=30)
ax.grid()
ax.tick_params(axis='both', which='major', labelsize=20)
ax.tick_params(axis='both', which='minor', labelsize=20)
ax.set_title(r'',  fontsize=20)

fig2, ax2 = plt.subplots()
#

this_batch = 'vary YoT vary Y t_HSA800 default H and B'

for i, imp in enumerate(data_dict[this_batch]['imp']):
    if imp==0 and data_dict[this_batch]['R'][i]!=0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Ldp'][i],data_dict[this_batch]['R'][i], marker='o', fillstyle='none', color='black',)
    elif imp==1 and data_dict[this_batch]['R'][i]!=0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Ldp'][i],data_dict[this_batch]['R'][i], marker='o', color='black')
    if imp==0 and data_dict[this_batch]['R'][i]==0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Ldp'][i],data_dict[this_batch]['R'][i], marker='s', fillstyle='none', color='black',)
    elif imp==1 and data_dict[this_batch]['R'][i]==0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Ldp'][i],data_dict[this_batch]['R'][i], marker='s', color='black')


idx = np.nonzero(data_dict[this_batch]['R'])
xdat = data_dict[this_batch]['YoT'][idx]
ydat = data_dict[this_batch]['Ldp'][idx]
zdat = data_dict[this_batch]['R'][idx]

xi = np.linspace(xdat.min(),xdat.max())
yi = np.linspace(ydat.min(),ydat.max())
zi = griddata((xdat, ydat), zdat, (xi[None,:], yi[:,None]), method='linear')
xig, yig = np.meshgrid(xi, yi)

surf = ax.plot_surface(xig, yig, zi, cmap='gist_gray', alpha=1, # use 0.7 using transparency
                        vmin=np.floor(np.nanmin(zi)), vmax=np.ceil(np.nanmax(zi)))

clb3 = fig.colorbar(surf, shrink=0.5, aspect=5, pad=0.015)
clb3.ax.set_title(r'$t_f = 20$' ' mm \n'
                  r'$t_w = 17.6$' ' mm \n'
                  r'$H = 400$' ' mm \n' 
                  r'$B = 200$' ' mm \n' 
                  r'$\lambda_p = [0.30, 0.42]$', fontsize=20, pad=35)
clb3.ax.tick_params(labelsize=20)

# 2D cut through 3D plot at YoT = 0.8
YoT_cut = 0.8
xi_strip = np.array(YoT_cut)
zi_strip = griddata((xdat, ydat), zdat, (xi_strip, yi[:,None]), method='linear')
xig_strip, yig_strip = np.meshgrid(xi_strip, yi)
# Plot 2D cut for given YoT
ax2.plot(yig_strip, zi_strip,
         marker='None',linestyle='-',linewidth=3,
         label=r'$R$ for $\sigma_y/\sigma_u={YoT}$; $t_f = {tf}$; $t_w = {tw}$; $H = {H}$; $B = {B}$ mm'
         .format(YoT=YoT_cut,tf=data_dict[this_batch]['tf'][0],tw=data_dict[this_batch]['tw'][0],H=data_dict[this_batch]['H'][0],B=data_dict[this_batch]['B'][0]),
         color='black')
# Plot parametric points
for i, imp in enumerate(data_dict[this_batch]['imp']):
    if data_dict[this_batch]['YoT'][i]==YoT_cut: 
        if imp==0 and data_dict[this_batch]['R'][i]!=0:
            ax2.plot(data_dict[this_batch]['Ldp'][i],data_dict[this_batch]['R'][i], marker='o', fillstyle='none', color='black', markersize=7, markeredgewidth=2)
        elif imp==1 and data_dict[this_batch]['R'][i]!=0:
            ax2.plot(data_dict[this_batch]['Ldp'][i],data_dict[this_batch]['R'][i], marker='o', color='black', markersize=7, markeredgewidth=2)
        if imp==0 and data_dict[this_batch]['R'][i]==0:
            ax2.plot(data_dict[this_batch]['Ldp'][i],data_dict[this_batch]['R'][i], marker='s', fillstyle='none', color='black', markersize=7, markeredgewidth=2)
        elif imp==1 and data_dict[this_batch]['R'][i]==0:
            ax2.plot(data_dict[this_batch]['Ldp'][i],data_dict[this_batch]['R'][i], marker='s', color='black', markersize=7, markeredgewidth=2)

# 2D cut through 3D plot at YoT = 0.89
YoT_cut = 0.89
xi_strip = np.array(YoT_cut)
zi_strip = griddata((xdat, ydat), zdat, (xi_strip, yi[:,None]), method='linear')
xig_strip, yig_strip = np.meshgrid(xi_strip, yi)
# Plot 2D cut for given YoT
ax2.plot(yig_strip, zi_strip, 
         marker='None',linestyle='--',linewidth=3,
         label=r'$R$ for $\sigma_y/\sigma_u={YoT}$; $t_f = {tf}$; $t_w = {tw}$; $H = {H}$; $B = {B}$ mm'
         .format(YoT=YoT_cut,tf=data_dict[this_batch]['tf'][0],tw=data_dict[this_batch]['tw'][0],H=data_dict[this_batch]['H'][0],B=data_dict[this_batch]['B'][0]),
         color='black')
# Plot parametric points
for i, imp in enumerate(data_dict[this_batch]['imp']):
    if data_dict[this_batch]['YoT'][i]==YoT_cut: 
        if imp==0 and data_dict[this_batch]['R'][i]!=0:
            ax2.plot(data_dict[this_batch]['Ldp'][i],data_dict[this_batch]['R'][i], marker='o', fillstyle='none', color='black', markersize=7, markeredgewidth=2)
        elif imp==1 and data_dict[this_batch]['R'][i]!=0:
            ax2.plot(data_dict[this_batch]['Ldp'][i],data_dict[this_batch]['R'][i], marker='o', color='black', markersize=7, markeredgewidth=2)
        if imp==0 and data_dict[this_batch]['R'][i]==0:
            ax2.plot(data_dict[this_batch]['Ldp'][i],data_dict[this_batch]['R'][i], marker='s', fillstyle='none', color='black', markersize=7, markeredgewidth=2)
        elif imp==1 and data_dict[this_batch]['R'][i]==0:
            ax2.plot(data_dict[this_batch]['Ldp'][i],data_dict[this_batch]['R'][i], marker='s', color='black', markersize=7, markeredgewidth=2)
            
# 2D cut through 3D plot at YoT = 0.99
YoT_cut = 0.99
xi_strip = np.array(YoT_cut)
zi_strip = griddata((xdat, ydat), zdat, (xi_strip, yi[:,None]), method='linear')
xig_strip, yig_strip = np.meshgrid(xi_strip, yi)
# Plot 2D cut for given YoT
ax2.plot(yig_strip, zi_strip, 
         marker='None',linestyle=':',linewidth=3,
         label=r'$R$ for $\sigma_y/\sigma_u={YoT}$; $t_f = {tf}$; $t_w = {tw}$; $H = {H}$; $B = {B}$ mm'
         .format(YoT=YoT_cut,tf=data_dict[this_batch]['tf'][0],tw=data_dict[this_batch]['tw'][0],H=data_dict[this_batch]['H'][0],B=data_dict[this_batch]['B'][0]),
         color='black')
# Plot parametric points
for i, imp in enumerate(data_dict[this_batch]['imp']):
    if data_dict[this_batch]['YoT'][i]==YoT_cut: 
        if imp==0 and data_dict[this_batch]['R'][i]!=0:
            ax2.plot(data_dict[this_batch]['Ldp'][i],data_dict[this_batch]['R'][i], marker='o', fillstyle='none', color='black', markersize=7, markeredgewidth=2)
        elif imp==1 and data_dict[this_batch]['R'][i]!=0:
            ax2.plot(data_dict[this_batch]['Ldp'][i],data_dict[this_batch]['R'][i], marker='o', color='black', markersize=7, markeredgewidth=2)
        if imp==0 and data_dict[this_batch]['R'][i]==0:
            ax2.plot(data_dict[this_batch]['Ldp'][i],data_dict[this_batch]['R'][i], marker='s', fillstyle='none', color='black', markersize=7, markeredgewidth=2)
        elif imp==1 and data_dict[this_batch]['R'][i]==0:
            ax2.plot(data_dict[this_batch]['Ldp'][i],data_dict[this_batch]['R'][i], marker='s', color='black', markersize=7, markeredgewidth=2)
#

this_batch = 'vary YoT vary Y t_SM490 default H and B'

# Plot parametric points
for i, imp in enumerate(data_dict[this_batch]['imp']):
    if imp==0 and data_dict[this_batch]['R'][i]!=0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Ldp'][i],data_dict[this_batch]['R'][i], marker='o', fillstyle='none', color='purple', markersize=7, markeredgewidth=2)
    elif imp==1 and data_dict[this_batch]['R'][i]!=0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Ldp'][i],data_dict[this_batch]['R'][i], marker='o', color='purple', markersize=7, markeredgewidth=2)
    if imp==0 and data_dict[this_batch]['R'][i]==0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Ldp'][i],data_dict[this_batch]['R'][i], marker='s', fillstyle='none', color='purple', markersize=7, markeredgewidth=2)
    elif imp==1 and data_dict[this_batch]['R'][i]==0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Ldp'][i],data_dict[this_batch]['R'][i], marker='s', color='purple', markersize=7, markeredgewidth=2)

idx = np.nonzero(data_dict[this_batch]['R'])
xdat = data_dict[this_batch]['YoT'][idx]
ydat = data_dict[this_batch]['Ldp'][idx]
zdat = data_dict[this_batch]['R'][idx]

xi = np.linspace(xdat.min(),xdat.max())
yi = np.linspace(ydat.min(),ydat.max())
zi = griddata((xdat, ydat), zdat, (xi[None,:], yi[:,None]), method='linear')
xig, yig = np.meshgrid(xi, yi)

surf = ax.plot_surface(xig, yig, zi, cmap='Purples_r', alpha=1, # use 0.7 using transparency
                       vmin=np.floor(np.nanmin(zi)), vmax=np.ceil(np.nanmax(zi)))

clb2 = fig.colorbar(surf, shrink=0.5, aspect=5, pad=0.05)
clb2.ax.set_title(r'$t_f = 15.0$' ' mm \n'
                  r'$t_w = 11.0$' ' mm \n'
                  r'$H = 400$' ' mm \n'  
                  r'$B = 220$' ' mm \n'
                  r'$\lambda_p = [0.46, 0.65]$', fontsize=20, pad=35)
clb2.ax.tick_params(labelsize=20)

# 2D cut through 3D plot at YoT = 0.8
YoT_cut = 0.8
xi_strip = np.array(YoT_cut)
zi_strip = griddata((xdat, ydat), zdat, (xi_strip, yi[:,None]), method='linear')
xig_strip, yig_strip = np.meshgrid(xi_strip, yi)
# Plot 2D cut for given YoT
ax2.plot(yig_strip, zi_strip, 
         marker='None',linestyle='-',linewidth=3,
         label=r'$R$ for $\sigma_y/\sigma_u={YoT}$; $t_f = {tf}$; $t_w = {tw}$; $H = {H}$; $B = {B}$ mm'
         .format(YoT=YoT_cut,tf=data_dict[this_batch]['tf'][0],tw=data_dict[this_batch]['tw'][0],H=data_dict[this_batch]['H'][0],B=data_dict[this_batch]['B'][0]),
         color='purple')
# Plot parametric points
for i, imp in enumerate(data_dict[this_batch]['imp']):
    if data_dict[this_batch]['YoT'][i]==YoT_cut: 
        if imp==0 and data_dict[this_batch]['R'][i]!=0:
            ax2.plot(data_dict[this_batch]['Ldp'][i],data_dict[this_batch]['R'][i], marker='o', fillstyle='none', color='purple', markersize=7, markeredgewidth=2)
        elif imp==1 and data_dict[this_batch]['R'][i]!=0:
            ax2.plot(data_dict[this_batch]['Ldp'][i],data_dict[this_batch]['R'][i], marker='o', color='purple', markersize=7, markeredgewidth=2)
        if imp==0 and data_dict[this_batch]['R'][i]==0:
            ax2.plot(data_dict[this_batch]['Ldp'][i],data_dict[this_batch]['R'][i], marker='s', fillstyle='none', color='purple', markersize=7, markeredgewidth=2)
        elif imp==1 and data_dict[this_batch]['R'][i]==0:
            ax2.plot(data_dict[this_batch]['Ldp'][i],data_dict[this_batch]['R'][i], marker='s', color='purple', markersize=7, markeredgewidth=2)
            
# 2D cut through 3D plot at YoT = 0.89
YoT_cut = 0.89
xi_strip = np.array(YoT_cut)
zi_strip = griddata((xdat, ydat), zdat, (xi_strip, yi[:,None]), method='linear')
xig_strip, yig_strip = np.meshgrid(xi_strip, yi)
# Plot 2D cut for given YoT
ax2.plot(yig_strip, zi_strip, 
         marker='None',linestyle='--',linewidth=3,
         label=r'$R$ for $\sigma_y/\sigma_u={YoT}$; $t_f = {tf}$; $t_w = {tw}$; $H = {H}$; $B = {B}$ mm'
         .format(YoT=YoT_cut,tf=data_dict[this_batch]['tf'][0],tw=data_dict[this_batch]['tw'][0],H=data_dict[this_batch]['H'][0],B=data_dict[this_batch]['B'][0]),
         color='purple')
# Plot parametric points
for i, imp in enumerate(data_dict[this_batch]['imp']):
    if data_dict[this_batch]['YoT'][i]==YoT_cut: 
        if imp==0 and data_dict[this_batch]['R'][i]!=0:
            ax2.plot(data_dict[this_batch]['Ldp'][i],data_dict[this_batch]['R'][i], marker='o', fillstyle='none', color='purple', markersize=7, markeredgewidth=2)
        elif imp==1 and data_dict[this_batch]['R'][i]!=0:
            ax2.plot(data_dict[this_batch]['Ldp'][i],data_dict[this_batch]['R'][i], marker='o', color='purple', markersize=7, markeredgewidth=2)
        if imp==0 and data_dict[this_batch]['R'][i]==0:
            ax2.plot(data_dict[this_batch]['Ldp'][i],data_dict[this_batch]['R'][i], marker='s', fillstyle='none', color='purple', markersize=7, markeredgewidth=2)
        elif imp==1 and data_dict[this_batch]['R'][i]==0:
            ax2.plot(data_dict[this_batch]['Ldp'][i],data_dict[this_batch]['R'][i], marker='s', color='purple', markersize=7, markeredgewidth=2)
            
# 2D cut through 3D plot at YoT = 0.99
YoT_cut = 0.99
xi_strip = np.array(YoT_cut)
zi_strip = griddata((xdat, ydat), zdat, (xi_strip, yi[:,None]), method='linear')
xig_strip, yig_strip = np.meshgrid(xi_strip, yi)
# Plot 2D cut for given YoT
ax2.plot(yig_strip, zi_strip, 
         marker='None',linestyle=':',linewidth=3,
         label=r'$R$ for $\sigma_y/\sigma_u={YoT}$; $t_f = {tf}$; $t_w = {tw}$; $H = {H}$; $B = {B}$ mm'
         .format(YoT=YoT_cut,tf=data_dict[this_batch]['tf'][0],tw=data_dict[this_batch]['tw'][0],H=data_dict[this_batch]['H'][0],B=data_dict[this_batch]['B'][0]),
         color='purple')
# Plot parametric points
for i, imp in enumerate(data_dict[this_batch]['imp']):
    if data_dict[this_batch]['YoT'][i]==YoT_cut: 
        if imp==0 and data_dict[this_batch]['R'][i]!=0:
            ax2.plot(data_dict[this_batch]['Ldp'][i],data_dict[this_batch]['R'][i], marker='o', fillstyle='none', color='purple', markersize=7, markeredgewidth=2)
        elif imp==1 and data_dict[this_batch]['R'][i]!=0:
            ax2.plot(data_dict[this_batch]['Ldp'][i],data_dict[this_batch]['R'][i], marker='o', color='purple', markersize=7, markeredgewidth=2)
        if imp==0 and data_dict[this_batch]['R'][i]==0:
            ax2.plot(data_dict[this_batch]['Ldp'][i],data_dict[this_batch]['R'][i], marker='s', fillstyle='none', color='purple', markersize=7, markeredgewidth=2)
        elif imp==1 and data_dict[this_batch]['R'][i]==0:
            ax2.plot(data_dict[this_batch]['Ldp'][i],data_dict[this_batch]['R'][i], marker='s', color='purple', markersize=7, markeredgewidth=2)

#

this_batch = 'vary YoT vary Y t_SM490_12mm default H and B'
# this_batch = 'all runs'

# Plot parametric points
for i, imp in enumerate(data_dict[this_batch]['imp']):
    if imp==0 and data_dict[this_batch]['R'][i]!=0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Ldp'][i],data_dict[this_batch]['R'][i], marker='o', fillstyle='none', color='orange')
    elif imp==1 and data_dict[this_batch]['R'][i]!=0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Ldp'][i],data_dict[this_batch]['R'][i], marker='o', color='orange')
    if imp==0 and data_dict[this_batch]['R'][i]==0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Ldp'][i],data_dict[this_batch]['R'][i], marker='s', fillstyle='none', color='orange')
    elif imp==1 and data_dict[this_batch]['R'][i]==0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Ldp'][i],data_dict[this_batch]['R'][i], marker='s', color='orange')

idx = np.nonzero(data_dict[this_batch]['R'])
xdat = data_dict[this_batch]['YoT'][idx]
ydat = data_dict[this_batch]['Ldp'][idx]
zdat = data_dict[this_batch]['R'][idx]

xi = np.linspace(xdat.min(),xdat.max())
yi = np.linspace(ydat.min(),ydat.max())
zi = griddata((xdat, ydat), zdat, (xi[None,:], yi[:,None]), method='linear')
xig, yig = np.meshgrid(xi, yi)

surf = ax.plot_surface(xig, yig, zi, cmap=my_cmap, alpha=1, # use 0.7 using transparency
                        vmin=floor_prec(np.nanmin(zi),1), vmax=ceil_prec(np.nanmax(zi),1))

clb1 = fig.colorbar(surf, shrink=0.5, aspect=5, pad=0.05)
clb1.ax.set_title(r'$t_f = 12$' ' mm \n'
                  r'$t_w = 11$' ' mm \n'
                  r'$H = 400$' ' mm \n' 
                  r'$B = 220$' ' mm \n' 
                  r'$\lambda_p = [0.54, 0.77]$', fontsize=20, pad=35)
clb1.ax.tick_params(labelsize=20)

# 2D cut through 3D plot at YoT = 0.8
YoT_cut = 0.8
xi_strip = np.array(YoT_cut)
zi_strip = griddata((xdat, ydat), zdat, (xi_strip, yi[:,None]), method='linear')
xig_strip, yig_strip = np.meshgrid(xi_strip, yi)

# Plot 2D cut for given YoT
ax2.plot(np.array([0.29,0.8]), np.array([3,3]),
          marker='None', linestyle='-',linewidth=2,
          # label=r'$R=3$',
          color='blue')
ax2.plot(yig_strip, zi_strip, 
         marker='None',linestyle='-',linewidth=3,
         label=r'$R$ for $\sigma_y/\sigma_u={YoT}$; $t_f = {tf}$; $t_w = {tw}$; $H = {H}$; $B = {B}$ mm'
         .format(YoT=YoT_cut,tf=data_dict[this_batch]['tf'][0],tw=data_dict[this_batch]['tw'][0],H=data_dict[this_batch]['H'][0],B=data_dict[this_batch]['B'][0]),
         color='orange')
# Plot parametric points
for i, imp in enumerate(data_dict[this_batch]['imp']):
    if data_dict[this_batch]['YoT'][i]==YoT_cut: 
        if imp==0 and data_dict[this_batch]['R'][i]!=0:
            ax2.plot(data_dict[this_batch]['Ldp'][i],data_dict[this_batch]['R'][i], marker='o', fillstyle='none', color='orange', markersize=7, markeredgewidth=2)
        elif imp==1 and data_dict[this_batch]['R'][i]!=0:
            ax2.plot(data_dict[this_batch]['Ldp'][i],data_dict[this_batch]['R'][i], marker='o', color='orange', markersize=7, markeredgewidth=2)
        if imp==0 and data_dict[this_batch]['R'][i]==0:
            ax2.plot(data_dict[this_batch]['Ldp'][i],data_dict[this_batch]['R'][i], marker='s', fillstyle='none', color='orange', markersize=7, markeredgewidth=2)
        elif imp==1 and data_dict[this_batch]['R'][i]==0:
            ax2.plot(data_dict[this_batch]['Ldp'][i],data_dict[this_batch]['R'][i], marker='s', color='orange', markersize=7, markeredgewidth=2)
            
# 2D cut through 3D plot at YoT = 0.89
YoT_cut = 0.89
xi_strip = np.array(YoT_cut)
zi_strip = griddata((xdat, ydat), zdat, (xi_strip, yi[:,None]), method='linear')
xig_strip, yig_strip = np.meshgrid(xi_strip, yi)

# Plot 2D cut for given YoT
ax2.plot(yig_strip, zi_strip, 
         marker='None',linestyle='--',linewidth=3,
         label=r'$R$ for $\sigma_y/\sigma_u={YoT}$; $t_f = {tf}$; $t_w = {tw}$; $H = {H}$; $B = {B}$ mm'
         .format(YoT=YoT_cut,tf=data_dict[this_batch]['tf'][0],tw=data_dict[this_batch]['tw'][0],H=data_dict[this_batch]['H'][0],B=data_dict[this_batch]['B'][0]),
         color='orange')
# Plot parametric points
for i, imp in enumerate(data_dict[this_batch]['imp']):
    if data_dict[this_batch]['YoT'][i]==YoT_cut: 
        if imp==0 and data_dict[this_batch]['R'][i]!=0:
            ax2.plot(data_dict[this_batch]['Ldp'][i],data_dict[this_batch]['R'][i], marker='o', fillstyle='none', color='orange', markersize=7, markeredgewidth=2)
        elif imp==1 and data_dict[this_batch]['R'][i]!=0:
            ax2.plot(data_dict[this_batch]['Ldp'][i],data_dict[this_batch]['R'][i], marker='o', color='orange', markersize=7, markeredgewidth=2)
        if imp==0 and data_dict[this_batch]['R'][i]==0:
            ax2.plot(data_dict[this_batch]['Ldp'][i],data_dict[this_batch]['R'][i], marker='s', fillstyle='none', color='orange', markersize=7, markeredgewidth=2)
        elif imp==1 and data_dict[this_batch]['R'][i]==0:
            ax2.plot(data_dict[this_batch]['Ldp'][i],data_dict[this_batch]['R'][i], marker='s', color='orange', markersize=7, markeredgewidth=2)
#

# 2D cut through 3D plot at YoT = 0.99
YoT_cut = 0.99
xi_strip = np.array(YoT_cut)
zi_strip = griddata((xdat, ydat), zdat, (xi_strip, yi[:,None]), method='linear')
xig_strip, yig_strip = np.meshgrid(xi_strip, yi)

# Plot 2D cut for given YoT
ax2.plot(yig_strip, zi_strip, 
         marker='None',linestyle=':',linewidth=3,
         label=r'$R$ for $\sigma_y/\sigma_u={YoT}$; $t_f = {tf}$; $t_w = {tw}$; $H = {H}$; $B = {B}$ mm'
         .format(YoT=YoT_cut,tf=data_dict[this_batch]['tf'][0],tw=data_dict[this_batch]['tw'][0],H=data_dict[this_batch]['H'][0],B=data_dict[this_batch]['B'][0]),
         color='orange')
# Plot parametric points
for i, imp in enumerate(data_dict[this_batch]['imp']):
    if data_dict[this_batch]['YoT'][i]==YoT_cut: 
        if imp==0 and data_dict[this_batch]['R'][i]!=0:
            ax2.plot(data_dict[this_batch]['Ldp'][i],data_dict[this_batch]['R'][i], marker='o', fillstyle='none', color='orange', markersize=7, markeredgewidth=2)
        elif imp==1 and data_dict[this_batch]['R'][i]!=0:
            ax2.plot(data_dict[this_batch]['Ldp'][i],data_dict[this_batch]['R'][i], marker='o', color='orange', markersize=7, markeredgewidth=2)
        if imp==0 and data_dict[this_batch]['R'][i]==0:
            ax2.plot(data_dict[this_batch]['Ldp'][i],data_dict[this_batch]['R'][i], marker='s', fillstyle='none', color='orange', markersize=7, markeredgewidth=2)
        elif imp==1 and data_dict[this_batch]['R'][i]==0:
            ax2.plot(data_dict[this_batch]['Ldp'][i],data_dict[this_batch]['R'][i], marker='s', color='orange', markersize=7, markeredgewidth=2)

ax2.set_xlabel(r'$\lambda_p$',  fontsize=30)
ax2.set_ylabel(r'$R$', fontsize=30)
ax2.grid(visible=True)
ax2.tick_params(axis='both', which='major', labelsize=20)
ax2.tick_params(axis='both', which='minor', labelsize=20)
ax2.set_title(r'',  fontsize=22)
ax2.legend(fontsize=20)
ax2.set_xlim([0.29, 0.8])
figManager = plt.get_current_fig_manager()
figManager.window.showMaximized()

# fig2.savefig('R_against_Ldp.eps', format='eps', bbox_inches='tight')

# fig2.savefig('R against Ldp.png', format='png', dpi=1200, bbox_inches='tight')
# fig2.savefig('R against Ldp.pdf', format='pdf', bbox_inches='tight')

# Plane through given R
xi = np.linspace(0.785,0.99,2)
yi = np.linspace(0.25,0.8,2)
xig, yig = np.meshgrid(xi, yi)
zig = 3*np.ones(xig.shape)
# surf = ax.plot_surface(xig, yig, zig, cmap='gist_gray', alpha=0.125, 
                        # vmin=np.floor(np.nanmin(zi)), vmax=np.ceil(np.nanmax(zi))) # for transparent surface
# surf = ax.plot_wireframe(xig, yig, zig, linewidth=0.5, color='black',rstride=1, cstride=1) # for outline
surf = ax.plot([0.785, 0.785], [0.25, 0.8], [3,3], linewidth=0.5, zorder=0,
                            color='black') 
surf = ax.plot([0.99, 0.99], [0.25, 0.8], [3,3], linewidth=0.5, zorder=1,
                            color='black') 
surf = ax.plot([0.785, 0.99], [0.25, 0.25], [3,3], linewidth=0.5, zorder=0,
                            color='black') 
surf = ax.plot([0.785, 0.99], [0.8, 0.8], [3,3], linewidth=0.5, zorder=1,
                            color='black') 

# Plane through given YoT
zi = np.linspace(0,14.2,2)
yig, zig = np.meshgrid(yi, zi)
xig = 0.94*np.ones(zig.shape)
# surf = ax.plot_surface(xig, yig, zig, cmap='gist_gray', alpha=0.125, 
                        # vmin=np.floor(np.nanmin(zi)), vmax=np.ceil(np.nanmax(zi))) # for transparent surface
# surf = ax.plot_wireframe(xig, yig, zig, linewidth=0.5, color='black',rstride=1, cstride=1) # for outline
surf = ax.plot([0.94, 0.94], [0.25, 0.25], [0,14.2], linewidth=0.5, zorder=0,
                            color='black') # vertical line
surf = ax.plot([0.94, 0.94], [0.8, 0.8], [0,14.2], linewidth=0.5, zorder=1,
                            color='black') # vertical line
surf = ax.plot([0.94, 0.94], [0.25, 0.8], [0,0], linewidth=0.5, zorder=0,
                            color='black') # horizontal line
surf = ax.plot([0.94, 0.94], [0.25, 0.8], [14.2,14.2], linewidth=0.5, zorder=1,
                            color='black') # horizontal line

# Arrows for plane through given YoT
ax.arrow3D(0.78,0.25,14.2,
           0.16,0,0,
           mutation_scale=20,
           arrowstyle='<->',
           linestyle='solid')
ax.arrow3D(0.94,0.25,14.2,
           0.075,0,0,
           mutation_scale=20,
           arrowstyle='<->',
           linestyle='solid')
ax.annotate3D(r'$\sigma_y/\sigma_u<0.94$', (0.92,0.25,17), xytext=(3, 3), textcoords='offset points', fontsize=20)
ax.annotate3D(r'$\sigma_y/\sigma_u>0.94$', (1.02,0.25,17), xytext=(3, 3), textcoords='offset points', fontsize=20)

# Arrows for plane through given R
ax.arrow3D(0.8,0.82,0,
           0,0,3.4,
           mutation_scale=20,
           arrowstyle='<->',
           linestyle='solid')
ax.arrow3D(0.8,0.82,3.2,
           0,0,12,
           mutation_scale=20,
           arrowstyle='<->',
           linestyle='solid')
ax.annotate3D(r'$R<3$', (0.82,0.75,1.1), xytext=(3, 3), textcoords='offset points', fontsize=20)
ax.annotate3D(r'$R>3$', (0.82,0.75,8.8), xytext=(3, 3), textcoords='offset points', fontsize=20)

# Legend for markers

filled_circle = mlines.Line2D([], [], color='black', marker='o', linestyle='None',
                          markersize=10, label='LBA peak in web; GMNIA achieves $M_{pl}$')
empty_circle = mlines.Line2D([], [], color='black', marker='o', fillstyle = 'none', linestyle='None',
                          markersize=10, label='LBA peak in flange; GMNIA achieves $M_{pl}$')
filled_square = mlines.Line2D([], [], color='black', marker='s', linestyle='None',
                          markersize=10, label='LBA peak in web; GMNIA achieves $M_{pl}$')
empty_square = mlines.Line2D([], [], color='black', marker='s', fillstyle = 'none', linestyle='None',
                          markersize=10, label='LBA peak in flange; GMNIA does not achieve $M_{pl}$')

plt.figure(fig.number)
# plt.legend(handles=[filled_circle, empty_circle, filled_square, empty_square],
           # loc='center left', bbox_to_anchor=(1.3, 0.1), fontsize=20)
plt.legend(handles=[empty_circle, empty_square],
            loc='center left', bbox_to_anchor=(0.995, 0.14), fontsize=20)

# using bbox_inches='tight' makes the color in the colobar displaced, so make code here makes figure appear as intended in window before saving
ax.view_init(elev=20, azim=45) # rotate 3D plot
plt.show()
plt.subplots_adjust(left=0, bottom=0, right=1, top=1, wspace=0, hspace=0) # to remove blank space
figManager = plt.get_current_fig_manager()
figManager.window.showMaximized()

# fig.savefig('R_against_Ldp_YoT.eps', format='eps') 

# fig.savefig('R against Ldp and YoT.png', format='png', dpi=1200, bbox_inches='tight')
# fig.savefig('R against Ldp and YoT.pdf', format='pdf', bbox_inches='tight')

# %% Plot parametric space for all runs

plt.close('all')

this_batch = 'all runs'

fig = plt.figure()
ax = plt.axes(projection='3d')
ax.scatter3D(data_dict[this_batch]['YoT'], data_dict[this_batch]['Y'], data_dict[this_batch]['Ldp'])
ax.set_xlabel(r'$\sigma_y/\sigma_u$',  fontsize=20)
ax.set_ylabel(r'$Y$', fontsize=20)
ax.set_zlabel(r'$\lambda_p$', fontsize=20)
ax.grid()
ax.tick_params(axis='both', which='major', labelsize=12)
ax.tick_params(axis='both', which='minor', labelsize=12)
ax.set_title(r'',  fontsize=20)

# %% * Plot rotation capacities and YoT and Ldp - 1 yield strength (S690) 1 geometry (HSA800 Ldw 40eps)

plt.close('all')

this_batch = 'vary B vary YoT t_HSA800 Y690 Lw_40p0eps'
# this_batch = 'all runs'

# Plot R against YoT and Ldp

fig = plt.figure()
ax = plt.axes(projection='3d', computed_zorder=False)

# xi = np.linspace(data_dict[this_batch]['YoT'].min(),data_dict[this_batch]['YoT'].max())
# yi = np.linspace(data_dict[this_batch]['Ldp'].min(),data_dict[this_batch]['Ldp'].max())
# zi = griddata((data_dict[this_batch]['YoT'], data_dict[this_batch]['Ldp']), data_dict[this_batch]['R'], (xi[None,:], yi[:,None]), method='linear')

# xig, yig = np.meshgrid(xi, yi)

# Do not include zeros in fitting of surface
idx = np.nonzero(data_dict[this_batch]['R'])
xdat = data_dict[this_batch]['YoT'][idx]
ydat = data_dict[this_batch]['Ldp'][idx]
zdat = data_dict[this_batch]['R'][idx]

xi = np.linspace(xdat.min(),xdat.max())
yi = np.linspace(ydat.min(),ydat.max())
zi = griddata((xdat, ydat), zdat, (xi[None,:], yi[:,None]), method='linear')
xig, yig = np.meshgrid(xi, yi)

surf = ax.plot_surface(xig, yig, zi, cmap='gist_gray', alpha=0.7, 
                        vmin=np.floor(np.nanmin(zi)), vmax=np.ceil(np.nanmax(zi)))

clb1 = fig.colorbar(surf, shrink=0.7, pad=0.13)
clb1.ax.set_title('S690', fontsize=18, pad=20)
# surf.set_clim(vmin=np.floor(np.nanmin(zi)), vmax=np.ceil(np.nanmax(zi)))

# ax.scatter3D(data_dict[this_batch]['YoT'], data_dict[this_batch]['Y'], data_dict[this_batch]['R'])

for i, imp in enumerate(data_dict[this_batch]['imp']):
    if imp==0 and data_dict[this_batch]['R'][i]!=0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Ldp'][i],data_dict[this_batch]['R'][i], marker='o', fillstyle='none', color='black')
    elif imp==1 and data_dict[this_batch]['R'][i]!=0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Ldp'][i],data_dict[this_batch]['R'][i], marker='o', color='black')
    elif imp==0 and data_dict[this_batch]['R'][i]==0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Ldp'][i],data_dict[this_batch]['R'][i], marker='s', fillstyle='none', color='black')
    elif imp==1 and data_dict[this_batch]['R'][i]==0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Ldp'][i],data_dict[this_batch]['R'][i], marker='s', color='black')
        
ax.set_xlabel(r'$\sigma_y/\sigma_u$',  fontsize=20, labelpad=10)
ax.set_ylabel(r'$\lambda_p$', fontsize=20, labelpad=10)
ax.set_zlabel(r'$R$', fontsize=20)
ax.grid()
ax.tick_params(axis='both', which='major', labelsize=12)
ax.tick_params(axis='both', which='minor', labelsize=12)
ax.set_title(r'',  fontsize=20)

plotsize=fig.get_size_inches()
plt.tight_layout()
# fig.savefig('R_against_YoT_and_Ldp_for_S690_Ldw_40eps.eps', format='eps')

# 2D cut through given YoT
xi_strip = np.array(0.92)
zi_strip = griddata((xdat, ydat), zdat, (xi_strip, yi[:,None]), method='linear')
xig_strip, yig_strip = np.meshgrid(xi_strip, yi)

# Plot 2D cut on 3d plot
# fig = plt.figure()
# ax3D = fig.add_subplot(projection='3d')
# p3d = ax.scatter(xig_strip, yig_strip, zi_strip, marker='x')

# 2D cut through given YoT 
fig, ax = plt.subplots()
ax.plot(yig_strip, zi_strip, 'o')
ax.set_xlabel(r'$\lambda_p$',  fontsize=20)
ax.set_ylabel(r'$R$', fontsize=20)
ax.grid()
ax.tick_params(axis='both', which='major', labelsize=12)
ax.tick_params(axis='both', which='minor', labelsize=12)
ax.set_title(r'',  fontsize=20)

# Plot M/Mpl against YoT and Ldp

fig = plt.figure()
ax = plt.axes(projection='3d')

# xi = np.linspace(data_dict[this_batch]['YoT'].min(),data_dict[this_batch]['YoT'].max())
# yi = np.linspace(data_dict[this_batch]['Ldp'].min(),data_dict[this_batch]['Ldp'].max())
# zi = griddata((data_dict[this_batch]['YoT'], data_dict[this_batch]['Ldp']), data_dict[this_batch]['R'], (xi[None,:], yi[:,None]), method='linear')

# xig, yig = np.meshgrid(xi, yi)

# Do not include zeros in fitting of surface
idx = np.nonzero(data_dict[this_batch]['Mnu'])
xdat = data_dict[this_batch]['YoT'][idx]
ydat = data_dict[this_batch]['Ldp'][idx]
zdat = data_dict[this_batch]['Mnu'][idx]

xi = np.linspace(xdat.min(),xdat.max())
yi = np.linspace(ydat.min(),ydat.max())
zi = griddata((xdat, ydat), zdat, (xi[None,:], yi[:,None]), method='linear')
xig, yig = np.meshgrid(xi, yi)

my_cmap = ma.cmap('gist_gray', 0.7, bg="white", set_under=None, set_over=None,
         set_bad=None, out_cmap_name=None)

surf = ax.plot_surface(xig, yig, zi, cmap=my_cmap, alpha=1, 
                       vmin=np.nanmin(zi), vmax=np.nanmax(zi))
# my_cmap is used to fake transparency, creating an equivalent opaque color, so here alpha=1.
# if not limited by format that cannot support transparency, directly use alpha=0.7 with cmap ='gist_gray'

clb1 = fig.colorbar(surf, shrink=0.5, aspect=5)
clb1.ax.set_title('S690')
# surf.set_clim(vmin=np.floor(np.nanmin(zi)), vmax=np.ceil(np.nanmax(zi)))

# ax.scatter3D(data_dict[this_batch]['YoT'], data_dict[this_batch]['Y'], data_dict[this_batch]['R'])

for i, imp in enumerate(data_dict[this_batch]['imp']):
    if imp==0 and data_dict[this_batch]['R'][i]!=0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Ldp'][i],data_dict[this_batch]['Mnu'][i], marker='o', fillstyle='none', color='black')
    elif imp==1 and data_dict[this_batch]['R'][i]!=0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Ldp'][i],data_dict[this_batch]['Mnu'][i], marker='o', color='black')
    elif imp==0 and data_dict[this_batch]['R'][i]==0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Ldp'][i],data_dict[this_batch]['Mnu'][i], marker='s', fillstyle='none', color='black')
    elif imp==1 and data_dict[this_batch]['R'][i]==0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Ldp'][i],data_dict[this_batch]['Mnu'][i], marker='s', color='black')
        
ax.set_xlabel(r'$\sigma_y/\sigma_u$',  fontsize=20)
ax.set_ylabel(r'$\lambda_p$', fontsize=20)
ax.set_zlabel(r'$M_u/M_{pl}$', fontsize=20)
ax.grid()
ax.tick_params(axis='both', which='major', labelsize=12)
ax.tick_params(axis='both', which='minor', labelsize=12)
ax.set_title(r'',  fontsize=20)      

# %% * Plot rotation capacities and YoT and Ldp - 1 yield strength (S890) 1 geometry (HSA800 Ldw 40eps)

plt.close('all')

this_batch = 'vary B vary YoT t_HSA800 Y890 Lw_40p0eps'
# this_batch = 'all runs'

# Plot R against YoT and Ldp

fig = plt.figure()
ax = plt.axes(projection='3d', computed_zorder=False)

# xi = np.linspace(data_dict[this_batch]['YoT'].min(),data_dict[this_batch]['YoT'].max())
# yi = np.linspace(data_dict[this_batch]['Ldp'].min(),data_dict[this_batch]['Ldp'].max())
# zi = griddata((data_dict[this_batch]['YoT'], data_dict[this_batch]['Ldp']), data_dict[this_batch]['R'], (xi[None,:], yi[:,None]), method='linear')

# xig, yig = np.meshgrid(xi, yi)

# Do not include zeros in fitting of surface
idx = np.nonzero(data_dict[this_batch]['R'])
xdat = data_dict[this_batch]['YoT'][idx]
ydat = data_dict[this_batch]['Ldp'][idx]
zdat = data_dict[this_batch]['R'][idx]

xi = np.linspace(xdat.min(),xdat.max())
yi = np.linspace(ydat.min(),ydat.max())
zi = griddata((xdat, ydat), zdat, (xi[None,:], yi[:,None]), method='linear')
xig, yig = np.meshgrid(xi, yi)

my_cmap = ma.cmap('Purples_r', 0.7, bg="white", set_under=None, set_over=None,
         set_bad=None, out_cmap_name=None)

surf = ax.plot_surface(xig, yig, zi, cmap=my_cmap, alpha=1, 
                        vmin=np.floor(np.nanmin(zi)), vmax=np.ceil(np.nanmax(zi)))
# my_cmap is used to fake transparency, creating an equivalent opaque color, so here alpha=1.
# if not limited by format that cannot support transparency, directly use alpha=0.7 with cmap ='Purples_r'

clb1 = fig.colorbar(surf, shrink=0.7, pad=0.13)
clb1.ax.set_title('S890', fontsize=18, pad=20)
# surf.set_clim(vmin=np.floor(np.nanmin(zi)), vmax=np.ceil(np.nanmax(zi)))

# ax.scatter3D(data_dict[this_batch]['YoT'], data_dict[this_batch]['Y'], data_dict[this_batch]['R'])

for i, imp in enumerate(data_dict[this_batch]['imp']):
    if imp==0 and data_dict[this_batch]['R'][i]!=0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Ldp'][i],data_dict[this_batch]['R'][i], marker='o', fillstyle='none', color='purple')
    elif imp==1 and data_dict[this_batch]['R'][i]!=0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Ldp'][i],data_dict[this_batch]['R'][i], marker='o', color='purple')
    elif imp==0 and data_dict[this_batch]['R'][i]==0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Ldp'][i],data_dict[this_batch]['R'][i], marker='s', fillstyle='none', color='purple')
    elif imp==1 and data_dict[this_batch]['R'][i]==0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Ldp'][i],data_dict[this_batch]['R'][i], marker='s', color='purple')
        
ax.set_xlabel(r'$\sigma_y/\sigma_u$',  fontsize=20, labelpad=10)
ax.set_ylabel(r'$\lambda_p$', fontsize=20, labelpad=10)
ax.set_zlabel(r'$R$', fontsize=20)
ax.grid()
ax.tick_params(axis='both', which='major', labelsize=12)
ax.tick_params(axis='both', which='minor', labelsize=12)
ax.set_title(r'',  fontsize=20)

plt.tight_layout()
# fig.savefig('R_against_YoT_and_Ldp_for_S890_Ldw_40eps.eps', format='eps')

# 2D cut through given YoT
xi_strip = np.array(0.92)
zi_strip = griddata((xdat, ydat), zdat, (xi_strip, yi[:,None]), method='linear')
xig_strip, yig_strip = np.meshgrid(xi_strip, yi)

# Plot 2D cut on 3d plot
# fig = plt.figure()
# ax3D = fig.add_subplot(projection='3d')
# p3d = ax.scatter(xig_strip, yig_strip, zi_strip, marker='x')

# 2D cut through given YoT 
fig, ax = plt.subplots()
ax.plot(yig_strip, zi_strip, 'o')
ax.set_xlabel(r'$\lambda_p$',  fontsize=20)
ax.set_ylabel(r'$R$', fontsize=20)
ax.grid()
ax.tick_params(axis='both', which='major', labelsize=12)
ax.tick_params(axis='both', which='minor', labelsize=12)
ax.set_title(r'',  fontsize=20)

# Plot M/Mpl against YoT and Ldp

fig = plt.figure()
ax = plt.axes(projection='3d')

# xi = np.linspace(data_dict[this_batch]['YoT'].min(),data_dict[this_batch]['YoT'].max())
# yi = np.linspace(data_dict[this_batch]['Ldp'].min(),data_dict[this_batch]['Ldp'].max())
# zi = griddata((data_dict[this_batch]['YoT'], data_dict[this_batch]['Ldp']), data_dict[this_batch]['R'], (xi[None,:], yi[:,None]), method='linear')

# xig, yig = np.meshgrid(xi, yi)

# Do not include zeros in fitting of surface
idx = np.nonzero(data_dict[this_batch]['Mnu'])
xdat = data_dict[this_batch]['YoT'][idx]
ydat = data_dict[this_batch]['Ldp'][idx]
zdat = data_dict[this_batch]['Mnu'][idx]

xi = np.linspace(xdat.min(),xdat.max())
yi = np.linspace(ydat.min(),ydat.max())
zi = griddata((xdat, ydat), zdat, (xi[None,:], yi[:,None]), method='linear')
xig, yig = np.meshgrid(xi, yi)

surf = ax.plot_surface(xig, yig, zi, cmap='gist_gray', alpha=0.7, 
                       vmin=np.nanmin(zi), vmax=np.nanmax(zi))

clb1 = fig.colorbar(surf, shrink=0.5, aspect=5)
clb1.ax.set_title('S890')
# surf.set_clim(vmin=np.floor(np.nanmin(zi)), vmax=np.ceil(np.nanmax(zi)))

# ax.scatter3D(data_dict[this_batch]['YoT'], data_dict[this_batch]['Y'], data_dict[this_batch]['R'])

for i, imp in enumerate(data_dict[this_batch]['imp']):
    if imp==0 and data_dict[this_batch]['R'][i]!=0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Ldp'][i],data_dict[this_batch]['Mnu'][i], marker='o', fillstyle='none', color='black')
    elif imp==1 and data_dict[this_batch]['R'][i]!=0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Ldp'][i],data_dict[this_batch]['Mnu'][i], marker='o', color='black')
    elif imp==0 and data_dict[this_batch]['R'][i]==0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Ldp'][i],data_dict[this_batch]['Mnu'][i], marker='s', fillstyle='none', color='black')
    elif imp==1 and data_dict[this_batch]['R'][i]==0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Ldp'][i],data_dict[this_batch]['Mnu'][i], marker='s', color='black')
        
ax.set_xlabel(r'$\sigma_y/\sigma_u$',  fontsize=20)
ax.set_ylabel(r'$\lambda_p$', fontsize=20)
ax.set_zlabel(r'$M_u/M_{pl}$', fontsize=20)
ax.grid()
ax.tick_params(axis='both', which='major', labelsize=12)
ax.tick_params(axis='both', which='minor', labelsize=12)
ax.set_title(r'',  fontsize=20)      

# %% * Plot rotation capacities and YoT and Ldp  - 2 yield strengths; 1 same geometry (HSA800 Ldw 40eps)

plt.close('all')

this_batch = 'vary B vary YoT t_HSA800 Y690 Lw_40p0eps'
# this_batch = 'all runs'

fig = plt.figure()
ax = plt.axes(projection='3d')

# xi = np.linspace(data_dict[this_batch]['YoT'].min(),data_dict[this_batch]['YoT'].max())
# yi = np.linspace(data_dict[this_batch]['Ldp'].min(),data_dict[this_batch]['Ldp'].max())
# zi = griddata((data_dict[this_batch]['YoT'], data_dict[this_batch]['Ldp']), data_dict[this_batch]['R'], (xi[None,:], yi[:,None]), method='linear')

# xig, yig = np.meshgrid(xi, yi)

# Do not include zeros in fitting of sur
idx = np.nonzero(data_dict[this_batch]['R'])
xdat = data_dict[this_batch]['YoT'][idx]
ydat = data_dict[this_batch]['Ldp'][idx]
zdat = data_dict[this_batch]['R'][idx]

xi = np.linspace(xdat.min(),xdat.max())
yi = np.linspace(ydat.min(),ydat.max())
zi = griddata((xdat, ydat), zdat, (xi[None,:], yi[:,None]), method='linear')
xig, yig = np.meshgrid(xi, yi)

surf = ax.plot_surface(xig, yig, zi, cmap='gist_gray', alpha=0.7,
                       vmin=np.floor(np.nanmin(zi)), vmax=np.ceil(np.nanmax(zi)))

# clb1 = plt.colorbar(surf, shrink=0.5)
# clb1.ax.set_title('S690')

# ax.scatter3D(data_dict[this_batch]['YoT'], data_dict[this_batch]['Y'], data_dict[this_batch]['R'])

for i, imp in enumerate(data_dict[this_batch]['imp']):
    if imp==0 and data_dict[this_batch]['R'][i]!=0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Ldp'][i],data_dict[this_batch]['R'][i], marker='o', fillstyle='none', color='black')
    elif imp==1 and data_dict[this_batch]['R'][i]!=0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Ldp'][i],data_dict[this_batch]['R'][i], marker='o', color='black')
    elif imp==0 and data_dict[this_batch]['R'][i]==0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Ldp'][i],data_dict[this_batch]['R'][i], marker='s', fillstyle='none', color='black')
    elif imp==1 and data_dict[this_batch]['R'][i]==0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Ldp'][i],data_dict[this_batch]['R'][i], marker='s', color='black')
ax.set_xlabel(r'$\sigma_y/\sigma_u$',  fontsize=20, labelpad=10)
ax.set_ylabel(r'$\lambda_p$', fontsize=20, labelpad=10)
ax.set_zlabel(r'$R$', fontsize=20)
ax.grid()
ax.tick_params(axis='both', which='major', labelsize=12)
ax.tick_params(axis='both', which='minor', labelsize=12)
ax.set_title(r'',  fontsize=20)      
# ax.set_yticklabels(np.arange(0.3,0.476,0.025))
ax.set_yticks([0.3,0.325,0.35,0.375,0.4,0.425,0.45,0.475])
ax.set_xticks([0.8,0.85,0.90,0.95])

# 2nd 

this_batch = 'vary B vary YoT t_HSA800 Y890 Lw_40p0eps' 

# xi = np.linspace(data_dict[this_batch]['YoT'].min(),data_dict[this_batch]['YoT'].max())
# yi = np.linspace(data_dict[this_batch]['Ldp'].min(),data_dict[this_batch]['Ldp'].max())
zi = griddata((data_dict[this_batch]['YoT'], data_dict[this_batch]['Ldp']), data_dict[this_batch]['R'], (xi[None,:], yi[:,None]), method='linear')

# xig, yig = np.meshgrid(xi, yi)

idx = np.nonzero(data_dict[this_batch]['R'])
xdat = data_dict[this_batch]['YoT'][idx]
ydat = data_dict[this_batch]['Ldp'][idx]
zdat = data_dict[this_batch]['R'][idx]

xi = np.linspace(xdat.min(),xdat.max())
yi = np.linspace(ydat.min(),ydat.max())
zi = griddata((xdat, ydat), zdat, (xi[None,:], yi[:,None]), method='linear')
xig, yig = np.meshgrid(xi, yi)

surf = ax.plot_surface(xig, yig, zi, cmap='Purples_r', alpha=0.7, 
                       vmin=np.floor(np.nanmin(zi)), vmax=np.ceil(np.nanmax(zi)))

# clb2=fig.colorbar(surf, shrink=0.5, pad=0.17)
# clb2.ax.set_title('S890')

# ax.scatter3D(data_dict[this_batch]['YoT'], data_dict[this_batch]['Y'], data_dict[this_batch]['R'])

for i, imp in enumerate(data_dict[this_batch]['imp']):
    if imp==0 and data_dict[this_batch]['R'][i]!=0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Ldp'][i],data_dict[this_batch]['R'][i], marker='o', fillstyle='none', color='purple')
    elif imp==1 and data_dict[this_batch]['R'][i]!=0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Ldp'][i],data_dict[this_batch]['R'][i], marker='o', color='purple')
    elif imp==0 and data_dict[this_batch]['R'][i]==0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Ldp'][i],data_dict[this_batch]['R'][i], marker='s', fillstyle='none', color='purple')
    elif imp==1 and data_dict[this_batch]['R'][i]==0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Ldp'][i],data_dict[this_batch]['R'][i], marker='s', color='purple')

fig.tight_layout()

# Save as .png because image has transparency:
# fig.savefig('R_against_YoT_and_Ldp_for_S690_and_S890_Ldw_40eps.png', format='png', dpi=1200)

# %% Plot rotation capacities and YoT and Ldp  - 2 yield strengths various geometries

plt.close('all')

this_batch = 'all 690 runs'
# this_batch = 'all runs'

fig = plt.figure()
ax = plt.axes(projection='3d')


# Do not include zeros in fitting of surface
idx = np.nonzero(data_dict[this_batch]['R'])
xdat = data_dict[this_batch]['YoT'][idx]
ydat = data_dict[this_batch]['Ldp'][idx]
zdat = data_dict[this_batch]['R'][idx]

xi = np.linspace(xdat.min(),xdat.max())
yi = np.linspace(ydat.min(),ydat.max())
zi = griddata((xdat, ydat), zdat, (xi[None,:], yi[:,None]), method='linear')
xig, yig = np.meshgrid(xi, yi)

# surf = ax.plot_surface(xig, yig, zi, cmap='Purples_r', alpha=0.7, 
#                        vmin=np.floor(np.nanmin(zi)), vmax=np.ceil(np.nanmax(zi)))

clb1 = fig.colorbar(surf, shrink=0.5, aspect=5)
clb1.ax.set_title('S690')

for i, imp in enumerate(data_dict[this_batch]['imp']):
    if imp==0 and data_dict[this_batch]['R'][i]!=0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Ldp'][i],data_dict[this_batch]['R'][i], marker='o', fillstyle='none', color='purple')
    elif imp==1 and data_dict[this_batch]['R'][i]!=0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Ldp'][i],data_dict[this_batch]['R'][i], marker='o', color='purple')
    elif imp==0 and data_dict[this_batch]['R'][i]==0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Ldp'][i],data_dict[this_batch]['R'][i], marker='s', fillstyle='none', color='purple')
    elif imp==1 and data_dict[this_batch]['R'][i]==0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Ldp'][i],data_dict[this_batch]['R'][i], marker='s', color='purple')
ax.set_xlabel(r'$\sigma_y/\sigma_u$',  fontsize=20)
ax.set_ylabel(r'$\lambda_p$', fontsize=20)
ax.set_zlabel(r'$R$', fontsize=20)
ax.grid()
ax.tick_params(axis='both', which='major', labelsize=12)
ax.tick_params(axis='both', which='minor', labelsize=12)
ax.set_title(r'',  fontsize=20)      


# 2nd 

this_batch = 'all 890 runs' 

idx = np.nonzero(data_dict[this_batch]['R'])
xdat = data_dict[this_batch]['YoT'][idx]
ydat = data_dict[this_batch]['Ldp'][idx]
zdat = data_dict[this_batch]['R'][idx]

xi = np.linspace(xdat.min(),xdat.max())
yi = np.linspace(ydat.min(),ydat.max())
zi = griddata((xdat, ydat), zdat, (xi[None,:], yi[:,None]), method='linear')
xig, yig = np.meshgrid(xi, yi)

# surf = ax.plot_surface(xig, yig, zi, cmap='gist_gray', alpha=0.7, 
#                        vmin=np.floor(np.nanmin(zi)), vmax=np.ceil(np.nanmax(zi)))

clb2=fig.colorbar(surf, shrink=0.5, aspect=5)
clb2.ax.set_title('S890')

for i, imp in enumerate(data_dict[this_batch]['imp']):
    if imp==0 and data_dict[this_batch]['R'][i]!=0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Ldp'][i],data_dict[this_batch]['R'][i], marker='o', fillstyle='none', color='black')
    elif imp==1 and data_dict[this_batch]['R'][i]!=0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Ldp'][i],data_dict[this_batch]['R'][i], marker='o', color='black')
    elif imp==0 and data_dict[this_batch]['R'][i]==0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Ldp'][i],data_dict[this_batch]['R'][i], marker='s', fillstyle='none', color='black')
    elif imp==1 and data_dict[this_batch]['R'][i]==0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Ldp'][i],data_dict[this_batch]['R'][i], marker='s', color='black')

# %% * Plot rotation capacities and YoT and Ldf  - HSA800 Ldw = 40.0eps S690 - UR I limit
# Assess UR I limit (plane at Ldf = 10.1eps)

plt.close('all')

this_batch = 'vary B vary YoT t_HSA800 Y690 Lw_40p0eps'
# this_batch = 'all runs'

fig1 = plt.figure()
ax = plt.axes(projection='3d', computed_zorder=False)

# Do not include zeros in fitting of surface
idx = np.nonzero(data_dict[this_batch]['R'])
xdat = data_dict[this_batch]['YoT'][idx]
ydat = data_dict[this_batch]['Ldf_eps'][idx]
zdat = data_dict[this_batch]['R'][idx]

xi = np.linspace(xdat.min(),xdat.max(),100)
yi = np.linspace(ydat.min(),ydat.max(),100)
zi = griddata((xdat, ydat), zdat, (xi[None,:], yi[:,None]), method='linear')
xig, yig = np.meshgrid(xi, yi)
# emulate transparency in eps:
my_cmap = ma.cmap('Purples_r', 0.7, bg="white", set_under=None, set_over=None,
         set_bad=None, out_cmap_name=None)
surf = ax.plot_surface(xig, yig, zi, cmap='Purples_r', alpha=1, 
                        vmin=np.floor(np.nanmin(zi)), vmax=np.ceil(np.nanmax(zi)))

# position of colorbar
# where arg is [left, bottom, width, height]
# cax = fig1.add_axes([0.9, 0.35, 0.03, 0.35]) # add argument cax=cax to fig1.colorbar() to use this
clb1 = fig1.colorbar(surf, orientation='vertical', shrink=0.7, pad=0.13)
clb1.ax.set_title('S690', fontsize=18, pad=20)
# plt.show()

for i, imp in enumerate(data_dict[this_batch]['imp']):
    if imp==0 and data_dict[this_batch]['R'][i]!=0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Ldf_eps'][i],data_dict[this_batch]['R'][i], marker='o', fillstyle='none', color='purple')
    elif imp==1 and data_dict[this_batch]['R'][i]!=0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Ldf_eps'][i],data_dict[this_batch]['R'][i], marker='o', color='purple')
    elif imp==0 and data_dict[this_batch]['R'][i]==0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Ldf_eps'][i],data_dict[this_batch]['R'][i], marker='s', fillstyle='none', color='purple')
    elif imp==1 and data_dict[this_batch]['R'][i]==0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Ldf_eps'][i],data_dict[this_batch]['R'][i], marker='s', color='purple')
ax.set_xlabel(r'$\sigma_y/\sigma_u$',  fontsize=20, labelpad=10)
ax.set_ylabel(r'$\lambda_f/\varepsilon$', fontsize=20, labelpad=10)
ax.set_zlabel(r'$R$', fontsize=20)
ax.grid()
ax.tick_params(axis='both', which='major', labelsize=12)
ax.tick_params(axis='both', which='minor', labelsize=12)
ax.set_title(r'',  fontsize=20)      
ax.set_xlim([0.8, 0.99])
ax.set_ylim([6,12])
plt.tight_layout()

# 2D cut through 3D plot at YoT = 0.94
xi_strip = np.array(0.94)
zi_strip = griddata((xdat, ydat), zdat, (xi_strip, yi[:,None]), method='linear')
xig_strip, yig_strip = np.meshgrid(xi_strip, yi)

# Plot 2D cut for given YoT
fig2, ax2 = plt.subplots()
ax2.plot(yig_strip, zi_strip, 
         marker='None',linestyle='-',
         label=r'$R$ for $\sigma_y/\sigma_u=0.94$'
         '\n'
         r'and $\sigma_y=690$ MPa',
         color='purple')
ax2.plot(np.array([6,12]), np.array([3,3]),
         marker='None', linestyle='-',
         label=r'$R=3$',
         color='black')
ax2.set_xlabel(r'$\lambda_f/\varepsilon$',  fontsize=20)
ax2.set_ylabel(r'$R$', fontsize=20)
ax2.grid()
ax2.tick_params(axis='both', which='major', labelsize=14)
ax2.tick_params(axis='both', which='minor', labelsize=14)
ax2.set_title(r'',  fontsize=20)
legend1=plt.legend(loc='upper right', fontsize=12)
# Add legend
filled_circle = mlines.Line2D([], [], color='purple', marker='o', linestyle='None',
                          markersize=10, label='LBA peak in web')
empty_circle = mlines.Line2D([], [], color='purple', marker='o', fillstyle = 'none', linestyle='None',
                          markersize=10, label='LBA peak in flange')
legend2=plt.legend(handles=[filled_circle, empty_circle],
            loc='lower right', fontsize=11)   
ax2.add_artist(legend1)
ax2.add_artist(legend2)

# Plot points from parametric runs; R against Ldf_eps when YoT= 0.94 (regardless of LBA peak happening in web or flange)
R_YoT_0p94 = data_dict[this_batch]['R'][data_dict[this_batch]['YoT']==0.94]
Ldf_eps_YoT_0p94 = data_dict[this_batch]['Ldf_eps'][data_dict[this_batch]['YoT']==0.94]
idx2 = np.nonzero(R_YoT_0p94) # Do not include zeros in fitting of surface
# ax2.plot(Ldf_eps_YoT_0p94[idx2],R_YoT_0p94[idx2],marker='o',fillstyle='none',linestyle='none',color='purple')

# Plot points from parametric runs; R against Ldf_eps when YoT= 0.94 (with different markers depending of location of LBA peak)
for i, imp in enumerate(data_dict[this_batch]['imp'][data_dict[this_batch]['YoT']==0.94]):
    if imp==0 and data_dict[this_batch]['R'][data_dict[this_batch]['YoT']==0.94][i]!=0:
        ax2.plot(data_dict[this_batch]['Ldf_eps'][data_dict[this_batch]['YoT']==0.94][i],
                 data_dict[this_batch]['R'][data_dict[this_batch]['YoT']==0.94][i],
                 marker='o', fillstyle='none', color='purple')
    elif imp==1 and data_dict[this_batch]['R'][data_dict[this_batch]['YoT']==0.94][i]!=0:
        ax2.plot(data_dict[this_batch]['Ldf_eps'][data_dict[this_batch]['YoT']==0.94][i],
                 data_dict[this_batch]['R'][data_dict[this_batch]['YoT']==0.94][i],
                 marker='o', color='purple')
        
ax2.set_xlim([min(Ldf_eps_YoT_0p94[idx2]-0.2),max(Ldf_eps_YoT_0p94[idx2])+0.2])
        
# Find critical Ldf for given YoT
def R_min_3_varyLdf(y):
    return griddata((xdat, ydat), zdat, (xi_strip, y), method='linear')-3
Ldf_crit = optimize.fsolve(R_min_3_varyLdf,10)
print(f"For S690, Ldw = 40eps, YoT = 0.94, {Ldf_crit=}")

# annotate with arrow
ax2.annotate(r'$\lambda_f/\varepsilon=$' + "{Ldf:.1f}".format(Ldf=Ldf_crit[0]), 
            xy=(Ldf_crit , 3),  xycoords='data',
            xytext=(0.5, 0.15), textcoords='axes fraction',
            arrowprops=dict(facecolor='black', shrink=0.05, width=1),
            horizontalalignment='right', verticalalignment='top', fontsize=14
            )

# 2D cut through 3D plot at Ldf = 10.1eps
yi_strip = np.array(10.1)
zi_strip = griddata((xdat, ydat), zdat, (xi[None,:], yi_strip), method='linear')
xig_strip, yig_strip = np.meshgrid(xi, yi_strip)

# Plot 2D cut for given Ldf
fig3, ax2 = plt.subplots()
ax2.plot(xig_strip.T, zi_strip.T,
         marker='None', linestyle='-',
         label=r'$R$ for $\lambda_f=10.1\varepsilon$'
             '\n'
             r'and $\sigma_y=690$ MPa',
         color='purple')
ax2.plot(np.array([0.78, 0.99]), np.array([3,3]),
         marker='None', linestyle='-',
         label=r'$R=3$',
         color='black')
ax2.set_xlabel(r'$\sigma_y/\sigma_u$',  fontsize=20)
ax2.set_ylabel(r'$R$', fontsize=20)
ax2.grid()
ax2.tick_params(axis='both', which='major', labelsize=14)
ax2.tick_params(axis='both', which='minor', labelsize=14)
ax2.set_title(r'',  fontsize=20)
ax2.legend(loc='upper right', fontsize=12)

# plot points from parametric runs; R against YoT when Ldf_eps = 10.1
R_Ldf_eps_10p1 = data_dict[this_batch]['R'][np.logical_and(data_dict[this_batch]['Ldf_eps']>10.09, data_dict[this_batch]['Ldf_eps']<10.11)]
YoT_Ldf_eps_10p1 = data_dict[this_batch]['YoT'][np.logical_and(data_dict[this_batch]['Ldf_eps']>10.09, data_dict[this_batch]['Ldf_eps']<10.11)]
idx2 = np.nonzero(R_Ldf_eps_10p1) # Do not include zeros in fitting of surface
ax2.plot(YoT_Ldf_eps_10p1[idx2],R_Ldf_eps_10p1[idx2],marker='o',fillstyle='none',linestyle='none',color='purple')

ax2.set_xlim([min(YoT_Ldf_eps_10p1[idx2]-0.01),max(YoT_Ldf_eps_10p1[idx2])+0.01])
ax2.set_xticks(np.arange(0.80,1,0.05))

# Find critical YoT for given Ldf
def R_min_3_varyYoT(x):
    return griddata((xdat, ydat), zdat, (x, yi_strip), method='linear')-3
YoT_crit = optimize.fsolve(R_min_3_varyYoT,0.9)
print(f"For S690, Ldw = 40eps, Ldf = 10.1eps, {YoT_crit=}")
        
# annotate with arrow
ax2.annotate(r'$\sigma_y/\sigma_u=$' + "{YoT_crit:.2f}".format(YoT_crit=YoT_crit[0]), 
            xy=(YoT_crit , 3),  xycoords='data',
            xytext=(0.5, 0.3), textcoords='axes fraction',
            arrowprops=dict(facecolor='black', shrink=0.05, width=1),
            horizontalalignment='right', verticalalignment='top', fontsize=14
            )

# Plane through given R
xi = np.linspace(0.795,0.99,2)
yi = np.linspace(6,12.1,2)
xig, yig = np.meshgrid(xi, yi)
zig = 3*np.ones(xig.shape)

# surf = ax.plot_surface(xig, yig, zig, cmap='gist_gray', alpha=0.3, 
                        # vmin=np.floor(np.nanmin(zi)), vmax=np.ceil(np.nanmax(zi)))
# surf = ax.plot_wireframe(xig, yig, zig, linewidth=0.75, zorder=0,
                          # color='C0',rstride=1, cstride=1)
surf = ax.plot([0.795, 0.795], [6, 12.1], [3,3], linewidth=0.75, zorder=0,
                            color='C0') 
surf = ax.plot([0.99, 0.99], [6, 12.1], [3,3], linewidth=0.75, zorder=1,
                            color='C0') 
surf = ax.plot([0.795, 0.99], [6, 6], [3,3], linewidth=0.75, zorder=0,
                            color='C0') 
surf = ax.plot([0.795, 0.99], [12.1, 12.1], [3,3], linewidth=0.75, zorder=0,
                            color='C0') 

# Plane through given Ldf
zi = np.linspace(0,6,2)
xig, zig = np.meshgrid(xi, zi)
yig = 10.1*np.ones(zig.shape)

# surf = ax.plot_surface(xig, yig, zig, cmap='gist_gray', alpha=0.3, 
#                         vmin=np.floor(np.nanmin(zi)), vmax=np.ceil(np.nanmax(zi)))
# surf = ax.plot_wireframe(xig, yig, zig, linewidth=0.75, zorder=0,
                         # color='C1',rstride=1, cstride=1)
surf = ax.plot([0.795, 0.795], [10.1, 10.1], [0,6], linewidth=0.75, zorder=0,
                            color='C1') # vertical line
surf = ax.plot([0.99, 0.99], [10.1, 10.1], [0,6], linewidth=0.75, zorder=1,
                            color='C1') # vertical line
surf = ax.plot([0.795, 0.99], [10.1, 10.1], [0,0], linewidth=0.75, zorder=0,
                            color='C1') # horizontal line
surf = ax.plot([0.795, 0.99], [10.1, 10.1], [6,6], linewidth=0.75, zorder=1,
                            color='C1') # horizontal line


# Plane through given YoT
zi = np.linspace(0,6.8,2)
yig, zig = np.meshgrid(yi, zi)
xig = 0.94*np.ones(zig.shape)

# surf = ax.plot_surface(xig, yig, zig, cmap='gist_gray', alpha=0.3, 
#                         vmin=np.floor(np.nanmin(zi)), vmax=np.ceil(np.nanmax(zi)))
# surf = ax.plot_wireframe(xig, yig, zig, linewidth=0.75, zorder=1,
                         # color='C2',rstride=1, cstride=1)
surf = ax.plot([0.94, 0.94], [6, 6], [0,6.8], linewidth=0.75, zorder=1,
                            color='C2') # vertical line
surf = ax.plot([0.94, 0.94], [12.1, 12.1], [0,6.8], linewidth=0.75, zorder=0,
                            color='C2') # vertical line
surf = ax.plot([0.94, 0.94], [6, 12.1], [0,0], linewidth=0.75, zorder=0,
                            color='C2') # horizontal line
surf = ax.plot([0.94, 0.94], [6, 12.1], [6.8,6.8], linewidth=0.75, zorder=1,
                            color='C2') # horizontal line

# Arrrow demarking YoT

ax.arrow3D(0.8,12,7,
           0.14,0,0,
           mutation_scale=20,
           arrowstyle='<->',
           linestyle='solid')

ax.arrow3D(0.94,12,7,
           0.06,0,0,
           mutation_scale=20,
           arrowstyle='<->',
           linestyle='solid')

ax.annotate3D(r'$\sigma_y/\sigma_u<0.94$', (0.84,12,7), xytext=(3, 3), textcoords='offset points', fontsize=12)
ax.annotate3D(r'$\sigma_y/\sigma_u>0.94$', (0.93,12,7.5), xytext=(3, 3), textcoords='offset points', fontsize=12)

# Arrrow demarking Ldf

ax.arrow3D(0.97,6,0,
           0,4.1,0,
           mutation_scale=20,
           arrowstyle='<->',
           linestyle='solid')

ax.arrow3D(0.97,10.1,0,
           0,1.9,0,
           mutation_scale=20,
           arrowstyle='<->',
           linestyle='solid')

ax.annotate3D(r'$\lambda_f/\varepsilon<10.1$', (0.91,7,0), xytext=(3, 3), textcoords='offset points', fontsize=12)
ax.annotate3D(r'$\lambda_f/\varepsilon>10.1$', (0.90,10,0), xytext=(3, 3), textcoords='offset points', fontsize=12)

# fig1.savefig('R_against_YoT_and_Ldf_for_S690_Ldw_40eps.eps', format='eps')
# fig2.savefig('R_against_Ldf_for_S690_Ldw_40eps.eps', format='eps', bbox_inches='tight')
# fig3.savefig('R_against_YoT_for_S690_Ldw_40eps.eps', format='eps', bbox_inches='tight')

# %% * Plot rotation capacities and YoT and Ldf  - HSA800 Ldw = 40.0eps S890 - UR I limit
# Assess UR I limit (plane at Ldf = 10.1eps)

plt.close('all')

this_batch = 'vary B vary YoT t_HSA800 Y890 Lw_40p0eps'
# this_batch = 'all runs'

fig1 = plt.figure()
ax = plt.axes(projection='3d', computed_zorder=False)


# Do not include zeros in fitting of surface
idx = np.nonzero(data_dict[this_batch]['R'])
xdat = data_dict[this_batch]['YoT'][idx]
ydat = data_dict[this_batch]['Ldf_eps'][idx]
zdat = data_dict[this_batch]['R'][idx]

xi = np.linspace(xdat.min(),xdat.max())
yi = np.linspace(ydat.min(),ydat.max())
zi = griddata((xdat, ydat), zdat, (xi[None,:], yi[:,None]), method='linear')
xig, yig = np.meshgrid(xi, yi)

# emulate transparency in eps:
my_cmap = ma.cmap('Purples_r', 0.7, bg="white", set_under=None, set_over=None,
         set_bad=None, out_cmap_name=None)
surf = ax.plot_surface(xig, yig, zi, cmap='Purples_r', alpha=1, 
                        vmin=np.floor(np.nanmin(zi)), vmax=np.ceil(np.nanmax(zi)))


# position of colorbar
# where arg is [left, bottom, width, height]
# cax = fig1.add_axes([0.9, 0.35, 0.03, 0.35]) # add argument cax=cax to fig1.colorbar() to use this
clb1 = fig1.colorbar(surf, orientation='vertical', shrink=0.7, pad=0.13)
clb1.ax.set_title('S890', fontsize=18, pad=20)
# plt.show()

for i, imp in enumerate(data_dict[this_batch]['imp']):
    if imp==0 and data_dict[this_batch]['R'][i]!=0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Ldf_eps'][i],data_dict[this_batch]['R'][i], marker='o', fillstyle='none', color='purple')
    elif imp==1 and data_dict[this_batch]['R'][i]!=0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Ldf_eps'][i],data_dict[this_batch]['R'][i], marker='o', color='purple')
    elif imp==0 and data_dict[this_batch]['R'][i]==0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Ldf_eps'][i],data_dict[this_batch]['R'][i], marker='s', fillstyle='none', color='purple')
    elif imp==1 and data_dict[this_batch]['R'][i]==0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Ldf_eps'][i],data_dict[this_batch]['R'][i], marker='s', color='purple')
ax.set_xlabel(r'$\sigma_y/\sigma_u$',  fontsize=20, labelpad=10)
ax.set_ylabel(r'$\lambda_f/\varepsilon$', fontsize=20, labelpad=10)
ax.set_zlabel(r'$R$', fontsize=20)
ax.grid()
ax.tick_params(axis='both', which='major', labelsize=12)
ax.tick_params(axis='both', which='minor', labelsize=12)
ax.set_title(r'',  fontsize=20)      
ax.set_xlim([0.8, 0.99])
ax.set_ylim([6,12])
plt.tight_layout()

# 2D cut through 3D plot at YoT = 0.94
xi_strip = np.array(0.94)
zi_strip = griddata((xdat, ydat), zdat, (xi_strip, yi[:,None]), method='linear')
xig_strip, yig_strip = np.meshgrid(xi_strip, yi)

fig2, ax2 = plt.subplots()
ax2.plot(yig_strip, zi_strip,
         marker='None', linestyle='-',
         label=r'$R$ for $\sigma_y/\sigma_u=0.94$'
         '\n'
         r'and $\sigma_y=890$ MPa',
         color='purple')
ax2.plot(np.array([6,12]), np.array([3,3]),
         marker='None', linestyle='-',
         label=r'$R=3$',
         color='black')
ax2.set_xlabel(r'$\lambda_f/\varepsilon$',  fontsize=20)
ax2.set_ylabel(r'$R$', fontsize=20)
ax2.grid()
ax2.tick_params(axis='both', which='major', labelsize=14)
ax2.tick_params(axis='both', which='minor', labelsize=14)
ax2.set_title(r'',  fontsize=20)
ax2.legend(loc='upper right', fontsize=12)

# Plot points from parametric runs; R against Ldf_eps when YoT= 0.94
R_YoT_0p94 = data_dict[this_batch]['R'][data_dict[this_batch]['YoT']==0.94]
Ldf_eps_YoT_0p94 = data_dict[this_batch]['Ldf_eps'][data_dict[this_batch]['YoT']==0.94]
idx2 = np.nonzero(R_YoT_0p94) # Do not include zeros in fitting of surface
ax2.plot(Ldf_eps_YoT_0p94[idx2],R_YoT_0p94[idx2],marker='o',fillstyle='none',linestyle='none',color='purple')

ax2.set_xlim([min(Ldf_eps_YoT_0p94[idx2]-0.2),max(Ldf_eps_YoT_0p94[idx2])+0.2])

# Find critical Ldf for given YoT
def R_min_3_varyLdf(y):
    return griddata((xdat, ydat), zdat, (xi_strip, y), method='linear')-3
Ldf_crit = optimize.fsolve(R_min_3_varyLdf,10)
print(f"For S890, Ldw = 40eps, YoT = 0.94, {Ldf_crit=}")

# annotate with arrow
ax2.annotate(r'$\lambda_f/\varepsilon=$' + "{Ldf:.1f}".format(Ldf=Ldf_crit[0]), 
            xy=(Ldf_crit , 3),  xycoords='data',
            xytext=(0.5, 0.4), textcoords='axes fraction',
            arrowprops=dict(facecolor='black', shrink=0.05, width=1),
            horizontalalignment='right', verticalalignment='top', fontsize=14
            )

# 2D cut through 3D plot at Ldf = 10.1eps
yi_strip = np.array(10.1)
zi_strip = griddata((xdat, ydat), zdat, (xi[None,:], yi_strip), method='linear')
xig_strip, yig_strip = np.meshgrid(xi, yi_strip)

fig3, ax2 = plt.subplots()
ax2.plot(xig_strip.T, zi_strip.T,
         marker='None', linestyle='-',
         label=r'$R$ for $\lambda_f=10.1\varepsilon$'
         '\n'
         'and $\sigma_y=890$ MPa',
         color='purple')
ax2.plot(np.array([0.78, 1]), np.array([3,3]),
         marker='None', linestyle='-',
         label=r'$R=3$',
         color='black')
ax2.set_xlabel(r'$\sigma_y/\sigma_u$',  fontsize=20)
ax2.set_ylabel(r'$R$', fontsize=20)
ax2.grid()
ax2.tick_params(axis='both', which='major', labelsize=14)
ax2.tick_params(axis='both', which='minor', labelsize=14)
ax2.set_title(r'',  fontsize=20)
ax2.legend(fontsize=12)

R_Ldf_eps_10p1 = data_dict[this_batch]['R'][np.logical_and(data_dict[this_batch]['Ldf_eps']>10.05, data_dict[this_batch]['Ldf_eps']<10.15)]
YoT_Ldf_eps_10p1 = data_dict[this_batch]['YoT'][np.logical_and(data_dict[this_batch]['Ldf_eps']>10.05, data_dict[this_batch]['Ldf_eps']<10.15)]
idx2 = np.nonzero(R_Ldf_eps_10p1) # Do not include zeros in fitting of surface
ax2.plot(YoT_Ldf_eps_10p1[idx2],R_Ldf_eps_10p1[idx2],marker='o',fillstyle='none',linestyle='none',color='purple')

ax2.set_xlim([min(YoT_Ldf_eps_10p1[idx2]-0.01),max(YoT_Ldf_eps_10p1[idx2])+0.01])
ax2.set_xticks(np.arange(0.80,1.01,0.05))

# Find critical YoT for given Ldf
def R_min_3_varyYoT(x):
    return griddata((xdat, ydat), zdat, (x, yi_strip), method='linear')-3
YoT_crit = optimize.fsolve(R_min_3_varyYoT,0.95)
print(f"For S890, Ldw = 40eps, Ldf = 10.1eps, {YoT_crit=}")
        
# annotate with arrow
ax2.annotate(r'$\sigma_y/\sigma_u=$' + "{YoT_crit:.2f}".format(YoT_crit=YoT_crit[0]), 
            xy=(YoT_crit , 3),  xycoords='data',
            xytext=(0.5, 0.4), textcoords='axes fraction',
            arrowprops=dict(facecolor='black', shrink=0.05, width=1),
            horizontalalignment='right', verticalalignment='top', fontsize=14
            )
        
# Plane through given R
xi = np.linspace(0.795,0.99,2)
yi = np.linspace(6,12.1,2)
xig, yig = np.meshgrid(xi, yi)
zig = 3*np.ones(xig.shape)

# surf = ax.plot_surface(xig, yig, zig, cmap='gist_gray', alpha=0.3, 
#                         vmin=np.floor(np.nanmin(zi)), vmax=np.ceil(np.nanmax(zi)))
# surf = ax.plot_wireframe(xig, yig, zig, linewidth=0.75, zorder=0,
                         # color='C0',rstride=1, cstride=1)
surf = ax.plot([0.795, 0.795], [6, 12.1], [3,3], linewidth=0.75, zorder=0,
                            color='C0') 
surf = ax.plot([0.99, 0.99], [6, 12.1], [3,3], linewidth=0.75, zorder=1,
                            color='C0') 
surf = ax.plot([0.795, 0.99], [6, 6], [3,3], linewidth=0.75, zorder=0,
                            color='C0') 
surf = ax.plot([0.795, 0.99], [12.1, 12.1], [3,3], linewidth=0.75, zorder=0,
                            color='C0') 

# Plane through given Ldf
zi = np.linspace(0,7,2)
xig, zig = np.meshgrid(xi, zi)
yig = 10.1*np.ones(zig.shape)

# surf = ax.plot_surface(xig, yig, zig, cmap='gist_gray', alpha=0.3, 
#                         vmin=np.floor(np.nanmin(zi)), vmax=np.ceil(np.nanmax(zi)))
# surf = ax.plot_wireframe(xig, yig, zig, linewidth=0.75, zorder=0,
                         # color='C1',rstride=1, cstride=1)
surf = ax.plot([0.795, 0.795], [10.1, 10.1], [0,7], linewidth=0.75, zorder=0,
                            color='C1') # vertical line
surf = ax.plot([0.99, 0.99], [10.1, 10.1], [0,7], linewidth=0.75, zorder=1,
                            color='C1') # vertical line
surf = ax.plot([0.795, 0.99], [10.1, 10.1], [0,0], linewidth=0.75, zorder=0,
                            color='C1') # horizontal line
surf = ax.plot([0.795, 0.99], [10.1, 10.1], [7,7], linewidth=0.75, zorder=1,
                            color='C1') # horizontal line

# Plane through given YoT
zi = np.linspace(0,6,2)
yig, zig = np.meshgrid(yi, zi)
xig = 0.94*np.ones(zig.shape)

# surf = ax.plot_surface(xig, yig, zig, cmap='gist_gray', alpha=0.3, 
#                         vmin=np.floor(np.nanmin(zi)), vmax=np.ceil(np.nanmax(zi)))
# surf = ax.plot_wireframe(xig, yig, zig, linewidth=0.75, zorder=1,
                         # color='C2',rstride=1, cstride=1)
surf = ax.plot([0.94, 0.94], [6, 6], [0,6], linewidth=0.75, zorder=1,
                            color='C2') # vertical line
surf = ax.plot([0.94, 0.94], [12.1, 12.1], [0,6], linewidth=0.75, zorder=0,
                            color='C2') # vertical line
surf = ax.plot([0.94, 0.94], [6, 12.1], [0,0], linewidth=0.75, zorder=0,
                            color='C2') # horizontal line
surf = ax.plot([0.94, 0.94], [6, 12.1], [6,6], linewidth=0.75, zorder=1,
                            color='C2') # horizontal line

# Arrrow demarking YoT

ax.arrow3D(0.8,12,7,
           0.14,0,0,
           mutation_scale=20,
           arrowstyle='<->',
           linestyle='solid')

ax.arrow3D(0.94,12,7,
           0.06,0,0,
           mutation_scale=20,
           arrowstyle='<->',
           linestyle='solid')

ax.annotate3D(r'$\sigma_y/\sigma_u<0.94$', (0.84,12,7), xytext=(3, 3), textcoords='offset points', fontsize=12)
ax.annotate3D(r'$\sigma_y/\sigma_u>0.94$', (0.93,12,7.5), xytext=(3, 3), textcoords='offset points', fontsize=12)

# Arrrow demarking Ldf

ax.arrow3D(0.97,6,0,
           0,4.1,0,
           mutation_scale=20,
           arrowstyle='<->',
           linestyle='solid')

ax.arrow3D(0.97,10.1,0,
           0,1.9,0,
           mutation_scale=20,
           arrowstyle='<->',
           linestyle='solid')

ax.annotate3D(r'$\lambda_f/\varepsilon<10.1$', (0.91,7,0), xytext=(3, 3), textcoords='offset points', fontsize=12)
ax.annotate3D(r'$\lambda_f/\varepsilon>10.1$', (0.90,10,0), xytext=(3, 3), textcoords='offset points', fontsize=12)

# fig1.savefig('R_against_YoT_and_Ldf_for_S890_Ldw_40eps.eps', format='eps')
# fig2.savefig('R_against_Ldf_for_S890_Ldw_40eps.eps', format='eps', bbox_inches='tight')
# fig3.savefig('R_against_YoT_for_S890_Ldw_40eps.eps', format='eps', bbox_inches='tight')

# %% Plot rotation capacities and YoT and Ldf  - HSA800 Ldw = 40.0eps S690 - EC3 limit
# Assess EC3 limit (plane at Ldf = 9eps and YoT = 0.91 and YoT = 0.95)

plt.close('all')

this_batch = 'vary B vary YoT t_HSA800 Y690 Lw_40p0eps'
# this_batch = 'all runs'

fig = plt.figure()
ax = plt.axes(projection='3d')


# Do not include zeros in fitting of surface
idx = np.nonzero(data_dict[this_batch]['R'])
xdat = data_dict[this_batch]['YoT'][idx]
ydat = data_dict[this_batch]['Ldf_eps'][idx]
zdat = data_dict[this_batch]['R'][idx]

xi = np.linspace(xdat.min(),xdat.max())
yi = np.linspace(ydat.min(),ydat.max())
zi = griddata((xdat, ydat), zdat, (xi[None,:], yi[:,None]), method='linear')
xig, yig = np.meshgrid(xi, yi)
# 
surf = ax.plot_surface(xig, yig, zi, cmap='Purples_r', alpha=0.7, 
                        vmin=np.floor(np.nanmin(zi)), vmax=np.ceil(np.nanmax(zi)))


# position of colorbar
# where arg is [left, bottom, width, height]
cax = fig.add_axes([0.9, 0.35, 0.03, 0.35])
clb1 = fig.colorbar(surf, orientation='vertical', cax=cax)
clb1.ax.set_title('S690')
# plt.show()

for i, imp in enumerate(data_dict[this_batch]['imp']):
    if imp==0 and data_dict[this_batch]['R'][i]!=0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Ldf_eps'][i],data_dict[this_batch]['R'][i], marker='o', fillstyle='none', color='purple')
    elif imp==1 and data_dict[this_batch]['R'][i]!=0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Ldf_eps'][i],data_dict[this_batch]['R'][i], marker='o', color='purple')
    elif imp==0 and data_dict[this_batch]['R'][i]==0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Ldf_eps'][i],data_dict[this_batch]['R'][i], marker='s', fillstyle='none', color='purple')
    elif imp==1 and data_dict[this_batch]['R'][i]==0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Ldf_eps'][i],data_dict[this_batch]['R'][i], marker='s', color='purple')
ax.set_xlabel(r'$\sigma_y/\sigma_u$',  fontsize=20)
ax.set_ylabel(r'$\lambda_f/\varepsilon$', fontsize=20)
ax.set_zlabel(r'$R$', fontsize=20)
ax.grid()
ax.tick_params(axis='both', which='major', labelsize=12)
ax.tick_params(axis='both', which='minor', labelsize=12)
ax.set_title(r'',  fontsize=20)      
ax.set_xlim([0.8, 0.99])
ax.set_ylim([6,11.2])

# Plane through R = 3
xi = np.linspace(0.795,0.99,2)
yi = np.linspace(6,11.3,2)
xig, yig = np.meshgrid(xi, yi)
zig = 3*np.ones(xig.shape)


surf = ax.plot_surface(xig, yig, zig, cmap='gist_gray', alpha=0.3, 
                        vmin=np.floor(np.nanmin(zi)), vmax=np.ceil(np.nanmax(zi)))

# Plane through given Ldf
zi = np.linspace(0,6,2)
xig, zig = np.meshgrid(xi, zi)
yig = 9*np.ones(zig.shape)

surf = ax.plot_surface(xig, yig, zig, cmap='gist_gray', alpha=0.3, 
                        vmin=np.floor(np.nanmin(zi)), vmax=np.ceil(np.nanmax(zi)))

# Plane through given YoT
zi = np.linspace(0,6.7,2)
yig, zig = np.meshgrid(yi, zi)
xig = 0.91*np.ones(zig.shape)

surf = ax.plot_surface(xig, yig, zig, cmap='gist_gray', alpha=0.3, 
                        vmin=np.floor(np.nanmin(zi)), vmax=np.ceil(np.nanmax(zi)))

# Plane through given YoT
zi = np.linspace(0,6.7,2)
yig, zig = np.meshgrid(yi, zi)
xig = 0.95*np.ones(zig.shape)

surf = ax.plot_surface(xig, yig, zig, cmap='gist_gray', alpha=0.3, 
                        vmin=np.floor(np.nanmin(zi)), vmax=np.ceil(np.nanmax(zi)))

# Arrrow demarking YoT

ax.arrow3D(0.8,12,7,
           0.11,0,0,
           mutation_scale=20,
           arrowstyle='<->',
           linestyle='solid')

# ax.arrow3D(0.91,12,7,
#            0.04,0,0,
#            mutation_scale=20,
#            arrowstyle='<->',
#            linestyle='solid')

ax.arrow3D(0.95,12,7,
           0.05,0,0,
           mutation_scale=20,
           arrowstyle='<->',
           linestyle='solid')

ax.annotate3D(r'$\sigma_y/\sigma_u<0.91$', (0.84,12,7), xytext=(3, 3), textcoords='offset points')
ax.annotate3D(r'$\sigma_y/\sigma_u>0.95$', (0.93,12,7.5), xytext=(3, 3), textcoords='offset points')

# Arrrow demarking Ldf

ax.arrow3D(0.97,6,0,
           0,3,0,
           mutation_scale=20,
           arrowstyle='<->',
           linestyle='solid')

ax.arrow3D(0.97,9,0,
           0,3,0,
           mutation_scale=20,
           arrowstyle='<->',
           linestyle='solid')

ax.annotate3D(r'$\lambda_f/\varepsilon<9$', (0.91,7,0), xytext=(3, 3), textcoords='offset points')
ax.annotate3D(r'$\lambda_f/\varepsilon>9$', (0.90,10,0), xytext=(3, 3), textcoords='offset points')

# %% Plot rotation capacities and YoT and Ldf  - HSA800 Ldw = 40.0eps S890 - EC3 limit
# Assess EC3 limit (plane at Ldf = 9eps and YoT = 0.91 and YoT = 0.95)

plt.close('all')

this_batch = 'vary B vary YoT t_HSA800 Y890 Lw_40p0eps'
# this_batch = 'all runs'

fig = plt.figure()
ax = plt.axes(projection='3d')


# Do not include zeros in fitting of surface
idx = np.nonzero(data_dict[this_batch]['R'])
xdat = data_dict[this_batch]['YoT'][idx]
ydat = data_dict[this_batch]['Ldf_eps'][idx]
zdat = data_dict[this_batch]['R'][idx]

xi = np.linspace(xdat.min(),xdat.max())
yi = np.linspace(ydat.min(),ydat.max())
zi = griddata((xdat, ydat), zdat, (xi[None,:], yi[:,None]), method='linear')
xig, yig = np.meshgrid(xi, yi)
# 
surf = ax.plot_surface(xig, yig, zi, cmap='Purples_r', alpha=0.7, 
                        vmin=np.floor(np.nanmin(zi)), vmax=np.ceil(np.nanmax(zi)))

# position of colorbar
# where arg is [left, bottom, width, height]
cax = fig.add_axes([0.9, 0.35, 0.03, 0.35])
clb1 = fig.colorbar(surf, orientation='vertical', cax=cax)
clb1.ax.set_title('S890')
# plt.show()

for i, imp in enumerate(data_dict[this_batch]['imp']):
    if imp==0 and data_dict[this_batch]['R'][i]!=0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Ldf_eps'][i],data_dict[this_batch]['R'][i], marker='o', fillstyle='none', color='purple')
    elif imp==1 and data_dict[this_batch]['R'][i]!=0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Ldf_eps'][i],data_dict[this_batch]['R'][i], marker='o', color='purple')
    elif imp==0 and data_dict[this_batch]['R'][i]==0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Ldf_eps'][i],data_dict[this_batch]['R'][i], marker='s', fillstyle='none', color='purple')
    elif imp==1 and data_dict[this_batch]['R'][i]==0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Ldf_eps'][i],data_dict[this_batch]['R'][i], marker='s', color='purple')
ax.set_xlabel(r'$\sigma_y/\sigma_u$',  fontsize=20)
ax.set_ylabel(r'$\lambda_f/\varepsilon$', fontsize=20)
ax.set_zlabel(r'$R$', fontsize=20)
ax.grid()
ax.tick_params(axis='both', which='major', labelsize=12)
ax.tick_params(axis='both', which='minor', labelsize=12)
ax.set_title(r'',  fontsize=20)      
ax.set_xlim([0.8, 0.99])
# ax.set_ylim([6,10.1])

# 2D cut through 3D plot at YoT = 0.91
xi_strip = np.array(0.91)
zi_strip = griddata((xdat, ydat), zdat, (xi_strip, yi[:,None]), method='linear')
xig_strip, yig_strip = np.meshgrid(xi_strip, yi)

fig, ax2 = plt.subplots()
ax2.plot(yig_strip, zi_strip, marker='None', linestyle='-')
ax2.plot(np.array([6.4,11.2]), np.array([3,3]), marker='None', linestyle='-')
ax2.set_xlabel(r'$\lambda_f$',  fontsize=20)
ax2.set_ylabel(r'$R$', fontsize=20)
ax2.grid()
ax2.tick_params(axis='both', which='major', labelsize=12)
ax2.tick_params(axis='both', which='minor', labelsize=12)
ax2.set_title(r'',  fontsize=20)

# 2D cut through 3D plot at YoT = 0.95
xi_strip = np.array(0.95)
zi_strip = griddata((xdat, ydat), zdat, (xi_strip, yi[:,None]), method='linear')
xig_strip, yig_strip = np.meshgrid(xi_strip, yi)

fig, ax2 = plt.subplots()
ax2.plot(yig_strip, zi_strip, marker='None', linestyle='-')
ax2.plot(np.array([7,11]), np.array([3,3]), marker='None', linestyle='-')
ax2.set_xlabel(r'$\lambda_f$',  fontsize=20)
ax2.set_ylabel(r'$R$', fontsize=20)
ax2.grid()
ax2.tick_params(axis='both', which='major', labelsize=12)
ax2.tick_params(axis='both', which='minor', labelsize=12)
ax2.set_title(r'',  fontsize=20)
        
# 2D cut through 3D plot at Ldf = 9eps
yi_strip = np.array(9)
zi_strip = griddata((xdat, ydat), zdat, (xi[None,:], yi_strip), method='linear')
xig_strip, yig_strip = np.meshgrid(xi, yi_strip)

fig, ax2 = plt.subplots()
ax2.plot(xig_strip.T, zi_strip.T,
         marker='None', linestyle='-',
         label=r'$R$ for $\lambda_f=9\varepsilon$')
ax2.plot(np.array([0.8, 0.99]), np.array([3,3]),
         marker='None', linestyle='-',
         label=r'$R=3$')
ax2.set_xlabel(r'$\sigma_y/\sigma_u$',  fontsize=20)
ax2.set_ylabel(r'$R$', fontsize=20)
ax2.grid()
ax2.tick_params(axis='both', which='major', labelsize=12)
ax2.tick_params(axis='both', which='minor', labelsize=12)
ax2.set_title(r'',  fontsize=20)
ax2.legend()

# Plane through given R
xi = np.linspace(0.795,0.99,2)
yi = np.linspace(6,10.1,2)
xig, yig = np.meshgrid(xi, yi)
zig = 3*np.ones(xig.shape)

surf = ax.plot_surface(xig, yig, zig, cmap='gist_gray', alpha=0.3, 
                        vmin=np.floor(np.nanmin(zi)), vmax=np.ceil(np.nanmax(zi)))

# Plane through given Ldf
zi = np.linspace(0,8.1,2)
xig, zig = np.meshgrid(xi, zi)
yig = 9*np.ones(zig.shape)

surf = ax.plot_surface(xig, yig, zig, cmap='gist_gray', alpha=0.3, 
                        vmin=np.floor(np.nanmin(zi)), vmax=np.ceil(np.nanmax(zi)))

# Plane through given YoT
zi = np.linspace(0,8.8,2)
yig, zig = np.meshgrid(yi, zi)
xig = 0.91*np.ones(zig.shape)

surf = ax.plot_surface(xig, yig, zig, cmap='gist_gray', alpha=0.3, 
                        vmin=np.floor(np.nanmin(zi)), vmax=np.ceil(np.nanmax(zi)))

# Plane through given YoT
zi = np.linspace(0,8.8,2)
yig, zig = np.meshgrid(yi, zi)
xig = 0.95*np.ones(zig.shape)

surf = ax.plot_surface(xig, yig, zig, cmap='gist_gray', alpha=0.3, 
                        vmin=np.floor(np.nanmin(zi)), vmax=np.ceil(np.nanmax(zi)))

# Arrrow demarking YoT

ax.arrow3D(0.8,10.1,8.9,
           0.11,0,0,
           mutation_scale=20,
           arrowstyle='<->',
           linestyle='solid')

# ax.arrow3D(0.91,12,7,
#            0.04,0,0,
#            mutation_scale=20,
#            arrowstyle='<->',
#            linestyle='solid')

ax.arrow3D(0.95,10.1,8.9,
           0.05,0,0,
           mutation_scale=20,
           arrowstyle='<->',
           linestyle='solid')

ax.annotate3D(r'$\sigma_y/\sigma_u<0.91$', (0.78,12,7), xytext=(3, 3), textcoords='offset points')
ax.annotate3D(r'$\sigma_y/\sigma_u>0.95$', (0.88,12,7.5), xytext=(3, 3), textcoords='offset points')

# Arrrow demarking Ldf

ax.arrow3D(0.97,6,0,
           0,3,0,
           mutation_scale=20,
           arrowstyle='<->',
           linestyle='solid')

ax.arrow3D(0.97,9,0,
           0,1.2,0,
           mutation_scale=20,
           arrowstyle='<->',
           linestyle='solid')

ax.annotate3D(r'$\lambda_f/\varepsilon<9$', (0.91,7,0), xytext=(3, 3), textcoords='offset points')
ax.annotate3D(r'$\lambda_f/\varepsilon>9$', (0.91,9.5,0), xytext=(3, 3), textcoords='offset points')

# %% * Create marker legend separately
# marker legend

plt.close('all')

# Black

filled_circle = mlines.Line2D([], [], color='black', marker='o', linestyle='None',
                          markersize=10, label='LBA peak in web; GMNIA achieves $M_{pl}$')
empty_circle = mlines.Line2D([], [], color='black', marker='o', fillstyle = 'none', linestyle='None',
                          markersize=10, label='LBA peak in flange; GMNIA achieves $M_{pl}$')
filled_square = mlines.Line2D([], [], color='black', marker='s', linestyle='None',
                          markersize=10, label='LBA peak in web; GMNIA does not achieve $M_{pl}$')
empty_square = mlines.Line2D([], [], color='black', marker='s', fillstyle = 'none', linestyle='None',
                          markersize=10, label='LBA peak in flange; GMNIA does not achieve $M_{pl}$')
fig1 = plt.figure()
plt.legend(handles=[filled_circle, empty_circle, filled_square, empty_square],
            loc='center', fontsize=11)   
plt.axis('off')

# fig1.savefig('Marker_legend_black.eps', format='eps', bbox_inches='tight')

# Purple

filled_circle = mlines.Line2D([], [], color='purple', marker='o', linestyle='None',
                          markersize=10, label='LBA peak in web; GMNIA achieves $M_{pl}$')
empty_circle = mlines.Line2D([], [], color='purple', marker='o', fillstyle = 'none', linestyle='None',
                          markersize=10, label='LBA peak in flange; GMNIA achieves $M_{pl}$')
filled_square = mlines.Line2D([], [], color='purple', marker='s', linestyle='None',
                          markersize=10, label='LBA peak in web; GMNIA does not achieve $M_{pl}$')
empty_square = mlines.Line2D([], [], color='purple', marker='s', fillstyle = 'none', linestyle='None',
                          markersize=10, label='LBA peak in flange; GMNIA does not achieve $M_{pl}$')
fig2 = plt.figure()
plotsize=fig2.get_size_inches()
fig2.set_size_inches(plotsize[0], 1.6, forward=True)
plt.legend(handles=[filled_circle, empty_circle, filled_square, empty_square],
            loc='center', fontsize=11)   
plt.axis('off')

# fig2.savefig('Marker_legend_purple.eps', format='eps', bbox_inches='tight')


# %% * Plot rotation capacities and YoT and Ldw  - SM490_12mm Ldf = 10.1eps S690 - UR I limit
# Assess UR I limit (plane at Ldw = 52.5eps and YoT = 0.94)

plt.close('all')

this_batch = 'vary H vary YoT t_SM490_12mm Y690 Lf_10p1eps'
# this_batch = 'all runs'

fig1 = plt.figure()
ax = plt.axes(projection='3d', computed_zorder=False)

# Do not include zeros in fitting of surface
idx = np.nonzero(data_dict[this_batch]['R'])
xdat = data_dict[this_batch]['YoT'][idx]
ydat = data_dict[this_batch]['Ldw_eps'][idx]
zdat = data_dict[this_batch]['R'][idx]

xi = np.linspace(xdat.min(),xdat.max())
yi = np.linspace(ydat.min(),ydat.max())
zi = griddata((xdat, ydat), zdat, (xi[None,:], yi[:,None]), method='linear')
xig, yig = np.meshgrid(xi, yi)
# emulate transparency in eps:
my_cmap = ma.cmap('Purples_r', 0.7, bg="white", set_under=None, set_over=None,
          set_bad=None, out_cmap_name=None)
surf = ax.plot_surface(xig, yig, zi, cmap=my_cmap, alpha=1, 
                        vmin=np.floor(np.nanmin(zi)), vmax=np.ceil(np.nanmax(zi)))

# position of colorbar
# where arg is [left, bottom, width, height]
# cax = fig1.add_axes([0.9, 0.35, 0.03, 0.35]) # add argument cax=cax to fig1.colorbar() to use this
clb1 = fig1.colorbar(surf, orientation='vertical', shrink=0.7, pad=0.13, location='left')
clb1.ax.set_title('S690', fontsize=18, pad=20)
# plt.show()


for i, imp in enumerate(data_dict[this_batch]['imp']):
    if imp==0 and data_dict[this_batch]['R'][i]!=0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Ldw_eps'][i],data_dict[this_batch]['R'][i], marker='o', fillstyle='none', color='purple')
    elif imp==1 and data_dict[this_batch]['R'][i]!=0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Ldw_eps'][i],data_dict[this_batch]['R'][i], marker='o', color='purple')
    elif imp==0 and data_dict[this_batch]['R'][i]==0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Ldw_eps'][i],data_dict[this_batch]['R'][i], marker='s', fillstyle='none', color='purple')
    elif imp==1 and data_dict[this_batch]['R'][i]==0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Ldw_eps'][i],data_dict[this_batch]['R'][i], marker='s', color='purple')
ax.set_xlabel(r'$\sigma_y/\sigma_u$',  fontsize=20, labelpad=10)
ax.set_ylabel(r'$\lambda_w/\varepsilon$', fontsize=20, labelpad=10)
ax.set_zlabel(r'$R$', fontsize=20)
ax.grid()
ax.tick_params(axis='both', which='major', labelsize=12)
ax.tick_params(axis='both', which='minor', labelsize=12)
ax.set_title(r'',  fontsize=20)
ax.view_init(elev=30, azim=45) # rotate 3D plot 
fig1.tight_layout()
# ax.set_xlim([0.8, 0.99])
# ax.set_ylim([6,10.1])

# 2D cut through 3D plot at YoT = 0.94
xi_strip = np.array(0.94)
zi_strip = griddata((xdat, ydat), zdat, (xi_strip, yi[:,None]), method='linear')
xig_strip, yig_strip = np.meshgrid(xi_strip, yi)

fig2, ax2 = plt.subplots()
ax2.plot(yig_strip, zi_strip,
         marker='None', linestyle='-',
         label=r'$R$ for $\sigma_y/\sigma_u=0.94$'
         '\n'
         r'and $\sigma_y=690$ MPa',
         color='purple')
ax2.plot(np.array([39,56]), np.array([3,3]),
         marker='None', linestyle='-',
         label=r'$R=3$',
         color='black')
ax2.set_xlabel(r'$\lambda_w/\varepsilon$',  fontsize=20)
ax2.set_ylabel(r'$R$', fontsize=20)
ax2.grid()
ax2.tick_params(axis='both', which='major', labelsize=14)
ax2.tick_params(axis='both', which='minor', labelsize=14)
ax2.set_title(r'',  fontsize=20)
ax2.legend(loc='lower left', fontsize=12)

# Plot points from parametric runs; R against Ldf_eps when YoT= 0.94
R_YoT_0p94 = data_dict[this_batch]['R'][data_dict[this_batch]['YoT']==0.94]
Ldw_eps_YoT_0p94 = data_dict[this_batch]['Ldw_eps'][data_dict[this_batch]['YoT']==0.94]
idx2 = np.nonzero(R_YoT_0p94) # Do not include zeros in fitting of surface
ax2.plot(Ldw_eps_YoT_0p94[idx2],R_YoT_0p94[idx2],marker='o',fillstyle='none',linestyle='none',color='purple')

ax2.set_xlim([min(Ldw_eps_YoT_0p94[idx2]-0.4),max(Ldw_eps_YoT_0p94[idx2])+0.4])

# Find critical Ldw for given YoT
def R_min_3_varyLdw(y):
    return griddata((xdat, ydat), zdat, (xi_strip, y), method='linear')-3
Ldw_crit = optimize.fsolve(R_min_3_varyLdw,50)
print(f"For S690, Ldf = 10.1eps, YoT = 0.94, {Ldw_crit=}")

# annotate with arrow
ax2.annotate(r'$\lambda_w/\varepsilon=$' + "{Ldw:.1f}".format(Ldw=Ldw_crit[0]), 
            xy=(Ldw_crit, 3),  xycoords='data',
            xytext=(0.8, 0.65), textcoords='axes fraction',
            arrowprops=dict(facecolor='black', shrink=0.05, width=1),
            horizontalalignment='right', verticalalignment='top', fontsize=14
            )
        
# 2D cut through 3D plot at Ldw = 52.5eps
yi_strip = np.array(52.5)
zi_strip = griddata((xdat, ydat), zdat, (xi[None,:], yi_strip), method='linear')
xig_strip, yig_strip = np.meshgrid(xi, yi_strip)

fig3, ax2 = plt.subplots()
ax2.plot(xig_strip.T, zi_strip.T,
         marker='None', linestyle='-',
         label=r'$R$ for $\lambda_w=52.5\varepsilon$'
         '\n' 
         r'and $\sigma_y=690$ MPa',
         color='purple')
ax2.plot(np.array([0.78, 0.99]),np.array([3,3]),
         marker='None', linestyle='-',
         label=r'$R=3$',
         color='black')
ax2.set_xlabel(r'$\sigma_y/\sigma_u$',  fontsize=20)
ax2.set_ylabel(r'$R$', fontsize=20)
ax2.grid()
ax2.tick_params(axis='both', which='major', labelsize=14)
ax2.tick_params(axis='both', which='minor', labelsize=14)
ax2.set_title(r'',  fontsize=20)
ax2.legend(loc='lower left', fontsize=12)
# plot points from parametric runs; R against YoT when Ldf_eps = 52.5
R_Ldf_eps_52p5 = data_dict[this_batch]['R'][np.logical_and(data_dict[this_batch]['Ldw_eps']>52.45, data_dict[this_batch]['Ldw_eps']<52.55)]
YoT_Ldf_eps_52p5 = data_dict[this_batch]['YoT'][np.logical_and(data_dict[this_batch]['Ldw_eps']>52.45, data_dict[this_batch]['Ldw_eps']<52.55)]
idx2 = np.nonzero(R_Ldf_eps_52p5) # Do not include zeros in fitting of surface
ax2.plot(YoT_Ldf_eps_52p5[idx2],R_Ldf_eps_52p5[idx2],marker='o',fillstyle='none',linestyle='none',color='purple')

ax2.set_xlim([min(YoT_Ldf_eps_52p5[idx2]-0.005),max(YoT_Ldf_eps_52p5[idx2])+0.005])
ax2.set_xticks(np.arange(0.80,0.96,0.05))

# Find critical YoT for given Ldw
def R_min_3_varyYoT(x):
    return griddata((xdat, ydat), zdat, (x, yi_strip), method='linear')-3
YoT_crit = optimize.fsolve(R_min_3_varyYoT,0.8)
print(f"For S890, Ldf = 10.1eps, Ldw = 52.5eps, {YoT_crit=}")
        
# annotate with arrow
ax2.annotate(r'$\sigma_y/\sigma_u=$' + "{YoT_crit:.2f}".format(YoT_crit=YoT_crit[0]), 
            xy=(YoT_crit , 3),  xycoords='data',
            xytext=(0.6, 0.8), textcoords='axes fraction',
            arrowprops=dict(facecolor='black', shrink=0.05, width=1),
            horizontalalignment='right', verticalalignment='top', fontsize=14
            )
        
# Plane through given R
xi = np.linspace(0.775,0.99,2)
yi = np.linspace(37.5,72.5,2)
xig, yig = np.meshgrid(xi, yi)
zig = 3*np.ones(xig.shape)

# surf = ax.plot_surface(xig, yig, zig, cmap='gist_gray', alpha=0.3, 
                        # vmin=np.floor(np.nanmin(zi)), vmax=np.ceil(np.nanmax(zi)))
# surf = ax.plot_wireframe(xig, yig, zig, linewidth=0.75, zorder=0,
                          # color='C0',rstride=1, cstride=1)
surf = ax.plot([0.775, 0.775], [37.5, 72.5], [3,3], linewidth=0.75, zorder=0,
                            color='C0') 
surf = ax.plot([0.99, 0.99], [37.5, 72.5], [3,3], linewidth=0.75, zorder=1,
                            color='C0') 
surf = ax.plot([0.775, 0.99], [37.5, 37.5], [3,3], linewidth=0.75, zorder=0,
                            color='C0') 
surf = ax.plot([0.775, 0.99], [72.5, 72.5], [3,3], linewidth=0.75, zorder=1,
                            color='C0') 

# Plane through given Ldw
zi = np.linspace(0,7,2)
xi = np.linspace(0.77,0.99,2)
xig, zig = np.meshgrid(xi, zi)
yig = 52.5*np.ones(zig.shape)

# surf = ax.plot_surface(xig, yig, zig, cmap='gist_gray', alpha=0.3, 
                        # vmin=np.floor(np.nanmin(zi)), vmax=np.ceil(np.nanmax(zi)))
# surf = ax.plot_wireframe(xig, yig, zig, linewidth=0.75, zorder=1,
                          # color='C1',rstride=1, cstride=1)
surf = ax.plot([0.77, 0.77], [52.5, 52.5], [0,7], linewidth=0.75, zorder=0,
                            color='C1') # vertical line
surf = ax.plot([0.99, 0.99], [52.5, 52.5], [0,7], linewidth=0.75, zorder=1,
                            color='C1') # vertical line
surf = ax.plot([0.77, 0.99], [52.5, 52.5], [0,0], linewidth=0.75, zorder=0,
                            color='C1') # horizontal line
surf = ax.plot([0.77, 0.99], [52.5, 52.5], [7,7], linewidth=0.75, zorder=1,
                            color='C1') # horizontal line

# Plane through given YoT
zi = np.linspace(0,7.5,2)
yig, zig = np.meshgrid(yi, zi)
xig = 0.94*np.ones(zig.shape)

# surf = ax.plot_surface(xig, yig, zig, cmap='gist_gray', alpha=0.3, 
                        # vmin=np.floor(np.nanmin(zi)), vmax=np.ceil(np.nanmax(zi)))
# surf = ax.plot_wireframe(xig, yig, zig, linewidth=0.75, zorder=1,
                            # color='C2',rstride=1, cstride=1)
surf = ax.plot([0.94, 0.94], [37.5, 37.5], [0,7.5], linewidth=0.75, zorder=0,
                            color='C2') # vertical line
surf = ax.plot([0.94, 0.94], [72.5, 72.5], [0,7.5], linewidth=0.75, zorder=1,
                            color='C2') # vertical line
surf = ax.plot([0.94, 0.94], [37.5, 72.5], [0,0], linewidth=0.75, zorder=0,
                            color='C2') # horizontal line
surf = ax.plot([0.94, 0.94], [37.5, 72.5], [7.5,7.5], linewidth=0.75, zorder=1,
                            color='C2') # horizontal line

# # Arrrow demarking YoT

ax.arrow3D(0.78,37.5,7.5,
            0.16,0,0,
            mutation_scale=20,
            arrowstyle='<->',
            linestyle='solid')

ax.arrow3D(0.94,37.5,7.5,
            0.06,0,0,
            mutation_scale=20,
            arrowstyle='<->',
            linestyle='solid')

ax.annotate3D(r'$\sigma_y/\sigma_u<0.94$', (0.92,37.5,9.5), xytext=(3, 3), textcoords='offset points')
ax.annotate3D(r'$\sigma_y/\sigma_u>0.94$', (1.02,37.5,9.5), xytext=(3, 3), textcoords='offset points')


# Arrrow demarking Ldw

ax.arrow3D(0.75,35,7,
            0,16.5,0,
            mutation_scale=20,
            arrowstyle='<->',
            linestyle='solid')

ax.arrow3D(0.75,51.5,7,
            0,20,0,
            mutation_scale=20,
            arrowstyle='<->',
            linestyle='solid')

ax.annotate3D(r'$\lambda_w/\varepsilon<52.5$', (0.75,41,7.3), xytext=(3, 3), textcoords='offset points')
ax.annotate3D(r'$\lambda_w/\varepsilon>52.5$', (0.75,55,7.1), xytext=(3, 3), textcoords='offset points')

# Arrows for plane through given R
ax.arrow3D(0.78,74,0,
           0,0,3.1,
           mutation_scale=20,
           arrowstyle='<->',
           linestyle='solid')
ax.arrow3D(0.78,74,3.2,
           0,0,4.7,
           mutation_scale=20,
           arrowstyle='<->',
           linestyle='solid')
ax.annotate3D(r'$R<3$', (0.8,74,2), xytext=(3, 3), textcoords='offset points')
ax.annotate3D(r'$R>3$', (0.8,74,5.5), xytext=(3, 3), textcoords='offset points')

# fig1.savefig('R_against_YoT_and_Ldw_for_S690_Ldf_10p1eps.eps', format='eps')
# fig2.savefig('R_against_Ldw_for_S690_Ldf_10p1eps.eps', format='eps', bbox_inches='tight')
# fig3.savefig('R_against_YoT_for_S690_Ldf_10p1eps.eps', format='eps', bbox_inches='tight')

# %% * Plot rotation capacities and YoT and Ldw  - SM490_12mm Ldf = 10.1eps S890 - UR I limit
# Assess UR I limit (plane at Ldw = 52.5eps and YoT = 0.94)

plt.close('all')

this_batch = 'vary H vary YoT t_SM490_12mm Y890 Lf_10p1eps'
# this_batch = 'all runs'

fig1 = plt.figure()
ax = plt.axes(projection='3d', computed_zorder=False)


# Do not include zeros in fitting of surface
idx = np.nonzero(data_dict[this_batch]['R'])
xdat = data_dict[this_batch]['YoT'][idx]
ydat = data_dict[this_batch]['Ldw_eps'][idx]
zdat = data_dict[this_batch]['R'][idx]

xi = np.linspace(xdat.min(),xdat.max())
yi = np.linspace(ydat.min(),ydat.max())
zi = griddata((xdat, ydat), zdat, (xi[None,:], yi[:,None]), method='linear')
xig, yig = np.meshgrid(xi, yi)
# emulate transparency in eps:
my_cmap = ma.cmap('Purples_r', 0.7, bg="white", set_under=None, set_over=None,
          set_bad=None, out_cmap_name=None)
surf = ax.plot_surface(xig, yig, zi, cmap=my_cmap, alpha=1, 
                        vmin=np.floor(np.nanmin(zi)), vmax=np.ceil(np.nanmax(zi)))

# position of colorbar
# where arg is [left, bottom, width, height]
# cax = fig1.add_axes([0.9, 0.35, 0.03, 0.35]) # add argument cax=cax to fig1.colorbar() to use this
clb1 = fig1.colorbar(surf, orientation='vertical', shrink=0.7, pad=0.13, location='left')
clb1.ax.set_title('S890', fontsize=18, pad=20)
# plt.show()

for i, imp in enumerate(data_dict[this_batch]['imp']):
    if imp==0 and data_dict[this_batch]['R'][i]!=0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Ldw_eps'][i],data_dict[this_batch]['R'][i], marker='o', fillstyle='none', color='purple')
    elif imp==1 and data_dict[this_batch]['R'][i]!=0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Ldw_eps'][i],data_dict[this_batch]['R'][i], marker='o', color='purple')
    elif imp==0 and data_dict[this_batch]['R'][i]==0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Ldw_eps'][i],data_dict[this_batch]['R'][i], marker='s', fillstyle='none', color='purple')
    elif imp==1 and data_dict[this_batch]['R'][i]==0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Ldw_eps'][i],data_dict[this_batch]['R'][i], marker='s', color='purple')
ax.set_xlabel(r'$\sigma_y/\sigma_u$',  fontsize=20, labelpad=10)
ax.set_ylabel(r'$\lambda_w/\varepsilon$', fontsize=20 ,labelpad=10)
ax.set_zlabel(r'$R$', fontsize=20)
ax.grid()
ax.tick_params(axis='both', which='major', labelsize=12)
ax.tick_params(axis='both', which='minor', labelsize=12)
ax.set_title(r'',  fontsize=20)  
plt.tight_layout()
ax.view_init(elev=30, azim=45) # rotate 3D plot     
# ax.set_xlim([0.8, 0.99])
# ax.set_ylim([6,10.1])

# 2D cut through 3D plot at YoT = 0.94
xi_strip = np.array(0.94)
zi_strip = griddata((xdat, ydat), zdat, (xi_strip, yi[:,None]), method='linear')
xig_strip, yig_strip = np.meshgrid(xi_strip, yi)

fig2, ax2 = plt.subplots()
ax2.plot(yig_strip, zi_strip,
         marker='None', linestyle='-',
         label=r'$R$ for $\sigma_y/\sigma_u=0.94$'
         '\n' 
         r'and $\sigma_y=890$ MPa',
         color='purple')
ax2.plot(np.array([39,57]), np.array([3,3]),
         marker='None', linestyle='-',
         label=r'$R=3$',
         color='black')
ax2.set_xlabel(r'$\lambda_w/\varepsilon$',  fontsize=20)
ax2.set_ylabel(r'$R$', fontsize=20)
ax2.grid()
ax2.tick_params(axis='both', which='major', labelsize=14)
ax2.tick_params(axis='both', which='minor', labelsize=14)
ax2.set_title(r'',  fontsize=20)
ax2.legend(loc='upper right', fontsize=12)
# Plot points from parametric runs; R against Ldf_eps when YoT= 0.94
R_YoT_0p94 = data_dict[this_batch]['R'][data_dict[this_batch]['YoT']==0.94]
Ldw_eps_YoT_0p94 = data_dict[this_batch]['Ldw_eps'][data_dict[this_batch]['YoT']==0.94]
idx2 = np.nonzero(R_YoT_0p94) # Do not include zeros in fitting of surface
ax2.plot(Ldw_eps_YoT_0p94[idx2],R_YoT_0p94[idx2],marker='o',fillstyle='none',linestyle='none',color='purple')

ax2.set_xlim([min(Ldw_eps_YoT_0p94[idx2]-0.4),max(Ldw_eps_YoT_0p94[idx2])+0.4])

# Find critical Ldw for given YoT
def R_min_3_varyLdw(y):
    return griddata((xdat, ydat), zdat, (xi_strip, y), method='linear')-3
Ldw_crit = optimize.fsolve(R_min_3_varyLdw,50)
print(f"For S890, Ldf = 10.1eps, YoT = 0.94, {Ldw_crit=}")

# annotate with arrow
ax2.annotate(r'$\lambda_w/\varepsilon=$' + "{Ldw:.1f}".format(Ldw=Ldw_crit[0]), 
            xy=(Ldw_crit, 3),  xycoords='data',
            xytext=(0.8, 0.5), textcoords='axes fraction',
            arrowprops=dict(facecolor='black', shrink=0.05, width=1),
            horizontalalignment='right', verticalalignment='top', fontsize=14
            )
        
# 2D cut through 3D plot at Ldw = 52.5eps
yi_strip = np.array(52.5)
zi_strip = griddata((xdat, ydat), zdat, (xi[None,:], yi_strip), method='linear')
xig_strip, yig_strip = np.meshgrid(xi, yi_strip)

fig3, ax2 = plt.subplots()
ax2.plot(xig_strip.T, zi_strip.T,
         marker='None', linestyle='-',
         label=r'$R$ for $\lambda_w=52.5\varepsilon$'
         '\n'
         r'and $\sigma_y=890$ MPa',
         color='purple')
ax2.plot(np.array([0.78, 0.99]), np.array([3,3]),
         marker='None', linestyle='-',
         label=r'$R=3$',
         color='black')
ax2.set_xlabel(r'$\sigma_y/\sigma_u$',  fontsize=20)
ax2.set_ylabel(r'$R$', fontsize=20)
ax2.grid()
ax2.tick_params(axis='both', which='major', labelsize=14)
ax2.tick_params(axis='both', which='minor', labelsize=14)
ax2.set_title(r'',  fontsize=20)
ax2.legend(fontsize=12)
# plot points from parametric runs; R against YoT when Ldf_eps = 52.5
R_Ldf_eps_52p5 = data_dict[this_batch]['R'][np.logical_and(data_dict[this_batch]['Ldw_eps']>52.45, data_dict[this_batch]['Ldw_eps']<52.55)]
YoT_Ldf_eps_52p5 = data_dict[this_batch]['YoT'][np.logical_and(data_dict[this_batch]['Ldw_eps']>52.45, data_dict[this_batch]['Ldw_eps']<52.55)]
idx2 = np.nonzero(R_Ldf_eps_52p5) # Do not include zeros in fitting of surface
ax2.plot(YoT_Ldf_eps_52p5[idx2],R_Ldf_eps_52p5[idx2],marker='o',fillstyle='none',linestyle='none',color='purple')

ax2.set_xlim([min(YoT_Ldf_eps_52p5[idx2]-0.005),max(YoT_Ldf_eps_52p5[idx2])+0.005])
ax2.set_xticks(np.arange(0.80,0.96,0.05))

# Find critical YoT for given Ldw
def R_min_3_varyYoT(x):
    return griddata((xdat, ydat), zdat, (x, yi_strip), method='linear')-3
YoT_crit = optimize.fsolve(R_min_3_varyYoT,0.9)
print(f"For S890, Ldf = 10.1eps, Ldw = 52.5eps, {YoT_crit=}")
        
# annotate with arrow
ax2.annotate(r'$\sigma_y/\sigma_u=$' + "{YoT_crit:.2f}".format(YoT_crit=YoT_crit[0]), 
            xy=(YoT_crit , 3),  xycoords='data',
            xytext=(0.8, 0.65), textcoords='axes fraction',
            arrowprops=dict(facecolor='black', shrink=0.05, width=1),
            horizontalalignment='right', verticalalignment='top', fontsize=14
            )

# Plane through given R
xi = np.linspace(0.765,0.99,2)
yi = np.linspace(37.5,72.5,2)
xig, yig = np.meshgrid(xi, yi)
zig = 3*np.ones(xig.shape)

# surf = ax.plot_surface(xig, yig, zig, cmap='gist_gray', alpha=0.3, 
                        # vmin=np.floor(np.nanmin(zi)), vmax=np.ceil(np.nanmax(zi)))
surf = ax.plot([0.765, 0.765], [37.5, 72.5], [3,3], linewidth=0.75, zorder=0,
                            color='C0') 
surf = ax.plot([0.99, 0.99], [37.5, 72.5], [3,3], linewidth=0.75, zorder=1,
                            color='C0') 
surf = ax.plot([0.765, 0.99], [37.5, 37.5], [3,3], linewidth=0.75, zorder=0,
                            color='C0') 
surf = ax.plot([0.765, 0.99], [72.5, 72.5], [3,3], linewidth=0.75, zorder=1,
                            color='C0') 

# Plane through given Ldw
zi = np.linspace(0,8,2)
xig, zig = np.meshgrid(xi, zi)
yig = 52.5*np.ones(zig.shape)

# surf = ax.plot_surface(xig, yig, zig, cmap='gist_gray', alpha=0.3, 
                        # vmin=np.floor(np.nanmin(zi)), vmax=np.ceil(np.nanmax(zi)))
surf = ax.plot([0.765, 0.765], [52.5, 52.5], [0,8], linewidth=0.75, zorder=0,
                            color='C1') # vertical line
surf = ax.plot([0.99, 0.99], [52.5, 52.5], [0,8], linewidth=0.75, zorder=1,
                            color='C1') # vertical line
surf = ax.plot([0.765, 0.99], [52.5, 52.5], [0,0], linewidth=0.75, zorder=0,
                            color='C1') # horizontal line
surf = ax.plot([0.765, 0.99], [52.5, 52.5], [8,8], linewidth=0.75, zorder=1,
                            color='C1') # horizontal line

# Plane through given YoT
zi = np.linspace(0,8,2)
yig, zig = np.meshgrid(yi, zi)
xig = 0.94*np.ones(zig.shape)

# surf = ax.plot_surface(xig, yig, zig, cmap='gist_gray', alpha=0.3, 
                        # vmin=np.floor(np.nanmin(zi)), vmax=np.ceil(np.nanmax(zi)))
surf = ax.plot([0.94, 0.94], [37.5, 37.5], [0,8], linewidth=0.75, zorder=0,
                            color='C2') # vertical line
surf = ax.plot([0.94, 0.94], [72.5, 72.5], [0,8], linewidth=0.75, zorder=1,
                            color='C2') # vertical line
surf = ax.plot([0.94, 0.94], [37.5, 72.5], [0,0], linewidth=0.75, zorder=0,
                            color='C2') # horizontal line
surf = ax.plot([0.94, 0.94], [37.5, 72.5], [8,8], linewidth=0.75, zorder=1,
                            color='C2') # horizontal line


# # Arrrow demarking YoT

ax.arrow3D(0.77,37.5,8.5,
            0.17,0,0,
            mutation_scale=20,
            arrowstyle='<->',
            linestyle='solid')

ax.arrow3D(0.94,37.5,8.5,
            0.08,0,0,
            mutation_scale=20,
            arrowstyle='<->',
            linestyle='solid')

ax.annotate3D(r'$\sigma_y/\sigma_u<0.94$', (0.94,37,10), xytext=(3, 3), textcoords='offset points')
ax.annotate3D(r'$\sigma_y/\sigma_u>0.94$', (1.04,37,10), xytext=(3, 3), textcoords='offset points')


# Arrrow demarking Ldw

ax.arrow3D(0.75,37.5,8,
            0,15,0,
            mutation_scale=20,
            arrowstyle='<->',
            linestyle='solid')

ax.arrow3D(0.75,52.5,8,
            0,18,0,
            mutation_scale=20,
            arrowstyle='<->',
            linestyle='solid')

ax.annotate3D(r'$\lambda_w/\varepsilon<52.5$', (0.75,43,8.6), xytext=(3, 3), textcoords='offset points')
ax.annotate3D(r'$\lambda_w/\varepsilon>52.5$', (0.75,55,8.3), xytext=(3, 3), textcoords='offset points')

# Arrows for plane through given R
ax.arrow3D(0.78,74,0,
           0,0,3.3,
           mutation_scale=20,
           arrowstyle='<->',
           linestyle='solid')
ax.arrow3D(0.78,74,3.4,
           0,0,5.5,
           mutation_scale=20,
           arrowstyle='<->',
           linestyle='solid')
ax.annotate3D(r'$R<3$', (0.8,74,1.7), xytext=(3, 3), textcoords='offset points')
ax.annotate3D(r'$R>3$', (0.8,74,5.8), xytext=(3, 3), textcoords='offset points')

# fig1.savefig('R_against_YoT_and_Ldw_for_S890_Ldf_10p1eps.eps', format='eps')
# fig2.savefig('R_against_Ldw_for_S890_Ldf_10p1eps.eps', format='eps', bbox_inches='tight')
# fig3.savefig('R_against_YoT_for_S890_Ldf_10p1eps.eps', format='eps', bbox_inches='tight')

# %% Find required Y/T against Y for HSA 800; Ldw = 40p0eps; Ldf = 10p1eps

plt.close('all')

this_batch = 'vary YoT vary Y t_HSA800 Lw_40p0eps Lf_10p1eps'

fig = plt.figure()
ax = plt.axes(projection='3d')


# Do not include zeros in fitting of surface
idx = np.nonzero(data_dict[this_batch]['R'])
xdat = data_dict[this_batch]['YoT'][idx]
ydat = data_dict[this_batch]['Y'][idx]
zdat = data_dict[this_batch]['R'][idx]

xi = np.linspace(xdat.min(),xdat.max(),80)
yi = np.linspace(ydat.min(),ydat.max(),50)
zi = griddata((xdat, ydat), zdat, (xi[None,:], yi[:,None]), method='linear')
xig, yig = np.meshgrid(xi, yi)
 
surf = ax.plot_surface(xig, yig, zi, cmap='Purples_r', alpha=0.7, 
                        vmin=np.floor(np.nanmin(zi)), vmax=np.ceil(np.nanmax(zi)))

# position of colorbar
# where arg is [left, bottom, width, height]
cax = fig.add_axes([0.9, 0.35, 0.03, 0.35])
clb1 = fig.colorbar(surf, orientation='vertical', cax=cax)
clb1.ax.set_title('')
# plt.show()

for i, imp in enumerate(data_dict[this_batch]['imp']):
    if imp==0 and data_dict[this_batch]['R'][i]!=0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Y'][i],data_dict[this_batch]['R'][i], marker='o', fillstyle='none', color='purple')
    elif imp==1 and data_dict[this_batch]['R'][i]!=0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Y'][i],data_dict[this_batch]['R'][i], marker='o', color='purple')
    elif imp==0 and data_dict[this_batch]['R'][i]==0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Y'][i],data_dict[this_batch]['R'][i], marker='s', fillstyle='none', color='purple')
    elif imp==1 and data_dict[this_batch]['R'][i]==0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Y'][i],data_dict[this_batch]['R'][i], marker='s', color='purple')
ax.set_xlabel(r'$\sigma_y/\sigma_u$',  fontsize=20)
ax.set_ylabel(r'$\sigma_y$', fontsize=20)
ax.set_zlabel(r'$R$', fontsize=20)
ax.grid()
ax.tick_params(axis='both', which='major', labelsize=12)
ax.tick_params(axis='both', which='minor', labelsize=12)
ax.set_title(r'',  fontsize=20)      
# ax.set_xlim([0.8, 0.99])
# ax.set_ylim([6,10.1])

# 2D cut through 3D plot at YoT = 0.94
zi_strip = 3
yi_strip = griddata((zdat, xdat), ydat, (zi_strip, xi[:,None]), method='linear')
zig_strip, xig_strip = np.meshgrid(zi_strip, xi)

fig, ax2 = plt.subplots()
ax2.plot(yi_strip, xi,
         marker='o', linestyle='-',
         label=r'required $\sigma_y/\sigma_u$ against $\sigma_y$',
         color='purple')
ax2.set_xlabel(r'$\sigma_y$',  fontsize=13)
ax2.set_ylabel(r'$\sigma_y/\sigma_u$', fontsize=13)
ax2.grid()
ax2.tick_params(axis='both', which='major', labelsize=12)
ax2.tick_params(axis='both', which='minor', labelsize=12)
ax2.set_title(r'',  fontsize=20)

# Plot the 2D cut on on the 3D plot
ax.plot(xi,yi_strip,3,label=r'$t_f$ = 20 mm; $t_w$ = 17.6 mm and $\lambda_{LT}$ = 0.1')
ax.legend()

sig_y = np.linspace(400,1100,100)
yield_ratio_fuku = sig_y/(0.83*sig_y+203.8)
yield_ratio_upper = 1/(1+2*(150/sig_y)**2.5);
yield_ratio_lower = 1/(1+3.25*(120/sig_y)**1.45);
yield_ratio_lang = 1-0.72*np.exp(-0.0027*sig_y);
yield_ratio_gard_hot = 1/(1+(200./sig_y)**1.75);
yield_ratio_gard_cold = 1/(1+(130./sig_y)**1.4);
ax2.plot(sig_y, yield_ratio_fuku, label=r'Fukumoto (1996)')
ax2.plot(sig_y, yield_ratio_upper, label=r'Bannister (1999) upper bound')
ax2.plot(sig_y, yield_ratio_lower, label=r'Bannister (1999) lower bound')
ax2.plot(sig_y, yield_ratio_lang, label=r'Langenberg (2008)')
ax2.plot(sig_y, yield_ratio_gard_hot, label=r'Yun & Gardner (2017) hot rolled')
ax2.plot(sig_y, yield_ratio_gard_cold, label=r'Gardner & Yun (2018) cold formed')
ax2.set_xlim([400,1000])

ax2.legend(loc='lower right')
ax2.set_title(r'required $\sigma_y/\sigma_u$ against $\sigma_y$ for achieving $R$=3 for $t_f$ = 20 mm; $t_w$ = 17.6 mm and $\lambda_{LT}$ = 0.1')

# %% Find required Y/T against Y for HSA 800; Ldw = 40p0eps; Ldf = 10p1eps (plotting sig_y on z axis)

plt.close('all')

this_batch = 'vary YoT vary Y t_HSA800 Lw_40p0eps Lf_10p1eps'
# this_batch = 'vary YoT vary Y t_SM490_12mm Lf_10p1eps Lw_52p5eps'

fig = plt.figure()
ax = plt.axes(projection='3d')


# Do not include zeros in fitting of surface
idx = np.nonzero(data_dict[this_batch]['R'])
xdat = data_dict[this_batch]['YoT'][idx]
zdat = data_dict[this_batch]['Y'][idx]
ydat = data_dict[this_batch]['R'][idx]

xi = np.linspace(xdat.min(),xdat.max(),500)
yi = np.linspace(ydat.min(),ydat.max(),500)
zi = griddata((xdat, ydat), zdat, (xi[None,:], yi[:,None]), method='linear')
xig, yig = np.meshgrid(xi, yi)
 
surf = ax.plot_surface(xig, yig, zi, cmap='Purples_r', alpha=0.7, 
                        vmin=np.floor(np.nanmin(zi)), vmax=np.ceil(np.nanmax(zi)))

# position of colorbar
# where arg is [left, bottom, width, height]
# cax = fig.add_axes([0.9, 0.35, 0.03, 0.35])
# clb1 = fig.colorbar(surf, orientation='vertical', cax=cax)
# clb1.ax.set_title('')
# plt.show()

for i, imp in enumerate(data_dict[this_batch]['imp']):
    if imp==0 and data_dict[this_batch]['R'][i]!=0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['R'][i],data_dict[this_batch]['Y'][i], marker='o', fillstyle='none', color='purple')
    elif imp==1 and data_dict[this_batch]['R'][i]!=0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['R'][i],data_dict[this_batch]['Y'][i], marker='o', color='purple')
    elif imp==0 and data_dict[this_batch]['R'][i]==0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['R'][i],data_dict[this_batch]['Y'][i], marker='s', fillstyle='none', color='purple')
    elif imp==1 and data_dict[this_batch]['R'][i]==0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['R'][i],data_dict[this_batch]['Y'][i], marker='s', color='purple')
ax.set_xlabel(r'$\sigma_y/\sigma_u$',  fontsize=20)
ax.set_ylabel(r'$R$', fontsize=20)
ax.set_zlabel(r'$\sigma_y$', fontsize=20)
ax.grid()
ax.tick_params(axis='both', which='major', labelsize=12)
ax.tick_params(axis='both', which='minor', labelsize=12)
ax.set_title(r'',  fontsize=20)      
# ax.set_xlim([0.8, 0.99])
# ax.set_ylim([6,10.1])

# 2D cut through 3D plot at YoT = 0.94
yi_strip = 3
zi_strip = griddata((ydat, xdat), zdat, (yi_strip, xi[:,None]), method='linear')
yig_strip, xig_strip = np.meshgrid(yi_strip, xi)

fig, ax2 = plt.subplots()
ax2.plot(zi_strip, xi,
         marker='o', linestyle='-',
         label=r'required $\sigma_y/\sigma_u$ against $\sigma_y$',
         color='purple')
ax2.set_xlabel(r'$\sigma_y$',  fontsize=13)
ax2.set_ylabel(r'$\sigma_y/\sigma_u$', fontsize=13)
ax2.grid()
ax2.tick_params(axis='both', which='major', labelsize=12)
ax2.tick_params(axis='both', which='minor', labelsize=12)
ax2.set_title(r'',  fontsize=20)

# Plot the 2D cut on on the 3D plot
# ax.plot(xi,zi_strip,label=r'$t_f$ = 20 mm; $t_w$ = 17.6 mm and $\lambda_{LT}$ = 0.1')
# ax.legend()

sig_y = np.linspace(400,1100,100)
yield_ratio_fuku = sig_y/(0.83*sig_y+203.8)
yield_ratio_upper = 1/(1+2*(150/sig_y)**2.5);
yield_ratio_lower = 1/(1+3.25*(120/sig_y)**1.45);
yield_ratio_lang = 1-0.72*np.exp(-0.0027*sig_y);
yield_ratio_gard_hot = 1/(1+(200./sig_y)**1.75);
yield_ratio_gard_cold = 1/(1+(130./sig_y)**1.4);
ax2.plot(sig_y, yield_ratio_fuku, label=r'Fukumoto (1996)')
ax2.plot(sig_y, yield_ratio_upper, label=r'Bannister (1999) upper bound')
ax2.plot(sig_y, yield_ratio_lower, label=r'Bannister (1999) lower bound')
ax2.plot(sig_y, yield_ratio_lang, label=r'Langenberg (2008)')
ax2.plot(sig_y, yield_ratio_gard_hot, label=r'Yun & Gardner (2017) hot rolled')
ax2.plot(sig_y, yield_ratio_gard_cold, label=r'Gardner & Yun (2018) cold formed')
ax2.set_xlim([400,1000])

ax2.legend(loc='lower right')
ax2.set_title(r'required $\sigma_y/\sigma_u$ against $\sigma_y$ for achieving $R$=3 for $t_f$ = 20 mm; $t_w$ = 17.6 mm and $\lambda_{LT}$ = 0.1')

# %% Find required Y/T against Y for SM 490 12 mm; Ldw = 52p5eps; Ldf = 10p1eps

plt.close('all')

this_batch = 'vary YoT vary Y t_SM490_12mm Lf_10p1eps Lw_52p5eps'

fig = plt.figure()
ax = plt.axes(projection='3d')

# Do not include zeros in fitting of surface
idx = np.nonzero(data_dict[this_batch]['R'])
xdat = data_dict[this_batch]['YoT'][idx]
ydat = data_dict[this_batch]['Y'][idx]
zdat = data_dict[this_batch]['R'][idx]

# Set interpolation points using xi:
xi = np.linspace(xdat.min(),xdat.max(),80) 
yi = np.linspace(ydat.min(),ydat.max(),50)
zi = griddata((xdat, ydat), zdat, (xi[None,:], yi[:,None]), method='linear')
xig, yig = np.meshgrid(xi, yi)
 
surf = ax.plot_surface(xig, yig, zi, cmap='Purples_r', alpha=0.7, 
                        vmin=np.floor(np.nanmin(zi)), vmax=np.ceil(np.nanmax(zi)))

# position of colorbar
# where arg is [left, bottom, width, height]
cax = fig.add_axes([0.9, 0.35, 0.03, 0.35])
clb1 = fig.colorbar(surf, orientation='vertical', cax=cax)
clb1.ax.set_title('')
# plt.show()

for i, imp in enumerate(data_dict[this_batch]['imp']):
    if imp==0 and data_dict[this_batch]['R'][i]!=0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Y'][i],data_dict[this_batch]['R'][i], marker='o', fillstyle='none', color='purple')
    elif imp==1 and data_dict[this_batch]['R'][i]!=0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Y'][i],data_dict[this_batch]['R'][i], marker='o', color='purple')
    elif imp==0 and data_dict[this_batch]['R'][i]==0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Y'][i],data_dict[this_batch]['R'][i], marker='s', fillstyle='none', color='purple')
    elif imp==1 and data_dict[this_batch]['R'][i]==0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Y'][i],data_dict[this_batch]['R'][i], marker='s', color='purple')
ax.set_xlabel(r'$\sigma_y/\sigma_u$',  fontsize=20)
ax.set_ylabel(r'$\sigma_y$', fontsize=20)
ax.set_zlabel(r'$R$', fontsize=20)
ax.grid()
ax.tick_params(axis='both', which='major', labelsize=12)
ax.tick_params(axis='both', which='minor', labelsize=12)
ax.set_title(r'',  fontsize=20)      
# ax.set_xlim([0.8, 0.99])
# ax.set_ylim([6,10.1])

# 2D cut through 3D plot at YoT = 0.94
zi_strip = 3
yi_strip = griddata((zdat, xdat), ydat, (zi_strip, xi[:,None]), method='linear')
zig_strip, xig_strip = np.meshgrid(zi_strip, xi)

fig, ax2 = plt.subplots()
ax2.plot(yi_strip, xi,
         marker='o', linestyle='-',
         label=r'required $\sigma_y/\sigma_u$ against $\sigma_y$',
         color='purple')
ax2.set_xlabel(r'$\sigma_y$',  fontsize=13)
ax2.set_ylabel(r'$\sigma_y/\sigma_u$', fontsize=13)
ax2.grid()
ax2.tick_params(axis='both', which='major', labelsize=12)
ax2.tick_params(axis='both', which='minor', labelsize=12)
ax2.set_title(r'',  fontsize=20)

# Plot the 2D cut on on the 3D plot
ax.plot(xi,yi_strip,3,label=r'$t_f$ = 12 mm; $t_w$ = 11 mm and $\lambda_{LT}$ = 0.1')
ax.legend()

sig_y = np.linspace(400,1100,100)
yield_ratio_fuku = sig_y/(0.83*sig_y+203.8)
yield_ratio_upper = 1/(1+2*(150/sig_y)**2.5);
yield_ratio_lower = 1/(1+3.25*(120/sig_y)**1.45);
yield_ratio_lang = 1-0.72*np.exp(-0.0027*sig_y);
yield_ratio_gard_hot = 1/(1+(200./sig_y)**1.75);
yield_ratio_gard_cold = 1/(1+(130./sig_y)**1.4);
ax2.plot(sig_y, yield_ratio_fuku, label=r'Fukumoto (1996)')
ax2.plot(sig_y, yield_ratio_upper, label=r'Bannister (1999) upper bound')
ax2.plot(sig_y, yield_ratio_lower, label=r'Bannister (1999) lower bound')
ax2.plot(sig_y, yield_ratio_lang, label=r'Langenberg (2008)')
ax2.plot(sig_y, yield_ratio_gard_hot, label=r'Yun & Gardner (2017) hot rolled')
ax2.plot(sig_y, yield_ratio_gard_cold, label=r'Gardner & Yun (2018) cold formed')
ax2.set_xlim([400,1000])

ax2.legend(loc='lower right')
ax2.set_title(r'required $\sigma_y/\sigma_u$ against $\sigma_y$ for achieving $R$=3 for $t_f$ = 12 mm; $t_w$ = 11 mm and $\lambda_{LT}$ = 0.1')

# %% * Find required Y/T against Y for HSA 800 and SM 490

plt.close('all')

# SM 490 12 mm; Ldw = 52p5eps; Ldf = 10p1eps
this_batch = 'vary YoT vary Y t_SM490_12mm Lf_10p1eps Lw_52p5eps'

fig = plt.figure()
ax = plt.axes(projection='3d')

# Do not include zeros in fitting of surface
idx = np.nonzero(data_dict[this_batch]['R'])
xdat = data_dict[this_batch]['YoT'][idx]
ydat = data_dict[this_batch]['Y'][idx]
zdat = data_dict[this_batch]['R'][idx]

# Set interpolation points using xi:
xi = np.linspace(xdat.min(),xdat.max(),1000) 
yi = np.linspace(ydat.min(),ydat.max(),1000)
zi = griddata((xdat, ydat), zdat, (xi[None,:], yi[:,None]), method='linear')
xig, yig = np.meshgrid(xi, yi)
 
surf = ax.plot_surface(xig, yig, zi, cmap='Purples_r', alpha=0.7, 
                        vmin=np.floor(np.nanmin(zi)), vmax=np.ceil(np.nanmax(zi)))

# position of colorbar
# where arg is [left, bottom, width, height]
cax = fig.add_axes([0.9, 0.35, 0.03, 0.35])
clb1 = fig.colorbar(surf, orientation='vertical', cax=cax)
clb1.ax.set_title('')
# plt.show()

for i, imp in enumerate(data_dict[this_batch]['imp']):
    if imp==0 and data_dict[this_batch]['R'][i]!=0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Y'][i],data_dict[this_batch]['R'][i], marker='o', fillstyle='none', color='purple')
    elif imp==1 and data_dict[this_batch]['R'][i]!=0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Y'][i],data_dict[this_batch]['R'][i], marker='o', color='purple')
    elif imp==0 and data_dict[this_batch]['R'][i]==0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Y'][i],data_dict[this_batch]['R'][i], marker='s', fillstyle='none', color='purple')
    elif imp==1 and data_dict[this_batch]['R'][i]==0:
        ax.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Y'][i],data_dict[this_batch]['R'][i], marker='s', color='purple')
ax.set_xlabel(r'$\sigma_y/\sigma_u$',  fontsize=20)
ax.set_ylabel(r'$\sigma_y$', fontsize=20)
ax.set_zlabel(r'$R$', fontsize=20)
ax.grid()
ax.tick_params(axis='both', which='major', labelsize=12)
ax.tick_params(axis='both', which='minor', labelsize=12)
ax.set_title(r'',  fontsize=20)      
# ax.set_xlim([0.8, 0.99])
# ax.set_ylim([6,10.1])

# 2D cut through 3D plot at R=3
zi_strip = np.array(3)
yi_strip = griddata((zdat, xdat), ydat, (zi_strip, xi[:,None]), method='linear')
zig_strip, xig_strip = np.meshgrid(zi_strip, xi)

fig2, ax2 = plt.subplots()
ax2.plot(yi_strip[~np.isnan(yi_strip)][:-29], xig_strip[~np.isnan(yi_strip)][:-29], # remove extrapolated values at high sig_y
         marker=None, linestyle='-', fillstyle='none',
         label=r'$R$ = 3 for $\lambda_f$ = 10.1 $\varepsilon$;'
         '\n'
         r'$\lambda_w$ = 52.5 $\varepsilon$; $t_f$ = 12 mm;' 
         '\n' 
         r'$t_w$ = 11 mm and $\lambda_{LT}$ = 0.1',
         color='red')
ax2.set_xlabel(r'$\sigma_y$',  fontsize=22)
ax2.set_ylabel(r'$\sigma_y/\sigma_u$', fontsize=22)
ax2.grid()
ax2.tick_params(axis='both', which='major', labelsize=18)
ax2.tick_params(axis='both', which='minor', labelsize=18)
ax2.set_title(r'',  fontsize=20)
ax2.set_yticks([0.8,0.825,0.85,0.875,0.9,0.925,0.95,0.975])

# Plot the 2D cut on on the 3D plot
ax.plot(xi,yi_strip,3,label=r'$t_f$ = 12 mm; $t_w$ = 11 mm and $\lambda_{LT}$ = 0.1')
ax.legend()

# HSA 800; Ldw = 40p0eps; Ldf = 10p1eps
this_batch = 'vary YoT vary Y t_HSA800 Lw_40p0eps Lf_10p1eps'

fig3 = plt.figure()
ax3 = plt.axes(projection='3d')

# Do not include zeros in fitting of surface
idx = np.nonzero(data_dict[this_batch]['R'])
xdat = data_dict[this_batch]['YoT'][idx]
ydat = data_dict[this_batch]['Y'][idx]
zdat = data_dict[this_batch]['R'][idx]

# xdat = data_dict[this_batch]['YoT']
# ydat = data_dict[this_batch]['Y']
# zdat = data_dict[this_batch]['R']

# Set interpolation points using xi:
xi = np.linspace(xdat.min(),xdat.max(),1000) 
yi = np.linspace(ydat.min(),ydat.max(),1000) 
zi = griddata((xdat, ydat), zdat, (xi[None,:], yi[:,None]), method='linear')
xig, yig = np.meshgrid(xi, yi)
 
surf = ax3.plot_surface(xig, yig, zi, cmap='Purples_r', alpha=0.7, 
                        vmin=np.floor(np.nanmin(zi)), vmax=np.ceil(np.nanmax(zi)))

# position of colorbar
# where arg is [left, bottom, width, height]
cax = fig3.add_axes([0.9, 0.35, 0.03, 0.35])
clb1 = fig.colorbar(surf, orientation='vertical', cax=cax)
clb1.ax.set_title('')
# plt.show()

for i, imp in enumerate(data_dict[this_batch]['imp']):
    if imp==0 and data_dict[this_batch]['R'][i]!=0:
        ax3.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Y'][i],data_dict[this_batch]['R'][i], marker='o', fillstyle='none', color='purple')
    elif imp==1 and data_dict[this_batch]['R'][i]!=0:
        ax3.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Y'][i],data_dict[this_batch]['R'][i], marker='o', color='purple')
    elif imp==0 and data_dict[this_batch]['R'][i]==0:
        ax3.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Y'][i],data_dict[this_batch]['R'][i], marker='s', fillstyle='none', color='purple')
    elif imp==1 and data_dict[this_batch]['R'][i]==0:
        ax3.plot(data_dict[this_batch]['YoT'][i],data_dict[this_batch]['Y'][i],data_dict[this_batch]['R'][i], marker='s', color='purple')
ax3.set_xlabel(r'$\sigma_y/\sigma_u$',  fontsize=20)
ax3.set_ylabel(r'$\sigma_y$', fontsize=20)
ax3.set_zlabel(r'$R$', fontsize=20)
ax3.grid()
ax3.tick_params(axis='both', which='major', labelsize=12)
ax3.tick_params(axis='both', which='minor', labelsize=12)
ax3.set_title(r'',  fontsize=20)      
# ax.set_xlim([0.8, 0.99])
# ax.set_ylim([6,10.1])

# 2D cut through 3D plot at R = 3
zi_strip = np.array(3)
yi_strip = griddata((zdat, xdat), ydat, (zi_strip, xi[:,None]), method='linear')
zig_strip, xig_strip = np.meshgrid(zi_strip, xi)

ax2.plot(yi_strip[~np.isnan(yi_strip)][59:], xig_strip[~np.isnan(yi_strip)][59:], # remove extrapolated values at low sig_y
         marker=None, linestyle='-', fillstyle='none',
         label=r'$R$ = 3 for $\lambda_f$ = 10.1 $\varepsilon$;'
         '\n'
         r'$\lambda_w$ = 40.0 $\varepsilon$; $t_f$ = 20 mm; '
         '\n'
         r'$t_w$ = 17.6 mm and $\lambda_{LT}$ = 0.1',
         color='purple')

# Plot the 2D cut on on the 3D plot
ax3.plot(xi,yi_strip,3,label=r'$t_f$ = 20 mm; $t_w$ = 17.6 mm and $\lambda_{LT}$ = 0.1')
ax3.legend()

# Observed Y/T and Y
sig_y = np.linspace(400,1100,100)
yield_ratio_fuku = sig_y/(0.83*sig_y+203.8)
yield_ratio_upper = 1/(1+2*(150/sig_y)**2.5);
yield_ratio_lower = 1/(1+3.25*(120/sig_y)**1.45);
yield_ratio_lang = 1-0.72*np.exp(-0.0027*sig_y);
yield_ratio_gard_hot = 1/(1+(200./sig_y)**1.75);
yield_ratio_gard_cold = 1/(1+(130./sig_y)**1.4);
# ax2.plot(sig_y, yield_ratio_fuku, label=r'Fukumoto (1996)')
ax2.plot(sig_y, yield_ratio_upper, label=r'Bannister (1999) upper bound', linestyle='--', color='C1')
ax2.plot(sig_y, yield_ratio_lower, label=r'Bannister (1999) lower bound', linestyle='--', color='C2')
ax2.plot(sig_y, yield_ratio_lang, label=r'Langenberg (2000)', linestyle='--', color='C4')
# ax2.plot(sig_y, yield_ratio_gard_hot, label=r'Yun & Gardner (2017) hot rolled')
# ax2.plot(sig_y, yield_ratio_gard_cold, label=r'Gardner & Yun (2018) cold formed')
ax2.set_xlim([460,960])
ax2.set_ylim([0.78,0.99])

box = ax.get_position()
# ax2.set_position([box.x0, box.y0, box.width*1.2, box.height])
# ax2.legend(loc='upper left', bbox_to_anchor=(1, 0.86),labelspacing = 2)

ax2.set_position([box.x0, box.y0, box.width*1.15, box.height])
ax2.legend(loc='upper left', bbox_to_anchor=(1, 0.85),labelspacing=1, fontsize=18)

# fig2.savefig('YoT_against_Y_competition.eps', format='eps', bbox_inches='tight')
# exportEmf(savePath, 'YoT against Y', fig=fig2, keepSVG=True)

# %% Plot specific moment-rotation graphs

YoT = 0.98
Y = 960
B = 217.5
H = 388.3
tf = 20
tw = 17.6
plt.close('all')

def plot_MR(Y,YoT,B,H,tf,tw,ax,this_batch):
    """
    Plot_MR given Y, YoT, B, H, tf, tw
    Outputs the length and index (frame number) near to which R occurs
    ax: axes to plot to
    """
    
    # initialize
    idx_R = np.NaN
    idx_Ru = np.NaN
    idx_Mel = np.NaN
    L = np.NaN
    R = np.NaN
    Ru = np.NaN
                
    for i, dataYoT in enumerate(data_dict[this_batch]['YoT']):
        if (dataYoT==YoT 
            and data_dict[this_batch]['Y'][i]==Y 
            and data_dict[this_batch]['B'][i]==B
            and data_dict[this_batch]['H'][i]==H
            and data_dict[this_batch] ['tf'][i]==tf
            and data_dict[this_batch]['tw'][i]==tw):
            
            L = data_dict[this_batch]['L'][i]
            R = data_dict[this_batch]['R'][i]
            Ru = data_dict[this_batch]['Ru'][i]
            
            ax.plot(data_dict[this_batch]['data_list'][i]['UR1'].values/data_dict[this_batch]['Tp'][i],
            data_dict[this_batch]['data_list'][i]['RM1']/data_dict[this_batch]['Mp'][i],
            label=rf'$\sigma_y={Y}; \sigma_y/\sigma_u={YoT}$'.format(Y=Y,YoT=YoT))
            
            idx_R = data_dict[this_batch]['idx_R'][i]
            if ~np.isnan(idx_R):
                ax.plot(data_dict[this_batch]['data_list'][i]['UR1'][idx_R]/data_dict[this_batch]['Tp'][i],
                        data_dict[this_batch]['data_list'][i]['RM1'][idx_R]/data_dict[this_batch]['Mp'][i],
                        marker='x')
            else:
                print('The moment does not reach Mpl.')

            idx_Ru = data_dict[this_batch]['idx_Ru'][i]
            ax.plot(data_dict[this_batch]['data_list'][i]['UR1'][idx_Ru]/data_dict[this_batch]['Tp'][i],
                    data_dict[this_batch]['data_list'][i]['RM1'][idx_Ru]/data_dict[this_batch]['Mp'][i],
                    marker='x')
            
            idx_Mel = data_dict[this_batch]['idx_Mel'][i]
            ax.plot(data_dict[this_batch]['data_list'][i]['UR1'][idx_Mel]/data_dict[this_batch]['Tp'][i],
                    data_dict[this_batch]['data_list'][i]['RM1'][idx_Mel]/data_dict[this_batch]['Mp'][i],
                    marker='x')
            
            print(f'Mel occurs at {idx_Mel} and Ru occurs at {idx_Ru} and R occurs at {idx_R}'.format(idx_Mel=idx_Mel,idx_Ru=idx_Ru,idx_R=idx_R))
    if np.isnan(idx_Mel):
        print('The given combination of Y, YoT, B, H, tf and tw does not exist in the parametric database.')
        
    return L, idx_R, R, Ru

fig, ax = plt.subplots()

this_batch = 'all runs'
plot_MR(Y,YoT,B,H,tf,tw,ax,this_batch)

ax.set_xlabel(r'$\theta/\theta_{pl}$',  fontsize=20)
ax.set_ylabel(r'$M/M_{pl}$', fontsize=20)
ax.grid()
ax.tick_params(axis='both', which='major', labelsize=12)
ax.tick_params(axis='both', which='minor', labelsize=12)
ax.set_title(r'',  fontsize=20)
ax.legend()

# %% Plot specific stress-strain curves

plt.close('all')

YoT = 1
Y = 690
n,K,E = fn.gen_n_K(YoT,Y)
epsY = Y/E

eps = np.linspace(Y/E,1,1000)
eps_pl = eps-Y/E
sig = K*eps**n

fig, ax = plt.subplots()
ax.plot([0,epsY], [0,Y], linestyle='-')
ax.plot(eps, sig, linestyle='-')
ax.set_xlabel(r'$\varepsilon$',  fontsize=20)
ax.set_ylabel(r'$\sigma$', fontsize=20)
ax.grid()
ax.tick_params(axis='both', which='major', labelsize=12)
ax.tick_params(axis='both', which='minor', labelsize=12)
ax.set_title(r'',  fontsize=20)
ax.set_xlim([0,0.1])
ax.set_ylim([0,900])

ax.plot(eps, sig, linestyle='-')

# %% * Number of runs used in paper
    
# this is the total number of runs
print(np.size(data_dict['all runs']['YoT']))

# old way of calculating, before the runs in the last two folders were added to use to access YoT requirement:
# this overcounts due to some duplicates unless it is used before duplicates are made in the relevant folders (see comments below)
print(np.size(data_dict['vary YoT vary Y t_HSA800 default H and B']['YoT']) + \
    np.size(data_dict['vary B vary YoT t_HSA800 Y690 Lw_40p0eps']['YoT']) + \
        np.size(data_dict['vary B vary YoT t_HSA800 Y890 Lw_40p0eps']['YoT']) + \
            np.size(data_dict['vary YoT vary Y t_SM490 default H and B']['YoT']) + \
                np.size(data_dict['vary YoT vary Y t_SM490_12mm default H and B']['YoT']) + \
                    np.size(data_dict['vary H vary YoT t_SM490_12mm Y690 Lf_10p1eps']['YoT']) + \
                        np.size(data_dict['vary H vary YoT t_SM490_12mm Y890 Lf_10p1eps']['YoT']) + \
                            # this contains duplicates copied from 'vary B vary YoT t_HSA800 Y690 Lw_40p0eps' and 'vary B vary YoT t_HSA800 Y890 Lw_40p0eps'
                            np.size(data_dict['vary YoT vary Y t_HSA800 Lw_40p0eps Lf_10p1eps']['YoT']) + \
                                # this contains duplicates copied from 'vary H vary YoT t_SM490_12mm Y690 Lf_10p1eps' and 'vary H vary YoT t_SM490_12mm Y890 Lf_10p1eps'
                                np.size(data_dict['vary YoT vary Y t_SM490_12mm Lf_10p1eps Lw_52p5eps']['YoT']))
    
# %% Back-calculating the actual LdLT to double check

this_batch = 'all runs'

LdLT = np.zeros(np.size(data_dict[this_batch]['L']))

for i, L in enumerate(data_dict[this_batch]['L']):
    LdLT[i] = fn.calc_LdLT(data_dict[this_batch]['Y'][i],
                            data_dict[this_batch]['Y'][i],
                            data_dict[this_batch]['tf'][i],
                            data_dict[this_batch]['tw'][i],
                            data_dict[this_batch]['B'][i],
                            data_dict[this_batch]['H'][i],
                            L)