# -*- coding: utf-8 -*-
"""
Created on Wed May 18 15:04:16 2022

@author: wjwong

"""
from scipy import optimize # To use fsolve
import numpy as np

def gen_pl_sig_eps(YoT,Y):
    
    E = float(2e5) # Young's modulus [MPa]
    epsY = Y/E
       
    def solve_n(YoT,Y):
        def YoT_min_f_YoT(n,YoT,Y): 

            	return YoT-1/(1+epsY)*(np.exp(1)*np.log(1+epsY)/n)**n
    
        return optimize.fsolve(YoT_min_f_YoT, x0=0.08, args=(YoT,Y))
    
    if YoT==1:
        n = np.log(epsY+1)
    else:
        n = solve_n(YoT,Y)
    K = Y/epsY**n
    
    eps = np.linspace(Y/E,1,1000)
    eps_pl = eps-Y/E
    sig = K*eps**n
    
    material_name = 'Y_' + "{Y:.0f}".format(Y=Y) + '_YoT_' + "{YoT:.2f}".format(YoT=YoT)
    material_name = material_name.replace('.','p') # Replace dots with 'p' for creating file name
    fid = open(material_name + '_SS.txt','w')
    fid.write('Stress [-], Plastic strain [MPa] \n')
    for i in range(eps_pl.size):
        fid.write('%10.8E, %10.8E \n'%(sig[i],eps_pl[i]))
    fid.close()
    return eps, sig

def gen_n_K(YoT,Y):
    
    E = float(2e5) # Young's modulus [MPa]
    epsY = Y/E
       
    def solve_n(YoT,Y):
        def YoT_min_f_YoT(n,YoT,Y): 

            	return YoT-1/(1+epsY)*(np.exp(1)*np.log(1+epsY)/n)**n
    
        return optimize.fsolve(YoT_min_f_YoT, x0=0.08, args=(YoT,Y))
    
    if YoT==1:
        n = np.log(epsY+1)
    else:
        n = solve_n(YoT,Y)
    K = Y/epsY**n
    
    return n,K,E

def calc_L(Yf,Yw,tf,tw,Bf,H,LdLT):
    
    # Work out L for which LdLT = specified
    E = 2e5
    nu = 0.3
    G = E/2/(1+nu) # shear modulus
    bf = Bf/2
    d = H-2*tf
    Mel = calc_Mel(Yf,Yw,tf,tw,Bf,H)
    Ix = tw*d**3/12+bf*tf*(d+tf)**2+bf*tf**3/3 # second moment of area
    Iw = Ix*d**2/4
    J = 1/3*(d*tw**3+4*bf*tf**3)
    Mcr_short = Mel/LdLT**2
    def func(L):
        return Mcr_short - np.pi/L*np.sqrt(E*Ix*G*J)*np.sqrt(1+np.pi**2/L**2*E*Iw/G/J)
    L_short = optimize.fsolve(func,500)
    return L_short[0] # [mm]

def calc_LdLT(Yf,Yw,tf,tw,Bf,H,L):
    E = 2e5
    nu = 0.3
    G = E/2/(1+nu) # shear modulus
    bf = Bf/2
    d = H-2*tf
    Mel = calc_Mel(Yf,Yw,tf,tw,Bf,H)
    Ix = tw*d**3/12+bf*tf*(d+tf)**2+bf*tf**3/3 # second moment of area
    Iw = Ix*d**2/4
    J = 1/3*(d*tw**3+4*bf*tf**3)
    Mcr = np.pi/L*np.sqrt(E*Ix*G*J)*np.sqrt(1+np.pi**2/L**2*E*Iw/G/J)
    LdLT = np.sqrt(Mel/Mcr)
    return LdLT

def calc_Mel(Yf,Yw,tf,tw,Bf,H):
    
    b = Bf/2 # [mm] - half the flange width
    d = H-2*tf # [mm] - depth of web
    
    # Elastic section properties
    Iwx = tw*d**3/12
    Ifx = b*tf**3/3 + b*tf*(tf+d)**2
    I = Ifx+Iwx
    S = I/(H/2)
    if Yw>=Yf or Yw<Yf<=H/2/(H/2-tf)*Yw:
        # flange yielding marks the end of linear stress distribution
        Mel = S*Yf
    
    if Yf>H/2/(H/2-tf)*Yw:
        # web yielding marks the end of linear stress distribution
        Mel = S*Yw
        
    # bf = Bf/2
    # hw = H-2*tf
    # J = tw*hw**3/12+bf*tf*(hw+tf)**2+bf*tf**3/3
        
    return Mel

def calc_Mpl(Yf,Yw,tf,tw,Bf,H):
    
    # x = (hw*tw+tf*bf-tp*bp)/(2*tw)
    
    # Mpl = x*tp*bp*sig_y + 0.5*x**2*tw*sig_y + (hw-x)*tf*bf + 0.5*(hw-x)**2*tw*sig_y
    
    hw = H-2*tf
    bf = Bf/2
    
    Mpl = 2*(bf*tf*(hw+tf)*Yf+hw**2/8*tw*Yw)
    
    return Mpl
    
def gen_inp_py_file(YoT,Y,tf,tw,Bf,H,LdLT):
    
    job_name = 'Y_' + "{Y:.0f}".format(Y=Y) + '_YoT_' + "{YoT:.2f}".format(YoT=YoT) + '_tf_' + \
        "{tf:.1f}".format(tf=tf) + '_tw_' + \
            "{tw:.1f}".format(tw=tw) + '_Bf_' + \
                "{Bf:.1f}".format(Bf=Bf) + '_H_' + \
                    "{H:.1f}".format(H=H) + '_LdLT_' + \
                        "{LdLT:.2f}".format(LdLT=LdLT)
    job_name = job_name.replace('.','p')
    
    material_name = 'Y_' + "{Y:.0f}".format(Y=Y) + '_YoT_' + "{YoT:.2f}".format(YoT=YoT)
    material_name = material_name.replace('.','p')
    
    LBA_name = 'LBA_Y_' + "{Y:.0f}".format(Y=Y) + '_tf_' + \
        "{tf:.1f}".format(tf=tf) + '_tw_' + \
            "{tw:.1f}".format(tw=tw) + '_Bf_' + \
                "{Bf:.1f}".format(Bf=Bf) + '_H_' + \
                    "{H:.1f}".format(H=H) + '_LdLT_' + \
                        "{LdLT:.2f}".format(LdLT=LdLT)
    LBA_name = LBA_name.replace('.','p')
                        
    eigenvalue_imp = np.loadtxt(LBA_name+'_out.txt', comments="#", delimiter=",", skiprows=1, unpack=False)
    
    imp = eigenvalue_imp[1]
        
    L = calc_L(Y,Y,tf,tw,Bf,H,LdLT)
    
    if imp==0: # LBA peak displacement is in flange
        imperfection = 0.8*Bf/100
    elif imp==1: # LBA peak displacement is in web
        imperfection = 0.8*H/100
    else:
        print('arg imp should be 0 (flange) or 1 (web)')
        
    context = {
        "job_name":job_name,
        "imperfection":imperfection,
        "Y":Y,
        "tf":tf,
        "tw":tw,
        "Bf":Bf,
        "H":H,
        "L":L,
        "LBA_name":LBA_name,
        "material_name":material_name
        }
    template = '''from abaqus import *
from abaqusConstants import *
from math import *
from caeModules import *
from driverUtils import executeOnCaeStartup
import sketch
import part
import math
from numpy import loadtxt

executeOnCaeStartup()
Mdb()

job_name = '{job_name}' # ABAQUS model .inp file name
Y = float({Y}) # Yield strength [MPa]
L = {L:6.1f} # Full length

E = float(2e5) # Young's modulus [MPa]

stress_strain = loadtxt('{material_name}'+'_SS.txt', comments="#", delimiter=",", skiprows=1, unpack=False)

nu = 0.3
sigy_w = Y
epsy_w = sigy_w/E
# sig_0p5_w = Esh*(0.5-epsy_w)+sigy_w

sigy_f = Y
epsy_f = sigy_f/E
# sig_0p5_f = Esh*(0.5-epsy_f)+sigy_f

max_time = 1.0
max_num_inc = 1000
max_inc = 0.0025
ini_inc = 0.0025

tf = float({tf})
tp = tf
tw = float({tw})
H = float({H})
hw = H-2*tf
Bf = float({Bf})
Bp = Bf
bp = Bp/2 # Half the flange width
bf = Bf/2
a_weld = 0.53*tw # Weld leg

# x = (hw*tw+tf*bf-tp*bp)/(2*tw)
# Mpl = x*tp*bp*sig_y + 0.5*x**2*tw*sig_y + (hw-x)*tf*bf + 0.5*(hw-x)**2*tw*sig_y

Mpl = 2*(bf*tf*(hw+tf)*sigy_f+hw**2/8*tw*sigy_w)
# Mpl = 1.58816e6*sig_y
# F = 8*Mpl/L

J = tw*hw**3/12+bf*tf*(hw+tf)**2+bf*tf**3/3

Tpl = Mpl*L/E/J

# part
s1 = mdb.models['Model-1'].ConstrainedSketch(name='__profile__', 
    sheetSize=200.0)
g, v, d, c = s1.geometry, s1.vertices, s1.dimensions, s1.constraints
s1.setPrimaryObject(option=STANDALONE)
s1.Spot(point=(0.0, 0.0))
s1.Spot(point=(bp, 0.0))
s1.Spot(point=(-bp, 0.0))
s1.Spot(point=(0.0, hw))
s1.Spot(point=(-bf, hw))
s1.Spot(point=(bf, hw))
session.viewports['Viewport: 1'].view.setValues(nearPlane=470.722, 
    farPlane=798.82, width=1309.27, height=785.434, cameraPosition=(65.6886, 
    129.458, 634.771), cameraTarget=(65.6886, 129.458, 0))
s1.Line(point1=(-bf, hw), point2=(0.0, hw))
s1.HorizontalConstraint(entity=g[2], addUndoState=False)
s1.Line(point1=(0.0, hw), point2=(bf, hw))
s1.HorizontalConstraint(entity=g[3], addUndoState=False)
s1.ParallelConstraint(entity1=g[2], entity2=g[3], addUndoState=False)
s1.Line(point1=(0.0, hw), point2=(0.0, 0.0))
s1.VerticalConstraint(entity=g[4], addUndoState=False)
s1.Line(point1=(-bp, 0.0), point2=(0.0, 0.0))
s1.HorizontalConstraint(entity=g[5], addUndoState=False)
s1.Line(point1=(0.0, 0.0), point2=(bp, 0.0))
s1.HorizontalConstraint(entity=g[6], addUndoState=False)
s1.ParallelConstraint(entity1=g[5], entity2=g[6], addUndoState=False)
p = mdb.models['Model-1'].Part(name='T', dimensionality=THREE_D, 
    type=DEFORMABLE_BODY)
p = mdb.models['Model-1'].parts['T']
p.BaseShellExtrude(sketch=s1, depth=L)
s1.unsetPrimaryObject()
p = mdb.models['Model-1'].parts['T']
session.viewports['Viewport: 1'].setValues(displayedObject=p)
del mdb.models['Model-1'].sketches['__profile__']

# Create datum planes for partitioning for residual stresses
# datum planes for partitioning flanges
p = mdb.models['Model-1'].parts['T']
p.DatumPlaneByPrincipalPlane(principalPlane=YZPLANE, offset=-7*Bf/16)
p = mdb.models['Model-1'].parts['T']
p.DatumPlaneByPrincipalPlane(principalPlane=YZPLANE, offset=7*Bf/16)
p = mdb.models['Model-1'].parts['T']
Bf_m = min(tw+5*a_weld,Bf/5) # Length of middle portion in flange with tensile residual stress
Bp_m = Bf_m # For symmetric I-sections
p.DatumPlaneByPrincipalPlane(principalPlane=YZPLANE, offset=Bf_m/2)
p = mdb.models['Model-1'].parts['T']
p.DatumPlaneByPrincipalPlane(principalPlane=YZPLANE, offset=-Bf_m/2)
# datum planes for partitioning webs
p = mdb.models['Model-1'].parts['T']
p.DatumPlaneByPrincipalPlane(principalPlane=XZPLANE, offset=hw/10-tp/2)
p = mdb.models['Model-1'].parts['T']
p.DatumPlaneByPrincipalPlane(principalPlane=XZPLANE, offset=9*hw/10+tf/2)

# material
mdb.models['Model-1'].Material(name='Elastic-plastic-flange')
mdb.models['Model-1'].materials['Elastic-plastic-flange'].Elastic(table=((E, 
    nu), ))
mdb.models['Model-1'].materials['Elastic-plastic-flange'].Plastic(table=stress_strain)
mdb.models['Model-1'].Material(name='Elastic-plastic-web')
mdb.models['Model-1'].materials['Elastic-plastic-web'].Elastic(table=((E, 
    nu), ))
mdb.models['Model-1'].materials['Elastic-plastic-web'].Plastic(table=stress_strain)

# create sections
mdb.models['Model-1'].HomogeneousShellSection(name='Section-Plate', preIntegrate=OFF, 
    material='Elastic-plastic-flange', thicknessType=UNIFORM, thickness=tp, 
    thicknessField='', nodalThicknessField='', idealization=NO_IDEALIZATION, 
    poissonDefinition=DEFAULT, thicknessModulus=None, temperature=GRADIENT, 
    useDensity=OFF, integrationRule=SIMPSON, numIntPts=25)
mdb.models['Model-1'].HomogeneousShellSection(name='Section-Flange', 
    preIntegrate=OFF, material='Elastic-plastic-flange', thicknessType=UNIFORM, 
    thickness=tf, thicknessField='', nodalThicknessField='', 
    idealization=NO_IDEALIZATION, poissonDefinition=DEFAULT, 
    thicknessModulus=None, temperature=GRADIENT, useDensity=OFF, 
    integrationRule=SIMPSON, numIntPts=25)
mdb.models['Model-1'].HomogeneousShellSection(name='Section-Web', 
    preIntegrate=OFF, material='Elastic-plastic-web', thicknessType=UNIFORM, 
    thickness=tw, thicknessField='', nodalThicknessField='', 
    idealization=NO_IDEALIZATION, poissonDefinition=DEFAULT, 
    thicknessModulus=None, temperature=GRADIENT, useDensity=OFF, 
    integrationRule=SIMPSON, numIntPts=25)
	
# assign sections
p = mdb.models['Model-1'].parts['T']
f = p.faces
faces = f.getSequenceFromMask(mask=('[#c ]', ), )
region = regionToolset.Region(faces=faces)
p = mdb.models['Model-1'].parts['T']
p.SectionAssignment(region=region, sectionName='Section-Flange', offset=0.0, 
    offsetType=MIDDLE_SURFACE, offsetField='', 
    thicknessAssignment=FROM_SECTION)
p = mdb.models['Model-1'].parts['T']
f = p.faces
faces = f.getSequenceFromMask(mask=('[#2 ]', ), )
region = regionToolset.Region(faces=faces)
p = mdb.models['Model-1'].parts['T']
p.SectionAssignment(region=region, sectionName='Section-Web', offset=0.0, 
    offsetType=MIDDLE_SURFACE, offsetField='', 
    thicknessAssignment=FROM_SECTION)
p = mdb.models['Model-1'].parts['T']
f = p.faces
faces = f.getSequenceFromMask(mask=('[#11 ]', ), )
region = regionToolset.Region(faces=faces)
p = mdb.models['Model-1'].parts['T']
p.SectionAssignment(region=region, sectionName='Section-Plate', offset=0.0, 
    offsetType=MIDDLE_SURFACE, offsetField='', 
    thicknessAssignment=FROM_SECTION)
	
# make shell orientation correct (for both flanges, attachment side is considered 'bottom')
p = mdb.models['Model-1'].parts['T']
f = p.faces
faces = f.getSequenceFromMask(mask=('[#8 ]', ), )
regions = regionToolset.Region(faces=faces)
p.flipNormal(regions=regions)

# offset top flange
mdb.models['Model-1'].parts['T'].sectionAssignments[0].setValues(
    offsetType=BOTTOM_SURFACE, offsetField='', offset=0.0)
	
# offset bottom flange
mdb.models['Model-1'].parts['T'].sectionAssignments[2].setValues(
    offsetType=BOTTOM_SURFACE, offsetField='', offset=0.0)

# create instance
a = mdb.models['Model-1'].rootAssembly
a.DatumCsysByDefault(CARTESIAN)
p = mdb.models['Model-1'].parts['T']
a.Instance(name='T-1', part=p, dependent=OFF)

# Create sets for boundary conditions and loading
# Rotated end set
a = mdb.models['Model-1'].rootAssembly
e1 = a.instances['T-1'].edges
edges1 = e1.getSequenceFromMask(mask=('[#9244 ]', ), )
a.Set(edges=edges1, name='Set-Rotated_end')
# Fixed end set
a = mdb.models['Model-1'].rootAssembly
e1 = a.instances['T-1'].edges
edges1 = e1.getSequenceFromMask(mask=('[#2491 ]', ), )
a.Set(edges=edges1, name='Set-Fixed_end')
# Rotated point (RP1) set
RP1 = a.ReferencePoint(point=(0.0, hw/2, 0.0))
a = mdb.models['Model-1'].rootAssembly
r1 = a.referencePoints
refPoints1=(r1[RP1.id], )
a.Set(referencePoints=refPoints1, name='RP1')

# coupling for boundaries
a = mdb.models['Model-1'].rootAssembly
region1=a.sets['RP1']
a = mdb.models['Model-1'].rootAssembly
region2=a.sets['Set-Rotated_end']
mdb.models['Model-1'].Coupling(name='Constraint-Rotated_end', 
    controlPoint=region1, surface=region2, influenceRadius=WHOLE_SURFACE, 
    couplingType=KINEMATIC, localCsys=None, u1=ON, u2=ON, u3=ON, ur1=ON, 
    ur2=ON, ur3=ON)
	
# boundary conditions
a = mdb.models['Model-1'].rootAssembly
region = a.sets['Set-Fixed_end']
mdb.models['Model-1'].EncastreBC(name='BC-Fixed_end', createStepName='Initial', 
    region=region, localCsys=None)
a = mdb.models['Model-1'].rootAssembly
region = a.sets['RP1']
mdb.models['Model-1'].DisplacementBC(name='BC-Rotated_end', createStepName='Initial', 
    region=region, u1=SET, u2=UNSET, u3=UNSET, ur1=UNSET, ur2=SET, ur3=SET, 
    amplitude=UNSET, distributionType=UNIFORM, fieldName='', localCsys=None)

# Partitioning by datum planes for the assignment of residual stresses
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
pickedFaces = f1.getSequenceFromMask(mask=('[#18 ]', ), )
d21 = a.instances['T-1'].datums
a.PartitionFaceByDatumPlane(datumPlane=d21[2], faces=pickedFaces)
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
pickedFaces = f1.getSequenceFromMask(mask=('[#14 ]', ), )
d31 = a.instances['T-1'].datums
a.PartitionFaceByDatumPlane(datumPlane=d31[3], faces=pickedFaces)
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
pickedFaces = f1.getSequenceFromMask(mask=('[#20 ]', ), )
d21 = a.instances['T-1'].datums
a.PartitionFaceByDatumPlane(datumPlane=d21[7], faces=pickedFaces)
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
pickedFaces = f1.getSequenceFromMask(mask=('[#1 ]', ), )
d31 = a.instances['T-1'].datums
a.PartitionFaceByDatumPlane(datumPlane=d31[6], faces=pickedFaces)
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
pickedFaces = f1.getSequenceFromMask(mask=('[#600 ]', ), )
d21 = a.instances['T-1'].datums
a.PartitionFaceByDatumPlane(datumPlane=d21[5], faces=pickedFaces)
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
pickedFaces = f1.getSequenceFromMask(mask=('[#30 ]', ), )
d21 = a.instances['T-1'].datums
a.PartitionFaceByDatumPlane(datumPlane=d21[4], faces=pickedFaces)

# Creating sets for the assignment of residual stresses
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
faces1 = f1.getSequenceFromMask(mask=('[#1100 ]', ), )
a.Set(faces=faces1, name='Set-Flange_tips')
#: The set 'Set-Flange_tips' has been created (2 faces).
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
faces1 = f1.getSequenceFromMask(mask=('[#84 ]', ), )
a.Set(faces=faces1, name='Set-Flange_intermediates')
#: The set 'Set-Flange_intermediates' has been created (2 faces).
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
faces1 = f1.getSequenceFromMask(mask=('[#2002 ]', ), )
a.Set(faces=faces1, name='Set-Flange_middle')
#: The set 'Set-Flange_middle' has been created (2 faces).
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
faces1 = f1.getSequenceFromMask(mask=('[#600 ]', ), )
a.Set(faces=faces1, name='Set-Plate_tips')
#: The set 'Set-Plate_tips' has been created (2 faces).
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
faces1 = f1.getSequenceFromMask(mask=('[#48 ]', ), )
a.Set(faces=faces1, name='Set-Plate_intermediates')
#: The set 'Set-Plate_intermediates' has been created (2 faces).
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
faces1 = f1.getSequenceFromMask(mask=('[#4001 ]', ), )
a.Set(faces=faces1, name='Set-Plate_middle')
#: The set 'Set-Plate_middle' has been created (2 faces).
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
faces1 = f1.getSequenceFromMask(mask=('[#20 ]', ), )
a.Set(faces=faces1, name='Set-Web_middle')
#: The set 'Set-Web_middle' has been created (1 face).
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
faces1 = f1.getSequenceFromMask(mask=('[#810 ]', ), )
a.Set(faces=faces1, name='Set-Web_junctions')
#: The set 'Set-Web_junctions' has been created (2 faces).

# Introduce residual stresses
sigy_r = min(sigy_f,sigy_w) # reference residual stress
# Flange tips residual stresses (tensile)
a = mdb.models['Model-1'].rootAssembly
region = a.sets['Set-Flange_tips']
mdb.models['Model-1'].Stress(name='Flange tips residual stress', region=region, 
    distributionType=UNIFORM, sigma11=0.0, sigma22=0.5*sigy_r*sqrt(float(235)/sigy_r), sigma12=0.0, 
    sigma33=None, sigma13=None, sigma23=None)
	# Flange middle residual stresses (tensile)
a = mdb.models['Model-1'].rootAssembly
region = a.sets['Set-Flange_middle']
mdb.models['Model-1'].Stress(name='Flange middle residual stress', region=region, 
    distributionType=UNIFORM, sigma11=0.0, sigma22=sigy_r*sqrt(float(235)/sigy_r), sigma12=0.0, 
    sigma33=None, sigma13=None, sigma23=None)
# Flange intermediates residual stresses (compressive)
# find compressive flange residual stresses by equilibrium with tensile flange residual stresses
Bf_t = Bf/8 + Bf_m # Length of portions of flange under tensile residual stress
Bf_c = Bf - Bf_t  # Length of portions of flange under compressive residual stress
sig_fc = (sigy_r*sqrt(float(235)/sigy_r) * (0.5*Bf/8 + Bf_m)) / Bf_c
region = a.sets['Set-Flange_intermediates']
mdb.models['Model-1'].Stress(name='Flange intermediates residual stress', region=region, 
    distributionType=UNIFORM, sigma11=0.0, sigma22=-sig_fc, sigma12=0.0, 
    sigma33=None, sigma13=None, sigma23=None)
# Plate (bottom flange) tips residual stresses (tensile)
a = mdb.models['Model-1'].rootAssembly
region = a.sets['Set-Plate_tips']
mdb.models['Model-1'].Stress(name='Plate tips residual stress', region=region, 
    distributionType=UNIFORM, sigma11=0.0, sigma22=0.5*sigy_r*sqrt(float(235)/sigy_r), sigma12=0.0, 
    sigma33=None, sigma13=None, sigma23=None)
# Plate (bottom flange) middle residual stresses (tensile)
a = mdb.models['Model-1'].rootAssembly
region = a.sets['Set-Plate_middle']
mdb.models['Model-1'].Stress(name='Plate middle residual stress', region=region, 
    distributionType=UNIFORM, sigma11=0.0, sigma22=sigy_r*sqrt(float(235)/sigy_r), sigma12=0.0, 
    sigma33=None, sigma13=None, sigma23=None)
# Plate (bottom flange) intermediates residual stresses (compressive)
# find compressive flange residual stresses by equilibrium with tensile flange residual stresses
Bp_t = Bp/8 + Bp_m # Length of portions of flange under tensile residual stress
Bp_c = Bp - Bp_t  # Length of portions of flange under compressive residual stress
sig_pc = (sigy_r*sqrt(float(235)/sigy_r) * (0.5*Bp/8 + Bp_m)) / Bp_c
region = a.sets['Set-Plate_intermediates']
mdb.models['Model-1'].Stress(name='Plate intermediates residual stress', region=region, 
    distributionType=UNIFORM, sigma11=0.0, sigma22=-sig_pc, sigma12=0.0, 
    sigma33=None, sigma13=None, sigma23=None)
# Web junctions residual stresses (tensile)
a = mdb.models['Model-1'].rootAssembly
region = a.sets['Set-Web_junctions']
mdb.models['Model-1'].Stress(name='Web junctions residual stress', region=region, 
    distributionType=UNIFORM, sigma11=sigy_r*sqrt(float(235)/sigy_r), sigma22=0.0, sigma12=0.0, 
    sigma33=None, sigma13=None, sigma23=None)
# Web middle residual stresses (compressive)
# find compressive web residual stresses by equilibrium with tensile web residual stresses
hw_t = hw/5-tf/2-tp/2 # Length of portions of web under tensile residual stress
hw_c = hw - hw_t  # Length of portions of web under compressive residual stress
sig_wc = sigy_r * sqrt(float(235)/sigy_r) * hw_t / hw_c
a = mdb.models['Model-1'].rootAssembly
region = a.sets['Set-Web_middle']
mdb.models['Model-1'].Stress(name='Web middle residual stress', region=region, 
    distributionType=UNIFORM, sigma11=-sig_wc, sigma22=0.0, sigma12=0.0, 
    sigma33=None, sigma13=None, sigma23=None)
	
# Mesh instance
# Flange (top)
a = mdb.models['Model-1'].rootAssembly
e1 = a.instances['T-1'].edges
pickedEdges = e1.getSequenceFromMask(mask=('[#7e000ff0 #f80 ]', ), )
a.seedEdgeBySize(edges=pickedEdges, size=tf/4, deviationFactor=0.1, 
    constraint=FINER)
# Plate (bottom)
a = mdb.models['Model-1'].rootAssembly
e1 = a.instances['T-1'].edges
pickedEdges = e1.getSequenceFromMask(mask=('[#81c0f00f #301f ]', ), )
a.seedEdgeBySize(edges=pickedEdges, size=tp/4, deviationFactor=0.1, 
    constraint=FINER)
# Web
a = mdb.models['Model-1'].rootAssembly
e1 = a.instances['T-1'].edges
pickedEdges = e1.getSequenceFromMask(mask=('[#3f0044 #60 ]', ), )
a.seedEdgeBySize(edges=pickedEdges, size=tw/4, deviationFactor=0.1, 
    constraint=FINER)
# Mesh	
a = mdb.models['Model-1'].rootAssembly
partInstances =(a.instances['T-1'], )
a.generateMesh(regions=partInstances)
	
# Create relaxation step
mdb.models['Model-1'].StaticStep(name='Step-Equilibrium', previous='Initial', 
    nlgeom=ON, description='Equilibrium after introduction of residual stresses')

# create loading step
mdb.models['Model-1'].StaticStep(name='Step-Static, General', 
    previous='Step-Equilibrium', maxNumInc=max_num_inc , initialInc=ini_inc, maxInc=max_inc, 
    nlgeom=ON, timePeriod=max_time, minInc=1e-08,
	stabilizationMagnitude=0.0001, stabilizationMethod=DISSIPATED_ENERGY_FRACTION,
	continueDampingFactors=False, adaptiveDampingRatio=0.01)

# loading
mdb.models['Model-1'].boundaryConditions['BC-Rotated_end'].setValuesInStep(
    stepName='Step-Static, General', ur1=Tpl*30)

# history output requests
regionDef=mdb.models['Model-1'].rootAssembly.sets['RP1']
mdb.models['Model-1'].HistoryOutputRequest(
    name='H-Output-Rotated_end', 
    createStepName='Step-Equilibrium', variables=('U2', 'U3', 'UR1', 'RF1', 
    'RF2', 'RF3', 'RM1', 'RM2', 'RM3', 'TF1', 'TF2', 'TF3', 'TM1', 'TM2', 
    'TM3'), region=regionDef, sectionPoints=DEFAULT, rebar=EXCLUDE)
regionDef=mdb.models['Model-1'].rootAssembly.sets['RP1']
mdb.models['Model-1'].historyOutputRequests['H-Output-Rotated_end'].setValuesInStep(
    stepName='Step-Static, General', variables=('U2', 'U3', 'UR1', 'RF1', 
    'RF2', 'RF3', 'RM1', 'RM2', 'RM3', 'TF1', 'TF2', 'TF3', 'TM1', 'TM2', 
    'TM3', 'VF1', 'VF2', 'VF3', 'VM1', 'VM2', 'VM3'))

# field output requests
mdb.models['Model-1'].FieldOutputRequest(name='F-Output-2', 
    createStepName='Step-Equilibrium', variables=PRESELECT)

# Mesh perturbation
mdb.models['Model-1'].keywordBlock.synchVersions(storeNodesAndElements=False)
mdb.models['Model-1'].keywordBlock.insert(49, """
*IMPERFECTION, FILE={LBA_name}, STEP=1
1,{imperfection:5.2e}""")

# Defining job
mdb.Job(name=job_name, model='Model-1', type=ANALYSIS, explicitPrecision=SINGLE, 
    nodalOutputPrecision=FULL, description='', 
    parallelizationMethodExplicit=DOMAIN, multiprocessingMode=MPI, 
    numDomains=16, userSubroutine='', numCpus=16, memory=90, 
    memoryUnits=PERCENTAGE, scratch='', echoPrint=OFF, modelPrint=OFF, 
    contactPrint=OFF, historyPrint=OFF)
mdb.jobs[job_name].writeInput(consistencyChecking=OFF)
'''
    with  open(job_name + '.py','w') as fid:
        fid.write(template.format(**context))

def gen_inp_py_file_exp_mag_SS_Sun(YoTf,Yf,YoTw,Yw,mag,tf,tw,Bf,H,LdLT,E,material_name_f,material_name_w):
    """
    Same as gen_inp_py_file_exp, except that magnitude of imperfection is directly
    specified, and the material stress strain curve is specified. Note YoT input
    arguments are not used.
    """
    job_name = 'Yf_' + "{Yf:.0f}".format(Yf=Yf) + '_YoTf_' + "{YoTf:.2f}".format(YoTf=YoTf) + \
        '_Yw_' + "{Yw:.0f}".format(Yw=Yw) + '_YoTw_' + "{YoTw:.2f}".format(YoTw=YoTw) + '_tf_' + \
        "{tf:.1f}".format(tf=tf) + '_tw_' + \
            "{tw:.1f}".format(tw=tw) + '_Bf_' + \
                "{Bf:.1f}".format(Bf=Bf) + '_H_' + \
                    "{H:.1f}".format(H=H) + '_LdLT_' + \
                        "{LdLT:.2f}".format(LdLT=LdLT)
    job_name = job_name.replace('.','p')
    
    LBA_name = 'LBA_' + \
        'Yf_' + "{Yf:.0f}".format(Yf=Yf) + \
        '_Yw_' + "{Yw:.0f}".format(Yw=Yw) + '_tf_' + \
        "{tf:.1f}".format(tf=tf) + '_tw_' + \
            "{tw:.1f}".format(tw=tw) + '_Bf_' + \
                "{Bf:.1f}".format(Bf=Bf) + '_H_' + \
                    "{H:.1f}".format(H=H) + '_LdLT_' + \
                        "{LdLT:.2f}".format(LdLT=LdLT)
    LBA_name = LBA_name.replace('.','p')
    
    L = calc_L(Yf,Yw,tf,tw,Bf,H,LdLT)
    
    imperfection = mag
        
    context = {
        "job_name":job_name,
        "imperfection":imperfection,
        "Yw":Yw,
        "Yf":Yf,
        "tf":tf,
        "tw":tw,
        "Bf":Bf,
        "H":H,
        "L":L,
        "LBA_name":LBA_name,
        "material_name_f":material_name_f,
        "material_name_w":material_name_w,
        "E":E
        }
    template = '''from abaqus import *
from abaqusConstants import *
from math import *
from caeModules import *
from driverUtils import executeOnCaeStartup
import sketch
import part
import math
from numpy import loadtxt

executeOnCaeStartup()
Mdb()

job_name = '{job_name}' # ABAQUS model .inp file name
Yw = float({Yw}) # Web yield strength [MPa]
Yf = float({Yf}) # Flange yield strength [MPa]
L = {L:6.1f} # Full length

E = float({E}) # Young's modulus [MPa]

stress_strain_f = loadtxt('{material_name_f}'+'_SS.txt', comments="#", delimiter=",", skiprows=1, unpack=False)
stress_strain_w = loadtxt('{material_name_w}'+'_SS.txt', comments="#", delimiter=",", skiprows=1, unpack=False)

nu = 0.3
sigy_w = Yw
epsy_w = sigy_w/E
# sig_0p5_w = Esh*(0.5-epsy_w)+sigy_w

sigy_f = Yf
epsy_f = sigy_f/E
# sig_0p5_f = Esh*(0.5-epsy_f)+sigy_f

max_time = 1.0
max_num_inc = 1000
max_inc = 0.0025
ini_inc = 0.0025

tf = float({tf})
tp = tf
tw = float({tw})
H = float({H})
hw = H-2*tf
Bf = float({Bf})
Bp = Bf
bp = Bp/2 # Half the flange width
bf = Bf/2
a_weld = 0.53*tw # Weld leg

# x = (hw*tw+tf*bf-tp*bp)/(2*tw)
# Mpl = x*tp*bp*sig_y + 0.5*x**2*tw*sig_y + (hw-x)*tf*bf + 0.5*(hw-x)**2*tw*sig_y

Mpl = 2*(bf*tf*(hw+tf)*sigy_f+hw**2/8*tw*sigy_w)
# Mpl = 1.58816e6*sig_y
# F = 8*Mpl/L

J = tw*hw**3/12+bf*tf*(hw+tf)**2+bf*tf**3/3

Tpl = Mpl*L/E/J

# part
s1 = mdb.models['Model-1'].ConstrainedSketch(name='__profile__', 
    sheetSize=200.0)
g, v, d, c = s1.geometry, s1.vertices, s1.dimensions, s1.constraints
s1.setPrimaryObject(option=STANDALONE)
s1.Spot(point=(0.0, 0.0))
s1.Spot(point=(bp, 0.0))
s1.Spot(point=(-bp, 0.0))
s1.Spot(point=(0.0, hw))
s1.Spot(point=(-bf, hw))
s1.Spot(point=(bf, hw))
session.viewports['Viewport: 1'].view.setValues(nearPlane=470.722, 
    farPlane=798.82, width=1309.27, height=785.434, cameraPosition=(65.6886, 
    129.458, 634.771), cameraTarget=(65.6886, 129.458, 0))
s1.Line(point1=(-bf, hw), point2=(0.0, hw))
s1.HorizontalConstraint(entity=g[2], addUndoState=False)
s1.Line(point1=(0.0, hw), point2=(bf, hw))
s1.HorizontalConstraint(entity=g[3], addUndoState=False)
s1.ParallelConstraint(entity1=g[2], entity2=g[3], addUndoState=False)
s1.Line(point1=(0.0, hw), point2=(0.0, 0.0))
s1.VerticalConstraint(entity=g[4], addUndoState=False)
s1.Line(point1=(-bp, 0.0), point2=(0.0, 0.0))
s1.HorizontalConstraint(entity=g[5], addUndoState=False)
s1.Line(point1=(0.0, 0.0), point2=(bp, 0.0))
s1.HorizontalConstraint(entity=g[6], addUndoState=False)
s1.ParallelConstraint(entity1=g[5], entity2=g[6], addUndoState=False)
p = mdb.models['Model-1'].Part(name='T', dimensionality=THREE_D, 
    type=DEFORMABLE_BODY)
p = mdb.models['Model-1'].parts['T']
p.BaseShellExtrude(sketch=s1, depth=L)
s1.unsetPrimaryObject()
p = mdb.models['Model-1'].parts['T']
session.viewports['Viewport: 1'].setValues(displayedObject=p)
del mdb.models['Model-1'].sketches['__profile__']

# Create datum planes for partitioning for residual stresses
# datum planes for partitioning flanges
p = mdb.models['Model-1'].parts['T']
p.DatumPlaneByPrincipalPlane(principalPlane=YZPLANE, offset=-7*Bf/16)
p = mdb.models['Model-1'].parts['T']
p.DatumPlaneByPrincipalPlane(principalPlane=YZPLANE, offset=7*Bf/16)
p = mdb.models['Model-1'].parts['T']
Bf_m = min(tw+5*a_weld,Bf/5) # Length of middle portion in flange with tensile residual stress
Bp_m = Bf_m # For symmetric I-sections
p.DatumPlaneByPrincipalPlane(principalPlane=YZPLANE, offset=Bf_m/2)
p = mdb.models['Model-1'].parts['T']
p.DatumPlaneByPrincipalPlane(principalPlane=YZPLANE, offset=-Bf_m/2)
# datum planes for partitioning webs
p = mdb.models['Model-1'].parts['T']
p.DatumPlaneByPrincipalPlane(principalPlane=XZPLANE, offset=hw/10-tp/2)
p = mdb.models['Model-1'].parts['T']
p.DatumPlaneByPrincipalPlane(principalPlane=XZPLANE, offset=9*hw/10+tf/2)

# material
mdb.models['Model-1'].Material(name='Elastic-plastic-flange')
mdb.models['Model-1'].materials['Elastic-plastic-flange'].Elastic(table=((E, 
    nu), ))
mdb.models['Model-1'].materials['Elastic-plastic-flange'].Plastic(table=stress_strain_f)
mdb.models['Model-1'].Material(name='Elastic-plastic-web')
mdb.models['Model-1'].materials['Elastic-plastic-web'].Elastic(table=((E, 
    nu), ))
mdb.models['Model-1'].materials['Elastic-plastic-web'].Plastic(table=stress_strain_w)

# create sections
mdb.models['Model-1'].HomogeneousShellSection(name='Section-Plate', preIntegrate=OFF, 
    material='Elastic-plastic-flange', thicknessType=UNIFORM, thickness=tp, 
    thicknessField='', nodalThicknessField='', idealization=NO_IDEALIZATION, 
    poissonDefinition=DEFAULT, thicknessModulus=None, temperature=GRADIENT, 
    useDensity=OFF, integrationRule=SIMPSON, numIntPts=25)
mdb.models['Model-1'].HomogeneousShellSection(name='Section-Flange', 
    preIntegrate=OFF, material='Elastic-plastic-flange', thicknessType=UNIFORM, 
    thickness=tf, thicknessField='', nodalThicknessField='', 
    idealization=NO_IDEALIZATION, poissonDefinition=DEFAULT, 
    thicknessModulus=None, temperature=GRADIENT, useDensity=OFF, 
    integrationRule=SIMPSON, numIntPts=25)
mdb.models['Model-1'].HomogeneousShellSection(name='Section-Web', 
    preIntegrate=OFF, material='Elastic-plastic-web', thicknessType=UNIFORM, 
    thickness=tw, thicknessField='', nodalThicknessField='', 
    idealization=NO_IDEALIZATION, poissonDefinition=DEFAULT, 
    thicknessModulus=None, temperature=GRADIENT, useDensity=OFF, 
    integrationRule=SIMPSON, numIntPts=25)
	
# assign sections
p = mdb.models['Model-1'].parts['T']
f = p.faces
faces = f.getSequenceFromMask(mask=('[#c ]', ), )
region = regionToolset.Region(faces=faces)
p = mdb.models['Model-1'].parts['T']
p.SectionAssignment(region=region, sectionName='Section-Flange', offset=0.0, 
    offsetType=MIDDLE_SURFACE, offsetField='', 
    thicknessAssignment=FROM_SECTION)
p = mdb.models['Model-1'].parts['T']
f = p.faces
faces = f.getSequenceFromMask(mask=('[#2 ]', ), )
region = regionToolset.Region(faces=faces)
p = mdb.models['Model-1'].parts['T']
p.SectionAssignment(region=region, sectionName='Section-Web', offset=0.0, 
    offsetType=MIDDLE_SURFACE, offsetField='', 
    thicknessAssignment=FROM_SECTION)
p = mdb.models['Model-1'].parts['T']
f = p.faces
faces = f.getSequenceFromMask(mask=('[#11 ]', ), )
region = regionToolset.Region(faces=faces)
p = mdb.models['Model-1'].parts['T']
p.SectionAssignment(region=region, sectionName='Section-Plate', offset=0.0, 
    offsetType=MIDDLE_SURFACE, offsetField='', 
    thicknessAssignment=FROM_SECTION)
	
# make shell orientation correct (for both flanges, attachment side is considered 'bottom')
p = mdb.models['Model-1'].parts['T']
f = p.faces
faces = f.getSequenceFromMask(mask=('[#8 ]', ), )
regions = regionToolset.Region(faces=faces)
p.flipNormal(regions=regions)

# offset top flange
mdb.models['Model-1'].parts['T'].sectionAssignments[0].setValues(
    offsetType=BOTTOM_SURFACE, offsetField='', offset=0.0)
	
# offset bottom flange
mdb.models['Model-1'].parts['T'].sectionAssignments[2].setValues(
    offsetType=BOTTOM_SURFACE, offsetField='', offset=0.0)

# create instance
a = mdb.models['Model-1'].rootAssembly
a.DatumCsysByDefault(CARTESIAN)
p = mdb.models['Model-1'].parts['T']
a.Instance(name='T-1', part=p, dependent=OFF)

# Create sets for boundary conditions and loading
# Rotated end set
a = mdb.models['Model-1'].rootAssembly
e1 = a.instances['T-1'].edges
edges1 = e1.getSequenceFromMask(mask=('[#9244 ]', ), )
a.Set(edges=edges1, name='Set-Rotated_end')
# Fixed end set
a = mdb.models['Model-1'].rootAssembly
e1 = a.instances['T-1'].edges
edges1 = e1.getSequenceFromMask(mask=('[#2491 ]', ), )
a.Set(edges=edges1, name='Set-Fixed_end')
# Rotated point (RP1) set
RP1 = a.ReferencePoint(point=(0.0, hw/2, 0.0))
a = mdb.models['Model-1'].rootAssembly
r1 = a.referencePoints
refPoints1=(r1[RP1.id], )
a.Set(referencePoints=refPoints1, name='RP1')

# coupling for boundaries
a = mdb.models['Model-1'].rootAssembly
region1=a.sets['RP1']
a = mdb.models['Model-1'].rootAssembly
region2=a.sets['Set-Rotated_end']
mdb.models['Model-1'].Coupling(name='Constraint-Rotated_end', 
    controlPoint=region1, surface=region2, influenceRadius=WHOLE_SURFACE, 
    couplingType=KINEMATIC, localCsys=None, u1=ON, u2=ON, u3=ON, ur1=ON, 
    ur2=ON, ur3=ON)
	
# boundary conditions
a = mdb.models['Model-1'].rootAssembly
region = a.sets['Set-Fixed_end']
mdb.models['Model-1'].EncastreBC(name='BC-Fixed_end', createStepName='Initial', 
    region=region, localCsys=None)
a = mdb.models['Model-1'].rootAssembly
region = a.sets['RP1']
mdb.models['Model-1'].DisplacementBC(name='BC-Rotated_end', createStepName='Initial', 
    region=region, u1=SET, u2=UNSET, u3=UNSET, ur1=UNSET, ur2=SET, ur3=SET, 
    amplitude=UNSET, distributionType=UNIFORM, fieldName='', localCsys=None)

# Partitioning by datum planes for the assignment of residual stresses
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
pickedFaces = f1.getSequenceFromMask(mask=('[#18 ]', ), )
d21 = a.instances['T-1'].datums
a.PartitionFaceByDatumPlane(datumPlane=d21[2], faces=pickedFaces)
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
pickedFaces = f1.getSequenceFromMask(mask=('[#14 ]', ), )
d31 = a.instances['T-1'].datums
a.PartitionFaceByDatumPlane(datumPlane=d31[3], faces=pickedFaces)
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
pickedFaces = f1.getSequenceFromMask(mask=('[#20 ]', ), )
d21 = a.instances['T-1'].datums
a.PartitionFaceByDatumPlane(datumPlane=d21[7], faces=pickedFaces)
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
pickedFaces = f1.getSequenceFromMask(mask=('[#1 ]', ), )
d31 = a.instances['T-1'].datums
a.PartitionFaceByDatumPlane(datumPlane=d31[6], faces=pickedFaces)
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
pickedFaces = f1.getSequenceFromMask(mask=('[#600 ]', ), )
d21 = a.instances['T-1'].datums
a.PartitionFaceByDatumPlane(datumPlane=d21[5], faces=pickedFaces)
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
pickedFaces = f1.getSequenceFromMask(mask=('[#30 ]', ), )
d21 = a.instances['T-1'].datums
a.PartitionFaceByDatumPlane(datumPlane=d21[4], faces=pickedFaces)

# Creating sets for the assignment of residual stresses
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
faces1 = f1.getSequenceFromMask(mask=('[#1100 ]', ), )
a.Set(faces=faces1, name='Set-Flange_tips')
#: The set 'Set-Flange_tips' has been created (2 faces).
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
faces1 = f1.getSequenceFromMask(mask=('[#84 ]', ), )
a.Set(faces=faces1, name='Set-Flange_intermediates')
#: The set 'Set-Flange_intermediates' has been created (2 faces).
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
faces1 = f1.getSequenceFromMask(mask=('[#2002 ]', ), )
a.Set(faces=faces1, name='Set-Flange_middle')
#: The set 'Set-Flange_middle' has been created (2 faces).
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
faces1 = f1.getSequenceFromMask(mask=('[#600 ]', ), )
a.Set(faces=faces1, name='Set-Plate_tips')
#: The set 'Set-Plate_tips' has been created (2 faces).
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
faces1 = f1.getSequenceFromMask(mask=('[#48 ]', ), )
a.Set(faces=faces1, name='Set-Plate_intermediates')
#: The set 'Set-Plate_intermediates' has been created (2 faces).
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
faces1 = f1.getSequenceFromMask(mask=('[#4001 ]', ), )
a.Set(faces=faces1, name='Set-Plate_middle')
#: The set 'Set-Plate_middle' has been created (2 faces).
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
faces1 = f1.getSequenceFromMask(mask=('[#20 ]', ), )
a.Set(faces=faces1, name='Set-Web_middle')
#: The set 'Set-Web_middle' has been created (1 face).
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
faces1 = f1.getSequenceFromMask(mask=('[#810 ]', ), )
a.Set(faces=faces1, name='Set-Web_junctions')
#: The set 'Set-Web_junctions' has been created (2 faces).

# Introduce residual stresses
sigy_r = min(sigy_f,sigy_w) # reference residual stress
# Flange tips residual stresses (tensile)
a = mdb.models['Model-1'].rootAssembly
region = a.sets['Set-Flange_tips']
mdb.models['Model-1'].Stress(name='Flange tips residual stress', region=region, 
    distributionType=UNIFORM, sigma11=0.0, sigma22=0.5*sigy_r*sqrt(float(235)/sigy_r), sigma12=0.0, 
    sigma33=None, sigma13=None, sigma23=None)
	# Flange middle residual stresses (tensile)
a = mdb.models['Model-1'].rootAssembly
region = a.sets['Set-Flange_middle']
mdb.models['Model-1'].Stress(name='Flange middle residual stress', region=region, 
    distributionType=UNIFORM, sigma11=0.0, sigma22=sigy_r*sqrt(float(235)/sigy_r), sigma12=0.0, 
    sigma33=None, sigma13=None, sigma23=None)
# Flange intermediates residual stresses (compressive)
# find compressive flange residual stresses by equilibrium with tensile flange residual stresses
Bf_t = Bf/8 + Bf_m # Length of portions of flange under tensile residual stress
Bf_c = Bf - Bf_t  # Length of portions of flange under compressive residual stress
sig_fc = (sigy_r*sqrt(float(235)/sigy_r) * (0.5*Bf/8 + Bf_m)) / Bf_c
region = a.sets['Set-Flange_intermediates']
mdb.models['Model-1'].Stress(name='Flange intermediates residual stress', region=region, 
    distributionType=UNIFORM, sigma11=0.0, sigma22=-sig_fc, sigma12=0.0, 
    sigma33=None, sigma13=None, sigma23=None)
# Plate (bottom flange) tips residual stresses (tensile)
a = mdb.models['Model-1'].rootAssembly
region = a.sets['Set-Plate_tips']
mdb.models['Model-1'].Stress(name='Plate tips residual stress', region=region, 
    distributionType=UNIFORM, sigma11=0.0, sigma22=0.5*sigy_r*sqrt(float(235)/sigy_r), sigma12=0.0, 
    sigma33=None, sigma13=None, sigma23=None)
# Plate (bottom flange) middle residual stresses (tensile)
a = mdb.models['Model-1'].rootAssembly
region = a.sets['Set-Plate_middle']
mdb.models['Model-1'].Stress(name='Plate middle residual stress', region=region, 
    distributionType=UNIFORM, sigma11=0.0, sigma22=sigy_r*sqrt(float(235)/sigy_r), sigma12=0.0, 
    sigma33=None, sigma13=None, sigma23=None)
# Plate (bottom flange) intermediates residual stresses (compressive)
# find compressive flange residual stresses by equilibrium with tensile flange residual stresses
Bp_t = Bp/8 + Bp_m # Length of portions of flange under tensile residual stress
Bp_c = Bp - Bp_t  # Length of portions of flange under compressive residual stress
sig_pc = (sigy_r*sqrt(float(235)/sigy_r) * (0.5*Bp/8 + Bp_m)) / Bp_c
region = a.sets['Set-Plate_intermediates']
mdb.models['Model-1'].Stress(name='Plate intermediates residual stress', region=region, 
    distributionType=UNIFORM, sigma11=0.0, sigma22=-sig_pc, sigma12=0.0, 
    sigma33=None, sigma13=None, sigma23=None)
# Web junctions residual stresses (tensile)
a = mdb.models['Model-1'].rootAssembly
region = a.sets['Set-Web_junctions']
mdb.models['Model-1'].Stress(name='Web junctions residual stress', region=region, 
    distributionType=UNIFORM, sigma11=sigy_r*sqrt(float(235)/sigy_r), sigma22=0.0, sigma12=0.0, 
    sigma33=None, sigma13=None, sigma23=None)
# Web middle residual stresses (compressive)
# find compressive web residual stresses by equilibrium with tensile web residual stresses
hw_t = hw/5-tf/2-tp/2 # Length of portions of web under tensile residual stress
hw_c = hw - hw_t  # Length of portions of web under compressive residual stress
sig_wc = sigy_r * sqrt(float(235)/sigy_r) * hw_t / hw_c
a = mdb.models['Model-1'].rootAssembly
region = a.sets['Set-Web_middle']
mdb.models['Model-1'].Stress(name='Web middle residual stress', region=region, 
    distributionType=UNIFORM, sigma11=-sig_wc, sigma22=0.0, sigma12=0.0, 
    sigma33=None, sigma13=None, sigma23=None)
	
# Mesh instance
# Flange (top)
a = mdb.models['Model-1'].rootAssembly
e1 = a.instances['T-1'].edges
pickedEdges = e1.getSequenceFromMask(mask=('[#7e000ff0 #f80 ]', ), )
a.seedEdgeBySize(edges=pickedEdges, size=tf/4, deviationFactor=0.1, 
    constraint=FINER)
# Plate (bottom)
a = mdb.models['Model-1'].rootAssembly
e1 = a.instances['T-1'].edges
pickedEdges = e1.getSequenceFromMask(mask=('[#81c0f00f #301f ]', ), )
a.seedEdgeBySize(edges=pickedEdges, size=tp/4, deviationFactor=0.1, 
    constraint=FINER)
# Web
a = mdb.models['Model-1'].rootAssembly
e1 = a.instances['T-1'].edges
pickedEdges = e1.getSequenceFromMask(mask=('[#3f0044 #60 ]', ), )
a.seedEdgeBySize(edges=pickedEdges, size=tw/4, deviationFactor=0.1, 
    constraint=FINER)
	
# Create relaxation step
mdb.models['Model-1'].StaticStep(name='Step-Equilibrium', previous='Initial', 
    nlgeom=ON, description='Equilibrium after introduction of residual stresses')

# create loading step
mdb.models['Model-1'].StaticStep(name='Step-Static, General', 
    previous='Step-Equilibrium', maxNumInc=max_num_inc , initialInc=ini_inc, maxInc=max_inc, 
    nlgeom=ON, timePeriod=max_time, minInc=1e-08,
	stabilizationMagnitude=0.0001, stabilizationMethod=DISSIPATED_ENERGY_FRACTION,
	continueDampingFactors=False, adaptiveDampingRatio=0.01)

# loading
mdb.models['Model-1'].boundaryConditions['BC-Rotated_end'].setValuesInStep(
    stepName='Step-Static, General', ur1=Tpl*30)

# history output requests
regionDef=mdb.models['Model-1'].rootAssembly.sets['RP1']
mdb.models['Model-1'].HistoryOutputRequest(
    name='H-Output-Rotated_end', 
    createStepName='Step-Equilibrium', variables=('U2', 'U3', 'UR1', 'RF1', 
    'RF2', 'RF3', 'RM1', 'RM2', 'RM3', 'TF1', 'TF2', 'TF3', 'TM1', 'TM2', 
    'TM3'), region=regionDef, sectionPoints=DEFAULT, rebar=EXCLUDE)
regionDef=mdb.models['Model-1'].rootAssembly.sets['RP1']
mdb.models['Model-1'].historyOutputRequests['H-Output-Rotated_end'].setValuesInStep(
    stepName='Step-Static, General', variables=('U2', 'U3', 'UR1', 'RF1', 
    'RF2', 'RF3', 'RM1', 'RM2', 'RM3', 'TF1', 'TF2', 'TF3', 'TM1', 'TM2', 
    'TM3', 'VF1', 'VF2', 'VF3', 'VM1', 'VM2', 'VM3'))
# Create field history output for RP1's coords
a = mdb.models['Model-1'].rootAssembly
session.viewports['Viewport: 1'].setValues(displayedObject=a)
regionDef=mdb.models['Model-1'].rootAssembly.sets['RP1']
mdb.models['Model-1'].FieldOutputRequest(name='F-Output-RP', 
    createStepName='Step-Static, General', variables=('COORD', ), 
    region=regionDef, sectionPoints=DEFAULT, rebar=EXCLUDE)

# Create datum plane at Z = 50 mm
a = mdb.models['Model-1'].rootAssembly
a.DatumPlaneByPrincipalPlane(principalPlane=XYPLANE, offset=50.0)
# Create datum plane at Z = L - 50 mm
a = mdb.models['Model-1'].rootAssembly
a.DatumPlaneByPrincipalPlane(principalPlane=XYPLANE, offset=L-50)
# Create datum plane at Y = hw/2
a = mdb.models['Model-1'].rootAssembly
a.DatumPlaneByPrincipalPlane(principalPlane=XZPLANE, offset=hw/2)
# Create partition at Y = hw/2
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
pickedFaces = f1.getSequenceFromMask(mask=('[#20 ]', ), )
d21 = a.datums
a.PartitionFaceByDatumPlane(datumPlane=d21[27], faces=pickedFaces)
# Create partition at Z = L - 50 mm
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
pickedFaces = f1.getSequenceFromMask(mask=('[#ffff ]', ), )
d1 = a.datums
a.PartitionFaceByDatumPlane(datumPlane=d1[26], faces=pickedFaces)
# Create partition at Z = 50 mm
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
pickedFaces = f1.getSequenceFromMask(mask=('[#776daa80 ]', ), )
d21 = a.datums
a.PartitionFaceByDatumPlane(datumPlane=d21[25], faces=pickedFaces)
# Create datum plane at Z = L/2
a = mdb.models['Model-1'].rootAssembly
a.DatumPlaneByPrincipalPlane(principalPlane=XYPLANE, offset=L/2)
# Partition edges at Z = L/2
a = mdb.models['Model-1'].rootAssembly
e1 = a.instances['T-1'].edges
pickedEdges = e1.getSequenceFromMask(mask=('[#850410a0 #14020 #30002140 #24 ]', 
    ), )
d1 = a.datums
a.PartitionEdgeByDatumPlane(datumPlane=d1[31], edges=pickedEdges)

# Create reference point at midspan and save it as Set-Midspan
a = mdb.models['Model-1'].rootAssembly
v1 = a.instances['T-1'].vertices
RP2 = a.ReferencePoint(point=v1[73])
a = mdb.models['Model-1'].rootAssembly
r1 = a.referencePoints
refPoints1=(r1[RP2.id], )
a.Set(referencePoints=refPoints1, name='Set-Midspan')
# Create kinematic coupling constraint between reference point and midspan point
a = mdb.models['Model-1'].rootAssembly
region1=a.sets['Set-Midspan']
a = mdb.models['Model-1'].rootAssembly
v1 = a.instances['T-1'].vertices
verts1 = v1.getSequenceFromMask(mask=('[#0:2 #200 ]', ), )
region2=regionToolset.Region(vertices=verts1)
mdb.models['Model-1'].Coupling(name='Constraint-Midspan', controlPoint=region1, 
    surface=region2, influenceRadius=WHOLE_SURFACE, couplingType=KINEMATIC, 
    localCsys=None, u1=ON, u2=ON, u3=ON, ur1=ON, ur2=ON, ur3=ON)
# Create history output request for Set-Midspan
regionDef=mdb.models['Model-1'].rootAssembly.sets['Set-Midspan']
mdb.models['Model-1'].HistoryOutputRequest(name='H-Output-Midspan', 
    createStepName='Step-Static, General', variables=('COOR1', 'COOR2', 
    'COOR3'), region=regionDef, sectionPoints=DEFAULT, rebar=EXCLUDE)
	
# Boundary conditions - lateral restraints
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
faces1 = f1.getSequenceFromMask(mask=('[#557f0000 #8892 ]', ), )
a.Set(faces=faces1, name='Set-Fixed_end_50mm')
#: The set 'Set-Fixed_end_50mm' has been created (16 faces).
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
faces1 = f1.getSequenceFromMask(mask=('[#a000b555 #2704 ]', ), )
a.Set(faces=faces1, name='Set-Rotated_end_50mm')
#: The set 'Set-Rotated_end_50mm' has been created (16 faces).
a = mdb.models['Model-1'].rootAssembly
region = a.sets['Set-Fixed_end_50mm']
mdb.models['Model-1'].DisplacementBC(name='BC-Fixed_end_50mm', 
    createStepName='Step-Static, General', region=region, u1=SET, u2=UNSET, u3=UNSET, 
    ur1=UNSET, ur2=UNSET, ur3=UNSET, amplitude=UNSET, distributionType=UNIFORM, 
    fieldName='', localCsys=None)
a = mdb.models['Model-1'].rootAssembly
region = a.sets['Set-Rotated_end_50mm']
mdb.models['Model-1'].DisplacementBC(name='BC-Rotated_end_50mm', 
    createStepName='Step-Static, General', region=region, u1=SET, u2=UNSET, u3=UNSET, 
    ur1=UNSET, ur2=UNSET, ur3=UNSET, amplitude=UNSET, distributionType=UNIFORM, 
    fieldName='', localCsys=None)
a = mdb.models['Model-1'].rootAssembly
region = a.sets['Set-Midspan']
mdb.models['Model-1'].DisplacementBC(name='BC-Midspan', 
    createStepName='Step-Static, General', region=region, u1=SET, u2=UNSET, u3=UNSET, 
    ur1=UNSET, ur2=UNSET, ur3=UNSET, amplitude=UNSET, distributionType=UNIFORM, 
    fieldName='', localCsys=None)

# field output requests
mdb.models['Model-1'].FieldOutputRequest(name='F-Output-2', 
    createStepName='Step-Equilibrium', variables=PRESELECT)

# Mesh	
a = mdb.models['Model-1'].rootAssembly
partInstances =(a.instances['T-1'], )
a.generateMesh(regions=partInstances)

# Mesh perturbation
mdb.models['Model-1'].keywordBlock.synchVersions(storeNodesAndElements=False)
mdb.models['Model-1'].keywordBlock.insert(60, """
*IMPERFECTION, FILE={LBA_name}, STEP=1
1,{imperfection:5.2e}""")

# Defining job
mdb.Job(name=job_name, model='Model-1', type=ANALYSIS, explicitPrecision=SINGLE, 
    nodalOutputPrecision=FULL, description='', 
    parallelizationMethodExplicit=DOMAIN, multiprocessingMode=MPI, 
    numDomains=16, userSubroutine='', numCpus=16, memory=90, 
    memoryUnits=PERCENTAGE, scratch='', echoPrint=OFF, modelPrint=OFF, 
    contactPrint=OFF, historyPrint=OFF)
mdb.jobs[job_name].writeInput(consistencyChecking=OFF)
'''
    with  open(job_name + '.py','w') as fid:
        fid.write(template.format(**context))

def gen_inp_py_file_exp_mag_SS_Lee(YoTf,Yf,YoTw,Yw,mag,tf,tw,Bf,H,LdLT,E,Lb2,material_name_f,material_name_w):
    """
    Same as gen_inp_py_file_exp, except that magnitude of imperfection is directly
    specified, and the material stress strain curve is specified. Note YoT input
    arguments are not used.
    """
    job_name = 'Yf_' + "{Yf:.0f}".format(Yf=Yf) + '_YoTf_' + "{YoTf:.2f}".format(YoTf=YoTf) + \
        '_Yw_' + "{Yw:.0f}".format(Yw=Yw) + '_YoTw_' + "{YoTw:.2f}".format(YoTw=YoTw) + '_tf_' + \
        "{tf:.1f}".format(tf=tf) + '_tw_' + \
            "{tw:.1f}".format(tw=tw) + '_Bf_' + \
                "{Bf:.1f}".format(Bf=Bf) + '_H_' + \
                    "{H:.1f}".format(H=H) + '_LdLT_' + \
                        "{LdLT:.2f}".format(LdLT=LdLT)
    job_name = job_name.replace('.','p')
    
    LBA_name = 'LBA_' + \
        'Yf_' + "{Yf:.0f}".format(Yf=Yf) + \
        '_Yw_' + "{Yw:.0f}".format(Yw=Yw) + '_tf_' + \
        "{tf:.1f}".format(tf=tf) + '_tw_' + \
            "{tw:.1f}".format(tw=tw) + '_Bf_' + \
                "{Bf:.1f}".format(Bf=Bf) + '_H_' + \
                    "{H:.1f}".format(H=H) + '_LdLT_' + \
                        "{LdLT:.2f}".format(LdLT=LdLT)
    LBA_name = LBA_name.replace('.','p')
    
    L = calc_L(Yf,Yw,tf,tw,Bf,H,LdLT)
    
    imperfection = mag
        
    context = {
        "job_name":job_name,
        "imperfection":imperfection,
        "Yw":Yw,
        "Yf":Yf,
        "tf":tf,
        "tw":tw,
        "Bf":Bf,
        "H":H,
        "L":L,
        "LBA_name":LBA_name,
        "material_name_f":material_name_f,
        "material_name_w":material_name_w,
        "E":E,
        "Lb2":Lb2
        }
    template = '''from abaqus import *
from abaqusConstants import *
from math import *
from caeModules import *
from driverUtils import executeOnCaeStartup
import sketch
import part
import math
from numpy import loadtxt

executeOnCaeStartup()
Mdb()

job_name = '{job_name}' # ABAQUS model .inp file name
Yw = float({Yw}) # Web yield strength [MPa]
Yf = float({Yf}) # Flange yield strength [MPa]
L = {L:6.1f} # Full length

E = float({E}) # Young's modulus [MPa]

stress_strain_f = loadtxt('{material_name_f}'+'_SS.txt', comments="#", delimiter=",", skiprows=1, unpack=False)
stress_strain_w = loadtxt('{material_name_w}'+'_SS.txt', comments="#", delimiter=",", skiprows=1, unpack=False)

nu = 0.3
sigy_w = Yw
epsy_w = sigy_w/E
# sig_0p5_w = Esh*(0.5-epsy_w)+sigy_w

sigy_f = Yf
epsy_f = sigy_f/E
# sig_0p5_f = Esh*(0.5-epsy_f)+sigy_f

max_time = 1.0
max_num_inc = 1000
max_inc = 0.0025
ini_inc = 0.0025

tf = float({tf})
tp = tf
tw = float({tw})
H = float({H})
hw = H-2*tf
Bf = float({Bf})
Bp = Bf
bp = Bp/2 # Half the flange width
bf = Bf/2
a_weld = 0.53*tw # Weld leg

Lb2 = float({Lb2}) # mm

# x = (hw*tw+tf*bf-tp*bp)/(2*tw)
# Mpl = x*tp*bp*sig_y + 0.5*x**2*tw*sig_y + (hw-x)*tf*bf + 0.5*(hw-x)**2*tw*sig_y

Mpl = 2*(bf*tf*(hw+tf)*sigy_f+hw**2/8*tw*sigy_w)
# Mpl = 1.58816e6*sig_y
# F = 8*Mpl/L

J = tw*hw**3/12+bf*tf*(hw+tf)**2+bf*tf**3/3

Tpl = Mpl*L/E/J

# part
s1 = mdb.models['Model-1'].ConstrainedSketch(name='__profile__', 
    sheetSize=200.0)
g, v, d, c = s1.geometry, s1.vertices, s1.dimensions, s1.constraints
s1.setPrimaryObject(option=STANDALONE)
s1.Spot(point=(0.0, 0.0))
s1.Spot(point=(bp, 0.0))
s1.Spot(point=(-bp, 0.0))
s1.Spot(point=(0.0, hw))
s1.Spot(point=(-bf, hw))
s1.Spot(point=(bf, hw))
session.viewports['Viewport: 1'].view.setValues(nearPlane=470.722, 
    farPlane=798.82, width=1309.27, height=785.434, cameraPosition=(65.6886, 
    129.458, 634.771), cameraTarget=(65.6886, 129.458, 0))
s1.Line(point1=(-bf, hw), point2=(0.0, hw))
s1.HorizontalConstraint(entity=g[2], addUndoState=False)
s1.Line(point1=(0.0, hw), point2=(bf, hw))
s1.HorizontalConstraint(entity=g[3], addUndoState=False)
s1.ParallelConstraint(entity1=g[2], entity2=g[3], addUndoState=False)
s1.Line(point1=(0.0, hw), point2=(0.0, 0.0))
s1.VerticalConstraint(entity=g[4], addUndoState=False)
s1.Line(point1=(-bp, 0.0), point2=(0.0, 0.0))
s1.HorizontalConstraint(entity=g[5], addUndoState=False)
s1.Line(point1=(0.0, 0.0), point2=(bp, 0.0))
s1.HorizontalConstraint(entity=g[6], addUndoState=False)
s1.ParallelConstraint(entity1=g[5], entity2=g[6], addUndoState=False)
p = mdb.models['Model-1'].Part(name='T', dimensionality=THREE_D, 
    type=DEFORMABLE_BODY)
p = mdb.models['Model-1'].parts['T']
p.BaseShellExtrude(sketch=s1, depth=L)
s1.unsetPrimaryObject()
p = mdb.models['Model-1'].parts['T']
session.viewports['Viewport: 1'].setValues(displayedObject=p)
del mdb.models['Model-1'].sketches['__profile__']

# Create datum planes for partitioning for residual stresses
# datum planes for partitioning flanges
p = mdb.models['Model-1'].parts['T']
p.DatumPlaneByPrincipalPlane(principalPlane=YZPLANE, offset=-7*Bf/16)
p = mdb.models['Model-1'].parts['T']
p.DatumPlaneByPrincipalPlane(principalPlane=YZPLANE, offset=7*Bf/16)
p = mdb.models['Model-1'].parts['T']
Bf_m = min(tw+5*a_weld,Bf/5) # Length of middle portion in flange with tensile residual stress
Bp_m = Bf_m # For symmetric I-sections
p.DatumPlaneByPrincipalPlane(principalPlane=YZPLANE, offset=Bf_m/2)
p = mdb.models['Model-1'].parts['T']
p.DatumPlaneByPrincipalPlane(principalPlane=YZPLANE, offset=-Bf_m/2)
# datum planes for partitioning webs
p = mdb.models['Model-1'].parts['T']
p.DatumPlaneByPrincipalPlane(principalPlane=XZPLANE, offset=hw/10-tp/2)
p = mdb.models['Model-1'].parts['T']
p.DatumPlaneByPrincipalPlane(principalPlane=XZPLANE, offset=9*hw/10+tf/2)

# material
mdb.models['Model-1'].Material(name='Elastic-plastic-flange')
mdb.models['Model-1'].materials['Elastic-plastic-flange'].Elastic(table=((E, 
    nu), ))
mdb.models['Model-1'].materials['Elastic-plastic-flange'].Plastic(table=stress_strain_f)
mdb.models['Model-1'].Material(name='Elastic-plastic-web')
mdb.models['Model-1'].materials['Elastic-plastic-web'].Elastic(table=((E, 
    nu), ))
mdb.models['Model-1'].materials['Elastic-plastic-web'].Plastic(table=stress_strain_w)

# create sections
mdb.models['Model-1'].HomogeneousShellSection(name='Section-Plate', preIntegrate=OFF, 
    material='Elastic-plastic-flange', thicknessType=UNIFORM, thickness=tp, 
    thicknessField='', nodalThicknessField='', idealization=NO_IDEALIZATION, 
    poissonDefinition=DEFAULT, thicknessModulus=None, temperature=GRADIENT, 
    useDensity=OFF, integrationRule=SIMPSON, numIntPts=25)
mdb.models['Model-1'].HomogeneousShellSection(name='Section-Flange', 
    preIntegrate=OFF, material='Elastic-plastic-flange', thicknessType=UNIFORM, 
    thickness=tf, thicknessField='', nodalThicknessField='', 
    idealization=NO_IDEALIZATION, poissonDefinition=DEFAULT, 
    thicknessModulus=None, temperature=GRADIENT, useDensity=OFF, 
    integrationRule=SIMPSON, numIntPts=25)
mdb.models['Model-1'].HomogeneousShellSection(name='Section-Web', 
    preIntegrate=OFF, material='Elastic-plastic-web', thicknessType=UNIFORM, 
    thickness=tw, thicknessField='', nodalThicknessField='', 
    idealization=NO_IDEALIZATION, poissonDefinition=DEFAULT, 
    thicknessModulus=None, temperature=GRADIENT, useDensity=OFF, 
    integrationRule=SIMPSON, numIntPts=25)
	
# assign sections
p = mdb.models['Model-1'].parts['T']
f = p.faces
faces = f.getSequenceFromMask(mask=('[#c ]', ), )
region = regionToolset.Region(faces=faces)
p = mdb.models['Model-1'].parts['T']
p.SectionAssignment(region=region, sectionName='Section-Flange', offset=0.0, 
    offsetType=MIDDLE_SURFACE, offsetField='', 
    thicknessAssignment=FROM_SECTION)
p = mdb.models['Model-1'].parts['T']
f = p.faces
faces = f.getSequenceFromMask(mask=('[#2 ]', ), )
region = regionToolset.Region(faces=faces)
p = mdb.models['Model-1'].parts['T']
p.SectionAssignment(region=region, sectionName='Section-Web', offset=0.0, 
    offsetType=MIDDLE_SURFACE, offsetField='', 
    thicknessAssignment=FROM_SECTION)
p = mdb.models['Model-1'].parts['T']
f = p.faces
faces = f.getSequenceFromMask(mask=('[#11 ]', ), )
region = regionToolset.Region(faces=faces)
p = mdb.models['Model-1'].parts['T']
p.SectionAssignment(region=region, sectionName='Section-Plate', offset=0.0, 
    offsetType=MIDDLE_SURFACE, offsetField='', 
    thicknessAssignment=FROM_SECTION)
	
# make shell orientation correct (for both flanges, attachment side is considered 'bottom')
p = mdb.models['Model-1'].parts['T']
f = p.faces
faces = f.getSequenceFromMask(mask=('[#8 ]', ), )
regions = regionToolset.Region(faces=faces)
p.flipNormal(regions=regions)

# offset top flange
mdb.models['Model-1'].parts['T'].sectionAssignments[0].setValues(
    offsetType=BOTTOM_SURFACE, offsetField='', offset=0.0)
	
# offset bottom flange
mdb.models['Model-1'].parts['T'].sectionAssignments[2].setValues(
    offsetType=BOTTOM_SURFACE, offsetField='', offset=0.0)

# create instance
a = mdb.models['Model-1'].rootAssembly
a.DatumCsysByDefault(CARTESIAN)
p = mdb.models['Model-1'].parts['T']
a.Instance(name='T-1', part=p, dependent=OFF)

# Create sets for boundary conditions and loading
# Rotated end set
a = mdb.models['Model-1'].rootAssembly
e1 = a.instances['T-1'].edges
edges1 = e1.getSequenceFromMask(mask=('[#9244 ]', ), )
a.Set(edges=edges1, name='Set-Rotated_end')
# Fixed end set
a = mdb.models['Model-1'].rootAssembly
e1 = a.instances['T-1'].edges
edges1 = e1.getSequenceFromMask(mask=('[#2491 ]', ), )
a.Set(edges=edges1, name='Set-Fixed_end')
# Rotated point (RP1) set
RP1 = a.ReferencePoint(point=(0.0, hw/2, 0.0))
a = mdb.models['Model-1'].rootAssembly
r1 = a.referencePoints
refPoints1=(r1[RP1.id], )
a.Set(referencePoints=refPoints1, name='RP1')

# coupling for boundaries
a = mdb.models['Model-1'].rootAssembly
region1=a.sets['RP1']
a = mdb.models['Model-1'].rootAssembly
region2=a.sets['Set-Rotated_end']
mdb.models['Model-1'].Coupling(name='Constraint-Rotated_end', 
    controlPoint=region1, surface=region2, influenceRadius=WHOLE_SURFACE, 
    couplingType=KINEMATIC, localCsys=None, u1=ON, u2=ON, u3=ON, ur1=ON, 
    ur2=ON, ur3=ON)
	
# boundary conditions
a = mdb.models['Model-1'].rootAssembly
region = a.sets['Set-Fixed_end']
mdb.models['Model-1'].EncastreBC(name='BC-Fixed_end', createStepName='Initial', 
    region=region, localCsys=None)
a = mdb.models['Model-1'].rootAssembly
region = a.sets['RP1']
mdb.models['Model-1'].DisplacementBC(name='BC-Rotated_end', createStepName='Initial', 
    region=region, u1=SET, u2=UNSET, u3=UNSET, ur1=UNSET, ur2=SET, ur3=SET, 
    amplitude=UNSET, distributionType=UNIFORM, fieldName='', localCsys=None)

# Partitioning by datum planes for the assignment of residual stresses
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
pickedFaces = f1.getSequenceFromMask(mask=('[#18 ]', ), )
d21 = a.instances['T-1'].datums
a.PartitionFaceByDatumPlane(datumPlane=d21[2], faces=pickedFaces)
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
pickedFaces = f1.getSequenceFromMask(mask=('[#14 ]', ), )
d31 = a.instances['T-1'].datums
a.PartitionFaceByDatumPlane(datumPlane=d31[3], faces=pickedFaces)
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
pickedFaces = f1.getSequenceFromMask(mask=('[#20 ]', ), )
d21 = a.instances['T-1'].datums
a.PartitionFaceByDatumPlane(datumPlane=d21[7], faces=pickedFaces)
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
pickedFaces = f1.getSequenceFromMask(mask=('[#1 ]', ), )
d31 = a.instances['T-1'].datums
a.PartitionFaceByDatumPlane(datumPlane=d31[6], faces=pickedFaces)
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
pickedFaces = f1.getSequenceFromMask(mask=('[#600 ]', ), )
d21 = a.instances['T-1'].datums
a.PartitionFaceByDatumPlane(datumPlane=d21[5], faces=pickedFaces)
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
pickedFaces = f1.getSequenceFromMask(mask=('[#30 ]', ), )
d21 = a.instances['T-1'].datums
a.PartitionFaceByDatumPlane(datumPlane=d21[4], faces=pickedFaces)

# Creating sets for the assignment of residual stresses
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
faces1 = f1.getSequenceFromMask(mask=('[#1100 ]', ), )
a.Set(faces=faces1, name='Set-Flange_tips')
#: The set 'Set-Flange_tips' has been created (2 faces).
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
faces1 = f1.getSequenceFromMask(mask=('[#84 ]', ), )
a.Set(faces=faces1, name='Set-Flange_intermediates')
#: The set 'Set-Flange_intermediates' has been created (2 faces).
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
faces1 = f1.getSequenceFromMask(mask=('[#2002 ]', ), )
a.Set(faces=faces1, name='Set-Flange_middle')
#: The set 'Set-Flange_middle' has been created (2 faces).
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
faces1 = f1.getSequenceFromMask(mask=('[#600 ]', ), )
a.Set(faces=faces1, name='Set-Plate_tips')
#: The set 'Set-Plate_tips' has been created (2 faces).
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
faces1 = f1.getSequenceFromMask(mask=('[#48 ]', ), )
a.Set(faces=faces1, name='Set-Plate_intermediates')
#: The set 'Set-Plate_intermediates' has been created (2 faces).
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
faces1 = f1.getSequenceFromMask(mask=('[#4001 ]', ), )
a.Set(faces=faces1, name='Set-Plate_middle')
#: The set 'Set-Plate_middle' has been created (2 faces).
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
faces1 = f1.getSequenceFromMask(mask=('[#20 ]', ), )
a.Set(faces=faces1, name='Set-Web_middle')
#: The set 'Set-Web_middle' has been created (1 face).
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
faces1 = f1.getSequenceFromMask(mask=('[#810 ]', ), )
a.Set(faces=faces1, name='Set-Web_junctions')
#: The set 'Set-Web_junctions' has been created (2 faces).

# Introduce residual stresses
sigy_r = min(sigy_f,sigy_w) # reference residual stress
# Flange tips residual stresses (tensile)
a = mdb.models['Model-1'].rootAssembly
region = a.sets['Set-Flange_tips']
mdb.models['Model-1'].Stress(name='Flange tips residual stress', region=region, 
    distributionType=UNIFORM, sigma11=0.0, sigma22=0.5*sigy_r*sqrt(float(235)/sigy_r), sigma12=0.0, 
    sigma33=None, sigma13=None, sigma23=None)
	# Flange middle residual stresses (tensile)
a = mdb.models['Model-1'].rootAssembly
region = a.sets['Set-Flange_middle']
mdb.models['Model-1'].Stress(name='Flange middle residual stress', region=region, 
    distributionType=UNIFORM, sigma11=0.0, sigma22=sigy_r*sqrt(float(235)/sigy_r), sigma12=0.0, 
    sigma33=None, sigma13=None, sigma23=None)
# Flange intermediates residual stresses (compressive)
# find compressive flange residual stresses by equilibrium with tensile flange residual stresses
Bf_t = Bf/8 + Bf_m # Length of portions of flange under tensile residual stress
Bf_c = Bf - Bf_t  # Length of portions of flange under compressive residual stress
sig_fc = (sigy_r*sqrt(float(235)/sigy_r) * (0.5*Bf/8 + Bf_m)) / Bf_c
region = a.sets['Set-Flange_intermediates']
mdb.models['Model-1'].Stress(name='Flange intermediates residual stress', region=region, 
    distributionType=UNIFORM, sigma11=0.0, sigma22=-sig_fc, sigma12=0.0, 
    sigma33=None, sigma13=None, sigma23=None)
# Plate (bottom flange) tips residual stresses (tensile)
a = mdb.models['Model-1'].rootAssembly
region = a.sets['Set-Plate_tips']
mdb.models['Model-1'].Stress(name='Plate tips residual stress', region=region, 
    distributionType=UNIFORM, sigma11=0.0, sigma22=0.5*sigy_r*sqrt(float(235)/sigy_r), sigma12=0.0, 
    sigma33=None, sigma13=None, sigma23=None)
# Plate (bottom flange) middle residual stresses (tensile)
a = mdb.models['Model-1'].rootAssembly
region = a.sets['Set-Plate_middle']
mdb.models['Model-1'].Stress(name='Plate middle residual stress', region=region, 
    distributionType=UNIFORM, sigma11=0.0, sigma22=sigy_r*sqrt(float(235)/sigy_r), sigma12=0.0, 
    sigma33=None, sigma13=None, sigma23=None)
# Plate (bottom flange) intermediates residual stresses (compressive)
# find compressive flange residual stresses by equilibrium with tensile flange residual stresses
Bp_t = Bp/8 + Bp_m # Length of portions of flange under tensile residual stress
Bp_c = Bp - Bp_t  # Length of portions of flange under compressive residual stress
sig_pc = (sigy_r*sqrt(float(235)/sigy_r) * (0.5*Bp/8 + Bp_m)) / Bp_c
region = a.sets['Set-Plate_intermediates']
mdb.models['Model-1'].Stress(name='Plate intermediates residual stress', region=region, 
    distributionType=UNIFORM, sigma11=0.0, sigma22=-sig_pc, sigma12=0.0, 
    sigma33=None, sigma13=None, sigma23=None)
# Web junctions residual stresses (tensile)
a = mdb.models['Model-1'].rootAssembly
region = a.sets['Set-Web_junctions']
mdb.models['Model-1'].Stress(name='Web junctions residual stress', region=region, 
    distributionType=UNIFORM, sigma11=sigy_r*sqrt(float(235)/sigy_r), sigma22=0.0, sigma12=0.0, 
    sigma33=None, sigma13=None, sigma23=None)
# Web middle residual stresses (compressive)
# find compressive web residual stresses by equilibrium with tensile web residual stresses
hw_t = hw/5-tf/2-tp/2 # Length of portions of web under tensile residual stress
hw_c = hw - hw_t  # Length of portions of web under compressive residual stress
sig_wc = sigy_r * sqrt(float(235)/sigy_r) * hw_t / hw_c
a = mdb.models['Model-1'].rootAssembly
region = a.sets['Set-Web_middle']
mdb.models['Model-1'].Stress(name='Web middle residual stress', region=region, 
    distributionType=UNIFORM, sigma11=-sig_wc, sigma22=0.0, sigma12=0.0, 
    sigma33=None, sigma13=None, sigma23=None)
	
# Mesh instance
# Flange (top)
a = mdb.models['Model-1'].rootAssembly
e1 = a.instances['T-1'].edges
pickedEdges = e1.getSequenceFromMask(mask=('[#7e000ff0 #f80 ]', ), )
a.seedEdgeBySize(edges=pickedEdges, size=tf/4, deviationFactor=0.1, 
    constraint=FINER)
# Plate (bottom)
a = mdb.models['Model-1'].rootAssembly
e1 = a.instances['T-1'].edges
pickedEdges = e1.getSequenceFromMask(mask=('[#81c0f00f #301f ]', ), )
a.seedEdgeBySize(edges=pickedEdges, size=tp/4, deviationFactor=0.1, 
    constraint=FINER)
# Web
a = mdb.models['Model-1'].rootAssembly
e1 = a.instances['T-1'].edges
pickedEdges = e1.getSequenceFromMask(mask=('[#3f0044 #60 ]', ), )
a.seedEdgeBySize(edges=pickedEdges, size=tw/4, deviationFactor=0.1, 
    constraint=FINER)
	
# Create relaxation step
mdb.models['Model-1'].StaticStep(name='Step-Equilibrium', previous='Initial', 
    nlgeom=ON, description='Equilibrium after introduction of residual stresses')

# create loading step
mdb.models['Model-1'].StaticStep(name='Step-Static, General', 
    previous='Step-Equilibrium', maxNumInc=max_num_inc , initialInc=ini_inc, maxInc=max_inc, 
    nlgeom=ON, timePeriod=max_time, minInc=1e-08,
	stabilizationMagnitude=0.0001, stabilizationMethod=DISSIPATED_ENERGY_FRACTION,
	continueDampingFactors=False, adaptiveDampingRatio=0.01)

# loading
mdb.models['Model-1'].boundaryConditions['BC-Rotated_end'].setValuesInStep(
    stepName='Step-Static, General', ur1=Tpl*30)

# history output requests
regionDef=mdb.models['Model-1'].rootAssembly.sets['RP1']
mdb.models['Model-1'].HistoryOutputRequest(
    name='H-Output-Rotated_end', 
    createStepName='Step-Equilibrium', variables=('U2', 'U3', 'UR1', 'RF1', 
    'RF2', 'RF3', 'RM1', 'RM2', 'RM3', 'TF1', 'TF2', 'TF3', 'TM1', 'TM2', 
    'TM3'), region=regionDef, sectionPoints=DEFAULT, rebar=EXCLUDE)
regionDef=mdb.models['Model-1'].rootAssembly.sets['RP1']
mdb.models['Model-1'].historyOutputRequests['H-Output-Rotated_end'].setValuesInStep(
    stepName='Step-Static, General', variables=('U2', 'U3', 'UR1', 'RF1', 
    'RF2', 'RF3', 'RM1', 'RM2', 'RM3', 'TF1', 'TF2', 'TF3', 'TM1', 'TM2', 
    'TM3', 'VF1', 'VF2', 'VF3', 'VM1', 'VM2', 'VM3'))
# Create field history output for RP1's coords
a = mdb.models['Model-1'].rootAssembly
session.viewports['Viewport: 1'].setValues(displayedObject=a)
regionDef=mdb.models['Model-1'].rootAssembly.sets['RP1']
mdb.models['Model-1'].FieldOutputRequest(name='F-Output-RP', 
    createStepName='Step-Static, General', variables=('COORD', ), 
    region=regionDef, sectionPoints=DEFAULT, rebar=EXCLUDE)

# Create datum plane at Z = Lb2
a = mdb.models['Model-1'].rootAssembly
a.DatumPlaneByPrincipalPlane(principalPlane=XYPLANE, offset=Lb2)
# Create datum plane at Z = L - Lb2
a = mdb.models['Model-1'].rootAssembly
a.DatumPlaneByPrincipalPlane(principalPlane=XYPLANE, offset=L-Lb2)
# Create datum plane at Y = hw/2
a = mdb.models['Model-1'].rootAssembly
a.DatumPlaneByPrincipalPlane(principalPlane=XZPLANE, offset=hw/2)
# Create partition at Y = hw/2
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
pickedFaces = f1.getSequenceFromMask(mask=('[#20 ]', ), )
d21 = a.datums
a.PartitionFaceByDatumPlane(datumPlane=d21[27], faces=pickedFaces)
# Create partition at Z = L - Lb2
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
pickedFaces = f1.getSequenceFromMask(mask=('[#ffff ]', ), )
d1 = a.datums
a.PartitionFaceByDatumPlane(datumPlane=d1[26], faces=pickedFaces)
# Create partition at Z = Lb2
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
pickedFaces = f1.getSequenceFromMask(mask=('[#776daa80 ]', ), )
d21 = a.datums
a.PartitionFaceByDatumPlane(datumPlane=d21[25], faces=pickedFaces)
# Create datum plane at Z = L/2
a = mdb.models['Model-1'].rootAssembly
a.DatumPlaneByPrincipalPlane(principalPlane=XYPLANE, offset=L/2)
# Partition edges at Z = L/2
a = mdb.models['Model-1'].rootAssembly
e1 = a.instances['T-1'].edges
pickedEdges = e1.getSequenceFromMask(mask=('[#850410a0 #14020 #30002140 #24 ]', 
    ), )
d1 = a.datums
a.PartitionEdgeByDatumPlane(datumPlane=d1[31], edges=pickedEdges)

# Create reference point at midspan and save it as Set-Midspan
a = mdb.models['Model-1'].rootAssembly
v1 = a.instances['T-1'].vertices
RP2 = a.ReferencePoint(point=v1[73])
a = mdb.models['Model-1'].rootAssembly
r1 = a.referencePoints
refPoints1=(r1[RP2.id], )
a.Set(referencePoints=refPoints1, name='Set-Midspan')
# Create kinematic coupling constraint between reference point and midspan point
a = mdb.models['Model-1'].rootAssembly
region1=a.sets['Set-Midspan']
a = mdb.models['Model-1'].rootAssembly
v1 = a.instances['T-1'].vertices
verts1 = v1.getSequenceFromMask(mask=('[#0:2 #200 ]', ), )
region2=regionToolset.Region(vertices=verts1)
mdb.models['Model-1'].Coupling(name='Constraint-Midspan', controlPoint=region1, 
    surface=region2, influenceRadius=WHOLE_SURFACE, couplingType=KINEMATIC, 
    localCsys=None, u1=ON, u2=ON, u3=ON, ur1=ON, ur2=ON, ur3=ON)
# Create history output request for Set-Midspan
regionDef=mdb.models['Model-1'].rootAssembly.sets['Set-Midspan']
mdb.models['Model-1'].HistoryOutputRequest(name='H-Output-Midspan', 
    createStepName='Step-Static, General', variables=('COOR1', 'COOR2', 
    'COOR3'), region=regionDef, sectionPoints=DEFAULT, rebar=EXCLUDE)

# Create reference point at midspan and save it as Set-Restraint_1
a = mdb.models['Model-1'].rootAssembly
v1 = a.instances['T-1'].vertices
RP3 = a.ReferencePoint(point=v1[27])
a = mdb.models['Model-1'].rootAssembly
r1 = a.referencePoints
refPoints1=(r1[RP3.id], )
a.Set(referencePoints=refPoints1, name='Set-Restraint_1')
# Create kinematic coupling constraint between reference point and midspan point
a = mdb.models['Model-1'].rootAssembly
region1=a.sets['Set-Restraint_1']
a = mdb.models['Model-1'].rootAssembly
v1 = a.instances['T-1'].vertices
verts1 = v1.getSequenceFromMask(mask=('[#8000000 ]', ), )
region2=regionToolset.Region(vertices=verts1)
mdb.models['Model-1'].Coupling(name='Constraint-Restraint_1', 
    controlPoint=region1, surface=region2, influenceRadius=WHOLE_SURFACE, 
    couplingType=KINEMATIC, localCsys=None, u1=ON, u2=ON, u3=ON, ur1=ON, 
    ur2=ON, ur3=ON)
	
# Create reference point at midspan and save it as Set-Restraint_2
a = mdb.models['Model-1'].rootAssembly
v1 = a.instances['T-1'].vertices
RP4 = a.ReferencePoint(point=v1[61])
a = mdb.models['Model-1'].rootAssembly
r1 = a.referencePoints
refPoints1=(r1[RP4.id], )
a.Set(referencePoints=refPoints1, name='Set-Restraint_2')
# Create kinematic coupling constraint between reference point and midspan point
a = mdb.models['Model-1'].rootAssembly
region1=a.sets['Set-Restraint_2']
a = mdb.models['Model-1'].rootAssembly
v1 = a.instances['T-1'].vertices
verts1 = v1.getSequenceFromMask(mask=('[#0 #20000000 ]', ), )
region2=regionToolset.Region(vertices=verts1)
mdb.models['Model-1'].Coupling(name='Constraint-Restraint_2', 
    controlPoint=region1, surface=region2, influenceRadius=WHOLE_SURFACE, 
    couplingType=KINEMATIC, localCsys=None, u1=ON, u2=ON, u3=ON, ur1=ON, 
    ur2=ON, ur3=ON)
	
# Boundary conditions - lateral restraints
a = mdb.models['Model-1'].rootAssembly
region = a.sets['Set-Midspan']
mdb.models['Model-1'].DisplacementBC(name='BC-Midspan', 
    createStepName='Step-Static, General', region=region, u1=SET, u2=UNSET, u3=UNSET, 
    ur1=UNSET, ur2=UNSET, ur3=UNSET, amplitude=UNSET, distributionType=UNIFORM, 
    fieldName='', localCsys=None)
a = mdb.models['Model-1'].rootAssembly
region = a.sets['Set-Restraint_1']
mdb.models['Model-1'].DisplacementBC(name='BC-Restraint_1', 
    createStepName='Step-Static, General', region=region, u1=0.0, u2=UNSET, 
    u3=UNSET, ur1=UNSET, ur2=UNSET, ur3=UNSET, amplitude=UNSET, fixed=OFF, 
    distributionType=UNIFORM, fieldName='', localCsys=None)
a = mdb.models['Model-1'].rootAssembly
region = a.sets['Set-Restraint_2']
mdb.models['Model-1'].DisplacementBC(name='BC-Restraint_2', 
    createStepName='Step-Static, General', region=region, u1=0.0, u2=UNSET, 
    u3=UNSET, ur1=UNSET, ur2=UNSET, ur3=UNSET, amplitude=UNSET, fixed=OFF, 
    distributionType=UNIFORM, fieldName='', localCsys=None)

# field output requests
mdb.models['Model-1'].FieldOutputRequest(name='F-Output-2', 
    createStepName='Step-Equilibrium', variables=PRESELECT)

# Mesh	
a = mdb.models['Model-1'].rootAssembly
partInstances =(a.instances['T-1'], )
a.generateMesh(regions=partInstances)

# Mesh perturbation
mdb.models['Model-1'].keywordBlock.synchVersions(storeNodesAndElements=False)
mdb.models['Model-1'].keywordBlock.insert(70, """
*IMPERFECTION, FILE={LBA_name}, STEP=1
1,{imperfection:5.2e}""")

# Defining job
mdb.Job(name=job_name, model='Model-1', type=ANALYSIS, explicitPrecision=SINGLE, 
    nodalOutputPrecision=FULL, description='', 
    parallelizationMethodExplicit=DOMAIN, multiprocessingMode=MPI, 
    numDomains=16, userSubroutine='', numCpus=16, memory=90, 
    memoryUnits=PERCENTAGE, scratch='', echoPrint=OFF, modelPrint=OFF, 
    contactPrint=OFF, historyPrint=OFF)
mdb.jobs[job_name].writeInput(consistencyChecking=OFF)
'''
    with  open(job_name + '.py','w') as fid:
        fid.write(template.format(**context))

def gen_postproc_py_file(YoT,Y,tf,tw,Bf,H,LdLT):
    
    # Parameters are used to generate the name so that the created
    # post-processing script script will access the correct odb file and 
    # save the output data to files with correct names
    
    job_name = 'Y_' + "{Y:.0f}".format(Y=Y) + '_YoT_' + "{YoT:.2f}".format(YoT=YoT) + '_tf_' + \
        "{tf:.1f}".format(tf=tf) + '_tw_' + \
            "{tw:.1f}".format(tw=tw) + '_Bf_' + \
                "{Bf:.1f}".format(Bf=Bf) + '_H_' + \
                    "{H:.1f}".format(H=H) + '_LdLT_' + \
                        "{LdLT:.2f}".format(LdLT=LdLT)
    job_name = job_name.replace('.','p')
    
    context = {
        "job_name":job_name
        }
    template = '''from odbAccess import *

job_name = '{job_name}'

odb = openOdb(job_name+'.odb', readOnly=True)

step = odb.steps['Step-Static, General']

# UR1
UR1 = step.historyRegions['Node ASSEMBLY.1'].historyOutputs['UR1'].data

# RM1
RM1 = step.historyRegions['Node ASSEMBLY.1'].historyOutputs['RM1'].data

# number of frames
m = len(step.frames)
fid1 = open (job_name + '_histories.csv', 'w')
fid1.write('UR1, RM1\\n')

# double precision data
for i in range(m): # loop through frames

	# write UR1 & RM1 to file
	fid1.write('%15.13E, %15.13E \\n'%(UR1[i][1],RM1[i][1]))
			
fid1.close()

'''
    with  open('extract_MR_' + job_name + '.py','w') as fid:
        fid.write(template.format(**context))

def gen_postproc_py_file_exp_Sun_Lee(YoTf,Yf,YoTw,Yw,tf,tw,Bf,H,LdLT):
    
    # Parameters are used to generate the name so that the created
    # post-processing script script will access the correct odb file and 
    # save the output data to files with correct names
    
    job_name = 'Yf_' + "{Yf:.0f}".format(Yf=Yf) + '_YoTf_' + "{YoTf:.2f}".format(YoTf=YoTf) + \
        '_Yw_' + "{Yw:.0f}".format(Yw=Yw) + '_YoTw_' + "{YoTw:.2f}".format(YoTw=YoTw) + '_tf_' + \
        "{tf:.1f}".format(tf=tf) + '_tw_' + \
            "{tw:.1f}".format(tw=tw) + '_Bf_' + \
                "{Bf:.1f}".format(Bf=Bf) + '_H_' + \
                    "{H:.1f}".format(H=H) + '_LdLT_' + \
                        "{LdLT:.2f}".format(LdLT=LdLT)
    job_name = job_name.replace('.','p')
    
    context = {
        "job_name":job_name
        }
    template = '''from odbAccess import *

job_name = '{job_name}'

odb = openOdb(job_name+'.odb', readOnly=True)

step = odb.steps['Step-Static, General']

# UR1
UR1 = step.historyRegions['Node ASSEMBLY.1'].historyOutputs['UR1'].data

# RM1
RM1 = step.historyRegions['Node ASSEMBLY.1'].historyOutputs['RM1'].data

# Midspan COOR1
MS_COOR1 = step.historyRegions['Node ASSEMBLY.2'].historyOutputs['COOR1'].data
# Midspan COOR2
MS_COOR2 = step.historyRegions['Node ASSEMBLY.2'].historyOutputs['COOR2'].data
# Midspan COOR3
MS_COOR3 = step.historyRegions['Node ASSEMBLY.2'].historyOutputs['COOR3'].data

# ALLSD
ALLSD = step.historyRegions['Assembly ASSEMBLY'].historyOutputs['ALLSD'].data

# ALLSE
ALLSE = step.historyRegions['Assembly ASSEMBLY'].historyOutputs['ALLSE'].data

# number of frames
m = len(step.frames)
fid1 = open (job_name + '_histories.csv', 'w')
fid1.write('UR1, RM1, MS_COOR1, MS_COOR2, MS_COOR3, RP_COOR1, RP_COOR2, RP_COOR3, ALLSD, ALLSE\\n')

# double precision data
for i in range(m): # loop through frames
	
	RP_COOR1_i = step.frames[i].fieldOutputs['COORD'].values[0].dataDouble[0]
	RP_COOR2_i = step.frames[i].fieldOutputs['COORD'].values[0].dataDouble[1]
	RP_COOR3_i = step.frames[i].fieldOutputs['COORD'].values[0].dataDouble[2]
	# write to file
	fid1.write('%15.13E, %15.13E, %15.13E, %15.13E, %15.13E, %15.13E, %15.13E, %15.13E, %15.13E, %15.13E \\n'%(UR1[i][1],RM1[i][1],
	MS_COOR1[i][1],MS_COOR2[i][1],MS_COOR3[i][1],
	RP_COOR1_i,RP_COOR2_i,RP_COOR3_i,
	ALLSD[i][1],ALLSE[i][1]))
			
fid1.close()

'''
    with  open('extract_MR_' + job_name + '.py','w') as fid:
        fid.write(template.format(**context))

def gen_metadata(YoT,Y,tf,tw,Bf,H,LdLT):
    """
    Save parameters associated with job to a file for eventual batch access
    """
    job_name = 'Y_' + "{Y:.0f}".format(Y=Y) + '_YoT_' + "{YoT:.2f}".format(YoT=YoT) + '_tf_' + \
        "{tf:.1f}".format(tf=tf) + '_tw_' + \
            "{tw:.1f}".format(tw=tw) + '_Bf_' + \
                "{Bf:.1f}".format(Bf=Bf) + '_H_' + \
                    "{H:.1f}".format(H=H) + '_LdLT_' + \
                        "{LdLT:.2f}".format(LdLT=LdLT)
    job_name = job_name.replace('.','p')
    
        
    LBA_name = 'LBA_Y_' + "{Y:.0f}".format(Y=Y) + '_tf_' + \
        "{tf:.1f}".format(tf=tf) + '_tw_' + \
            "{tw:.1f}".format(tw=tw) + '_Bf_' + \
                "{Bf:.1f}".format(Bf=Bf) + '_H_' + \
                    "{H:.1f}".format(H=H) + '_LdLT_' + \
                        "{LdLT:.2f}".format(LdLT=LdLT)
    LBA_name = LBA_name.replace('.','p')
                        
    eigenvalue_imp = np.loadtxt(LBA_name+'_out.txt', comments="#", delimiter=",", skiprows=1, unpack=False)
    
    Ldp = np.sqrt(1/eigenvalue_imp[0])
    imp = eigenvalue_imp[1]
    
    fid = open(job_name + '_meta.txt','w')
    fid.write('YoT, Y, imp, tf, tw, Bf, H, LdLT, Ldp \n')
    fid.write('%3.2f, %4.0f, %g, %4.1f, %4.1f, %6.1f, %6.1f, %.2f, %.3f'%(YoT,Y,imp,tf,tw,Bf,H,LdLT,Ldp))
    fid.close()
    
def gen_metadata_exp_mag(YoTf,Yf,YoTw,Yw,mag,tf,tw,Bf,H,LdLT,E):
    """
    Save parameters associated with job to a file for eventual batch access
    
    In a parametric batch run series, this function is to be called by function which
    generates the GMNIA py script, so that the metadata is generated upon the
    creation of the model-generation py script.
    """
    job_name = 'Yf_' + "{Yf:.0f}".format(Yf=Yf) + '_YoTf_' + "{YoTf:.2f}".format(YoTf=YoTf) + \
        '_Yw_' + "{Yw:.0f}".format(Yw=Yw) + '_YoTw_' + "{YoTw:.2f}".format(YoTw=YoTw) + '_tf_' + \
        "{tf:.1f}".format(tf=tf) + '_tw_' + \
            "{tw:.1f}".format(tw=tw) + '_Bf_' + \
                "{Bf:.1f}".format(Bf=Bf) + '_H_' + \
                    "{H:.1f}".format(H=H) + '_LdLT_' + \
                        "{LdLT:.2f}".format(LdLT=LdLT)
    job_name = job_name.replace('.','p')
    
    fid = open(job_name + '_meta.txt','w')
    fid.write('YoTf, Yf, YoTw, Yw, mag, tf, tw, Bf, H, LdLT, E \n')
    fid.write('%3.2f, %4.0f, %3.2f, %4.0f, %.3f, %4.1f, %4.1f, %6.1f, %6.1f, %.2f, %.4e'%(YoTf,Yf,YoTw,Yw,mag,tf,tw,Bf,H,LdLT,E))
    fid.close()

def gen_LBA_py_file(Y,tf,tw,Bf,H,LdLT):
    
    # Y affects L and is used to name the file, but does not affect LBA
    # as a material property
    # Y also used to calculate reference applied moment
    
    LBA_name = 'LBA_Y_' + "{Y:.0f}".format(Y=Y) + '_tf_' + \
        "{tf:.1f}".format(tf=tf) + '_tw_' + \
            "{tw:.1f}".format(tw=tw) + '_Bf_' + \
                "{Bf:.1f}".format(Bf=Bf) + '_H_' + \
                    "{H:.1f}".format(H=H) + '_LdLT_' + \
                        "{LdLT:.2f}".format(LdLT=LdLT)
    LBA_name = LBA_name.replace('.','p')
    
    L = calc_L(Y,Y,tf,tw,Bf,H,LdLT)
    
    Mel = calc_Mel(Y,Y,tf,tw,Bf,H)

    context = {
        "Y":Y,
        "tf":tf,
        "tw":tw,
        "Bf":Bf,
        "H":H,
        "L":L,
        "LBA_name":LBA_name,
        "Mel":Mel
        }
    
    template = '''from abaqus import *
from abaqusConstants import *
from math import *
from caeModules import *
from driverUtils import executeOnCaeStartup
import sketch
import part
import math
executeOnCaeStartup()
Mdb()

job_name = '{LBA_name}' # ABAQUS model .inp file name
L = {L:6.1f} # [mm] - Full length
E = float(2e5)
nu = 0.3

tf = float({tf})
tp = tf
tw = float({tw})
H = float({H})
hw = H-2*tf
Bf = float({Bf})
Bp = Bf
bp = Bp/2 # Half the flange width
bf = Bf/2
a_weld = 0.53*tw # Weld leg

# part
s1 = mdb.models['Model-1'].ConstrainedSketch(name='__profile__', 
    sheetSize=200.0)
g, v, d, c = s1.geometry, s1.vertices, s1.dimensions, s1.constraints
s1.setPrimaryObject(option=STANDALONE)
s1.Spot(point=(0.0, 0.0))
s1.Spot(point=(bp, 0.0))
s1.Spot(point=(-bp, 0.0))
s1.Spot(point=(0.0, hw))
s1.Spot(point=(-bf, hw))
s1.Spot(point=(bf, hw))
session.viewports['Viewport: 1'].view.setValues(nearPlane=470.722, 
    farPlane=798.82, width=1309.27, height=785.434, cameraPosition=(65.6886, 
    129.458, 634.771), cameraTarget=(65.6886, 129.458, 0))
s1.Line(point1=(-bf, hw), point2=(0.0, hw))
s1.HorizontalConstraint(entity=g[2], addUndoState=False)
s1.Line(point1=(0.0, hw), point2=(bf, hw))
s1.HorizontalConstraint(entity=g[3], addUndoState=False)
s1.ParallelConstraint(entity1=g[2], entity2=g[3], addUndoState=False)
s1.Line(point1=(0.0, hw), point2=(0.0, 0.0))
s1.VerticalConstraint(entity=g[4], addUndoState=False)
s1.Line(point1=(-bp, 0.0), point2=(0.0, 0.0))
s1.HorizontalConstraint(entity=g[5], addUndoState=False)
s1.Line(point1=(0.0, 0.0), point2=(bp, 0.0))
s1.HorizontalConstraint(entity=g[6], addUndoState=False)
s1.ParallelConstraint(entity1=g[5], entity2=g[6], addUndoState=False)
p = mdb.models['Model-1'].Part(name='T', dimensionality=THREE_D, 
    type=DEFORMABLE_BODY)
p = mdb.models['Model-1'].parts['T']
p.BaseShellExtrude(sketch=s1, depth=L)
s1.unsetPrimaryObject()
p = mdb.models['Model-1'].parts['T']
session.viewports['Viewport: 1'].setValues(displayedObject=p)
del mdb.models['Model-1'].sketches['__profile__']

# Create datum planes for partitioning for residual stresses
# datum planes for partitioning flanges
p = mdb.models['Model-1'].parts['T']
p.DatumPlaneByPrincipalPlane(principalPlane=YZPLANE, offset=-7*Bf/16)
p = mdb.models['Model-1'].parts['T']
p.DatumPlaneByPrincipalPlane(principalPlane=YZPLANE, offset=7*Bf/16)
p = mdb.models['Model-1'].parts['T']
Bf_m = min(tw+5*a_weld,Bf/5) # Length of middle portion in flange with tensile residual stress
Bp_m = Bf_m # For symmetric I-sections
p.DatumPlaneByPrincipalPlane(principalPlane=YZPLANE, offset=Bf_m/2)
p = mdb.models['Model-1'].parts['T']
p.DatumPlaneByPrincipalPlane(principalPlane=YZPLANE, offset=-Bf_m/2)
# datum planes for partitioning webs
p = mdb.models['Model-1'].parts['T']
p.DatumPlaneByPrincipalPlane(principalPlane=XZPLANE, offset=hw/10-tp/2)
p = mdb.models['Model-1'].parts['T']
p.DatumPlaneByPrincipalPlane(principalPlane=XZPLANE, offset=9*hw/10+tf/2)

# material
mdb.models['Model-1'].Material(name='Elastic-plastic-flange')
mdb.models['Model-1'].materials['Elastic-plastic-flange'].Elastic(table=((E, 
    nu), ))
mdb.models['Model-1'].Material(name='Elastic-plastic-web')
mdb.models['Model-1'].materials['Elastic-plastic-web'].Elastic(table=((E, 
    nu), ))

# create sections
mdb.models['Model-1'].HomogeneousShellSection(name='Section-Plate', preIntegrate=OFF, 
    material='Elastic-plastic-flange', thicknessType=UNIFORM, thickness=tp, 
    thicknessField='', nodalThicknessField='', idealization=NO_IDEALIZATION, 
    poissonDefinition=DEFAULT, thicknessModulus=None, temperature=GRADIENT, 
    useDensity=OFF, integrationRule=SIMPSON, numIntPts=25)
mdb.models['Model-1'].HomogeneousShellSection(name='Section-Flange', 
    preIntegrate=OFF, material='Elastic-plastic-flange', thicknessType=UNIFORM, 
    thickness=tf, thicknessField='', nodalThicknessField='', 
    idealization=NO_IDEALIZATION, poissonDefinition=DEFAULT, 
    thicknessModulus=None, temperature=GRADIENT, useDensity=OFF, 
    integrationRule=SIMPSON, numIntPts=25)
mdb.models['Model-1'].HomogeneousShellSection(name='Section-Web', 
    preIntegrate=OFF, material='Elastic-plastic-web', thicknessType=UNIFORM, 
    thickness=tw, thicknessField='', nodalThicknessField='', 
    idealization=NO_IDEALIZATION, poissonDefinition=DEFAULT, 
    thicknessModulus=None, temperature=GRADIENT, useDensity=OFF, 
    integrationRule=SIMPSON, numIntPts=25)
	
# assign sections
p = mdb.models['Model-1'].parts['T']
f = p.faces
faces = f.getSequenceFromMask(mask=('[#c ]', ), )
region = regionToolset.Region(faces=faces)
p = mdb.models['Model-1'].parts['T']
p.SectionAssignment(region=region, sectionName='Section-Flange', offset=0.0, 
    offsetType=MIDDLE_SURFACE, offsetField='', 
    thicknessAssignment=FROM_SECTION)
p = mdb.models['Model-1'].parts['T']
f = p.faces
faces = f.getSequenceFromMask(mask=('[#2 ]', ), )
region = regionToolset.Region(faces=faces)
p = mdb.models['Model-1'].parts['T']
p.SectionAssignment(region=region, sectionName='Section-Web', offset=0.0, 
    offsetType=MIDDLE_SURFACE, offsetField='', 
    thicknessAssignment=FROM_SECTION)
p = mdb.models['Model-1'].parts['T']
f = p.faces
faces = f.getSequenceFromMask(mask=('[#11 ]', ), )
region = regionToolset.Region(faces=faces)
p = mdb.models['Model-1'].parts['T']
p.SectionAssignment(region=region, sectionName='Section-Plate', offset=0.0, 
    offsetType=MIDDLE_SURFACE, offsetField='', 
    thicknessAssignment=FROM_SECTION)
	
# make shell orientation correct (for both flanges, attachment side is considered 'bottom')
p = mdb.models['Model-1'].parts['T']
f = p.faces
faces = f.getSequenceFromMask(mask=('[#8 ]', ), )
regions = regionToolset.Region(faces=faces)
p.flipNormal(regions=regions)

# offset top flange
mdb.models['Model-1'].parts['T'].sectionAssignments[0].setValues(
    offsetType=BOTTOM_SURFACE, offsetField='', offset=0.0)
	
# offset bottom flange
mdb.models['Model-1'].parts['T'].sectionAssignments[2].setValues(
    offsetType=BOTTOM_SURFACE, offsetField='', offset=0.0)

# create instance
a = mdb.models['Model-1'].rootAssembly
a.DatumCsysByDefault(CARTESIAN)
p = mdb.models['Model-1'].parts['T']
a.Instance(name='T-1', part=p, dependent=OFF)

# reference points and sets for coupling at boundaries
RP1 = a.ReferencePoint(point=(0.0, hw/2, 0.0))
a = mdb.models['Model-1'].rootAssembly
r1 = a.referencePoints
refPoints1=(r1[RP1.id], )
a.Set(referencePoints=refPoints1, name='RP1')
a = mdb.models['Model-1'].rootAssembly
e1 = a.instances['T-1'].edges
edges1 = e1.getSequenceFromMask(mask=('[#9244 ]', ), )
a.Set(edges=edges1, name='Set-Rotated_end')
a = mdb.models['Model-1'].rootAssembly
e1 = a.instances['T-1'].edges
edges1 = e1.getSequenceFromMask(mask=('[#2491 ]', ), )
a.Set(edges=edges1, name='Set-Fixed_end')

# coupling for boundaries
a = mdb.models['Model-1'].rootAssembly
region1=a.sets['RP1']
a = mdb.models['Model-1'].rootAssembly
region2=a.sets['Set-Rotated_end']
mdb.models['Model-1'].Coupling(name='Constraint-Rotated_end', 
    controlPoint=region1, surface=region2, influenceRadius=WHOLE_SURFACE, 
    couplingType=KINEMATIC, localCsys=None, u1=ON, u2=ON, u3=ON, ur1=ON, 
    ur2=ON, ur3=ON)
	
# boundary conditions
a = mdb.models['Model-1'].rootAssembly
region = a.sets['Set-Fixed_end']
mdb.models['Model-1'].EncastreBC(name='BC-Fixed_end', createStepName='Initial', 
    region=region, localCsys=None)
a = mdb.models['Model-1'].rootAssembly
region = a.sets['RP1']
mdb.models['Model-1'].DisplacementBC(name='BC-Rotated_end', createStepName='Initial', 
    region=region, u1=SET, u2=UNSET, u3=UNSET, ur1=UNSET, ur2=SET, ur3=SET, 
    amplitude=UNSET, distributionType=UNIFORM, fieldName='', localCsys=None)

# Partitioning by datum planes for the assignment of residual stresses
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
pickedFaces = f1.getSequenceFromMask(mask=('[#18 ]', ), )
d21 = a.instances['T-1'].datums
a.PartitionFaceByDatumPlane(datumPlane=d21[2], faces=pickedFaces)
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
pickedFaces = f1.getSequenceFromMask(mask=('[#14 ]', ), )
d31 = a.instances['T-1'].datums
a.PartitionFaceByDatumPlane(datumPlane=d31[3], faces=pickedFaces)
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
pickedFaces = f1.getSequenceFromMask(mask=('[#20 ]', ), )
d21 = a.instances['T-1'].datums
a.PartitionFaceByDatumPlane(datumPlane=d21[7], faces=pickedFaces)
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
pickedFaces = f1.getSequenceFromMask(mask=('[#1 ]', ), )
d31 = a.instances['T-1'].datums
a.PartitionFaceByDatumPlane(datumPlane=d31[6], faces=pickedFaces)
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
pickedFaces = f1.getSequenceFromMask(mask=('[#600 ]', ), )
d21 = a.instances['T-1'].datums
a.PartitionFaceByDatumPlane(datumPlane=d21[5], faces=pickedFaces)
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
pickedFaces = f1.getSequenceFromMask(mask=('[#30 ]', ), )
d21 = a.instances['T-1'].datums
a.PartitionFaceByDatumPlane(datumPlane=d21[4], faces=pickedFaces)

# Creating sets for the assignment of residual stresses
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
faces1 = f1.getSequenceFromMask(mask=('[#1100 ]', ), )
a.Set(faces=faces1, name='Set-Flange_tips')
#: The set 'Set-Flange_tips' has been created (2 faces).
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
faces1 = f1.getSequenceFromMask(mask=('[#84 ]', ), )
a.Set(faces=faces1, name='Set-Flange_intermediates')
#: The set 'Set-Flange_intermediates' has been created (2 faces).
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
faces1 = f1.getSequenceFromMask(mask=('[#2002 ]', ), )
a.Set(faces=faces1, name='Set-Flange_middle')
#: The set 'Set-Flange_middle' has been created (2 faces).
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
faces1 = f1.getSequenceFromMask(mask=('[#600 ]', ), )
a.Set(faces=faces1, name='Set-Plate_tips')
#: The set 'Set-Plate_tips' has been created (2 faces).
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
faces1 = f1.getSequenceFromMask(mask=('[#48 ]', ), )
a.Set(faces=faces1, name='Set-Plate_intermediates')
#: The set 'Set-Plate_intermediates' has been created (2 faces).
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
faces1 = f1.getSequenceFromMask(mask=('[#4001 ]', ), )
a.Set(faces=faces1, name='Set-Plate_middle')
#: The set 'Set-Plate_middle' has been created (2 faces).
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
faces1 = f1.getSequenceFromMask(mask=('[#20 ]', ), )
a.Set(faces=faces1, name='Set-Web_middle')
#: The set 'Set-Web_middle' has been created (1 face).
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
faces1 = f1.getSequenceFromMask(mask=('[#810 ]', ), )
a.Set(faces=faces1, name='Set-Web_junctions')
#: The set 'Set-Web_junctions' has been created (2 faces).

# Mesh instance
# Flange (top)
a = mdb.models['Model-1'].rootAssembly
e1 = a.instances['T-1'].edges
pickedEdges = e1.getSequenceFromMask(mask=('[#7e000ff0 #f80 ]', ), )
a.seedEdgeBySize(edges=pickedEdges, size=tf/4, deviationFactor=0.1, 
    constraint=FINER)
# Plate (bottom)
a = mdb.models['Model-1'].rootAssembly
e1 = a.instances['T-1'].edges
pickedEdges = e1.getSequenceFromMask(mask=('[#81c0f00f #301f ]', ), )
a.seedEdgeBySize(edges=pickedEdges, size=tp/4, deviationFactor=0.1, 
    constraint=FINER)
# Web
a = mdb.models['Model-1'].rootAssembly
e1 = a.instances['T-1'].edges
pickedEdges = e1.getSequenceFromMask(mask=('[#3f0044 #60 ]', ), )
a.seedEdgeBySize(edges=pickedEdges, size=tw/4, deviationFactor=0.1, 
    constraint=FINER)
# Mesh	
a = mdb.models['Model-1'].rootAssembly
partInstances =(a.instances['T-1'], )
a.generateMesh(regions=partInstances)

# create step
mdb.models['Model-1'].BuckleStep(name='Step-LBA', previous='Initial', 
    numEigen=1, eigensolver=LANCZOS, minEigen=0.0, blockSize=7, 
    maxBlocks=DEFAULT)

# loading
a = mdb.models['Model-1'].rootAssembly
region = a.sets['RP1']
mdb.models['Model-1'].Moment(name='Load-1', createStepName='Step-LBA', 
    region=region, cm1={Mel}, distributionType=UNIFORM, field='', localCsys=None)

# Save buckled shape
mdb.models['Model-1'].keywordBlock.synchVersions(storeNodesAndElements=False)
mdb.models['Model-1'].keywordBlock.insert(73, """
*NODE FILE, LAST MODE=1
U""")

# Add COORD to field output request so that location of maximum amplitude can be found in post-processing
a = mdb.models['Model-1'].rootAssembly
session.viewports['Viewport: 1'].setValues(displayedObject=a)
mdb.models['Model-1'].fieldOutputRequests['F-Output-1'].setValues(variables=(
    'U', 'COORD'))

# Defining job
mdb.Job(name=job_name, model='Model-1', type=ANALYSIS, explicitPrecision=SINGLE, 
    nodalOutputPrecision=FULL, description='', 
    parallelizationMethodExplicit=DOMAIN, multiprocessingMode=MPI, 
    numDomains=8, userSubroutine='', numCpus=8, memory=90, 
    memoryUnits=PERCENTAGE, scratch='', echoPrint=OFF, modelPrint=OFF, 
    contactPrint=OFF, historyPrint=OFF)
mdb.jobs[job_name].writeInput(consistencyChecking=OFF)
'''
    with  open(LBA_name + '.py','w') as fid:
        fid.write(template.format(**context))
        
def gen_LBA_py_file_exp_Sun(Yf,Yw,tf,tw,Bf,H,LdLT,E):
    """
    same as gen_LBA_py_file but takes Yw and Yf instead of just Y
    Yw and Yf does not affect LBA as material properties but affect the length
    of the model because L depends on Yw and Yf for a given LdLT
    """
    
    LBA_name = 'LBA_' + \
        'Yf_' + "{Yf:.0f}".format(Yf=Yf) + \
        '_Yw_' + "{Yw:.0f}".format(Yw=Yw) + '_tf_' + \
        "{tf:.1f}".format(tf=tf) + '_tw_' + \
            "{tw:.1f}".format(tw=tw) + '_Bf_' + \
                "{Bf:.1f}".format(Bf=Bf) + '_H_' + \
                    "{H:.1f}".format(H=H) + '_LdLT_' + \
                        "{LdLT:.2f}".format(LdLT=LdLT)
    LBA_name = LBA_name.replace('.','p')

    L = calc_L(Yf,Yw,tf,tw,Bf,H,LdLT)
    
    Mel = calc_Mel(Yf,Yw,tf,tw,Bf,H)
    
    context = {
        "tf":tf,
        "tw":tw,
        "Bf":Bf,
        "H":H,
        "L":L,
        "LBA_name":LBA_name,
        "Mel":Mel,
        "E":E
        }
    
    template = '''from abaqus import *
from abaqusConstants import *
from math import *
from caeModules import *
from driverUtils import executeOnCaeStartup
import sketch
import part
import math
executeOnCaeStartup()
Mdb()

job_name = '{LBA_name}' # ABAQUS model .inp file name
L = {L:6.1f} # [mm] - Full length
E = float({E})
nu = 0.3

tf = float({tf})
tp = tf
tw = float({tw})
H = float({H})
hw = H-2*tf
Bf = float({Bf})
Bp = Bf
bp = Bp/2 # Half the flange width
bf = Bf/2
a_weld = 0.53*tw # Weld leg

# part
s1 = mdb.models['Model-1'].ConstrainedSketch(name='__profile__', 
    sheetSize=200.0)
g, v, d, c = s1.geometry, s1.vertices, s1.dimensions, s1.constraints
s1.setPrimaryObject(option=STANDALONE)
s1.Spot(point=(0.0, 0.0))
s1.Spot(point=(bp, 0.0))
s1.Spot(point=(-bp, 0.0))
s1.Spot(point=(0.0, hw))
s1.Spot(point=(-bf, hw))
s1.Spot(point=(bf, hw))
session.viewports['Viewport: 1'].view.setValues(nearPlane=470.722, 
    farPlane=798.82, width=1309.27, height=785.434, cameraPosition=(65.6886, 
    129.458, 634.771), cameraTarget=(65.6886, 129.458, 0))
s1.Line(point1=(-bf, hw), point2=(0.0, hw))
s1.HorizontalConstraint(entity=g[2], addUndoState=False)
s1.Line(point1=(0.0, hw), point2=(bf, hw))
s1.HorizontalConstraint(entity=g[3], addUndoState=False)
s1.ParallelConstraint(entity1=g[2], entity2=g[3], addUndoState=False)
s1.Line(point1=(0.0, hw), point2=(0.0, 0.0))
s1.VerticalConstraint(entity=g[4], addUndoState=False)
s1.Line(point1=(-bp, 0.0), point2=(0.0, 0.0))
s1.HorizontalConstraint(entity=g[5], addUndoState=False)
s1.Line(point1=(0.0, 0.0), point2=(bp, 0.0))
s1.HorizontalConstraint(entity=g[6], addUndoState=False)
s1.ParallelConstraint(entity1=g[5], entity2=g[6], addUndoState=False)
p = mdb.models['Model-1'].Part(name='T', dimensionality=THREE_D, 
    type=DEFORMABLE_BODY)
p = mdb.models['Model-1'].parts['T']
p.BaseShellExtrude(sketch=s1, depth=L)
s1.unsetPrimaryObject()
p = mdb.models['Model-1'].parts['T']
session.viewports['Viewport: 1'].setValues(displayedObject=p)
del mdb.models['Model-1'].sketches['__profile__']

# Create datum planes for partitioning for residual stresses
# datum planes for partitioning flanges
p = mdb.models['Model-1'].parts['T']
p.DatumPlaneByPrincipalPlane(principalPlane=YZPLANE, offset=-7*Bf/16)
p = mdb.models['Model-1'].parts['T']
p.DatumPlaneByPrincipalPlane(principalPlane=YZPLANE, offset=7*Bf/16)
p = mdb.models['Model-1'].parts['T']
Bf_m = min(tw+5*a_weld,Bf/5) # Length of middle portion in flange with tensile residual stress
Bp_m = Bf_m # For symmetric I-sections
p.DatumPlaneByPrincipalPlane(principalPlane=YZPLANE, offset=Bf_m/2)
p = mdb.models['Model-1'].parts['T']
p.DatumPlaneByPrincipalPlane(principalPlane=YZPLANE, offset=-Bf_m/2)
# datum planes for partitioning webs
p = mdb.models['Model-1'].parts['T']
p.DatumPlaneByPrincipalPlane(principalPlane=XZPLANE, offset=hw/10-tp/2)
p = mdb.models['Model-1'].parts['T']
p.DatumPlaneByPrincipalPlane(principalPlane=XZPLANE, offset=9*hw/10+tf/2)

# material
mdb.models['Model-1'].Material(name='Elastic-plastic-flange')
mdb.models['Model-1'].materials['Elastic-plastic-flange'].Elastic(table=((E, 
    nu), ))
mdb.models['Model-1'].Material(name='Elastic-plastic-web')
mdb.models['Model-1'].materials['Elastic-plastic-web'].Elastic(table=((E, 
    nu), ))

# create sections
mdb.models['Model-1'].HomogeneousShellSection(name='Section-Plate', preIntegrate=OFF, 
    material='Elastic-plastic-flange', thicknessType=UNIFORM, thickness=tp, 
    thicknessField='', nodalThicknessField='', idealization=NO_IDEALIZATION, 
    poissonDefinition=DEFAULT, thicknessModulus=None, temperature=GRADIENT, 
    useDensity=OFF, integrationRule=SIMPSON, numIntPts=25)
mdb.models['Model-1'].HomogeneousShellSection(name='Section-Flange', 
    preIntegrate=OFF, material='Elastic-plastic-flange', thicknessType=UNIFORM, 
    thickness=tf, thicknessField='', nodalThicknessField='', 
    idealization=NO_IDEALIZATION, poissonDefinition=DEFAULT, 
    thicknessModulus=None, temperature=GRADIENT, useDensity=OFF, 
    integrationRule=SIMPSON, numIntPts=25)
mdb.models['Model-1'].HomogeneousShellSection(name='Section-Web', 
    preIntegrate=OFF, material='Elastic-plastic-web', thicknessType=UNIFORM, 
    thickness=tw, thicknessField='', nodalThicknessField='', 
    idealization=NO_IDEALIZATION, poissonDefinition=DEFAULT, 
    thicknessModulus=None, temperature=GRADIENT, useDensity=OFF, 
    integrationRule=SIMPSON, numIntPts=25)
	
# assign sections
p = mdb.models['Model-1'].parts['T']
f = p.faces
faces = f.getSequenceFromMask(mask=('[#c ]', ), )
region = regionToolset.Region(faces=faces)
p = mdb.models['Model-1'].parts['T']
p.SectionAssignment(region=region, sectionName='Section-Flange', offset=0.0, 
    offsetType=MIDDLE_SURFACE, offsetField='', 
    thicknessAssignment=FROM_SECTION)
p = mdb.models['Model-1'].parts['T']
f = p.faces
faces = f.getSequenceFromMask(mask=('[#2 ]', ), )
region = regionToolset.Region(faces=faces)
p = mdb.models['Model-1'].parts['T']
p.SectionAssignment(region=region, sectionName='Section-Web', offset=0.0, 
    offsetType=MIDDLE_SURFACE, offsetField='', 
    thicknessAssignment=FROM_SECTION)
p = mdb.models['Model-1'].parts['T']
f = p.faces
faces = f.getSequenceFromMask(mask=('[#11 ]', ), )
region = regionToolset.Region(faces=faces)
p = mdb.models['Model-1'].parts['T']
p.SectionAssignment(region=region, sectionName='Section-Plate', offset=0.0, 
    offsetType=MIDDLE_SURFACE, offsetField='', 
    thicknessAssignment=FROM_SECTION)
	
# make shell orientation correct (for both flanges, attachment side is considered 'bottom')
p = mdb.models['Model-1'].parts['T']
f = p.faces
faces = f.getSequenceFromMask(mask=('[#8 ]', ), )
regions = regionToolset.Region(faces=faces)
p.flipNormal(regions=regions)

# offset top flange
mdb.models['Model-1'].parts['T'].sectionAssignments[0].setValues(
    offsetType=BOTTOM_SURFACE, offsetField='', offset=0.0)
	
# offset bottom flange
mdb.models['Model-1'].parts['T'].sectionAssignments[2].setValues(
    offsetType=BOTTOM_SURFACE, offsetField='', offset=0.0)

# create instance
a = mdb.models['Model-1'].rootAssembly
a.DatumCsysByDefault(CARTESIAN)
p = mdb.models['Model-1'].parts['T']
a.Instance(name='T-1', part=p, dependent=OFF)

# reference points and sets for coupling at boundaries
RP1 = a.ReferencePoint(point=(0.0, hw/2, 0.0))
a = mdb.models['Model-1'].rootAssembly
r1 = a.referencePoints
refPoints1=(r1[RP1.id], )
a.Set(referencePoints=refPoints1, name='RP1')
a = mdb.models['Model-1'].rootAssembly
e1 = a.instances['T-1'].edges
edges1 = e1.getSequenceFromMask(mask=('[#9244 ]', ), )
a.Set(edges=edges1, name='Set-Rotated_end')
a = mdb.models['Model-1'].rootAssembly
e1 = a.instances['T-1'].edges
edges1 = e1.getSequenceFromMask(mask=('[#2491 ]', ), )
a.Set(edges=edges1, name='Set-Fixed_end')

# coupling for boundaries
a = mdb.models['Model-1'].rootAssembly
region1=a.sets['RP1']
a = mdb.models['Model-1'].rootAssembly
region2=a.sets['Set-Rotated_end']
mdb.models['Model-1'].Coupling(name='Constraint-Rotated_end', 
    controlPoint=region1, surface=region2, influenceRadius=WHOLE_SURFACE, 
    couplingType=KINEMATIC, localCsys=None, u1=ON, u2=ON, u3=ON, ur1=ON, 
    ur2=ON, ur3=ON)
	
# boundary conditions
a = mdb.models['Model-1'].rootAssembly
region = a.sets['Set-Fixed_end']
mdb.models['Model-1'].EncastreBC(name='BC-Fixed_end', createStepName='Initial', 
    region=region, localCsys=None)
a = mdb.models['Model-1'].rootAssembly
region = a.sets['RP1']
mdb.models['Model-1'].DisplacementBC(name='BC-Rotated_end', createStepName='Initial', 
    region=region, u1=SET, u2=UNSET, u3=UNSET, ur1=UNSET, ur2=SET, ur3=SET, 
    amplitude=UNSET, distributionType=UNIFORM, fieldName='', localCsys=None)

# Partitioning by datum planes for the assignment of residual stresses
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
pickedFaces = f1.getSequenceFromMask(mask=('[#18 ]', ), )
d21 = a.instances['T-1'].datums
a.PartitionFaceByDatumPlane(datumPlane=d21[2], faces=pickedFaces)
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
pickedFaces = f1.getSequenceFromMask(mask=('[#14 ]', ), )
d31 = a.instances['T-1'].datums
a.PartitionFaceByDatumPlane(datumPlane=d31[3], faces=pickedFaces)
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
pickedFaces = f1.getSequenceFromMask(mask=('[#20 ]', ), )
d21 = a.instances['T-1'].datums
a.PartitionFaceByDatumPlane(datumPlane=d21[7], faces=pickedFaces)
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
pickedFaces = f1.getSequenceFromMask(mask=('[#1 ]', ), )
d31 = a.instances['T-1'].datums
a.PartitionFaceByDatumPlane(datumPlane=d31[6], faces=pickedFaces)
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
pickedFaces = f1.getSequenceFromMask(mask=('[#600 ]', ), )
d21 = a.instances['T-1'].datums
a.PartitionFaceByDatumPlane(datumPlane=d21[5], faces=pickedFaces)
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
pickedFaces = f1.getSequenceFromMask(mask=('[#30 ]', ), )
d21 = a.instances['T-1'].datums
a.PartitionFaceByDatumPlane(datumPlane=d21[4], faces=pickedFaces)

# Creating sets for the assignment of residual stresses
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
faces1 = f1.getSequenceFromMask(mask=('[#1100 ]', ), )
a.Set(faces=faces1, name='Set-Flange_tips')
#: The set 'Set-Flange_tips' has been created (2 faces).
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
faces1 = f1.getSequenceFromMask(mask=('[#84 ]', ), )
a.Set(faces=faces1, name='Set-Flange_intermediates')
#: The set 'Set-Flange_intermediates' has been created (2 faces).
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
faces1 = f1.getSequenceFromMask(mask=('[#2002 ]', ), )
a.Set(faces=faces1, name='Set-Flange_middle')
#: The set 'Set-Flange_middle' has been created (2 faces).
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
faces1 = f1.getSequenceFromMask(mask=('[#600 ]', ), )
a.Set(faces=faces1, name='Set-Plate_tips')
#: The set 'Set-Plate_tips' has been created (2 faces).
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
faces1 = f1.getSequenceFromMask(mask=('[#48 ]', ), )
a.Set(faces=faces1, name='Set-Plate_intermediates')
#: The set 'Set-Plate_intermediates' has been created (2 faces).
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
faces1 = f1.getSequenceFromMask(mask=('[#4001 ]', ), )
a.Set(faces=faces1, name='Set-Plate_middle')
#: The set 'Set-Plate_middle' has been created (2 faces).
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
faces1 = f1.getSequenceFromMask(mask=('[#20 ]', ), )
a.Set(faces=faces1, name='Set-Web_middle')
#: The set 'Set-Web_middle' has been created (1 face).
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
faces1 = f1.getSequenceFromMask(mask=('[#810 ]', ), )
a.Set(faces=faces1, name='Set-Web_junctions')
#: The set 'Set-Web_junctions' has been created (2 faces).

# Mesh instance
# Flange (top)
a = mdb.models['Model-1'].rootAssembly
e1 = a.instances['T-1'].edges
pickedEdges = e1.getSequenceFromMask(mask=('[#7e000ff0 #f80 ]', ), )
a.seedEdgeBySize(edges=pickedEdges, size=tf/4, deviationFactor=0.1, 
    constraint=FINER)
# Plate (bottom)
a = mdb.models['Model-1'].rootAssembly
e1 = a.instances['T-1'].edges
pickedEdges = e1.getSequenceFromMask(mask=('[#81c0f00f #301f ]', ), )
a.seedEdgeBySize(edges=pickedEdges, size=tp/4, deviationFactor=0.1, 
    constraint=FINER)
# Web
a = mdb.models['Model-1'].rootAssembly
e1 = a.instances['T-1'].edges
pickedEdges = e1.getSequenceFromMask(mask=('[#3f0044 #60 ]', ), )
a.seedEdgeBySize(edges=pickedEdges, size=tw/4, deviationFactor=0.1, 
    constraint=FINER)

# Create datum plane at Z = 50 mm
a = mdb.models['Model-1'].rootAssembly
a.DatumPlaneByPrincipalPlane(principalPlane=XYPLANE, offset=50.0)
# Create datum plane at Z = L - 50 mm
a = mdb.models['Model-1'].rootAssembly
a.DatumPlaneByPrincipalPlane(principalPlane=XYPLANE, offset=L-50)
# Create datum plane at Y = hw/2
a = mdb.models['Model-1'].rootAssembly
a.DatumPlaneByPrincipalPlane(principalPlane=XZPLANE, offset=hw/2)
# Create partition at Y = hw/2
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
pickedFaces = f1.getSequenceFromMask(mask=('[#20 ]', ), )
d21 = a.datums
a.PartitionFaceByDatumPlane(datumPlane=d21[27], faces=pickedFaces)
# Create partition at Z = L - 50 mm
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
pickedFaces = f1.getSequenceFromMask(mask=('[#ffff ]', ), )
d1 = a.datums
a.PartitionFaceByDatumPlane(datumPlane=d1[26], faces=pickedFaces)
# Create partition at Z = 50 mm
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
pickedFaces = f1.getSequenceFromMask(mask=('[#776daa80 ]', ), )
d21 = a.datums
a.PartitionFaceByDatumPlane(datumPlane=d21[25], faces=pickedFaces)
# Create datum plane at Z = L/2
a = mdb.models['Model-1'].rootAssembly
a.DatumPlaneByPrincipalPlane(principalPlane=XYPLANE, offset=L/2)
# Partition edges at Z = L/2
a = mdb.models['Model-1'].rootAssembly
e1 = a.instances['T-1'].edges
pickedEdges = e1.getSequenceFromMask(mask=('[#850410a0 #14020 #30002140 #24 ]', 
    ), )
d1 = a.datums
a.PartitionEdgeByDatumPlane(datumPlane=d1[31], edges=pickedEdges)

# Create reference point at midspan and save it as Set-Midspan
a = mdb.models['Model-1'].rootAssembly
v1 = a.instances['T-1'].vertices
RP2 = a.ReferencePoint(point=v1[73])
a = mdb.models['Model-1'].rootAssembly
r1 = a.referencePoints
refPoints1=(r1[RP2.id], )
a.Set(referencePoints=refPoints1, name='Set-Midspan')
# Create kinematic coupling constraint between reference point and midspan point
a = mdb.models['Model-1'].rootAssembly
region1=a.sets['Set-Midspan']
a = mdb.models['Model-1'].rootAssembly
v1 = a.instances['T-1'].vertices
verts1 = v1.getSequenceFromMask(mask=('[#0:2 #200 ]', ), )
region2=regionToolset.Region(vertices=verts1)
mdb.models['Model-1'].Coupling(name='Constraint-Midspan', controlPoint=region1, 
    surface=region2, influenceRadius=WHOLE_SURFACE, couplingType=KINEMATIC, 
    localCsys=None, u1=ON, u2=ON, u3=ON, ur1=ON, ur2=ON, ur3=ON)

# create step
mdb.models['Model-1'].BuckleStep(name='Step-LBA', previous='Initial', 
    numEigen=1, eigensolver=LANCZOS, minEigen=0.0, blockSize=7, 
    maxBlocks=DEFAULT)

# loading
a = mdb.models['Model-1'].rootAssembly
region = a.sets['RP1']
mdb.models['Model-1'].Moment(name='Load-1', createStepName='Step-LBA', 
    region=region, cm1={Mel}, distributionType=UNIFORM, field='', localCsys=None)

# Mesh	
a = mdb.models['Model-1'].rootAssembly
partInstances =(a.instances['T-1'], )
a.generateMesh(regions=partInstances)

# Save buckled shape
mdb.models['Model-1'].keywordBlock.synchVersions(storeNodesAndElements=False)
mdb.models['Model-1'].keywordBlock.insert(80, """
*NODE FILE, LAST MODE=1
U""")

# Add COORD to field output request so that location of maximum amplitude can be found in post-processing
a = mdb.models['Model-1'].rootAssembly
session.viewports['Viewport: 1'].setValues(displayedObject=a)
mdb.models['Model-1'].fieldOutputRequests['F-Output-1'].setValues(variables=(
    'U', 'COORD'))

# Defining job
mdb.Job(name=job_name, model='Model-1', type=ANALYSIS, explicitPrecision=SINGLE, 
    nodalOutputPrecision=FULL, description='', 
    parallelizationMethodExplicit=DOMAIN, multiprocessingMode=MPI, 
    numDomains=8, userSubroutine='', numCpus=8, memory=90, 
    memoryUnits=PERCENTAGE, scratch='', echoPrint=OFF, modelPrint=OFF, 
    contactPrint=OFF, historyPrint=OFF)
mdb.jobs[job_name].writeInput(consistencyChecking=OFF)
'''
    with  open(LBA_name + '.py','w') as fid:
        fid.write(template.format(**context))

def gen_LBA_py_file_exp_Lee(Yf,Yw,tf,tw,Bf,H,LdLT,E,Lb2):
    """
    same as gen_LBA_py_file but takes Yw and Yf instead of just Y
    Yw and Yf does not affect LBA as material properties but affect the length
    of the model because L depends on Yw and Yf for a given LdLT
    """
    
    LBA_name = 'LBA_' + \
        'Yf_' + "{Yf:.0f}".format(Yf=Yf) + \
        '_Yw_' + "{Yw:.0f}".format(Yw=Yw) + '_tf_' + \
        "{tf:.1f}".format(tf=tf) + '_tw_' + \
            "{tw:.1f}".format(tw=tw) + '_Bf_' + \
                "{Bf:.1f}".format(Bf=Bf) + '_H_' + \
                    "{H:.1f}".format(H=H) + '_LdLT_' + \
                        "{LdLT:.2f}".format(LdLT=LdLT)
    LBA_name = LBA_name.replace('.','p')

    L = calc_L(Yf,Yw,tf,tw,Bf,H,LdLT)
    
    Mel = calc_Mel(Yf,Yw,tf,tw,Bf,H)
    
    context = {
        "tf":tf,
        "tw":tw,
        "Bf":Bf,
        "H":H,
        "L":L,
        "LBA_name":LBA_name,
        "Mel":Mel,
        "E":E,
        "Lb2":Lb2,
        }
    
    template = '''from abaqus import *
from abaqusConstants import *
from math import *
from caeModules import *
from driverUtils import executeOnCaeStartup
import sketch
import part
import math
executeOnCaeStartup()
Mdb()

job_name = '{LBA_name}' # ABAQUS model .inp file name
L = {L:6.1f} # [mm] - Full length
E = float({E})
nu = 0.3

tf = float({tf})
tp = tf
tw = float({tw})
H = float({H})
hw = H-2*tf
Bf = float({Bf})
Bp = Bf
bp = Bp/2 # Half the flange width
bf = Bf/2
a_weld = 0.53*tw # Weld leg

Lb2 = float({Lb2}) # mm

# part
s1 = mdb.models['Model-1'].ConstrainedSketch(name='__profile__', 
    sheetSize=200.0)
g, v, d, c = s1.geometry, s1.vertices, s1.dimensions, s1.constraints
s1.setPrimaryObject(option=STANDALONE)
s1.Spot(point=(0.0, 0.0))
s1.Spot(point=(bp, 0.0))
s1.Spot(point=(-bp, 0.0))
s1.Spot(point=(0.0, hw))
s1.Spot(point=(-bf, hw))
s1.Spot(point=(bf, hw))
session.viewports['Viewport: 1'].view.setValues(nearPlane=470.722, 
    farPlane=798.82, width=1309.27, height=785.434, cameraPosition=(65.6886, 
    129.458, 634.771), cameraTarget=(65.6886, 129.458, 0))
s1.Line(point1=(-bf, hw), point2=(0.0, hw))
s1.HorizontalConstraint(entity=g[2], addUndoState=False)
s1.Line(point1=(0.0, hw), point2=(bf, hw))
s1.HorizontalConstraint(entity=g[3], addUndoState=False)
s1.ParallelConstraint(entity1=g[2], entity2=g[3], addUndoState=False)
s1.Line(point1=(0.0, hw), point2=(0.0, 0.0))
s1.VerticalConstraint(entity=g[4], addUndoState=False)
s1.Line(point1=(-bp, 0.0), point2=(0.0, 0.0))
s1.HorizontalConstraint(entity=g[5], addUndoState=False)
s1.Line(point1=(0.0, 0.0), point2=(bp, 0.0))
s1.HorizontalConstraint(entity=g[6], addUndoState=False)
s1.ParallelConstraint(entity1=g[5], entity2=g[6], addUndoState=False)
p = mdb.models['Model-1'].Part(name='T', dimensionality=THREE_D, 
    type=DEFORMABLE_BODY)
p = mdb.models['Model-1'].parts['T']
p.BaseShellExtrude(sketch=s1, depth=L)
s1.unsetPrimaryObject()
p = mdb.models['Model-1'].parts['T']
session.viewports['Viewport: 1'].setValues(displayedObject=p)
del mdb.models['Model-1'].sketches['__profile__']

# Create datum planes for partitioning for residual stresses
# datum planes for partitioning flanges
p = mdb.models['Model-1'].parts['T']
p.DatumPlaneByPrincipalPlane(principalPlane=YZPLANE, offset=-7*Bf/16)
p = mdb.models['Model-1'].parts['T']
p.DatumPlaneByPrincipalPlane(principalPlane=YZPLANE, offset=7*Bf/16)
p = mdb.models['Model-1'].parts['T']
Bf_m = min(tw+5*a_weld,Bf/5) # Length of middle portion in flange with tensile residual stress
Bp_m = Bf_m # For symmetric I-sections
p.DatumPlaneByPrincipalPlane(principalPlane=YZPLANE, offset=Bf_m/2)
p = mdb.models['Model-1'].parts['T']
p.DatumPlaneByPrincipalPlane(principalPlane=YZPLANE, offset=-Bf_m/2)
# datum planes for partitioning webs
p = mdb.models['Model-1'].parts['T']
p.DatumPlaneByPrincipalPlane(principalPlane=XZPLANE, offset=hw/10-tp/2)
p = mdb.models['Model-1'].parts['T']
p.DatumPlaneByPrincipalPlane(principalPlane=XZPLANE, offset=9*hw/10+tf/2)

# material
mdb.models['Model-1'].Material(name='Elastic-plastic-flange')
mdb.models['Model-1'].materials['Elastic-plastic-flange'].Elastic(table=((E, 
    nu), ))
mdb.models['Model-1'].Material(name='Elastic-plastic-web')
mdb.models['Model-1'].materials['Elastic-plastic-web'].Elastic(table=((E, 
    nu), ))

# create sections
mdb.models['Model-1'].HomogeneousShellSection(name='Section-Plate', preIntegrate=OFF, 
    material='Elastic-plastic-flange', thicknessType=UNIFORM, thickness=tp, 
    thicknessField='', nodalThicknessField='', idealization=NO_IDEALIZATION, 
    poissonDefinition=DEFAULT, thicknessModulus=None, temperature=GRADIENT, 
    useDensity=OFF, integrationRule=SIMPSON, numIntPts=25)
mdb.models['Model-1'].HomogeneousShellSection(name='Section-Flange', 
    preIntegrate=OFF, material='Elastic-plastic-flange', thicknessType=UNIFORM, 
    thickness=tf, thicknessField='', nodalThicknessField='', 
    idealization=NO_IDEALIZATION, poissonDefinition=DEFAULT, 
    thicknessModulus=None, temperature=GRADIENT, useDensity=OFF, 
    integrationRule=SIMPSON, numIntPts=25)
mdb.models['Model-1'].HomogeneousShellSection(name='Section-Web', 
    preIntegrate=OFF, material='Elastic-plastic-web', thicknessType=UNIFORM, 
    thickness=tw, thicknessField='', nodalThicknessField='', 
    idealization=NO_IDEALIZATION, poissonDefinition=DEFAULT, 
    thicknessModulus=None, temperature=GRADIENT, useDensity=OFF, 
    integrationRule=SIMPSON, numIntPts=25)
	
# assign sections
p = mdb.models['Model-1'].parts['T']
f = p.faces
faces = f.getSequenceFromMask(mask=('[#c ]', ), )
region = regionToolset.Region(faces=faces)
p = mdb.models['Model-1'].parts['T']
p.SectionAssignment(region=region, sectionName='Section-Flange', offset=0.0, 
    offsetType=MIDDLE_SURFACE, offsetField='', 
    thicknessAssignment=FROM_SECTION)
p = mdb.models['Model-1'].parts['T']
f = p.faces
faces = f.getSequenceFromMask(mask=('[#2 ]', ), )
region = regionToolset.Region(faces=faces)
p = mdb.models['Model-1'].parts['T']
p.SectionAssignment(region=region, sectionName='Section-Web', offset=0.0, 
    offsetType=MIDDLE_SURFACE, offsetField='', 
    thicknessAssignment=FROM_SECTION)
p = mdb.models['Model-1'].parts['T']
f = p.faces
faces = f.getSequenceFromMask(mask=('[#11 ]', ), )
region = regionToolset.Region(faces=faces)
p = mdb.models['Model-1'].parts['T']
p.SectionAssignment(region=region, sectionName='Section-Plate', offset=0.0, 
    offsetType=MIDDLE_SURFACE, offsetField='', 
    thicknessAssignment=FROM_SECTION)
	
# make shell orientation correct (for both flanges, attachment side is considered 'bottom')
p = mdb.models['Model-1'].parts['T']
f = p.faces
faces = f.getSequenceFromMask(mask=('[#8 ]', ), )
regions = regionToolset.Region(faces=faces)
p.flipNormal(regions=regions)

# offset top flange
mdb.models['Model-1'].parts['T'].sectionAssignments[0].setValues(
    offsetType=BOTTOM_SURFACE, offsetField='', offset=0.0)
	
# offset bottom flange
mdb.models['Model-1'].parts['T'].sectionAssignments[2].setValues(
    offsetType=BOTTOM_SURFACE, offsetField='', offset=0.0)

# create instance
a = mdb.models['Model-1'].rootAssembly
a.DatumCsysByDefault(CARTESIAN)
p = mdb.models['Model-1'].parts['T']
a.Instance(name='T-1', part=p, dependent=OFF)

# reference points and sets for coupling at boundaries
RP1 = a.ReferencePoint(point=(0.0, hw/2, 0.0))
a = mdb.models['Model-1'].rootAssembly
r1 = a.referencePoints
refPoints1=(r1[RP1.id], )
a.Set(referencePoints=refPoints1, name='RP1')
a = mdb.models['Model-1'].rootAssembly
e1 = a.instances['T-1'].edges
edges1 = e1.getSequenceFromMask(mask=('[#9244 ]', ), )
a.Set(edges=edges1, name='Set-Rotated_end')
a = mdb.models['Model-1'].rootAssembly
e1 = a.instances['T-1'].edges
edges1 = e1.getSequenceFromMask(mask=('[#2491 ]', ), )
a.Set(edges=edges1, name='Set-Fixed_end')

# coupling for boundaries
a = mdb.models['Model-1'].rootAssembly
region1=a.sets['RP1']
a = mdb.models['Model-1'].rootAssembly
region2=a.sets['Set-Rotated_end']
mdb.models['Model-1'].Coupling(name='Constraint-Rotated_end', 
    controlPoint=region1, surface=region2, influenceRadius=WHOLE_SURFACE, 
    couplingType=KINEMATIC, localCsys=None, u1=ON, u2=ON, u3=ON, ur1=ON, 
    ur2=ON, ur3=ON)
	
# boundary conditions
a = mdb.models['Model-1'].rootAssembly
region = a.sets['Set-Fixed_end']
mdb.models['Model-1'].EncastreBC(name='BC-Fixed_end', createStepName='Initial', 
    region=region, localCsys=None)
a = mdb.models['Model-1'].rootAssembly
region = a.sets['RP1']
mdb.models['Model-1'].DisplacementBC(name='BC-Rotated_end', createStepName='Initial', 
    region=region, u1=SET, u2=UNSET, u3=UNSET, ur1=UNSET, ur2=SET, ur3=SET, 
    amplitude=UNSET, distributionType=UNIFORM, fieldName='', localCsys=None)

# Partitioning by datum planes for the assignment of residual stresses
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
pickedFaces = f1.getSequenceFromMask(mask=('[#18 ]', ), )
d21 = a.instances['T-1'].datums
a.PartitionFaceByDatumPlane(datumPlane=d21[2], faces=pickedFaces)
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
pickedFaces = f1.getSequenceFromMask(mask=('[#14 ]', ), )
d31 = a.instances['T-1'].datums
a.PartitionFaceByDatumPlane(datumPlane=d31[3], faces=pickedFaces)
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
pickedFaces = f1.getSequenceFromMask(mask=('[#20 ]', ), )
d21 = a.instances['T-1'].datums
a.PartitionFaceByDatumPlane(datumPlane=d21[7], faces=pickedFaces)
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
pickedFaces = f1.getSequenceFromMask(mask=('[#1 ]', ), )
d31 = a.instances['T-1'].datums
a.PartitionFaceByDatumPlane(datumPlane=d31[6], faces=pickedFaces)
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
pickedFaces = f1.getSequenceFromMask(mask=('[#600 ]', ), )
d21 = a.instances['T-1'].datums
a.PartitionFaceByDatumPlane(datumPlane=d21[5], faces=pickedFaces)
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
pickedFaces = f1.getSequenceFromMask(mask=('[#30 ]', ), )
d21 = a.instances['T-1'].datums
a.PartitionFaceByDatumPlane(datumPlane=d21[4], faces=pickedFaces)

# Creating sets for the assignment of residual stresses
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
faces1 = f1.getSequenceFromMask(mask=('[#1100 ]', ), )
a.Set(faces=faces1, name='Set-Flange_tips')
#: The set 'Set-Flange_tips' has been created (2 faces).
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
faces1 = f1.getSequenceFromMask(mask=('[#84 ]', ), )
a.Set(faces=faces1, name='Set-Flange_intermediates')
#: The set 'Set-Flange_intermediates' has been created (2 faces).
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
faces1 = f1.getSequenceFromMask(mask=('[#2002 ]', ), )
a.Set(faces=faces1, name='Set-Flange_middle')
#: The set 'Set-Flange_middle' has been created (2 faces).
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
faces1 = f1.getSequenceFromMask(mask=('[#600 ]', ), )
a.Set(faces=faces1, name='Set-Plate_tips')
#: The set 'Set-Plate_tips' has been created (2 faces).
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
faces1 = f1.getSequenceFromMask(mask=('[#48 ]', ), )
a.Set(faces=faces1, name='Set-Plate_intermediates')
#: The set 'Set-Plate_intermediates' has been created (2 faces).
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
faces1 = f1.getSequenceFromMask(mask=('[#4001 ]', ), )
a.Set(faces=faces1, name='Set-Plate_middle')
#: The set 'Set-Plate_middle' has been created (2 faces).
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
faces1 = f1.getSequenceFromMask(mask=('[#20 ]', ), )
a.Set(faces=faces1, name='Set-Web_middle')
#: The set 'Set-Web_middle' has been created (1 face).
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
faces1 = f1.getSequenceFromMask(mask=('[#810 ]', ), )
a.Set(faces=faces1, name='Set-Web_junctions')
#: The set 'Set-Web_junctions' has been created (2 faces).

# Mesh instance
# Flange (top)
a = mdb.models['Model-1'].rootAssembly
e1 = a.instances['T-1'].edges
pickedEdges = e1.getSequenceFromMask(mask=('[#7e000ff0 #f80 ]', ), )
a.seedEdgeBySize(edges=pickedEdges, size=tf/4, deviationFactor=0.1, 
    constraint=FINER)
# Plate (bottom)
a = mdb.models['Model-1'].rootAssembly
e1 = a.instances['T-1'].edges
pickedEdges = e1.getSequenceFromMask(mask=('[#81c0f00f #301f ]', ), )
a.seedEdgeBySize(edges=pickedEdges, size=tp/4, deviationFactor=0.1, 
    constraint=FINER)
# Web
a = mdb.models['Model-1'].rootAssembly
e1 = a.instances['T-1'].edges
pickedEdges = e1.getSequenceFromMask(mask=('[#3f0044 #60 ]', ), )
a.seedEdgeBySize(edges=pickedEdges, size=tw/4, deviationFactor=0.1, 
    constraint=FINER)

# Create datum plane at Z = Lb2 mm
a = mdb.models['Model-1'].rootAssembly
a.DatumPlaneByPrincipalPlane(principalPlane=XYPLANE, offset=Lb2)
# Create datum plane at Z = L - Lb2
a = mdb.models['Model-1'].rootAssembly
a.DatumPlaneByPrincipalPlane(principalPlane=XYPLANE, offset=L-Lb2)
# Create datum plane at Y = hw/2
a = mdb.models['Model-1'].rootAssembly
a.DatumPlaneByPrincipalPlane(principalPlane=XZPLANE, offset=hw/2)
# Create partition at Y = hw/2
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
pickedFaces = f1.getSequenceFromMask(mask=('[#20 ]', ), )
d21 = a.datums
a.PartitionFaceByDatumPlane(datumPlane=d21[27], faces=pickedFaces)
# Create partition at Z = L - Lb2
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
pickedFaces = f1.getSequenceFromMask(mask=('[#ffff ]', ), )
d1 = a.datums
a.PartitionFaceByDatumPlane(datumPlane=d1[26], faces=pickedFaces)
# Create partition at Z = Lb2
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
pickedFaces = f1.getSequenceFromMask(mask=('[#776daa80 ]', ), )
d21 = a.datums
a.PartitionFaceByDatumPlane(datumPlane=d21[25], faces=pickedFaces)
# Create datum plane at Z = L/2
a = mdb.models['Model-1'].rootAssembly
a.DatumPlaneByPrincipalPlane(principalPlane=XYPLANE, offset=L/2)
# Partition edges at Z = L/2
a = mdb.models['Model-1'].rootAssembly
e1 = a.instances['T-1'].edges
pickedEdges = e1.getSequenceFromMask(mask=('[#850410a0 #14020 #30002140 #24 ]', 
    ), )
d1 = a.datums
a.PartitionEdgeByDatumPlane(datumPlane=d1[31], edges=pickedEdges)

# Create reference point at midspan and save it as Set-Midspan
a = mdb.models['Model-1'].rootAssembly
v1 = a.instances['T-1'].vertices
RP2 = a.ReferencePoint(point=v1[73])
a = mdb.models['Model-1'].rootAssembly
r1 = a.referencePoints
refPoints1=(r1[RP2.id], )
a.Set(referencePoints=refPoints1, name='Set-Midspan')
# Create kinematic coupling constraint between reference point and midspan point
a = mdb.models['Model-1'].rootAssembly
region1=a.sets['Set-Midspan']
a = mdb.models['Model-1'].rootAssembly
v1 = a.instances['T-1'].vertices
verts1 = v1.getSequenceFromMask(mask=('[#0:2 #200 ]', ), )
region2=regionToolset.Region(vertices=verts1)
mdb.models['Model-1'].Coupling(name='Constraint-Midspan', controlPoint=region1, 
    surface=region2, influenceRadius=WHOLE_SURFACE, couplingType=KINEMATIC, 
    localCsys=None, u1=ON, u2=ON, u3=ON, ur1=ON, ur2=ON, ur3=ON)

# Create reference point at midspan and save it as Set-Restraint_1
a = mdb.models['Model-1'].rootAssembly
v1 = a.instances['T-1'].vertices
RP3 = a.ReferencePoint(point=v1[27])
a = mdb.models['Model-1'].rootAssembly
r1 = a.referencePoints
refPoints1=(r1[RP3.id], )
a.Set(referencePoints=refPoints1, name='Set-Restraint_1')
# Create kinematic coupling constraint between reference point and midspan point
a = mdb.models['Model-1'].rootAssembly
region1=a.sets['Set-Restraint_1']
a = mdb.models['Model-1'].rootAssembly
v1 = a.instances['T-1'].vertices
verts1 = v1.getSequenceFromMask(mask=('[#8000000 ]', ), )
region2=regionToolset.Region(vertices=verts1)
mdb.models['Model-1'].Coupling(name='Constraint-Restraint_1', 
    controlPoint=region1, surface=region2, influenceRadius=WHOLE_SURFACE, 
    couplingType=KINEMATIC, localCsys=None, u1=ON, u2=ON, u3=ON, ur1=ON, 
    ur2=ON, ur3=ON)
	
# Create reference point at midspan and save it as Set-Restraint_2
a = mdb.models['Model-1'].rootAssembly
v1 = a.instances['T-1'].vertices
RP4 = a.ReferencePoint(point=v1[61])
a = mdb.models['Model-1'].rootAssembly
r1 = a.referencePoints
refPoints1=(r1[RP4.id], )
a.Set(referencePoints=refPoints1, name='Set-Restraint_2')
# Create kinematic coupling constraint between reference point and midspan point
a = mdb.models['Model-1'].rootAssembly
region1=a.sets['Set-Restraint_2']
a = mdb.models['Model-1'].rootAssembly
v1 = a.instances['T-1'].vertices
verts1 = v1.getSequenceFromMask(mask=('[#0 #20000000 ]', ), )
region2=regionToolset.Region(vertices=verts1)
mdb.models['Model-1'].Coupling(name='Constraint-Restraint_2', 
    controlPoint=region1, surface=region2, influenceRadius=WHOLE_SURFACE, 
    couplingType=KINEMATIC, localCsys=None, u1=ON, u2=ON, u3=ON, ur1=ON, 
    ur2=ON, ur3=ON)

# create step
mdb.models['Model-1'].BuckleStep(name='Step-LBA', previous='Initial', 
    numEigen=1, eigensolver=LANCZOS, minEigen=0.0, blockSize=7, 
    maxBlocks=DEFAULT)

# loading
a = mdb.models['Model-1'].rootAssembly
region = a.sets['RP1']
mdb.models['Model-1'].Moment(name='Load-1', createStepName='Step-LBA', 
    region=region, cm1={Mel}, distributionType=UNIFORM, field='', localCsys=None)

# Mesh	
a = mdb.models['Model-1'].rootAssembly
partInstances =(a.instances['T-1'], )
a.generateMesh(regions=partInstances)

# Save buckled shape
mdb.models['Model-1'].keywordBlock.synchVersions(storeNodesAndElements=False)
mdb.models['Model-1'].keywordBlock.insert(94, """
*NODE FILE, LAST MODE=1
U""")

# Add COORD to field output request so that location of maximum amplitude can be found in post-processing
a = mdb.models['Model-1'].rootAssembly
session.viewports['Viewport: 1'].setValues(displayedObject=a)
mdb.models['Model-1'].fieldOutputRequests['F-Output-1'].setValues(variables=(
    'U', 'COORD'))

# Defining job
mdb.Job(name=job_name, model='Model-1', type=ANALYSIS, explicitPrecision=SINGLE, 
    nodalOutputPrecision=FULL, description='', 
    parallelizationMethodExplicit=DOMAIN, multiprocessingMode=MPI, 
    numDomains=8, userSubroutine='', numCpus=8, memory=90, 
    memoryUnits=PERCENTAGE, scratch='', echoPrint=OFF, modelPrint=OFF, 
    contactPrint=OFF, historyPrint=OFF)
mdb.jobs[job_name].writeInput(consistencyChecking=OFF)
'''
    with  open(LBA_name + '.py','w') as fid:
        fid.write(template.format(**context))

def gen_postproc_LBA_py_file(Y,tf,tw,Bf,H,LdLT):
    """
    Function to generate post-processing py file which opens the odb of the LBA
    and prints the eigenvalue of the first mode and the location of the maximum
    amplitude (imp=0 for flange; imp=1 for web) to a csv file.
    
    In a parametric batch run series, this function is to be called by function which
    generates the GMNIA py script, which also then runs the generated script,
    so that the GMNIA py script can use the correct imp for its input.
    """
    
    job_name = 'LBA_Y_' + "{Y:.0f}".format(Y=Y) + '_tf_' + \
        "{tf:.1f}".format(tf=tf) + '_tw_' + \
            "{tw:.1f}".format(tw=tw) + '_Bf_' + \
                "{Bf:.1f}".format(Bf=Bf) + '_H_' + \
                    "{H:.1f}".format(H=H) + '_LdLT_' + \
                        "{LdLT:.2f}".format(LdLT=LdLT)
    job_name = job_name.replace('.','p')
    
    context = {
        "job_name":job_name,
        "tf":tf,
        "H":H
        }
    template = '''from odbAccess import *
import numpy as np

tf = float({tf})
H = float({H})
d = H-2*tf

job_name = '{job_name}'

odb = openOdb(job_name+'.odb', readOnly=True)

step = odb.steps['Step-LBA']

# Find eigenvalue
fd = step.frames[1].description # frame description
eigenvalue = float(fd[28:48])

# Find total number of nodes
n = len(step.frames[0].fieldOutputs['COORD'].getSubset(region = odb.rootAssembly.nodeSets[' ALL NODES']).values)

# load original coordinates of all nodes
COORD = step.frames[0].fieldOutputs['COORD'].getSubset(region = odb.rootAssembly.nodeSets[' ALL NODES'])

# Load mode 1 displacements of all nodes 
U = step.frames[1].fieldOutputs['U'].getSubset(region = odb.rootAssembly.nodeSets[' ALL NODES'])

# Initialize array of displacement magnitudes
Umag = np.zeros(n)

# double precision data
for i in range(n): # loop through all the nodes
	
	v = U.values[i].dataDouble # Displacements of this node
	Umag[i] = np.sqrt(v.dot(v)) # magnitude of displacement of this node

# Find maximums of Umag
Umax = np.amax(Umag) 

# Find indices of maximums of Umag
Umax_ind = np.where(Umag==Umax)[0][0]

# U.values[Umax_ind].dataDouble # For double checking the Umag

# Coordinates of node with Umax
Umax_coord = COORD.values[Umax_ind].dataDouble

if abs(Umax_coord[1]-d)<0.1*tf:
     # If y-coordinate of maximum is at y-coordinate of top flange
     # Note that the relevant offset for shell flange had been applied
     # in model generation's section assignments
	imp = 0
else:
	imp = 1
	
# write eigenvalue & RM1 to file
fid = open(job_name + '_out.txt', 'w')
fid.write('eigenvalue, imp \\n')
fid.write('%.5f, %g'%(eigenvalue,imp))
fid.close()

'''
    with  open('postproc_' + job_name + '.py','w') as fid:
        fid.write(template.format(**context))