# -*- coding: utf-8 -*-
"""
Created on Wed Jun  8 16:26:04 2022

@author: wjwong
"""
import numpy as np
from scipy.interpolate import interp1d
from scipy import optimize
from os import listdir
from os.path import isfile, join
import pandas as pd
import functions as fn

def solve_R(UR1,RM1):
    """
    Find rotation capacity by finding intersection of moment-rotation curve with M=Mp
    
    Args:
        Normalized rotation
    """
    # Create array of one's with same size as RM1
    RM1_pl = np.ones(np.size(RM1)) 
    # Look for intersections to give estimate on trial value for fsolve
    idx = np.argwhere(np.diff(np.sign(RM1 - RM1_pl))).flatten() 
    if idx.size==1:
        print('Moment-rotation curve has intersected M=Mp only once. Check the curve.')
        return np.nan, np.nan
    elif idx.size==2:
        UR1_trial = UR1[idx[1]]
        fn = interp1d(UR1,RM1)
        def fn_interpolation(x):
            return fn(x)-1 # minus 1 so that solving this function = 0 gives the intersection of the moment-rotation curve with 1
        R_aval = optimize.fsolve(fn_interpolation,UR1_trial)-1 # minus 1 because this is in the definition of rotation capacity
        return R_aval, idx[1]
    elif idx.size==0:
        # print('Moment-rotation curve has not reached Mp. Check the curve.')
        return 0, np.nan
    else:
        print('Moment-rotation curve has intersected M=Mp more than twice. Check the curve.')
        return np.nan, np.nan

def load_data(mypath):
    
    csv_files = [f for f in listdir(mypath) if isfile(join(mypath, f)) and f.endswith(".csv")]
    
    job_names = csv_files
    for i, file in enumerate(csv_files):
        job_names[i] = job_names[i] .replace("_histories.csv","")
    
    # meta_files = [f for f in listdir(mypath) if isfile(join(mypath, f)) and f.endswith(".txt")]
    
    meta_list = list()
    data_list = list()
    Mp = np.zeros(np.size(job_names))
    Tp = np.zeros(np.size(job_names))
    L = np.zeros(np.size(job_names))
    Ix = np.zeros(np.size(job_names))
    Iw = np.zeros(np.size(job_names))
    J = np.zeros(np.size(job_names))
    R = np.zeros(np.size(job_names))
    YoT = np.zeros(np.size(job_names))
    Y = np.zeros(np.size(job_names))
    Ldp = np.zeros(np.size(job_names))
    Ldf = np.zeros(np.size(job_names))
    Ldf_eps = np.zeros(np.size(job_names))
    Ldw = np.zeros(np.size(job_names))
    Ldw_eps = np.zeros(np.size(job_names))
    LdLT = np.zeros(np.size(job_names))
    B = np.zeros(np.size(job_names))
    H = np.zeros(np.size(job_names))
    b = np.zeros(np.size(job_names))
    d = np.zeros(np.size(job_names))
    tf = np.zeros(np.size(job_names))
    tw = np.zeros(np.size(job_names))
    imp =  np.zeros(np.size(job_names))
    Mnu =  np.zeros(np.size(job_names))
    Ru =  np.zeros(np.size(job_names))
    idx_R = np.zeros(np.size(job_names))
    idx_Ru = np.zeros(np.size(job_names),dtype=int)
    Mel = np.zeros(np.size(job_names))
    idx_Mel = np.zeros(np.size(job_names))
    
    for i, name in enumerate(job_names):
        meta_list.append(pd.read_fwf(mypath + name + '_meta.txt',
                                     delimiter=',', widths=[5,6,3,6,6,8,8,6,7],  header=0, names=['YoT', 'Y', 'imp', 'tf', 'tw', 'Bf', 'H', 'LdLT', 'Ldp']))
        data_list.append(pd.read_fwf(mypath + name+'_histories.csv',
                                     delimiter=", ", header=0,names=['UR1','RM1']))
        
        YoT[i] = meta_list[i].YoT.values
        Y[i] = meta_list[i].Y.values
        Ldp[i] = meta_list[i].Ldp.values
        B[i] = meta_list[i].Bf.values # [mm] - full flange width
        H[i] = meta_list[i].H.values
        tf[i] = meta_list[i].tf.values
        tw[i] = meta_list[i].tw.values
        imp[i] = meta_list[i].imp.values
        b[i] = B[i]/2 # [mm] - half the flange width
        d[i] = H[i]-2*tf[i] # [mm] - depth of web
        Ldf[i] = (B[i]-tw[i])/2/tf[i]
        Ldf_eps[i] = Ldf[i]/np.sqrt(235/Y[i])
        Ldw[i] = d[i]/tw[i]
        Ldw_eps[i] = Ldw[i]/np.sqrt(235/Y[i])
        LdLT = meta_list[i].LdLT.values
        L[i] = fn.calc_L(Y[i],Y[i],tf[i],tw[i],B[i],H[i],LdLT)
        sigy_f = meta_list[i].Y.values
        sigy_w = meta_list[i].Y.values
        Mel[i] = fn.calc_Mel(sigy_f,sigy_w,tf[i],tw[i],B[i],H[i])
        
       
        # Plastic section properties
        Mp[i] = 2*(b[i]*tf[i]*(d[i]+tf[i])*sigy_f+d[i]**2/8*tw[i]*sigy_w)
        Ix[i] = tw[i]*d[i]**3/12+b[i]*tf[i]*(d[i]+tf[i])**2+b[i]*tf[i]**3/3 # second moment of area
        Iw[i] = Ix[i]*d[i]**2/4
        J[i] = 1/3*(d[i]*tw[i]**3+4*b[i]*tf[i]**3)
        
        # Mcr_short = Mel[i]/LdLT**2
        
        E = 2e5
        # nu = 0.3
        # G = E/2/(1+nu) # shear modulus
        
        # def func(L,Ix,Iw,J):
            # return Mcr_short - np.pi/L*np.sqrt(E*Ix*G*J)*np.sqrt(1+np.pi**2/L**2*E*Iw/G/J)
        
        # L[i] = optimize.fsolve(func,500,args=(Ix[i],Iw[i],J[i]))
        Tp[i] = Mp[i]*L[i]/E/Ix[i]
    
        R[i], idx_R[i] = solve_R(data_list[i]['UR1'].values/Tp[i],
                                    data_list[i]['RM1'].values/Mp[i])
        
        # Find the maximum moment and the index and the rotation corresponding to it
        Mnu[i] = max(data_list[i]['RM1'].values)/Mp[i]
        idx_Ru[i] = np.argmax(data_list[i]['RM1'].values)
        Ru[i] = data_list[i]['UR1'].values[idx_Ru[i]]/Tp[i]-1
        
        # import pdb; pdb.set_trace()
        # Find the index corresponding to Mel
        idx_Mel[i] = np.abs(data_list[i]['RM1'].values[:idx_Ru[i]] - Mel[i]).argmin()
        
    return YoT, Y, Ldp, B, H, tf, tw, imp, Ldf, Ldw, Mp, Tp, L, R , Ldf_eps, Ldw_eps, Mnu, Ru, idx_R, idx_Ru, Mel, idx_Mel, data_list