# -*- coding: utf-8 -*-
"""
Created on Thu May 19 15:36:19 2022

@author: wjwong
"""

import numpy as np
import matplotlib.pyplot as plt
import pandas as pd
from scipy import optimize
from scipy.interpolate import interp1d
from cycler import cycler # for black and white plotting scheme
import functions as fn

# Function for saving images as emf for use in word
# First save as svg, then use Inkscape to convert to emf
import subprocess
import os
inkscapePath = r'C:\Program Files\Inkscape\bin\inkscape.exe'
savePath= os.getcwd()+'\Images'
def exportEmf(savePath, plotName, fig=None, keepSVG=False):
    """Save a figure as an emf file
    
    Parameters
    ----------
    savePath : str, the path to the directory you want the image saved in
    plotName : str, the name of the image 
    fig : matplotlib figure, (optional, default uses gca)
    keepSVG : bool, whether to keep the interim svg file
    """

    figFolder = savePath + r"\{}.{}"
    svgFile = figFolder.format(plotName,"svg")
    emfFile = figFolder.format(plotName,"emf")
    if fig:
        use=fig
    else:
        use=plt
    use.savefig(svgFile, bbox_inches='tight')
    subprocess.run([inkscapePath, svgFile, '--export-filename', emfFile])
 
    if not keepSVG:
        os.system('del "{}"'.format(svgFile))

plt.close('all')

E = 2e5 # MPa

# %% Finite element data 

job_names_validation = ['Yf_609_YoTf_0p88_Yw_609_YoTw_0p88_tf_13p0_tw_10p0_Bf_152p0_H_305p0_LdLT_0p11']
                        
R_aval_validation = np.zeros(np.size(job_names_validation))

exp_name = ['Specimen 7 (Green et al. 2002)']
R_aval_exp = np.zeros(np.size(exp_name))

# %% Find rotation capacity

def solve_R(UR1,RM1):
    """
    Find rotation capacity by finding intersection of moment-rotation curve with M=Mp
    
    Args:
        Normalized rotation
        Normalized moment
    """
    # Create array of one's with same size as RM1
    RM1_pl = np.ones(np.size(RM1)) 
    # Look for intersections to give estimate on trial value for fsolve
    idx = np.argwhere(np.diff(np.sign(RM1 - RM1_pl))).flatten() 
    if idx.size==1:
        # print('Moment-rotation curve has intersected M=Mp only once. Check the curve.')
        return np.nan
    elif idx.size==2:
        UR1_trial = UR1[idx[1]]
    else:
        # print('Moment-rotation curve has intersected M=Mp more than twice. Check the curve.')
        return np.nan
    fn = interp1d(UR1,RM1)
    def fn_interpolation(x):
        return fn(x)-1 # minus 1 so that solving this function = 0 gives the intersection of the moment-rotation curve with 1
    R_aval = optimize.fsolve(fn_interpolation,UR1_trial)-1 # minus 1 because this is in the definition of rotation capacity
    return R_aval

# %% Calculate analytical M/Mpl

def calc_MoMp(kappa,E,Y,n):
    eps_out = kappa 

# %% Material and geometrical properties

# Y13-4

sigy_f_HSA800_C = 609 # [MPa] - initial yield stress flange
sigy_w_HSA800_C = 609 # [MPa] - initial yield stress web

B_HSA800_C = 152
b_HSA800_C = B_HSA800_C/2
t_HSA800_C = 13 # [mm] - thickness of flange
H_HSA800_C = 305 # [mm] - total depth of beam
d_HSA800_C = H_HSA800_C-2*t_HSA800_C # [mm] - depth of web
v_HSA800_C = 10 # [mm] - web thickness
L_HSA800_C = 915

# Plastic section properties
Mp_HSA800_C = 2*(b_HSA800_C*t_HSA800_C*(d_HSA800_C+t_HSA800_C)*sigy_f_HSA800_C+d_HSA800_C**2/8*v_HSA800_C*sigy_w_HSA800_C)
Ix_HSA800_C = v_HSA800_C*d_HSA800_C**3/12+b_HSA800_C*t_HSA800_C*(d_HSA800_C+t_HSA800_C)**2+b_HSA800_C*t_HSA800_C**3/3 # second moment of area
kappa_HSA800_C = Mp_HSA800_C/E/Ix_HSA800_C
Tp_HSA800_C = Mp_HSA800_C*L_HSA800_C/E/Ix_HSA800_C
Mel_HSA800_C = fn.calc_Mel(sigy_f_HSA800_C, sigy_w_HSA800_C, t_HSA800_C, v_HSA800_C, B_HSA800_C, H_HSA800_C)
Tel_HSA800_C =  Mel_HSA800_C*L_HSA800_C/E/Ix_HSA800_C

# Y15-4 

sigy_f_HSA800_NC = 754 # [MPa] - initial yield stress flange
sigy_w_HSA800_NC = 781 # [MPa] - initial yield stress web

B_HSA800_NC = 161.07
b_HSA800_NC = B_HSA800_NC/2
t_HSA800_NC = 16.24 # [mm] - thickness of flange
H_HSA800_NC = 353.03 # [mm] - total depth of beam
d_HSA800_NC = H_HSA800_NC-2*t_HSA800_NC # [mm] - depth of web
v_HSA800_NC = 9.58 # [mm] - web thickness
L_HSA800_NC = 1500

# Plastic section properties
Mp_HSA800_NC = 2*(b_HSA800_NC*t_HSA800_NC*(d_HSA800_NC+t_HSA800_NC)*sigy_f_HSA800_NC+d_HSA800_NC**2/8*v_HSA800_NC*sigy_w_HSA800_NC)
Ix_HSA800_NC = v_HSA800_NC*d_HSA800_NC**3/12+b_HSA800_NC*t_HSA800_NC*(d_HSA800_NC+t_HSA800_NC)**2+b_HSA800_NC*t_HSA800_NC**3/3 # second moment of area
kappa_HSA800_NC = Mp_HSA800_NC/E/Ix_HSA800_NC
Tp_HSA800_NC = Mp_HSA800_NC*L_HSA800_NC/E/Ix_HSA800_NC
Mel_HSA800_NC = fn.calc_Mel(sigy_f_HSA800_NC, sigy_w_HSA800_NC, t_HSA800_NC, v_HSA800_NC, B_HSA800_NC, H_HSA800_NC)
Tel_HSA800_NC =  Mel_HSA800_NC*L_HSA800_NC/E/Ix_HSA800_NC

# HSB800-NC-LP-4-A

sigy_f_HSB800_NC = 991 # [MPa] - initial yield stress flange
sigy_w_HSB800_NC = 991 # [MPa] - initial yield stress web

B_HSB800_NC = 220
b_HSB800_NC = B_HSB800_NC/2
t_HSB800_NC = 17.6 # [mm] - thickness of flange
H_HSB800_NC = 400 # [mm] - total depth of beam
d_HSB800_NC = H_HSB800_NC-2*t_HSB800_NC # [mm] - depth of web
v_HSB800_NC = 17.6 # [mm] - web thickness
L_HSB800_NC = 1400

# Plastic section properties
Mp_HSB800_NC = 2*(b_HSB800_NC*t_HSB800_NC*(d_HSB800_NC+t_HSB800_NC)*sigy_f_HSB800_NC+d_HSB800_NC**2/8*v_HSB800_NC*sigy_w_HSB800_NC)
Ix_HSB800_NC = v_HSB800_NC*d_HSB800_NC**3/12+b_HSB800_NC*t_HSB800_NC*(d_HSB800_NC+t_HSB800_NC)**2+b_HSB800_NC*t_HSB800_NC**3/3 # second moment of area
kappa_HSB800_NC = Mp_HSB800_NC/E/Ix_HSB800_NC
Tp_HSB800_NC = Mp_HSB800_NC*L_HSB800_NC/E/Ix_HSB800_NC
Mel_HSB800_NC = fn.calc_Mel(sigy_f_HSB800_NC, sigy_w_HSB800_NC, t_HSB800_NC, v_HSB800_NC, B_HSB800_NC, H_HSB800_NC)
Tel_HSB800_NC =  Mel_HSB800_NC*L_HSB800_NC/E/Ix_HSB800_NC

Mp_val = np.array([Mp_HSA800_C, Mp_HSA800_NC, Mp_HSB800_NC])
Tp_val = np.array([Tp_HSA800_C, Tp_HSA800_NC, Tp_HSB800_NC])
kappa_val = np.array([kappa_HSA800_C, kappa_HSA800_NC, kappa_HSB800_NC])
Mel_val = np.array([Mel_HSA800_C,Mel_HSA800_NC,Mel_HSB800_NC])
Tel_val = np.array([Tel_HSA800_C,Tel_HSA800_NC,Tel_HSB800_NC])

datalist_validation = list()
zerod_UR1norm = list() # UR1norm after applying offset
zerod_UR1norm_el = list() # UR1norm after applying offset
RM1norm_FE = list() # RM1norm from FE model
RM1norm_el_FE = list() # RM1norm from FE model
altUR1norm = list() # attempt to calculate normalized rotation by estimating beam end rotation of 4-point bending system
zerod_altUR1norm = list()
exp_data = list()

Lout = [915]
Lin = [915]

Mel_o_Mp = np.zeros(np.size(job_names_validation))
offset_UR1 = np.zeros(np.size(job_names_validation))

RM1norm_max_FE = np.zeros(np.size(job_names_validation)) # maximum normalized moment
RM1norm_max_exp = np.zeros(np.size(exp_name)) 
RM1norm_diff = np.zeros(np.size(exp_name))
idx_RM1norm_max_FE = np.zeros(np.size(job_names_validation), dtype=int)
idx_RM1norm_max_exp = np.zeros(np.size(exp_name), dtype=int) 
UR1norm_for_RM1norm_max_FE = np.zeros(np.size(job_names_validation)) # UR1norm when RM1norm is max
UR1norm_for_RM1norm_max_exp = np.zeros(np.size(exp_name)) # UR1norm when RM1norm is max
UR1norm_for_RM1norm_max_diff_abs = np.zeros(np.size(exp_name))
UR1norm_for_RM1norm_max_diff = np.zeros(np.size(exp_name))
UR1norm_aval_FE = np.zeros(np.size(job_names_validation)) # UR1norm when RM1norm drops below 1
UR1norm_aval_exp = np.zeros(np.size(exp_name)) # UR1norm when RM1norm drops below 1
UR1norm_capacity_exp = np.zeros(np.size(exp_name)) # The UR1norm over which RM1norm drops from max to below RMnorm=1
RM1norm_reserve_exp = np.zeros(np.size(exp_name)) # RM1norm excess of RM1norm=1
UR1norm_aval_FE_asper_exp = np.zeros(np.size(job_names_validation)) # UR1norm at which RM1norm has dropped by an amount equal to RM1norm_reserve_exp
idx_UR1norm_aval_FE_asper_exp = np.zeros(np.size(job_names_validation), dtype=int) # The index corresponding to the point in the FE curve when the moment has dropped by an amount equal to RM1norm_reserve_exp
UR1norm_capacity_FE_asper_exp = np.zeros(np.size(job_names_validation)) # The UR1norm over which RM1norm drops by an amount equal to RM1norm_reserve_exp
UR1norm_capacity_diff = np.zeros(np.size(exp_name)) # difference (as a ratio) between capacity of experiment and FE, where capacity is the UR1norm over which the RM1 drops by an amount equal to RM1norm_reserve_exp

# based on altUR1:
altUR1norm_for_RM1norm_max_FE = np.zeros(np.size(exp_name))
altUR1norm_for_RM1norm_max_diff_abs = np.zeros(np.size(exp_name))
altUR1norm_for_RM1norm_max_diff = np.zeros(np.size(exp_name))
altUR1norm_aval_FE_asper_exp = np.zeros(np.size(exp_name))
altUR1norm_capacity_FE_asper_exp = np.zeros(np.size(exp_name))
altUR1norm_capacity_diff = np.zeros(np.size(exp_name))
R_exp = np.zeros(np.size(exp_name))
R_FE = np.zeros(np.size(exp_name))
R_diff = np.zeros(np.size(exp_name))

fig, ax = plt.subplots() # normalize against pl with both rotation definitions
figb, axb = plt.subplots() # normalize against el with both rotation definitions
figc, axc = plt.subplots() # normalize against pl with alt rotation definition

color_cycle = ['C1','C2','C4']
marker_cycle = ['.','o','x']

for i, name in enumerate(job_names_validation):
    
    # ax.plot([0, 100], [1, 1], label='Plastic moment')

    exp_data.append(pd.read_csv(exp_name[i] + '.csv', header=0,names=['UR1norm','RM1norm']))

    datalist_validation.append(pd.read_csv(name+'_histories.csv', delimiter=", ",engine='python',
                                           header=0,names=['UR1','RM1', 'MS_COOR1', 'MS_COOR2', 'MS_COOR3', 'RP_COOR1', 'RP_COOR2', 'RP_COOR3', 'ALLSD', 'ALLSE']))

    # Attempt to work out theoretical 4-point-bending-beam-end rotation by 
    # adding elastic rotations 
    # of the non-uniform moment region to the rotation at the end of the 
    # uniform moment region
    UR1new = np.zeros(np.size(datalist_validation[i]['UR1']))
    for j,UR1 in enumerate(datalist_validation[i]['UR1']):
        # UR1new[j] = (UR1 + min(UR1,Tp_val[i]))/Tp_val[i]
        # UR1new[j] = UR1/Lout/kappa_val[i] + min(UR1,Tp_val[i])/Tp_val[i]
        UR1new[j] = (0.5*UR1 + 0.5*Lout[i]/Lin[i]*min(UR1,Tp_val[i]))/(0.5*Lout[i]*kappa_val[i]+0.5*Tp_val[i])
    
    altUR1norm.append(UR1new)
    zerod_altUR1norm.append(UR1new - UR1new[0])
    
    # correct for small rotation that occurs in relaxation of residual stresses
    offset_UR1[i] = datalist_validation[i]['UR1'].values[0]
    
    zerod_UR1norm.append((datalist_validation[i]['UR1'].values-offset_UR1[i])/Tp_val[i])
    zerod_UR1norm_el.append((datalist_validation[i]['UR1'].values-offset_UR1[i])/Tel_val[i])
    RM1norm_FE.append(datalist_validation[i]['RM1'].values/Mp_val[i])
    RM1norm_el_FE.append(datalist_validation[i]['RM1'].values/Mel_val[i])
    
    # fig# - normalize against pl with both rotation definitions
    ax.plot(zerod_UR1norm[i], RM1norm_FE[i],
            label=exp_name[i] + ' FE rotation in modelled length', color=color_cycle[i],
            linestyle=':')
    ax.plot(zerod_altUR1norm[i], datalist_validation[i]['RM1'].values/Mp_val[i],
            label=exp_name[i] + ' FE rotation at end by superimposition', color=color_cycle[i],
            linestyle='-')
    ax.plot(exp_data[i]['UR1norm'].values, exp_data[i]['RM1norm'].values,
            label=exp_name[i] + ' experiment', color=color_cycle[i],
            marker='.', linestyle='--')
    
    # figb -# normalize against el with both rotation definitions
    axb.plot(zerod_UR1norm_el[i], RM1norm_el_FE[i],
             label=exp_name[i] + ' FE rotation in modelled length', color=color_cycle[i],
             linestyle=':')
    axb.plot(zerod_altUR1norm[i]*Tp_val[i]/Tel_val[i], RM1norm_el_FE[i],
             label=exp_name[i] + ' FE rotation at end by superimposition', color=color_cycle[i],
             linestyle='-')    
    axb.plot(exp_data[i]['UR1norm'].values*Tp_val[i]/Tel_val[i], exp_data[i]['RM1norm'].values*Mp_val[i]/Mel_val[i],
        label=exp_name[i] + ' experiment', color=color_cycle[i], marker='.', linestyle='--')
    
    # figc - normalize against pl with alt rotation definition
    axc.plot(zerod_altUR1norm[i], datalist_validation[i]['RM1'].values/Mp_val[i],
            label=exp_name[i] + ' FE', color=color_cycle[i],
            linestyle='-')
    axc.plot(exp_data[i]['UR1norm'].values, exp_data[i]['RM1norm'].values,
            label=exp_name[i] + ' experiment', color=color_cycle[i],
            marker='.', linestyle='--')
    
    # ax2 = ax.twiny()
    # new_tick_locations = np.append(np.array([0, Tel_val[i]/Tp_val[i]]), np.arange(1.5,3.5,0.5))
    
    # Mel_o_Mp[i] = Mel_val[i]/Mp_val[i]
    
    # def tick_function(X):
    #     V = X*Tp_val[i]/Tel_val[i]
    #     return ["%.1f" % z for z in V]
    
    # ax2.set_xticks(new_tick_locations)
    # ax2.set_xticklabels(tick_function(new_tick_locations))
    # ax2.set_xlim(ax.get_xlim())
    # ax2.set_ylim(ax.get_ylim())
    # ax2.set_xlabel(r"$\theta/\theta_{el}$", fontsize=20)
    # ax2.tick_params(axis='both', which='major', labelsize=11)
    # ax2.tick_params(axis='both', which='minor', labelsize=11)
    
# %% Compare maximum moments and UR1norm_capacity

    R_aval_validation[i] = solve_R(zerod_UR1norm[i],
                                datalist_validation[i]['RM1'].values/Mp_val[i])
    
    R_aval_exp[i] = solve_R(exp_data[i]['UR1norm'].values,exp_data[i]['RM1norm'].values)
    
    UR1norm_aval_FE[i] = R_aval_validation[i] + 1
    UR1norm_aval_exp[i] = R_aval_exp[i] + 1
    
    RM1norm_max_FE[i] = max(datalist_validation[i]['RM1'].values/Mp_val[i])
    RM1norm_max_exp[i] = max(exp_data[i]['RM1norm'].values)
    RM1norm_diff[i] = (RM1norm_max_exp[i] - RM1norm_max_FE[i]) / RM1norm_max_FE[i]
    RM1norm_reserve_exp[i] = RM1norm_max_exp[i] - 1
    
    idx_RM1norm_max_FE[i] = np.argmax(RM1norm_FE[i])
    idx_RM1norm_max_exp[i] = np.argmax(exp_data[i]['RM1norm'].values)
    
    UR1norm_for_RM1norm_max_FE[i] = zerod_UR1norm[i][idx_RM1norm_max_FE[i]]
    UR1norm_for_RM1norm_max_exp[i] = exp_data[i]['UR1norm'].values[idx_RM1norm_max_exp[i]]
    UR1norm_for_RM1norm_max_diff_abs[i] = (UR1norm_for_RM1norm_max_exp[i] - UR1norm_for_RM1norm_max_FE[i])
    UR1norm_for_RM1norm_max_diff[i] = (UR1norm_for_RM1norm_max_exp[i] - UR1norm_for_RM1norm_max_FE[i]) / UR1norm_for_RM1norm_max_exp[i]  
    
    UR1norm_capacity_exp[i] = UR1norm_aval_exp[i] - UR1norm_for_RM1norm_max_exp[i]
    
    idx_UR1norm_aval_FE_asper_exp[i] = np.argmin(abs(RM1norm_FE[i] - (RM1norm_max_FE[i]-RM1norm_reserve_exp[i])))
    
    UR1norm_aval_FE_asper_exp[i] = zerod_UR1norm[i][idx_UR1norm_aval_FE_asper_exp[i]]
    UR1norm_capacity_FE_asper_exp[i] = UR1norm_aval_FE_asper_exp[i] -  UR1norm_for_RM1norm_max_FE[i]
    
    UR1norm_capacity_diff[i] = (UR1norm_capacity_exp[i] - UR1norm_capacity_FE_asper_exp[i]) / UR1norm_capacity_exp[i]
    
    # %% for alternate zerod_UR1norm
    
    altUR1norm_for_RM1norm_max_FE[i] = zerod_altUR1norm[i][idx_RM1norm_max_FE[i]]
    altUR1norm_for_RM1norm_max_diff_abs[i] = (UR1norm_for_RM1norm_max_exp[i] - altUR1norm_for_RM1norm_max_FE[i])
    altUR1norm_for_RM1norm_max_diff[i] = (UR1norm_for_RM1norm_max_exp[i] - altUR1norm_for_RM1norm_max_FE[i]) / UR1norm_for_RM1norm_max_exp[i]  
    altUR1norm_aval_FE_asper_exp[i] = zerod_altUR1norm[i][idx_UR1norm_aval_FE_asper_exp[i]]
    altUR1norm_capacity_FE_asper_exp[i] = altUR1norm_aval_FE_asper_exp[i] -  altUR1norm_for_RM1norm_max_FE[i]
    altUR1norm_capacity_diff[i] = (UR1norm_capacity_exp[i] - altUR1norm_capacity_FE_asper_exp[i]) / UR1norm_capacity_exp[i]
    
    R_exp[i] = solve_R(exp_data[i]['UR1norm'].values, exp_data[i]['RM1norm'].values)
    R_FE[i] = solve_R(zerod_altUR1norm[i], datalist_validation[i]['RM1'].values/Mp_val[i])
    R_diff[i] = (R_exp[i] - R_FE[i])/R_exp[i]
    
ax.set_xlabel(r'$\theta/\theta_{pl}$',  fontsize=20)
ax.set_ylabel(r'$M/M_{pl}$', fontsize=20)
ax.grid()
# ax.set_xlim(0, exp_data[i]['UR1norm'].values[-1])
ax.set_xlim(0, exp_data[i]['UR1norm'].values[-1]+5)
ax.set_ylim(bottom=0)
ax.tick_params(axis='both', which='major', labelsize=12)
ax.tick_params(axis='both', which='minor', labelsize=12)
ax.legend(fontsize=10, framealpha=1) 

axb.set_xlabel(r'$\theta/\theta_{el}$',  fontsize=20)
axb.set_ylabel(r'$M/M_{el}$', fontsize=20)
axb.grid()
# axb.set_xlim(0, exp_data[i]['UR1norm'].values[-1])
# axb.set_ylim(bottom=0)
axb.tick_params(axis='both', which='major', labelsize=12)
axb.tick_params(axis='both', which='minor', labelsize=12)
axb.legend(fontsize=10, framealpha=1) 

axc.set_xlabel(r'$\theta/\theta_{pl}$',  fontsize=20)
axc.set_ylabel(r'$M/M_{pl}$', fontsize=20)
axc.grid()
axc.set_xlim(0, 5.5)
axc.set_ylim(bottom=0)
axc.tick_params(axis='both', which='major', labelsize=12)
axc.tick_params(axis='both', which='minor', labelsize=12)
axc.legend(fontsize=11, framealpha=1) 

# figc.savefig('MR_Green.eps', format='eps', bbox_inches='tight')
# exportEmf(savePath, 'MR_Green', fig=figc, keepSVG=True)

# %%

print(['RM1norm_diff', RM1norm_diff])
print(['UR1norm_for_RM1norm_max_diff', UR1norm_for_RM1norm_max_diff])
print(['altUR1norm_for_RM1norm_max_diff', altUR1norm_for_RM1norm_max_diff])
print(['UR1norm_capacity_diff', UR1norm_capacity_diff])
print(['altUR1norm_capacity_diff', altUR1norm_capacity_diff])
print(['R_diff', R_diff])